@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #     
@@ -37,7 +37,7 @@ def Print(string):
   print string,
   f.write(string+' ')
   
-def Write(string):
+def write(string):
   f.write(string)
   f.write('\n')
   
