@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #     
@@ -26,13 +26,13 @@ import petscconf
 import log
 import check
 
-def Check(conf,directory,libs):
+def Check(conf,vars,cmake,tmpdir,directory,libs):
   
-  log.Write('='*80)
+  log.write('='*80)
   log.Println('Checking PRIMME library...')
 
   if petscconf.PRECISION == 'single':
-    sys.exit('ERROR: PRIMME does not support single precision.')
+    log.Exit('ERROR: PRIMME does not support single precision.')
  
   functions_base = ['primme_set_method','primme_Free','primme_initialize']
   if directory:
@@ -55,13 +55,16 @@ def Check(conf,directory,libs):
     else:
       l =  libs
       f = []
-    if check.Link(functions,[],l+f):
-      conf.write('SLEPC_HAVE_PRIMME = -DSLEPC_HAVE_PRIMME\n')
-      conf.write('PRIMME_LIB =' + str.join(' ', l) + '\n')
-      conf.write('PRIMME_FLAGS =' + str.join(' ', f) + '\n')
+    if check.Link(tmpdir,functions,[],l+f):
+      conf.write('#ifndef SLEPC_HAVE_PRIMME\n#define SLEPC_HAVE_PRIMME 1\n#endif\n\n')
+      vars.write('PRIMME_LIB = ' + str.join(' ', l) + '\n')
+      vars.write('PRIMME_FLAGS = ' + str.join(' ', f) + '\n')
+      cmake.write('set (SLEPC_HAVE_PRIMME YES)\n')
+      cmake.write('find_library (PRIMME_LIB primme HINTS '+ d +')\n')
+      cmake.write('find_path (PRIMME_INCLUDE primme.h ' + d + '/PRIMMESRC/COMMONSRC)\n')
       return l+f
 
   log.Println('ERROR: Unable to link with PRIMME library')
-  print 'ERROR: In directories',dirs
-  print 'ERROR: With flags',libs,
+  log.Println('ERROR: In directories '+''.join([s+' ' for s in dirs]))
+  log.Println('ERROR: With flags '+''.join([s+' ' for s in libs]))
   log.Exit('')
