@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #     
@@ -25,7 +25,7 @@ import sys
 import petscconf
 import check
 
-def Check(conf,directory,libs):
+def Check(conf,vars,cmake,tmpdir,directory,libs):
 
   if petscconf.SCALAR == 'complex':
     sys.exit('ERROR: TRLAN does not support complex numbers.') 
@@ -44,4 +44,4 @@ def Check(conf,directory,libs):
   else:
     dirs = check.GenerateGuesses('TRLan')
     
-  return check.FortranLib(conf,'TRLAN',dirs,libs,functions)
+  return check.FortranLib(tmpdir,conf,vars,cmake,'TRLAN',dirs,libs,functions)
