@@ -4,7 +4,7 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !     
@@ -29,13 +29,11 @@
       PetscEnum EPS_CONVERGED_TOL        
       PetscEnum EPS_DIVERGED_ITS
       PetscEnum EPS_DIVERGED_BREAKDOWN
-      PetscEnum EPS_DIVERGED_NONSYMMETRIC
       PetscEnum EPS_CONVERGED_ITERATING
 
       parameter (EPS_CONVERGED_TOL          =  2)
       parameter (EPS_DIVERGED_ITS           = -3)
       parameter (EPS_DIVERGED_BREAKDOWN     = -4)
-      parameter (EPS_DIVERGED_NONSYMMETRIC  = -5)
       parameter (EPS_CONVERGED_ITERATING    =  0)
 
       PetscEnum EPS_HEP
@@ -59,6 +57,7 @@
       PetscEnum EPS_TARGET_MAGNITUDE
       PetscEnum EPS_TARGET_REAL
       PetscEnum EPS_TARGET_IMAGINARY
+      PetscEnum EPS_ALL
       PetscEnum EPS_WHICH_USER
 
       parameter (EPS_LARGEST_MAGNITUDE      =  1)
@@ -70,7 +69,8 @@
       parameter (EPS_TARGET_MAGNITUDE       =  7)
       parameter (EPS_TARGET_REAL            =  8)
       parameter (EPS_TARGET_IMAGINARY       =  9)
-      parameter (EPS_WHICH_USER             = 10)
+      parameter (EPS_ALL                    = 10)
+      parameter (EPS_WHICH_USER             = 11)
        
       PetscEnum EPS_BALANCE_NONE
       PetscEnum EPS_BALANCE_ONESIDE
@@ -170,12 +170,15 @@
       parameter (EPS_PRIMME_PRECOND_NONE        =  0)
       parameter (EPS_PRIMME_PRECOND_DIAGONAL    =  1)
 
+!
+!   Possible arguments to EPSMonitorSet()
+!
       external EPSMONITORALL
       external EPSMONITORLG
+      external EPSMONITORLGALL
       external EPSMONITORCONVERGED
       external EPSMONITORFIRST
 
-!PETSC_DEC_ATTRIBUTES(EPSMONITORALL,'_EPSMONITORALL')
-!PETSC_DEC_ATTRIBUTES(EPSMONITORLG,'_EPSMONITORLG')
-!PETSC_DEC_ATTRIBUTES(EPSMONITORCONVERGED,'_EPSMONITORCONVERGED')
-!PETSC_DEC_ATTRIBUTES(EPSMONITORFIRST,'_EPSMONITORFIRST')
+!
+!  End of Fortran include file for the EPS package in SLEPc
+!
