@@ -4,7 +4,7 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !     
@@ -28,8 +28,7 @@
 #define IP PetscFortranAddr
 #endif
 
-#define IPOrthogonalizationType PetscEnum
-#define IPOrthogonalizationRefinementType PetscEnum
-#define IPBilinearForm PetscEnum
+#define IPOrthogType       PetscEnum
+#define IPOrthogRefineType PetscEnum
 
 #endif
