@@ -4,7 +4,7 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+!  Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !     
@@ -47,3 +47,16 @@
 
       parameter (SVD_LARGEST                =  0)
       parameter (SVD_SMALLEST               =  1)
+
+!
+!   Possible arguments to SVDMonitorSet()
+!
+      external SVDMONITORALL
+      external SVDMONITORLG
+      external SVDMONITORLGALL
+      external SVDMONITORCONVERGED
+      external SVDMONITORFIRST
+
+!
+!  End of Fortran include file for the SVD package in SLEPc
+!
