@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -22,7 +22,7 @@
 #ifndef _EPSIMPL
 #define _EPSIMPL
 
-#include "slepceps.h"
+#include <slepceps.h>
 
 extern PetscFList EPSList;
 extern PetscLogEvent EPS_SetUp, EPS_Solve, EPS_Dense;
@@ -35,6 +35,7 @@ struct _EPSOps {
   PetscErrorCode  (*setfromoptions)(EPS);
   PetscErrorCode  (*publishoptions)(EPS);
   PetscErrorCode  (*destroy)(EPS);
+  PetscErrorCode  (*reset)(EPS);
   PetscErrorCode  (*view)(EPS,PetscViewer);
   PetscErrorCode  (*backtransform)(EPS);
   PetscErrorCode  (*computevectors)(EPS);
@@ -63,18 +64,19 @@ struct _p_EPS {
   PetscErrorCode (*conv_func)(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
   void           *conv_ctx;
   EPSWhich       which;            /* which part of the spectrum to be sought */
-  PetscTruth     leftvecs;         /* if left eigenvectors are requested */
+  PetscBool      leftvecs;         /* if left eigenvectors are requested */
   PetscErrorCode (*which_func)(EPS,PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*,void*);
   void           *which_ctx;
+  PetscReal      inta, intb;       /* interval [a,b] for spectrum slicing */
   EPSProblemType problem_type;     /* which kind of problem to be solved */
   EPSExtraction  extraction;       /* which kind of extraction to be applied */
   EPSBalance     balance;          /* the balancing method */
   PetscInt       balance_its;      /* number of iterations of the balancing method */
   PetscReal      balance_cutoff;   /* cutoff value for balancing */
   PetscReal      nrma, nrmb;       /* matrix norms */
-  PetscTruth     adaptive;         /* whether matrix norms are adaptively improved */
-  PetscTruth     trueres;          /* whether the true residual norm must be computed */
-  PetscTruth     trackall;         /* whether all the residuals must be computed */
+  PetscBool      adaptive;         /* whether matrix norms are adaptively improved */
+  PetscBool      trueres;          /* whether the true residual norm must be computed */
+  PetscBool      trackall;         /* whether all the residuals must be computed */
 
   /*------------------------- Working data --------------------------*/
   Vec         D,                /* diagonal matrix for balancing */
@@ -96,67 +98,52 @@ struct _p_EPS {
               nv,               /* size of current Schur decomposition */
               n, nloc,          /* problem dimensions (global, local) */
               allocated_ncv;    /* number of basis vectors allocated */
-  PetscTruth  evecsavailable;   /* computed eigenvectors */
+  PetscBool   evecsavailable;   /* computed eigenvectors */
   PetscRandom rand;             /* random number generator */
+  Vec         t;                /* template vector */
 
   /* ---------------- Default work-area and status vars -------------------- */
   PetscInt   nwork;
   Vec        *work;
 
-  PetscTruth ds_ortho;         /* if DS vectors have been stored and orthonormalized */  
+  PetscBool  ds_ortho;         /* if DS vectors have been stored and orthonormalized */  
   PetscInt   setupcalled;
-  PetscTruth isgeneralized,
+  PetscBool  isgeneralized,
              ispositive,
              ishermitian;
   EPSConvergedReason reason;     
 
   PetscErrorCode (*monitor[MAXEPSMONITORS])(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*); 
-  PetscErrorCode (*monitordestroy[MAXEPSMONITORS])(void*);
+  PetscErrorCode (*monitordestroy[MAXEPSMONITORS])(void**);
   void       *monitorcontext[MAXEPSMONITORS];
   PetscInt    numbermonitors; 
 };
 
-#define EPSMonitor(eps,it,nconv,eigr,eigi,errest,nest) \
-        { PetscErrorCode _ierr; PetscInt _i,_im = eps->numbermonitors; \
-          for ( _i=0; _i<_im; _i++ ) {\
-            _ierr=(*eps->monitor[_i])(eps,it,nconv,eigr,eigi,errest,nest,eps->monitorcontext[_i]);\
-            CHKERRQ(_ierr); \
-	  } \
-	}
+extern PetscErrorCode EPSMonitor(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt);
 
-/* context for EPSMonitorConverged */
-typedef struct {
-  PetscViewerASCIIMonitor viewer;
-  PetscInt oldnconv;
-} EPSMONITOR_CONV;
-EXTERN PetscErrorCode EPSMonitorDestroy_Converged(EPSMONITOR_CONV*);
-
-EXTERN PetscErrorCode EPSRegisterAll(char *);
-EXTERN PetscErrorCode EPSInitializePackage(char *);
-EXTERN PetscErrorCode EPSFinalizePackage(void);
-
-EXTERN PetscErrorCode EPSDestroy_Default(EPS);
-EXTERN PetscErrorCode EPSDefaultGetWork(EPS,PetscInt);
-EXTERN PetscErrorCode EPSDefaultFreeWork(EPS);
-EXTERN PetscErrorCode EPSAllocateSolution(EPS);
-EXTERN PetscErrorCode EPSFreeSolution(EPS);
-EXTERN PetscErrorCode EPSBackTransform_Default(EPS);
-EXTERN PetscErrorCode EPSComputeVectors_Default(EPS);
-EXTERN PetscErrorCode EPSComputeVectors_Hermitian(EPS);
-EXTERN PetscErrorCode EPSComputeVectors_Schur(EPS);
-EXTERN PetscErrorCode EPSComputeResidualNorm_Private(EPS,PetscScalar,PetscScalar,Vec,Vec,PetscReal*);
-EXTERN PetscErrorCode EPSComputeRelativeError_Private(EPS,PetscScalar,PetscScalar,Vec,Vec,PetscReal*);
-EXTERN PetscErrorCode EPSComputeTrueResidual(EPS,PetscScalar,PetscScalar,PetscScalar*,Vec*,PetscInt,PetscReal*);
+extern PetscErrorCode EPSReset_Default(EPS);
+extern PetscErrorCode EPSDefaultGetWork(EPS,PetscInt);
+extern PetscErrorCode EPSDefaultFreeWork(EPS);
+extern PetscErrorCode EPSDefaultSetWhich(EPS);
+extern PetscErrorCode EPSAllocateSolution(EPS);
+extern PetscErrorCode EPSFreeSolution(EPS);
+extern PetscErrorCode EPSBackTransform_Default(EPS);
+extern PetscErrorCode EPSComputeVectors_Default(EPS);
+extern PetscErrorCode EPSComputeVectors_Hermitian(EPS);
+extern PetscErrorCode EPSComputeVectors_Schur(EPS);
+extern PetscErrorCode EPSComputeResidualNorm_Private(EPS,PetscScalar,PetscScalar,Vec,Vec,PetscReal*);
+extern PetscErrorCode EPSComputeRelativeError_Private(EPS,PetscScalar,PetscScalar,Vec,Vec,PetscReal*);
+extern PetscErrorCode EPSComputeTrueResidual(EPS,PetscScalar,PetscScalar,PetscScalar*,Vec*,PetscInt,PetscReal*);
 
 /* Private functions of the solver implementations */
 
-EXTERN PetscErrorCode EPSBasicArnoldi(EPS,PetscTruth,PetscScalar*,PetscInt,Vec*,PetscInt,PetscInt*,Vec,PetscReal*,PetscTruth*);
-EXTERN PetscErrorCode EPSDelayedArnoldi(EPS,PetscScalar*,PetscInt,Vec*,PetscInt,PetscInt*,Vec,PetscReal*,PetscTruth*);
-EXTERN PetscErrorCode EPSDelayedArnoldi1(EPS,PetscScalar*,PetscInt,Vec*,PetscInt,PetscInt*,Vec,PetscReal*,PetscTruth*);
-EXTERN PetscErrorCode EPSKrylovConvergence(EPS,PetscTruth,PetscInt,PetscInt,PetscScalar*,PetscInt,PetscScalar*,Vec*,PetscInt,PetscReal,PetscReal,PetscInt*,PetscScalar*);
-EXTERN PetscErrorCode EPSFullLanczos(EPS,PetscReal*,PetscReal*,Vec*,PetscInt,PetscInt*,Vec,PetscTruth*);
-EXTERN PetscErrorCode EPSTranslateHarmonic(PetscInt,PetscScalar*,PetscInt,PetscScalar,PetscScalar,PetscScalar*,PetscScalar*);
-EXTERN PetscErrorCode EPSBuildBalance_Krylov(EPS);
-EXTERN PetscErrorCode EPSProjectedKSNonsym(EPS,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscInt);
+extern PetscErrorCode EPSBasicArnoldi(EPS,PetscBool,PetscScalar*,PetscInt,Vec*,PetscInt,PetscInt*,Vec,PetscReal*,PetscBool*);
+extern PetscErrorCode EPSDelayedArnoldi(EPS,PetscScalar*,PetscInt,Vec*,PetscInt,PetscInt*,Vec,PetscReal*,PetscBool*);
+extern PetscErrorCode EPSDelayedArnoldi1(EPS,PetscScalar*,PetscInt,Vec*,PetscInt,PetscInt*,Vec,PetscReal*,PetscBool*);
+extern PetscErrorCode EPSKrylovConvergence(EPS,PetscBool,PetscBool,PetscInt,PetscInt,PetscScalar*,PetscInt,PetscScalar*,Vec*,PetscInt,PetscReal,PetscReal,PetscInt*,PetscScalar*);
+extern PetscErrorCode EPSFullLanczos(EPS,PetscReal*,PetscReal*,Vec*,PetscInt,PetscInt*,Vec,PetscBool*);
+extern PetscErrorCode EPSTranslateHarmonic(PetscInt,PetscScalar*,PetscInt,PetscScalar,PetscScalar,PetscScalar*,PetscScalar*);
+extern PetscErrorCode EPSBuildBalance_Krylov(EPS);
+extern PetscErrorCode EPSProjectedKSNonsym(EPS,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscInt);
 
 #endif
