@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -22,10 +22,18 @@
 #ifndef _SLEPCIMPL
 #define _SLEPCIMPL
 
-#include "slepcsys.h"
+#include <slepcsys.h>
+
+/* context for monitors of type XXXMonitorConverged */
+struct _n_SlepcConvMonitor {
+  PetscViewer viewer;
+  PetscInt    oldnconv;
+};
+typedef struct _n_SlepcConvMonitor* SlepcConvMonitor;
 
 /* Private functions that are shared by several classes */
 
-EXTERN PetscErrorCode DenseSelectedEvec(PetscScalar*,PetscInt,PetscScalar*,PetscScalar*,PetscInt,PetscTruth,PetscInt,PetscScalar*);
+extern PetscErrorCode DenseSelectedEvec(PetscScalar*,PetscInt,PetscScalar*,PetscScalar*,PetscInt,PetscBool,PetscInt,PetscScalar*);
+extern PetscErrorCode SlepcConvMonitorDestroy(SlepcConvMonitor *ctx);
 
 #endif
