@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -22,8 +22,8 @@
 #ifndef _SVDIMPL
 #define _SVDIMPL
 
-#include "slepcsvd.h"
-#include "slepcip.h"
+#include <slepcsvd.h>
+#include <slepcip.h>
 
 extern PetscFList SVDList;
 extern PetscLogEvent SVD_SetUp, SVD_Solve, SVD_Dense;
@@ -36,6 +36,7 @@ struct _SVDOps {
   PetscErrorCode  (*setfromoptions)(SVD);
   PetscErrorCode  (*publishoptions)(SVD);
   PetscErrorCode  (*destroy)(SVD);
+  PetscErrorCode  (*reset)(SVD);
   PetscErrorCode  (*view)(SVD,PetscViewer);
 };
 
@@ -50,16 +51,16 @@ struct _SVDOps {
 struct _p_SVD {
   PETSCHEADER(struct _SVDOps);
   Mat              OP;          /* problem matrix */
-  Mat              A;	        /* problem matrix (m>n) */
+  Mat              A;           /* problem matrix (m>n) */
   Mat              AT;          /* transposed matrix */
   SVDTransposeMode transmode;   /* transpose mode */
-  PetscReal        *sigma;	/* singular values */
+  PetscReal        *sigma;      /* singular values */
   PetscInt         *perm;       /* permutation for singular value ordering */
-  Vec              *U,*V;	/* left and right singular vectors */
+  Vec              *U,*V;       /* left and right singular vectors */
   Vec              *IS;         /* placeholder for references to user-provided initial space */
   PetscInt         n;           /* maximun size of descomposition */
   SVDWhich         which;       /* which singular values are computed */
-  PetscInt         nconv;	/* number of converged values */
+  PetscInt         nconv;       /* number of converged values */
   PetscInt         nsv;         /* number of requested values */
   PetscInt         ncv;         /* basis size */
   PetscInt         mpd;         /* maximum dimension of projected problem */
@@ -69,45 +70,28 @@ struct _p_SVD {
   PetscReal        tol;         /* tolerance */
   PetscReal        *errest;     /* error estimates */
   PetscRandom      rand;        /* random number generator */
-  void             *data;	/* placeholder for misc stuff associated
-                   		   with a particular solver */
+  Vec              tl,tr;       /* template vectors */
+  void             *data;       /* placeholder for misc stuff associated
+                                   with a particular solver */
   PetscInt         setupcalled;
   SVDConvergedReason reason;
   IP               ip;
-  PetscTruth       trackall;
+  PetscBool        trackall;
   
   PetscErrorCode  (*monitor[MAXSVDMONITORS])(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,void*);
-  PetscErrorCode  (*monitordestroy[MAXSVDMONITORS])(void*);
+  PetscErrorCode  (*monitordestroy[MAXSVDMONITORS])(void**);
   void            *monitorcontext[MAXSVDMONITORS];
   PetscInt        numbermonitors;
   
   PetscInt        matvecs;
 };
 
-EXTERN PetscErrorCode SVDRegisterAll(char *);
-EXTERN PetscErrorCode SVDInitializePackage(char*);
-EXTERN PetscErrorCode SVDFinalizePackage(void);
+extern PetscErrorCode SVDMonitor(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt);
 
-#define SVDMonitor(svd,it,nconv,sigma,errest,nest) \
-        { PetscErrorCode _ierr; PetscInt _i,_im = svd->numbermonitors; \
-          for ( _i=0; _i<_im; _i++ ) {\
-            _ierr=(*svd->monitor[_i])(svd,it,nconv,sigma,errest,nest,svd->monitorcontext[_i]);\
-            CHKERRQ(_ierr); \
-	  } \
-	}
-
-/* context for SVDMonitorConverged */
-typedef struct {
-  PetscViewerASCIIMonitor viewer;
-  PetscInt oldnconv;
-} SVDMONITOR_CONV;
-EXTERN PetscErrorCode SVDMonitorDestroy_Converged(SVDMONITOR_CONV*);
-
-EXTERN PetscErrorCode SVDDestroy_Default(SVD);
-EXTERN PetscErrorCode SVDMatMult(SVD,PetscTruth,Vec,Vec);
-EXTERN PetscErrorCode SVDMatGetVecs(SVD,Vec*,Vec*);
-EXTERN PetscErrorCode SVDMatGetSize(SVD,PetscInt*,PetscInt*);
-EXTERN PetscErrorCode SVDMatGetLocalSize(SVD,PetscInt*,PetscInt*);
-EXTERN PetscErrorCode SVDTwoSideLanczos(SVD,PetscReal*,PetscReal*,Vec*,Vec,Vec*,PetscInt,PetscInt,PetscScalar*);
+extern PetscErrorCode SVDMatMult(SVD,PetscBool,Vec,Vec);
+extern PetscErrorCode SVDMatGetVecs(SVD,Vec*,Vec*);
+extern PetscErrorCode SVDMatGetSize(SVD,PetscInt*,PetscInt*);
+extern PetscErrorCode SVDMatGetLocalSize(SVD,PetscInt*,PetscInt*);
+extern PetscErrorCode SVDTwoSideLanczos(SVD,PetscReal*,PetscReal*,Vec*,Vec,Vec*,PetscInt,PetscInt,PetscScalar*);
 
 #endif
