@@ -3,14 +3,19 @@
 
 #define SLEPC_VERSION_RELEASE    1
 #define SLEPC_VERSION_MAJOR      3
-#define SLEPC_VERSION_MINOR      1
+#define SLEPC_VERSION_MINOR      2
 #define SLEPC_VERSION_SUBMINOR   0
-#define SLEPC_VERSION_PATCH      5
-#define SLEPC_VERSION_DATE       "August 4, 2010"
-#define SLEPC_VERSION_PATCH_DATE "December 13, 2010"
-#define SLEPC_AUTHOR_INFO        "        The SLEPc Team\n\
-   slepc-maint@grycap.upv.es\n\
- http://www.grycap.upv.es/slepc\n"
+#define SLEPC_VERSION_PATCH      3
+#define SLEPC_VERSION_DATE       "October 28, 2011"
+#define SLEPC_VERSION_PATCH_DATE "November 29, 2011"
+
+#if !defined (SLEPC_VERSION_SVN)
+#define SLEPC_VERSION_SVN        "unknown"
+#endif
+
+#if !defined(SLEPC_VERSION_DATE_SVN)
+#define SLEPC_VERSION_DATE_SVN   "unknown"
+#endif
 
 #define SLEPC_VERSION_(MAJOR,MINOR,SUBMINOR) \
  ((SLEPC_VERSION_MAJOR == (MAJOR)) &&       \
