@@ -7,7 +7,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -31,10 +31,9 @@
    options.
 */
 
-#include "petsc.h"
-#include "private/epsimpl.h"
-#include "private/stimpl.h"
-
+#include <private/epsimpl.h>         /*I "slepceps.h" I*/
+#include <private/stimpl.h>          /*I "slepcst.h" I*/
+#include <slepcblaslapack.h>
 
 typedef struct _dvdFunctionList {
   PetscErrorCode (*f)(void*);
@@ -77,22 +76,14 @@ typedef enum {
 } InitType_t;
 
 typedef enum {
-  DVD_PROJ_KBXX,
-  DVD_PROJ_KBXY,
-  DVD_PROJ_KBXZ,
-  DVD_PROJ_KBXZY
+  DVD_PROJ_KXX,
+  DVD_PROJ_KZX
 } ProjType_t;
 
 typedef enum {
-    DVD_MT_IDENTITY,/* without transformation */
-    DVD_MT_pX,       /* using the projected problem eigenvectors */
-    DVD_MT_ORTHO    /* using an orthonormal transformation */
-} MT_type_t;
-
-typedef enum {
-    DVD_ORTHOV_NONE,/* V isn't orthonormalized */
-    DVD_ORTHOV_I,   /* V is orthonormalized */     
-    DVD_ORTHOV_B    /* V is B-orthonormalized */
+    DVD_ORTHOV_I,        /* V is orthonormalized */     
+    DVD_ORTHOV_B,        /* V is B-orthonormalized */
+    DVD_ORTHOV_BOneMV    /* V is B-orthonormalized using IPBOrthogonalize */
 } orthoV_type_t;
 
 
@@ -107,7 +98,7 @@ typedef struct _dvdDashboard {
   void *calcPairs_data;
 
   /* Eigenpair test for convergence */
-  PetscTruth (*testConv)(struct _dvdDashboard*, PetscScalar eigvr,
+  PetscBool (*testConv)(struct _dvdDashboard*, PetscScalar eigvr,
        PetscScalar eigvi, PetscReal res, PetscReal *error);
   void *testConv_data;
 
@@ -123,7 +114,7 @@ typedef struct _dvdDashboard {
   void *improveX_data;
 
   /* Check for restarting */
-  PetscTruth (*isRestarting)(struct _dvdDashboard*);
+  PetscBool (*isRestarting)(struct _dvdDashboard*);
   void *isRestarting_data;
 
   /* Perform restarting */
@@ -140,34 +131,51 @@ typedef struct _dvdDashboard {
   EPType_t sEP;     /* Problem specifications */
   PetscInt nev;     /* number of eigenpairs */
   EPSWhich which;   /* spectrum selection */
-  PetscTruth
+  PetscBool
     withTarget;     /* if there is a target */
   PetscScalar
     target[2];         /* target value */
   PetscReal tol;    /* tolerance */
-  PetscTruth
-    correctXnorm;   /* if true, tol < |r|/|x| */
+  PetscBool
+    correctXnorm;   /* if true, norm of X are computed */
 
   /**** Subspaces specification ****/
   Vec *V,           /* searching subspace */
+    *real_V,        /* original V */
     *W,             /* testing subspace */
+    *real_W,        /* original W */
     *cX,            /* converged right eigenvectors */
     *cY,            /* converged left eigenvectors */
     *BcX,           /* basis of B*cX */
     *AV,            /* A*V */
     *real_AV,       /* original A*V space */
     *BV,            /* B*V */
-    *real_BV;       /* original B*V space */
+    *real_BV,       /* original B*V space */
+    *BDS;           /* B * eps->DV */
   PetscInt size_V,  /* size of V */
+    size_real_V,    /* original size of V */
+    size_W,         /* size of W */
+    size_real_W,    /* original size of W */
     size_AV,        /* size of AV */
+    size_real_AV,   /* original size of AV */
     size_BV,        /* size of BV */
+    size_BDS,       /* size of BDS */
+    size_real_BV,   /* original size of BV */
     size_cX,        /* size of cX */
     size_cY,        /* size of cY */
     size_D,         /* active vectors */
+    size_BcX,       /* size of BcX */
+    size_real_eigr, /* size of original eigr, eigi, nR, errest */
     max_size_V,     /* max size of V */
-    max_size_X,     /* max size of X */
+    max_size_W,     /* max size of W */
+    max_size_X,     /* max new vectors in V */
     max_size_AV,    /* max size of AV */
-    max_size_BV;    /* max size of BV */
+    max_size_BV,    /* max size of BV */
+    max_size_proj,  /* max size projected problem */
+    max_cX_in_proj, /* max vectors from cX in the projected problem */
+    max_cX_in_impr, /* max vectros from cX in the projector */
+    max_size_P,     /* max unconverged vectors in the projector */
+    bs;             /* max vectors that expands the subspace every iteration */
   EPS eps;          /* Connection to SLEPc */
  
   /**** Auxiliary space ****/
@@ -181,11 +189,16 @@ typedef struct _dvdDashboard {
   /**** Eigenvalues and errors ****/
   PetscScalar
     *ceigr, *ceigi, /* converged eigenvalues */
-    *eigr, *eigi;   /* current eigenvalues */
+    *eigr, *eigi,   /* current eigenvalues */
+    *real_eigr,
+    *real_eigi;     /* original eigr and eigi */
   PetscReal
     *nR,            /* residual norm */
+    *real_nR,       /* original nR */
     *nX,            /* X norm */
-    *errest;        /* relative error eigenpairs */
+    *real_nX,       /* original nX */
+    *errest,        /* relative error eigenpairs */
+    *real_errest;   /* original errest */
 
   /**** Shared function and variables ****/
   PetscErrorCode (*e_Vchanged)(struct _dvdDashboard*, PetscInt s_imm,
@@ -193,19 +206,17 @@ typedef struct _dvdDashboard {
   void *e_Vchanged_data;
 
   PetscErrorCode (*calcpairs_residual)(struct _dvdDashboard*, PetscInt s, PetscInt e,
-                                 Vec *R, PetscScalar *auxS, Vec auxV);
+                                 Vec *R);
+  PetscErrorCode (*calcpairs_residual_eig)(struct _dvdDashboard*, PetscInt s, PetscInt e,
+                                 Vec *R);
   PetscErrorCode (*calcpairs_selectPairs)(struct _dvdDashboard*, PetscInt n);
   void *calcpairs_residual_data;
-  PetscErrorCode (*calcpairs_X)(struct _dvdDashboard*, PetscInt s, PetscInt e,
-                          Vec *X);
-  PetscErrorCode (*calcpairs_Y)(struct _dvdDashboard*, PetscInt s, PetscInt e,
-                          Vec *Y);
   PetscErrorCode (*improvex_precond)(struct _dvdDashboard*, PetscInt i, Vec x,
                   Vec Px);
   void *improvex_precond_data;
   PetscErrorCode (*improvex_jd_proj_uv)(struct _dvdDashboard*, PetscInt i_s,
-                                        PetscInt i_e, Vec **u, Vec **v,
-                                        Vec **kr, Vec **auxV_,
+                                        PetscInt i_e, Vec *u, Vec *v,
+                                        Vec *kr, Vec *auxV,
                                         PetscScalar *theta,
                                         PetscScalar *thetai,
                                         PetscScalar *pX, PetscScalar *pY,
@@ -248,8 +259,6 @@ typedef struct _dvdDashboard {
     *cS,            /* first Schur matrix of converged pairs */
     *cT;            /* second Schur matrix of converged pairs */
   MatType_t
-    pX_type,        /* pX properties */
-    pY_type,        /* pY properties */
     sH,             /* H properties */
     sG;             /* G properties */
   PetscInt ldH,     /* leading dimension of H */
@@ -263,51 +272,96 @@ typedef struct _dvdDashboard {
     ldcT,           /* leading dimension of cT */
     size_H,         /* rows and columns in H */
     size_G,         /* rows and columns in G */
-    size_MT;        /* rows in MT */
+    size_MT,        /* rows in MT */
+    size_cS,        /* dimension of cS */
+    size_cT,        /* dimension of cT */
+    max_size_cS,    /* max size of cS and cT */
+    cX_in_V,        /* number of converged vectors in V */
+    cX_in_W,        /* number of converged vectors in W */
+    cX_in_AV,       /* number of converged vectors in AV */
+    cX_in_BV,       /* number of converged vectors in BV */
+    cX_in_H,        /* number of converged vectors in H */
+    cX_in_G;        /* number of converged vectors in G */
 
-  PetscInt V_imm_s,
-    V_imm_e,        /* unchanged V columns, V_imm_s:V_imm_e-1 */
+  PetscInt 
     V_tra_s,
-    V_tra_e,        /* V(V_tra_e:) = V*MT(V_tra_s:V_tra_e-1) */
+    V_tra_e,        /* cX <- [cX V*MT(0:V_tra_s-1)], V <- V*MT(V_tra_s:V_tra_e) */
     V_new_s,
     V_new_e;        /* added to V the columns V_new_s:V_new_e */
+  PetscBool
+    BV_shift,       /* if true BV is shifted when vectors converge */
+    W_shift;        /* if true W is shifted when vectors converge */
 
-  MT_type_t MT_type;
   orthoV_type_t orthoV_type;
 
-  PetscRandom rand; /* random seed */
   void* prof_data;  /* profiler data */
 } dvdDashboard;
 
-#define DVD_FL_ADD(list, fun) { \
+/* Add the function fun at the beginning of list */
+#define DVD_FL_ADD_BEGIN(list, fun) { \
   dvdFunctionList *fl=(list); \
   PetscErrorCode ierr; \
   ierr = PetscMalloc(sizeof(dvdFunctionList), &(list)); CHKERRQ(ierr); \
   (list)->f = (PetscErrorCode(*)(void*))(fun); \
   (list)->next = fl; }
 
+/* Add the function fun at the end of list */
+#define DVD_FL_ADD_END(list, fun) { \
+  if ((list)) {DVD_FL_ADD_END0(list, fun);} \
+  else {DVD_FL_ADD_BEGIN(list, fun);} }
+
+#define DVD_FL_ADD_END0(list, fun) { \
+  dvdFunctionList *fl=(list); \
+  PetscErrorCode ierr; \
+  for(;fl->next; fl = fl->next); \
+  ierr = PetscMalloc(sizeof(dvdFunctionList), &fl->next); CHKERRQ(ierr); \
+  fl->next->f = (PetscErrorCode(*)(void*))(fun); \
+  fl->next->next = PETSC_NULL; }
+
+#define DVD_FL_ADD(list, fun) DVD_FL_ADD_END(list, fun)
+
 #define DVD_FL_CALL(list, arg0) { \
   dvdFunctionList *fl; \
-  for(fl=(list); fl; fl=fl->next) (*(dvdCallback)fl->f)((arg0)); }
+  for(fl=(list); fl; fl=fl->next) \
+    if(*(dvdCallback)fl->f) (*(dvdCallback)fl->f)((arg0)); }
 
 #define DVD_FL_DEL(list) { \
   dvdFunctionList *fl=(list), *oldfl; \
   PetscErrorCode ierr; \
   while(fl) { \
-    oldfl = fl; fl = fl->next; ierr = PetscFree(oldfl); CHKERRQ(ierr); }}
+    oldfl = fl; fl = fl->next; ierr = PetscFree(oldfl); CHKERRQ(ierr); } \
+  (list) = PETSC_NULL;}
 
 /*
   The blackboard configuration structure: saves information about the memory
-  and other requirements
+  and other requirements.
+
+  The starting memory structure:
+
+  V           W?          AV          BV?          tKZ     
+  |-----------|-----------|-----------|------------|-------|
+  nev+mpd     nev+mpd     scP+mpd     nev?+mpd     sP+scP  
+              scP+mpd                 scP+mpd
+
+  The final memory structure considering W_shift and BV_shift:
+
+  cX  V       cY?  W?     cAV AV      BcX? BV?     KZ  tKZ 
+  |---|-------|----|------|---|-------|----|-------|---|---|
+  nev mpd     nev  mpd    scP mpd     nev  mpd     scP sP    <- shift
+              scP                     scP                    <- !shift
 */
 typedef struct {
-  PetscInt max_size_V,  /* max size of the searching subspace */
-    max_size_X,         /* max size of X */
-    size_V,             /* real size of V */
+  PetscInt max_size_V,  /* max size of the searching subspace (mpd) */
+    max_size_X,         /* max size of X (bs) */
+    size_V,             /* real size of V (nev+size_P+mpd) */
     max_size_oldX,      /* max size of oldX */
     max_size_auxV,      /* max size of auxiliary vecs */
     max_size_auxS,      /* max size of auxiliary scalars */ 
     max_nev,            /* max number of converged pairs */
+    max_size_P,         /* number of computed vectors for the projector */
+    max_size_cP,        /* number of converged vectors in the projectors */
+    max_size_proj,      /* max size projected problem */
+    max_size_cX_proj,   /* max converged vectors in the projected problem */
     own_vecs,           /* number of global vecs */
     own_scalars;        /* number of local scalars */
   Vec *free_vecs;       /* free global vectors */
@@ -325,12 +379,11 @@ typedef struct {
 #define DVD_STATE_RUN 2
 
 /* Shared types */
-typedef void* dvdPrecondData; // DEPRECATED!!
 typedef PetscErrorCode (*dvdPrecond)(dvdDashboard*, PetscInt i, Vec x, Vec Px);
 typedef PetscErrorCode (*dvdCallback)(dvdDashboard*);
 typedef PetscErrorCode (*e_Vchanged_type)(dvdDashboard*, PetscInt s_imm,
                          PetscInt e_imm, PetscInt s_new, PetscInt e_new);
-typedef PetscTruth (*isRestarting_type)(dvdDashboard*);
+typedef PetscBool (*isRestarting_type)(dvdDashboard*);
 typedef PetscErrorCode (*e_newIteration_type)(dvdDashboard*);
 typedef PetscErrorCode (*improveX_type)(dvdDashboard*, Vec *D, PetscInt max_size_D,
                                   PetscInt r_s, PetscInt r_e, PetscInt *size_D);
@@ -367,18 +420,17 @@ typedef struct {
 } DvdMult_copy_func;
 
 /* Routines for initV step */
-PetscErrorCode dvd_initV_classic(dvdDashboard *d, dvdBlackboard *b, PetscInt k);
-PetscErrorCode dvd_initV_user(dvdDashboard *d, dvdBlackboard *b,
-                        PetscInt size_userV, PetscInt k);
-PetscErrorCode dvd_initV_krylov(dvdDashboard *d, dvdBlackboard *b, PetscInt k);
+PetscErrorCode dvd_initV(dvdDashboard *d, dvdBlackboard *b, PetscInt k,
+                         PetscInt user, PetscBool krylov);
 
 /* Routines for calcPairs step */
-PetscErrorCode dvd_calcpairs_rr(dvdDashboard *d, dvdBlackboard *b);
-PetscErrorCode dvd_calcpairs_qz(dvdDashboard *d, dvdBlackboard *b, IP ip);
+PetscErrorCode dvd_calcpairs_qz(dvdDashboard *d, dvdBlackboard *b,
+                                orthoV_type_t orth, IP ipI,
+                                PetscInt cX_proj, PetscBool harm);
 
 /* Routines for improveX step */
 PetscErrorCode dvd_improvex_jd(dvdDashboard *d, dvdBlackboard *b, KSP ksp,
-                         PetscInt max_bs);
+                         PetscInt max_bs, PetscInt cX_impr, PetscBool dynamic);
 PetscErrorCode dvd_improvex_jd_proj_uv(dvdDashboard *d, dvdBlackboard *b,
                                  ProjType_t p);
 PetscErrorCode dvd_improvex_jd_lit_const(dvdDashboard *d, dvdBlackboard *b,
@@ -391,10 +443,10 @@ PetscErrorCode dvd_testconv_slepc(dvdDashboard *d, dvdBlackboard *b);
 
 /* Routines for management of V */
 PetscErrorCode dvd_managementV_basic(dvdDashboard *d, dvdBlackboard *b,
-                                     PetscInt bs, PetscInt max_size_V,
-                                     PetscInt mpd, PetscInt min_size_V,
-                                     PetscInt plusk, PetscTruth harm,
-                                     PetscTruth allResiduals);
+                                     PetscInt bs, PetscInt mpd,
+                                     PetscInt min_size_V,
+                                     PetscInt plusk, PetscBool harm,
+                                     PetscBool allResiduals);
 
 /* Some utilities */
 PetscErrorCode dvd_static_precond_PC(dvdDashboard *d, dvdBlackboard *b, PC pc);
@@ -402,31 +454,33 @@ PetscErrorCode dvd_jacobi_precond(dvdDashboard *d, dvdBlackboard *b);
 PetscErrorCode dvd_profiler(dvdDashboard *d, dvdBlackboard *b);
 PetscErrorCode dvd_prof_init();
 PetscErrorCode dvd_harm_conf(dvdDashboard *d, dvdBlackboard *b,
-                             HarmType_t mode, PetscTruth fixedTarget,
+                             HarmType_t mode, PetscBool fixedTarget,
                              PetscScalar t);
 
 /* Methods */
 PetscErrorCode dvd_schm_basic_preconf(dvdDashboard *d, dvdBlackboard *b,
-  PetscInt max_size_V, PetscInt mpd, PetscInt min_size_V, PetscInt bs,
-  PetscInt ini_size_V, PetscInt size_initV, PetscInt plusk, PC pc,
-  HarmType_t harmMode, KSP ksp, InitType_t init, PetscTruth allResiduals);
+  PetscInt mpd, PetscInt min_size_V, PetscInt bs,
+  PetscInt ini_size_V, PetscInt size_initV, PetscInt plusk,
+  HarmType_t harmMode, KSP ksp, InitType_t init, PetscBool allResiduals,
+  orthoV_type_t orth, PetscInt cX_proj, PetscInt cX_impr);
 PetscErrorCode dvd_schm_basic_conf(dvdDashboard *d, dvdBlackboard *b,
-  PetscInt max_size_V, PetscInt mpd, PetscInt min_size_V, PetscInt bs,
-  PetscInt ini_size_V, PetscInt size_initV, PetscInt plusk, PC pc,
-  IP ip, HarmType_t harmMode, PetscTruth fixedTarget, PetscScalar t, KSP ksp,
-  PetscReal fix, InitType_t init, PetscTruth allResiduals);
+  PetscInt mpd, PetscInt min_size_V, PetscInt bs,
+  PetscInt ini_size_V, PetscInt size_initV, PetscInt plusk,
+  IP ip, HarmType_t harmMode, PetscBool fixedTarget, PetscScalar t, KSP ksp,
+  PetscReal fix, InitType_t init, PetscBool allResiduals, orthoV_type_t orth,
+  PetscInt cX_proj, PetscInt cX_impr, PetscBool dynamic);
 
 /* BLAS routines */
 PetscErrorCode SlepcDenseMatProd(PetscScalar *C, PetscInt _ldC, PetscScalar b,
   PetscScalar a,
-  const PetscScalar *A, PetscInt _ldA, PetscInt rA, PetscInt cA, PetscTruth At,
-  const PetscScalar *B, PetscInt _ldB, PetscInt rB, PetscInt cB, PetscTruth Bt);
+  const PetscScalar *A, PetscInt _ldA, PetscInt rA, PetscInt cA, PetscBool At,
+  const PetscScalar *B, PetscInt _ldB, PetscInt rB, PetscInt cB, PetscBool Bt);
 PetscErrorCode SlepcDenseMatProdTriang(
   PetscScalar *C, MatType_t sC, PetscInt ldC,
   const PetscScalar *A, MatType_t sA, PetscInt ldA, PetscInt rA, PetscInt cA,
-  PetscTruth At,
+  PetscBool At,
   const PetscScalar *B, MatType_t sB, PetscInt ldB, PetscInt rB, PetscInt cB,
-  PetscTruth Bt);
+  PetscBool Bt);
 PetscErrorCode SlepcDenseNorm(PetscScalar *A, PetscInt ldA, PetscInt _rA,
                               PetscInt cA, PetscScalar *eigi);
 PetscErrorCode SlepcDenseOrth(PetscScalar *A, PetscInt _ldA, PetscInt _rA,
@@ -471,49 +525,41 @@ PetscErrorCode SlepcAllReduceSum(DvdReduction *r, PetscInt size_in,
 PetscErrorCode SlepcAllReduceSumEnd(DvdReduction *r);
 PetscErrorCode dvd_orthV(IP ip, Vec *DS, PetscInt size_DS, Vec *cX,
                          PetscInt size_cX, Vec *V, PetscInt V_new_s,
-                         PetscInt V_new_e, PetscScalar *auxS, Vec auxV,
+                         PetscInt V_new_e, PetscScalar *auxS,
                          PetscRandom rand);
+PetscErrorCode dvd_BorthV(IP ip, Vec *DS, Vec *BDS, PetscInt size_DS, Vec *cX,
+                         Vec *BcX, PetscInt size_cX, Vec *V, Vec *BV,
+                         PetscInt V_new_s, PetscInt V_new_e,
+                         PetscScalar *auxS, PetscRandom rand);
 PetscErrorCode dvd_compute_eigenvectors(PetscInt n_, PetscScalar *S,
   PetscInt ldS_, PetscScalar *T, PetscInt ldT_, PetscScalar *pX,
   PetscInt ldpX_, PetscScalar *pY, PetscInt ldpY_, PetscScalar *auxS,
-  PetscInt size_auxS, PetscTruth doProd);
+  PetscInt size_auxS, PetscBool doProd);
+PETSC_EXTERN_CXX_BEGIN
+PetscErrorCode EPSSortDenseHEP(EPS eps, PetscInt n, PetscInt k, PetscScalar *w, PetscScalar *V, PetscInt ldV);
+PetscErrorCode EPSCleanDenseSchur(PetscInt n,PetscInt k,PetscScalar *S,PetscInt ldS,PetscScalar *T,PetscInt ldT,PetscScalar *eigi,PetscScalar *X,PetscInt ldX,PetscBool doProd);
+PETSC_EXTERN_CXX_END
 
 /* SLEPc interface routines */
 PetscErrorCode SLEPcNotImplemented();
-PetscErrorCode EPSCreate_DAVIDSON(EPS eps);
-PetscErrorCode EPSDestroy_DAVIDSON(EPS eps);
-PetscErrorCode EPSSetUp_DAVIDSON(EPS eps);
-PetscErrorCode EPSSolve_DAVIDSON(EPS eps);
-PetscErrorCode EPSComputeVectors_QZ(EPS eps);
-PetscErrorCode EPSDAVIDSONSetKrylovStart_DAVIDSON(EPS eps,PetscTruth krylovstart);
-PetscErrorCode EPSDAVIDSONGetKrylovStart_DAVIDSON(EPS eps,PetscTruth *krylovstart);
-PetscErrorCode EPSDAVIDSONSetBlockSize_DAVIDSON(EPS eps,PetscInt blocksize);
-PetscErrorCode EPSDAVIDSONGetBlockSize_DAVIDSON(EPS eps,PetscInt *blocksize);
-PetscErrorCode EPSDAVIDSONSetRestart_DAVIDSON(EPS eps,PetscInt minv,PetscInt plusk);
-PetscErrorCode EPSDAVIDSONGetRestart_DAVIDSON(EPS eps,PetscInt *minv,PetscInt *plusk);
-PetscErrorCode EPSDAVIDSONGetInitialSize_DAVIDSON(EPS eps,PetscInt *initialsize);
-PetscErrorCode EPSDAVIDSONSetInitialSize_DAVIDSON(EPS eps,PetscInt initialsize);
-PetscErrorCode EPSDAVIDSONGetFix_DAVIDSON(EPS eps,PetscReal *fix);
-PetscErrorCode EPSDAVIDSONSetFix_DAVIDSON(EPS eps,PetscReal fix);
-
-typedef struct {
-  /**** Solver options ****/
-  PetscInt blocksize,     /* block size */
-    initialsize,          /* initial size of V */
-    minv,                 /* size of V after restarting */
-    plusk;                /* keep plusk eigenvectors from the last iteration */
-  PetscTruth ipB;         /* truth if V'B*V=I */
-  PetscInt   method;      /* method for improving the approximate solution */
-  PetscReal  fix;         /* the fix parameter */
-  PetscTruth krylovstart; /* true if the starting subspace is a Krylov basis */
-
-  /**** Solver data ****/
-  dvdDashboard ddb;
-
-  /**** Things to destroy ****/
-  PetscScalar *wS;
-  Vec         *wV;
-  PetscInt    size_wV;
-  PC          pc;         /* pc extracted from st->ksp */
-} EPS_DAVIDSON;
-
+PetscErrorCode EPSCreate_Davidson(EPS eps);
+PetscErrorCode EPSReset_Davidson(EPS eps);
+PetscErrorCode EPSSetUp_Davidson(EPS eps);
+PetscErrorCode EPSSolve_Davidson(EPS eps);
+PetscErrorCode EPSComputeVectors_Davidson(EPS eps);
+PetscErrorCode EPSDavidsonSetKrylovStart_Davidson(EPS eps,PetscBool krylovstart);
+PetscErrorCode EPSDavidsonGetKrylovStart_Davidson(EPS eps,PetscBool *krylovstart);
+PetscErrorCode EPSDavidsonSetBlockSize_Davidson(EPS eps,PetscInt blocksize);
+PetscErrorCode EPSDavidsonGetBlockSize_Davidson(EPS eps,PetscInt *blocksize);
+PetscErrorCode EPSDavidsonSetRestart_Davidson(EPS eps,PetscInt minv,PetscInt plusk);
+PetscErrorCode EPSDavidsonGetRestart_Davidson(EPS eps,PetscInt *minv,PetscInt *plusk);
+PetscErrorCode EPSDavidsonGetInitialSize_Davidson(EPS eps,PetscInt *initialsize);
+PetscErrorCode EPSDavidsonSetInitialSize_Davidson(EPS eps,PetscInt initialsize);
+PetscErrorCode EPSDavidsonGetFix_Davidson(EPS eps,PetscReal *fix);
+PetscErrorCode EPSDavidsonSetFix_Davidson(EPS eps,PetscReal fix);
+PetscErrorCode EPSDavidsonSetBOrth_Davidson(EPS eps,PetscBool borth);
+PetscErrorCode EPSDavidsonGetBOrth_Davidson(EPS eps,PetscBool *borth);
+PetscErrorCode EPSDavidsonSetConstantCorrectionTolerance_Davidson(EPS eps,PetscBool constant);
+PetscErrorCode EPSDavidsonGetConstantCorrectionTolerance_Davidson(EPS eps,PetscBool *constant);
+PetscErrorCode EPSDavidsonSetWindowSizes_Davidson(EPS eps,PetscInt pwindow,PetscInt qwindow);
+PetscErrorCode EPSDavidsonGetWindowSizes_Davidson(EPS eps,PetscInt *pwindow,PetscInt *qwindow);
