@@ -5,7 +5,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -37,7 +37,7 @@ typedef struct {
 /*
   Create a static preconditioner from a PC
 */
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_static_precond_PC"
 PetscErrorCode dvd_static_precond_PC(dvdDashboard *d, dvdBlackboard *b, PC pc)
 {
@@ -61,10 +61,12 @@ PetscErrorCode dvd_static_precond_PC(dvdDashboard *d, dvdBlackboard *b, PC pc)
       /* PC saves the matrix associated with the linear system, and it has to
          be initialize to a valid matrix */
       ierr = PCGetOperators(pc, PETSC_NULL, &P, &str); CHKERRQ(ierr);
-      ierr = PetscObjectReference((PetscObject)P); CHKERRQ(ierr);
-      ierr = PCSetOperators(pc, P, P, str); CHKERRQ(ierr);
-      ierr = MatDestroy(P); CHKERRQ(ierr);
-      ierr = PCSetUp(pc); CHKERRQ(ierr);
+      if (P) {
+        ierr = PetscObjectReference((PetscObject)P); CHKERRQ(ierr);
+        ierr = PCSetOperators(pc, P, P, str); CHKERRQ(ierr);
+        ierr = MatDestroy(&P); CHKERRQ(ierr);
+      } else
+        d->improvex_precond = dvd_precond_none;
 
       DVD_FL_ADD(d->destroyList, dvd_improvex_precond_d);
 
@@ -76,7 +78,7 @@ PetscErrorCode dvd_static_precond_PC(dvdDashboard *d, dvdBlackboard *b, PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_improvex_precond_d"
 PetscErrorCode dvd_improvex_precond_d(dvdDashboard *d)
 {
@@ -86,14 +88,15 @@ PetscErrorCode dvd_improvex_precond_d(dvdDashboard *d)
   PetscFunctionBegin;
 
   /* Free local data */
-  if (dvdpc->pc) { ierr = PCDestroy(dvdpc->pc); CHKERRQ(ierr); }
+  if (dvdpc->pc) { ierr = PCDestroy(&dvdpc->pc);CHKERRQ(ierr); }
   ierr = PetscFree(d->improvex_precond_data); CHKERRQ(ierr);
-  d->improvex_precond_data = PETSC_NULL;
 
   PetscFunctionReturn(0);
 }
 
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_static_precond_PC_0"
 PetscErrorCode dvd_static_precond_PC_0(dvdDashboard *d, PetscInt i, Vec x,
                                        Vec Px)
 {
@@ -110,6 +113,9 @@ PetscErrorCode dvd_static_precond_PC_0(dvdDashboard *d, PetscInt i, Vec x,
 typedef struct {
   Vec diagA, diagB;
 } dvdJacobiPrecond;
+
+#undef __FUNCT__
+#define __FUNCT__ "dvd_jacobi_precond"
 /*
   Create the Jacobi preconditioner for Generalized Eigenproblems
 */
@@ -118,7 +124,7 @@ PetscErrorCode dvd_jacobi_precond(dvdDashboard *d, dvdBlackboard *b)
   PetscErrorCode  ierr;
   dvdJacobiPrecond
                   *dvdjp;
-  PetscTruth      t;
+  PetscBool       t;
 
   PetscFunctionBegin;
 
@@ -155,6 +161,8 @@ PetscErrorCode dvd_jacobi_precond(dvdDashboard *d, dvdBlackboard *b)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_jacobi_precond_0"
 PetscErrorCode dvd_jacobi_precond_0(dvdDashboard *d, PetscInt i, Vec x, Vec Px)
 {
   PetscErrorCode  ierr;
@@ -180,6 +188,8 @@ PetscErrorCode dvd_jacobi_precond_0(dvdDashboard *d, PetscInt i, Vec x, Vec Px)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_precond_none"
 /*
   Create a trivial preconditioner
 */
@@ -223,30 +233,28 @@ PetscLogStage stages[6] = {0,0,0,0,0,0};
 
 /*** Other things ****/
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_prof_init"
-PetscErrorCode dvd_prof_init() {
+PetscErrorCode dvd_prof_init()
+{
   PetscErrorCode  ierr;
 
   PetscFunctionBegin;
-
   if (!stages[0]) {
     ierr = PetscLogStageRegister("Dvd_step_initV", &stages[DVD_STAGE_INITV]);
     CHKERRQ(ierr);
-    ierr = PetscLogStageRegister("Dvd_step_calcPairs",
-  		               &stages[DVD_STAGE_CALCPAIRS]); CHKERRQ(ierr);
-    ierr = PetscLogStageRegister("Dvd_step_improveX",
-  		               &stages[DVD_STAGE_IMPROVEX]); CHKERRQ(ierr);
-    ierr = PetscLogStageRegister("Dvd_step_updateV",
-  		               &stages[DVD_STAGE_UPDATEV]); CHKERRQ(ierr);
-    ierr = PetscLogStageRegister("Dvd_step_orthV",
-  		               &stages[DVD_STAGE_ORTHV]); CHKERRQ(ierr);
+    ierr = PetscLogStageRegister("Dvd_step_calcPairs",&stages[DVD_STAGE_CALCPAIRS]);CHKERRQ(ierr);
+    ierr = PetscLogStageRegister("Dvd_step_improveX",&stages[DVD_STAGE_IMPROVEX]);CHKERRQ(ierr);
+    ierr = PetscLogStageRegister("Dvd_step_updateV",&stages[DVD_STAGE_UPDATEV]);CHKERRQ(ierr);
+    ierr = PetscLogStageRegister("Dvd_step_orthV",&stages[DVD_STAGE_ORTHV]);CHKERRQ(ierr);
   }
-  
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode dvd_initV_prof(dvdDashboard* d) {
+#undef __FUNCT__
+#define __FUNCT__ "dvd_initV_prof"
+PetscErrorCode dvd_initV_prof(dvdDashboard* d)
+{
   DvdProfiler     *p = (DvdProfiler*)d->prof_data;
   PetscErrorCode  ierr;
 
@@ -259,7 +267,10 @@ PetscErrorCode dvd_initV_prof(dvdDashboard* d) {
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode dvd_calcPairs_prof(dvdDashboard* d) {
+#undef __FUNCT__
+#define __FUNCT__ "dvd_calcPairs_prof"
+PetscErrorCode dvd_calcPairs_prof(dvdDashboard* d)
+{
   DvdProfiler     *p = (DvdProfiler*)d->prof_data;
   PetscErrorCode  ierr;
 
@@ -272,8 +283,11 @@ PetscErrorCode dvd_calcPairs_prof(dvdDashboard* d) {
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_improveX_prof"
 PetscErrorCode dvd_improveX_prof(dvdDashboard* d, Vec *D, PetscInt max_size_D,
-                       PetscInt r_s, PetscInt r_e, PetscInt *size_D) {
+                       PetscInt r_s, PetscInt r_e, PetscInt *size_D)
+{
   DvdProfiler     *p = (DvdProfiler*)d->prof_data;
   PetscErrorCode  ierr;
 
@@ -286,7 +300,10 @@ PetscErrorCode dvd_improveX_prof(dvdDashboard* d, Vec *D, PetscInt max_size_D,
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode dvd_updateV_prof(dvdDashboard *d) {
+#undef __FUNCT__
+#define __FUNCT__ "dvd_updateV_prof"
+PetscErrorCode dvd_updateV_prof(dvdDashboard *d)
+{
   DvdProfiler     *p = (DvdProfiler*)d->prof_data;
   PetscErrorCode  ierr;
 
@@ -299,7 +316,10 @@ PetscErrorCode dvd_updateV_prof(dvdDashboard *d) {
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode dvd_orthV_prof(dvdDashboard *d) {
+#undef __FUNCT__
+#define __FUNCT__ "dvd_orthV_prof"
+PetscErrorCode dvd_orthV_prof(dvdDashboard *d)
+{
   DvdProfiler     *p = (DvdProfiler*)d->prof_data;
   PetscErrorCode  ierr;
 
@@ -312,7 +332,7 @@ PetscErrorCode dvd_orthV_prof(dvdDashboard *d) {
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_profiler"
 PetscErrorCode dvd_profiler(dvdDashboard *d, dvdBlackboard *b)
 {
@@ -323,9 +343,7 @@ PetscErrorCode dvd_profiler(dvdDashboard *d, dvdBlackboard *b)
 
   /* Setup the step */
   if (b->state >= DVD_STATE_CONF) {
-    if (d->prof_data) {
-      ierr = PetscFree(d->prof_data); CHKERRQ(ierr);
-    }
+    ierr = PetscFree(d->prof_data); CHKERRQ(ierr);
     ierr = PetscMalloc(sizeof(DvdProfiler), &p); CHKERRQ(ierr);
     d->prof_data = p;
     p->old_initV = d->initV; d->initV = dvd_initV_prof;
@@ -339,7 +357,7 @@ PetscErrorCode dvd_profiler(dvdDashboard *d, dvdBlackboard *b)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_profiler_d"
 PetscErrorCode dvd_profiler_d(dvdDashboard *d)
 {
@@ -350,7 +368,6 @@ PetscErrorCode dvd_profiler_d(dvdDashboard *d)
 
   /* Free local data */
   ierr = PetscFree(p); CHKERRQ(ierr);
-  d->prof_data = PETSC_NULL;
 
   PetscFunctionReturn(0);
 }
@@ -372,7 +389,7 @@ typedef struct {
   PetscScalar
     Wa, Wb,       /* span{W} = span{Wa*AV - Wb*BV} */
     Pa, Pb;       /* H=W'*(Pa*AV - Pb*BV), G=W'*(Wa*AV - Wb*BV) */
-  PetscTruth
+  PetscBool 
     withTarget;
   HarmType_t
     mode;
@@ -398,8 +415,10 @@ PetscErrorCode dvd_harm_sort(EPS eps, PetscScalar ar, PetscScalar ai,
                              void *ctx);
 PetscErrorCode dvd_harm_eigs_trans(dvdDashboard *d);
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_harm_conf"
 PetscErrorCode dvd_harm_conf(dvdDashboard *d, dvdBlackboard *b,
-                             HarmType_t mode, PetscTruth fixedTarget,
+                             HarmType_t mode, PetscBool fixedTarget,
                              PetscScalar t)
 {
   PetscErrorCode  ierr;
@@ -407,8 +426,8 @@ PetscErrorCode dvd_harm_conf(dvdDashboard *d, dvdBlackboard *b,
 
   PetscFunctionBegin;
 
-  /* Set the problem to GNHEP */
-  // TODO: d->G maybe is upper triangular due to biorthogonality of V and W
+  /* Set the problem to GNHEP:
+     d->G maybe is upper triangular due to biorthogonality of V and W */
   d->sEP = d->sA = d->sB = 0;
 
   /* Setup the step */
@@ -431,7 +450,7 @@ PetscErrorCode dvd_harm_conf(dvdDashboard *d, dvdBlackboard *b,
 }
 
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_harm_d"
 PetscErrorCode dvd_harm_d(dvdDashboard *d)
 {
@@ -441,13 +460,12 @@ PetscErrorCode dvd_harm_d(dvdDashboard *d)
 
   /* Free local data */
   ierr = PetscFree(d->calcpairs_W_data); CHKERRQ(ierr);
-  d->calcpairs_W_data = PETSC_NULL;
 
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_harm_start"
 PetscErrorCode dvd_harm_start(dvdDashboard *d)
 {
@@ -467,7 +485,7 @@ PetscErrorCode dvd_harm_start(dvdDashboard *d)
 }
 
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "dvd_harm_end"
 PetscErrorCode dvd_harm_end(dvdDashboard *d)
 {
@@ -484,6 +502,8 @@ PetscErrorCode dvd_harm_end(dvdDashboard *d)
 }
 
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_harm_transf"
 PetscErrorCode dvd_harm_transf(dvdHarmonic *dvdh, PetscScalar t)
 {
   PetscFunctionBegin;
@@ -500,7 +520,7 @@ PetscErrorCode dvd_harm_transf(dvdHarmonic *dvdh, PetscScalar t)
     dvdh->Wa = 0.0; dvdh->Wb = 1.0; dvdh->Pa = 1.0; dvdh->Pb = 0.0; break;
   case DVD_HARM_NONE:
   default:
-    SETERRQ(1, "The harmonic type is not supported!");
+    SETERRQ(PETSC_COMM_WORLD,1, "Harmonic type not supported");
   }
 
   /* Check the transformation does not change the sign of the imaginary part */
@@ -512,6 +532,8 @@ PetscErrorCode dvd_harm_transf(dvdHarmonic *dvdh, PetscScalar t)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_harm_updateW"
 PetscErrorCode dvd_harm_updateW(dvdDashboard *d)
 {
   dvdHarmonic     *data = (dvdHarmonic*)d->calcpairs_W_data;
@@ -533,6 +555,8 @@ PetscErrorCode dvd_harm_updateW(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_harm_proj"
 PetscErrorCode dvd_harm_proj(dvdDashboard *d)
 {
   dvdHarmonic     *data = (dvdHarmonic*)d->calcpairs_W_data;
@@ -541,22 +565,22 @@ PetscErrorCode dvd_harm_proj(dvdDashboard *d)
   PetscFunctionBegin;
 
   if (d->sH != d->sG) {
-    SETERRQ(1, "Error: Projected matrices H and G must have the same structure!");
+    SETERRQ(PETSC_COMM_WORLD,1,"Projected matrices H and G must have the same structure");
     PetscFunctionReturn(1);
   }
 
   /* [H G] <- [Pa*H - Pb*G, Wa*H - Wb*G] */
   if (DVD_ISNOT(d->sH,DVD_MAT_LTRIANG))     /* Upper triangular part */
-    for(i=d->V_new_s; i<d->V_new_e; i++)
+    for(i=d->V_new_s+d->cX_in_H; i<d->V_new_e+d->cX_in_H; i++)
       for(j=0; j<=i; j++) {
         PetscScalar h = d->H[d->ldH*i+j], g = d->G[d->ldH*i+j];
         d->H[d->ldH*i+j] = data->Pa*h - data->Pb*g;
         d->G[d->ldH*i+j] = data->Wa*h - data->Wb*g;
       }
   if (DVD_ISNOT(d->sH,DVD_MAT_UTRIANG))     /* Lower triangular part */
-    for(i=0; i<d->V_new_e; i++)
-      for(j=PetscMax(d->V_new_s,i+(DVD_ISNOT(d->sH,DVD_MAT_LTRIANG)?1:0));
-          j<d->V_new_e; j++) {
+    for(i=0; i<d->V_new_e+d->cX_in_H; i++)
+      for(j=PetscMax(d->V_new_s+d->cX_in_H,i+(DVD_ISNOT(d->sH,DVD_MAT_LTRIANG)?1:0));
+          j<d->V_new_e+d->cX_in_H; j++) {
         PetscScalar h = d->H[d->ldH*i+j], g = d->G[d->ldH*i+j];
         d->H[d->ldH*i+j] = data->Pa*h - data->Pb*g;
         d->G[d->ldH*i+j] = data->Wa*h - data->Wb*g;
@@ -565,6 +589,8 @@ PetscErrorCode dvd_harm_proj(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_harm_backtrans"
 PetscErrorCode dvd_harm_backtrans(dvdHarmonic *data, PetscScalar *ar,
                                   PetscScalar *ai)
 {
@@ -574,12 +600,10 @@ PetscErrorCode dvd_harm_backtrans(dvdHarmonic *data, PetscScalar *ar,
 #endif
 
   PetscFunctionBegin;
-
-  if(!ar) SETERRQ(1, "The real part has to be present!");
+  PetscValidPointer(ar,2);
   xr = *ar;
-
 #if !defined(PETSC_USE_COMPLEX)
-  if(!ai) SETERRQ(1, "The imaginary part has to be present!");
+  PetscValidPointer(ai,3);
   xi = *ai;
 
   if (xi != 0.0) {
@@ -596,6 +620,8 @@ PetscErrorCode dvd_harm_backtrans(dvdHarmonic *data, PetscScalar *ar,
 }
 
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_harm_sort"
 PetscErrorCode dvd_harm_sort(EPS eps, PetscScalar ar, PetscScalar ai,
                              PetscScalar br, PetscScalar bi, PetscInt *r,
                              void *ctx)
@@ -623,6 +649,8 @@ PetscErrorCode dvd_harm_sort(EPS eps, PetscScalar ar, PetscScalar ai,
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__
+#define __FUNCT__ "dvd_harm_eigs_trans"
 PetscErrorCode dvd_harm_eigs_trans(dvdDashboard *d)
 {
   dvdHarmonic     *data = (dvdHarmonic*)d->calcpairs_W_data;
@@ -631,7 +659,7 @@ PetscErrorCode dvd_harm_eigs_trans(dvdDashboard *d)
   PetscFunctionBegin;
 
   for(i=0; i<d->size_H; i++)
-    dvd_harm_backtrans(data, &d->eigr[i], &d->eigi[i]);
+    dvd_harm_backtrans(data, &d->eigr[i-d->cX_in_H], &d->eigi[i-d->cX_in_H]);
 
   PetscFunctionReturn(0);
 }
