@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,13 +21,13 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/stimpl.h"
-#include "private/epsimpl.h"
+#include <private/stimpl.h>       /*I "slepcst.h" I*/
+#include <private/epsimpl.h>      /*I "slepceps.h" I*/
 #include "slepc-interface.h"
-#include "lobpcg.h"
-#include "interpreter.h"
-#include "multivector.h"
-#include "temp_multivector.h"
+#include <blopex_lobpcg.h>
+#include <blopex_interpreter.h>
+#include <blopex_multivector.h>
+#include <blopex_temp_multivector.h>
 
 PetscErrorCode EPSSolve_BLOPEX(EPS);
 
@@ -38,23 +38,22 @@ typedef struct {
   mv_MultiVectorPtr          Y;
   mv_InterfaceInterpreter    ii;
   KSP                        ksp;
+  Vec                        w;
 } EPS_BLOPEX;
 
-
 #undef __FUNCT__  
 #define __FUNCT__ "Precond_FnSingleVector"
 static void Precond_FnSingleVector(void *data,void *x,void *y)
 {
-  PetscErrorCode  ierr;
-  EPS             eps = (EPS)data;
-  EPS_BLOPEX      *blopex = (EPS_BLOPEX*)eps->data;
-  PetscInt        lits;
+  PetscErrorCode ierr;
+  EPS            eps = (EPS)data;
+  EPS_BLOPEX     *blopex = (EPS_BLOPEX*)eps->data;
+  PetscInt       lits;
       
   PetscFunctionBegin;
   ierr = KSPSolve(blopex->ksp,(Vec)x,(Vec)y); CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  ierr = KSPGetIterationNumber(blopex->ksp, &lits); CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  ierr = KSPGetIterationNumber(blopex->ksp,&lits); CHKERRABORT(PETSC_COMM_WORLD,ierr);
   eps->OP->lineariterations+= lits;
-
   PetscFunctionReturnVoid();
 }
 
@@ -62,8 +61,8 @@ static void Precond_FnSingleVector(void *data,void *x,void *y)
 #define __FUNCT__ "Precond_FnMultiVector"
 static void Precond_FnMultiVector(void *data,void *x,void *y)
 {
-  EPS             eps = (EPS)data;
-  EPS_BLOPEX      *blopex = (EPS_BLOPEX*)eps->data;
+  EPS        eps = (EPS)data;
+  EPS_BLOPEX *blopex = (EPS_BLOPEX*)eps->data;
 
   PetscFunctionBegin;
   blopex->ii.Eval(Precond_FnSingleVector,data,x,y);
@@ -74,13 +73,19 @@ static void Precond_FnMultiVector(void *data,void *x,void *y)
 #define __FUNCT__ "OperatorASingleVector"
 static void OperatorASingleVector(void *data,void *x,void *y)
 {
-  PetscErrorCode  ierr;
-  EPS		  eps = (EPS)data;
-  Mat             A;
+  PetscErrorCode ierr;
+  EPS            eps = (EPS)data;
+  EPS_BLOPEX     *blopex = (EPS_BLOPEX*)eps->data;
+  Mat            A,B;
  
   PetscFunctionBegin;
-  ierr = STGetOperators(eps->OP,&A,PETSC_NULL); CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  ierr = MatMult(A,(Vec)x,(Vec)y); CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  ierr = STGetOperators(eps->OP,&A,&B);CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  ierr = MatMult(A,(Vec)x,(Vec)y);CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  if (eps->OP->sigma != 0.0) {
+    if (B) { ierr = MatMult(B,(Vec)x,blopex->w);CHKERRABORT(PETSC_COMM_WORLD,ierr); }
+    else { ierr = VecCopy((Vec)x,blopex->w);CHKERRABORT(PETSC_COMM_WORLD,ierr); }
+    ierr = VecAXPY((Vec)y,-eps->OP->sigma,blopex->w);CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  }
   PetscFunctionReturnVoid();
 }
 
@@ -88,8 +93,8 @@ static void OperatorASingleVector(void *data,void *x,void *y)
 #define __FUNCT__ "OperatorAMultiVector"
 static void OperatorAMultiVector(void *data,void *x,void *y)
 {
-  EPS		  eps = (EPS)data;
-  EPS_BLOPEX	  *blopex = (EPS_BLOPEX*)eps->data;
+  EPS        eps = (EPS)data;
+  EPS_BLOPEX *blopex = (EPS_BLOPEX*)eps->data;
 
   PetscFunctionBegin;
   blopex->ii.Eval(OperatorASingleVector,data,x,y);
@@ -100,13 +105,13 @@ static void OperatorAMultiVector(void *data,void *x,void *y)
 #define __FUNCT__ "OperatorBSingleVector"
 static void OperatorBSingleVector(void *data,void *x,void *y)
 {
-  PetscErrorCode  ierr;
-  EPS		  eps = (EPS)data;
-  Mat             B;
+  PetscErrorCode ierr;
+  EPS            eps = (EPS)data;
+  Mat            B;
   
   PetscFunctionBegin;
-  ierr = STGetOperators(eps->OP,PETSC_NULL,&B); CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  ierr = MatMult(B,(Vec)x,(Vec)y); CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  ierr = STGetOperators(eps->OP,PETSC_NULL,&B);CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  ierr = MatMult(B,(Vec)x,(Vec)y);CHKERRABORT(PETSC_COMM_WORLD,ierr);
   PetscFunctionReturnVoid();
 }
 
@@ -114,67 +119,66 @@ static void OperatorBSingleVector(void *data,void *x,void *y)
 #define __FUNCT__ "OperatorBMultiVector"
 static void OperatorBMultiVector(void *data,void *x,void *y)
 {
-  EPS		  eps = (EPS)data;
-  EPS_BLOPEX	  *blopex = (EPS_BLOPEX*)eps->data;
+  EPS        eps = (EPS)data;
+  EPS_BLOPEX *blopex = (EPS_BLOPEX*)eps->data;
 
   PetscFunctionBegin;
   blopex->ii.Eval(OperatorBSingleVector,data,x,y);
   PetscFunctionReturnVoid();
 }
 
-
 #undef __FUNCT__  
 #define __FUNCT__ "EPSSetUp_BLOPEX"
 PetscErrorCode EPSSetUp_BLOPEX(EPS eps)
 {
-  PetscErrorCode  ierr;
-  EPS_BLOPEX      *blopex = (EPS_BLOPEX *)eps->data;
-  PetscTruth      isPrecond, isPreonly;
+  PetscErrorCode ierr;
+  PetscInt       i;
+  EPS_BLOPEX     *blopex = (EPS_BLOPEX *)eps->data;
+  PetscBool      isPrecond;
 
   PetscFunctionBegin;
   if (!eps->ishermitian) { 
-    SETERRQ(PETSC_ERR_SUP,"blopex only works for hermitian problems"); 
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"blopex only works for hermitian problems"); 
   }
   if (!eps->which) eps->which = EPS_SMALLEST_REAL;
   if (eps->which!=EPS_SMALLEST_REAL) {
-    SETERRQ(1,"Wrong value of eps->which");
+    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
   }
 
   /* Change the default sigma to inf if necessary */
   if (eps->which == EPS_LARGEST_MAGNITUDE || eps->which == EPS_LARGEST_REAL ||
       eps->which == EPS_LARGEST_IMAGINARY) {
-    ierr = STSetDefaultShift(eps->OP, 3e300); CHKERRQ(ierr);
+    ierr = STSetDefaultShift(eps->OP,3e300);CHKERRQ(ierr);
   }
 
-  ierr = STSetUp(eps->OP); CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)eps->OP, STPRECOND, &isPrecond); CHKERRQ(ierr);
-  if (!isPrecond) SETERRQ(PETSC_ERR_SUP, "blopex only works with precond spectral transformation");
-  ierr = STGetKSP(eps->OP, &blopex->ksp); CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)blopex->ksp, KSPPREONLY, &isPreonly); CHKERRQ(ierr);
-  if (!isPreonly)
-    SETERRQ(PETSC_ERR_SUP, "blopex only works with preonly ksp of the spectral transformation");
+  ierr = STSetUp(eps->OP);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)eps->OP,STPRECOND,&isPrecond);CHKERRQ(ierr);
+  if (!isPrecond) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"blopex only works with STPRECOND");
+  ierr = STGetKSP(eps->OP,&blopex->ksp);CHKERRQ(ierr);
 
   eps->ncv = eps->nev = PetscMin(eps->nev,eps->n);
-  if (eps->mpd) PetscInfo(eps,"Warning: parameter mpd ignored\n");
+  if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
 
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
   ierr = EPSDefaultGetWork(eps,1);CHKERRQ(ierr);
   
-  blopex->tol.absolute = eps->tol;
+  blopex->tol.absolute = eps->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:eps->tol;
   blopex->tol.relative = 1e-50;
   
-  LOBPCG_InitRandomContext();
   SLEPCSetupInterpreter(&blopex->ii);
   blopex->eigenvectors = mv_MultiVectorCreateFromSampleVector(&blopex->ii,eps->ncv,eps->V);
+  for (i=0;i<eps->ncv;i++) { ierr = PetscObjectReference((PetscObject)eps->V[i]);CHKERRQ(ierr); }
   mv_MultiVectorSetRandom(blopex->eigenvectors,1234);
+  ierr = VecDuplicate(eps->V[0],&blopex->w);CHKERRQ(ierr);
 
   if (eps->nds > 0) {
-    blopex->Y = mv_MultiVectorCreateFromSampleVector(&blopex->ii, eps->nds, eps->DS);
+    blopex->Y = mv_MultiVectorCreateFromSampleVector(&blopex->ii,eps->nds,eps->DS);
+    for (i=0;i<eps->nds;i++) { ierr = PetscObjectReference((PetscObject)eps->DS[i]);CHKERRQ(ierr); }
   } else
     blopex->Y = PETSC_NULL;
 
-#ifdef PETSC_USE_COMPLEX
+#if defined(PETSC_USE_COMPLEX)
   blopex->blap_fn.zpotrf = PETSC_zpotrf_interface;
   blopex->blap_fn.zhegv = PETSC_zsygv_interface;
 #else
@@ -182,12 +186,10 @@ PetscErrorCode EPSSetUp_BLOPEX(EPS eps)
   blopex->blap_fn.dsygv = PETSC_dsygv_interface;
 #endif
 
-  if (eps->extraction) {
-     ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr);
-  }
+  if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  if (eps->leftvecs) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Left vectors not supported in this solver");
   eps->ops->solve = EPSSolve_BLOPEX;
   PetscFunctionReturn(0);
 }
@@ -196,31 +198,64 @@ PetscErrorCode EPSSetUp_BLOPEX(EPS eps)
 #define __FUNCT__ "EPSSolve_BLOPEX"
 PetscErrorCode EPSSolve_BLOPEX(EPS eps)
 {
-  EPS_BLOPEX      *blopex = (EPS_BLOPEX *)eps->data;
-  int             info,its;
+  EPS_BLOPEX *blopex = (EPS_BLOPEX *)eps->data;
+  int        i,j,info,its,nconv;
+  double     *lambdahist=PETSC_NULL,*residhist=PETSC_NULL;
+  PetscErrorCode ierr;
   
   PetscFunctionBegin;
-  
-#ifdef PETSC_USE_COMPLEX
+  if (eps->numbermonitors>0) {
+    ierr = PetscMalloc(eps->ncv*(eps->max_it+1)*sizeof(double),&lambdahist);CHKERRQ(ierr);
+    ierr = PetscMalloc(eps->ncv*(eps->max_it+1)*sizeof(double),&residhist);CHKERRQ(ierr);
+  }
+
+#if defined(PETSC_USE_COMPLEX)
   info = lobpcg_solve_complex(blopex->eigenvectors,eps,OperatorAMultiVector,
-  	eps->isgeneralized?eps:PETSC_NULL,eps->isgeneralized?OperatorBMultiVector:PETSC_NULL,
+        eps->isgeneralized?eps:PETSC_NULL,eps->isgeneralized?OperatorBMultiVector:PETSC_NULL,
         eps,Precond_FnMultiVector,blopex->Y,
         blopex->blap_fn,blopex->tol,eps->max_it,0,&its,
-        (komplex*)eps->eigr,PETSC_NULL,0,eps->errest,PETSC_NULL,0);
+        (komplex*)eps->eigr,lambdahist,eps->ncv,eps->errest,residhist,eps->ncv);
 #else
   info = lobpcg_solve_double(blopex->eigenvectors,eps,OperatorAMultiVector,
-  	eps->isgeneralized?eps:PETSC_NULL,eps->isgeneralized?OperatorBMultiVector:PETSC_NULL,
+        eps->isgeneralized?eps:PETSC_NULL,eps->isgeneralized?OperatorBMultiVector:PETSC_NULL,
         eps,Precond_FnMultiVector,blopex->Y,
         blopex->blap_fn,blopex->tol,eps->max_it,0,&its,
-        eps->eigr,PETSC_NULL,0,eps->errest,PETSC_NULL,0);
+        eps->eigr,lambdahist,eps->ncv,eps->errest,residhist,eps->ncv);
 #endif
-  if (info>0) SETERRQ1(PETSC_ERR_LIB,"Error in blopex (code=%d)",info); 
+  if (info>0) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in blopex (code=%d)",info); 
+
+  if (eps->numbermonitors>0) {
+    for (i=0;i<its;i++) {
+      nconv = 0;
+      for (j=0;j<eps->ncv;j++) { if (residhist[j+i*eps->ncv]>eps->tol) break; else nconv++; }
+      ierr = EPSMonitor(eps,i,nconv,lambdahist+i*eps->ncv,eps->eigi,residhist+i*eps->ncv,eps->ncv);CHKERRQ(ierr);
+    }
+    ierr = PetscFree(lambdahist);CHKERRQ(ierr); 
+    ierr = PetscFree(residhist);CHKERRQ(ierr); 
+  }
 
   eps->its = its;
   eps->nconv = eps->ncv;
+  if (eps->OP->sigma != 0.0) {
+    for (i=0;i<eps->nconv;i++) eps->eigr[i]+=eps->OP->sigma;
+  }
   if (info==-1) eps->reason = EPS_DIVERGED_ITS;
   else eps->reason = EPS_CONVERGED_TOL;
-  
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSReset_BLOPEX"
+PetscErrorCode EPSReset_BLOPEX(EPS eps)
+{
+  PetscErrorCode ierr;
+  EPS_BLOPEX     *blopex = (EPS_BLOPEX *)eps->data;
+
+  PetscFunctionBegin;
+  mv_MultiVectorDestroy(blopex->eigenvectors);
+  mv_MultiVectorDestroy(blopex->Y);
+  ierr = VecDestroy(&blopex->w);CHKERRQ(ierr);
+  ierr = EPSReset_Default(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -229,15 +264,23 @@ PetscErrorCode EPSSolve_BLOPEX(EPS eps)
 PetscErrorCode EPSDestroy_BLOPEX(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_BLOPEX     *blopex = (EPS_BLOPEX *)eps->data;
 
   PetscFunctionBegin;
   LOBPCG_DestroyRandomContext();
-  SLEPCSetupInterpreterForDignifiedDeath(&blopex->ii);
-  mv_MultiVectorDestroy(blopex->eigenvectors);
-  mv_MultiVectorDestroy(blopex->Y);
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
-  ierr = EPSDestroy_Default(eps);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSSetFromOptions_BLOPEX"
+PetscErrorCode EPSSetFromOptions_BLOPEX(EPS eps)
+{
+  PetscErrorCode  ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscOptionsHead("EPS BLOPEX Options");CHKERRQ(ierr);
+  LOBPCG_SetFromOptionsRandomContext();
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -247,26 +290,18 @@ EXTERN_C_BEGIN
 PetscErrorCode EPSCreate_BLOPEX(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_BLOPEX     *blopex;
-  const char*    prefix;
 
   PetscFunctionBegin;
-
-  ierr = STSetType(eps->OP, STPRECOND); CHKERRQ(ierr);
-  ierr = STPrecondSetKSPHasMat(eps->OP, PETSC_TRUE); CHKERRQ(ierr);
-
-  ierr = PetscNew(EPS_BLOPEX,&blopex);CHKERRQ(ierr);
-  PetscLogObjectMemory(eps,sizeof(EPS_BLOPEX));
-  ierr = KSPCreate(((PetscObject)eps)->comm,&blopex->ksp);CHKERRQ(ierr);
-  ierr = EPSGetOptionsPrefix(eps,&prefix);CHKERRQ(ierr);
-  ierr = KSPSetOptionsPrefix(blopex->ksp,prefix);CHKERRQ(ierr);
-  ierr = KSPAppendOptionsPrefix(blopex->ksp,"eps_blopex_");CHKERRQ(ierr);
-  eps->data                      = (void *) blopex;
+  ierr = PetscNewLog(eps,EPS_BLOPEX,&eps->data);CHKERRQ(ierr);
   eps->ops->setup                = EPSSetUp_BLOPEX;
-  eps->ops->setfromoptions       = PETSC_NULL;
+  eps->ops->setfromoptions       = EPSSetFromOptions_BLOPEX;
   eps->ops->destroy              = EPSDestroy_BLOPEX;
+  eps->ops->reset                = EPSReset_BLOPEX;
   eps->ops->backtransform        = EPSBackTransform_Default;
   eps->ops->computevectors       = EPSComputeVectors_Default;
+  ierr = STSetType(eps->OP,STPRECOND);CHKERRQ(ierr);
+  ierr = STPrecondSetKSPHasMat(eps->OP,PETSC_TRUE);CHKERRQ(ierr);
+  LOBPCG_InitRandomContext(((PetscObject)eps)->comm,eps->rand);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
