@@ -4,7 +4,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
  
    This file is part of SLEPc.
       
@@ -26,12 +26,8 @@
 #ifndef SLEPC_INTERFACE_HEADER
 #define SLEPC_INTERFACE_HEADER
 
-#include "lobpcg.h"
-#ifdef BLOPEX_DIR
+#include <blopex_lobpcg.h>
 #include "petsc-interface.h"
-#else
-#include "../src/contrib/blopex/petsc-interface/petsc-interface.h"
-#endif
 
 extern int
 SLEPCSetupInterpreter( mv_InterfaceInterpreter *ii );
