@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,8 +21,9 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "src/eps/impls/external/blzpack/blzpackp.h"   /*I "slepceps.h" I*/
-#include "private/stimpl.h"
+#include <private/epsimpl.h>    /*I "slepceps.h" I*/
+#include <private/stimpl.h>     /*I "slepcst.h" I*/
+#include <../src/eps/impls/external/blzpack/blzpackp.h>
 
 PetscErrorCode EPSSolve_BLZPACK(EPS);
 
@@ -67,39 +68,42 @@ const char* blzpack_error[33] = {
 PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscInt       listor, lrstor, ncuv, k1, k2, k3, k4;
+  PetscInt       listor,lrstor,ncuv,k1,k2,k3,k4;
   EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
-  PetscTruth     flg;
-  KSP            ksp;
-  PC             pc;
+  PetscBool      issinv;
 
   PetscFunctionBegin;
   if (eps->ncv) {
-    if( eps->ncv < PetscMin(eps->nev+10,eps->nev*2) )
-      SETERRQ(0,"Warning: BLZpack recommends that ncv be larger than min(nev+10,nev*2)");
+    if (eps->ncv < PetscMin(eps->nev+10,eps->nev*2))
+      SETERRQ(((PetscObject)eps)->comm,0,"Warning: BLZpack recommends that ncv be larger than min(nev+10,nev*2)");
   }
   else eps->ncv = PetscMin(eps->nev+10,eps->nev*2);
-  if (eps->mpd) PetscInfo(eps,"Warning: parameter mpd ignored\n");
+  if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->max_it) eps->max_it = PetscMax(1000,eps->n);
 
   if (!eps->ishermitian)
-    SETERRQ(PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
-  if (blz->slice || eps->isgeneralized) {
-    ierr = PetscTypeCompare((PetscObject)eps->OP,STSINVERT,&flg);CHKERRQ(ierr);
-    if (!flg)
-      SETERRQ(PETSC_ERR_SUP,"Shift-and-invert ST is needed for generalized problems or spectrum slicing");
-    ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
-    ierr = PetscTypeCompare((PetscObject)ksp,KSPPREONLY,&flg);CHKERRQ(ierr);
-    if (!flg)
-      SETERRQ(PETSC_ERR_SUP,"Preonly KSP is needed for generalized problems or spectrum slicing");
-    ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
-    ierr = PetscTypeCompare((PetscObject)pc,PCCHOLESKY,&flg);CHKERRQ(ierr);
-    if (!flg)
-      SETERRQ(PETSC_ERR_SUP,"Cholesky PC is needed for generalized problems or spectrum slicing");
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
+  if (eps->which==EPS_ALL) {
+    if (eps->inta==0.0 && eps->intb==0.0) SETERRQ(((PetscObject)eps)->comm,1,"Must define a computational interval when using EPS_ALL"); 
+    blz->slice = 1;
   }
-  if (!eps->which) eps->which = EPS_SMALLEST_REAL;
-  if (eps->which!=EPS_SMALLEST_REAL)
-    SETERRQ(1,"Wrong value of eps->which");
+  ierr = PetscTypeCompare((PetscObject)eps->OP,STSINVERT,&issinv);CHKERRQ(ierr);
+  if (blz->slice || eps->isgeneralized) {
+    if (!issinv) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Shift-and-invert ST is needed for generalized problems or spectrum slicing");
+  }
+  if (blz->slice) {
+    if (eps->intb >= PETSC_MAX_REAL) { /* right-open interval */
+      if (eps->inta <= PETSC_MIN_REAL) SETERRQ(((PetscObject)eps)->comm,1,"The defined computational interval should have at least one of their sides bounded");
+      ierr = STSetDefaultShift(eps->OP,eps->inta);CHKERRQ(ierr);
+    }
+    else { ierr = STSetDefaultShift(eps->OP,eps->intb);CHKERRQ(ierr); }
+  }
+  if (!eps->which) {
+    if (issinv) eps->which = EPS_TARGET_REAL;
+    else eps->which = EPS_SMALLEST_REAL;
+  }
+  if ((issinv && eps->which!=EPS_TARGET_REAL && eps->which!=EPS_TARGET_MAGNITUDE && eps->which!=EPS_ALL) || (!issinv && eps->which!=EPS_SMALLEST_REAL))
+    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
 
   k1 = PetscMin(eps->n,180);
   k2 = blz->block_size;
@@ -113,6 +117,7 @@ PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
 
   if (blz->slice) lrstor = eps->nloc*(k2*4+k1*2+k4)+k3;
   else lrstor = eps->nloc*(k2*4+k1)+k3;
+lrstor*=10;
   ierr = PetscFree(blz->rstor);CHKERRQ(ierr);
   ierr = PetscMalloc((4+lrstor)*sizeof(PetscReal),&blz->rstor);CHKERRQ(ierr);
   blz->rstor[3] = lrstor;
@@ -126,14 +131,12 @@ PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
   ierr = PetscFree(blz->eig);CHKERRQ(ierr);
   ierr = PetscMalloc(2*eps->ncv*sizeof(PetscReal),&blz->eig);CHKERRQ(ierr);
 
-  if (eps->extraction) {
-     ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr);
-  }
+  if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
 
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  if (eps->leftvecs) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Left vectors not supported in this solver");
   eps->ops->solve = EPSSolve_BLZPACK;
   PetscFunctionReturn(0);
 }
@@ -145,15 +148,14 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
   PetscErrorCode ierr;
   EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
   PetscInt       nn;
-  PetscBLASInt   i, nneig, lflag, nvopu;      
-  Vec            x, y;                           
+  PetscBLASInt   i,nneig,lflag,nvopu;      
+  Vec            x,y;                           
   PetscScalar    sigma,*pV;                      
   Mat            A;                              
   KSP            ksp;                            
   PC             pc;                             
   
   PetscFunctionBegin;
-
   ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,eps->nloc,PETSC_DECIDE,PETSC_NULL,&x);CHKERRQ(ierr);
   ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,eps->nloc,PETSC_DECIDE,PETSC_NULL,&y);CHKERRQ(ierr);
   ierr = VecGetArray(eps->V[0],&pV);CHKERRQ(ierr);
@@ -164,10 +166,10 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
     blz->rstor[1]  = sigma;        /* upper limit of eigenvalue interval */
   } else {
     sigma = 0.0;
-    blz->rstor[0]  = blz->initial; /* lower limit of eigenvalue interval */
-    blz->rstor[1]  = blz->final;   /* upper limit of eigenvalue interval */
+    blz->rstor[0]  = eps->inta;    /* lower limit of eigenvalue interval */
+    blz->rstor[1]  = eps->intb;    /* upper limit of eigenvalue interval */
   }
-  nneig = 0;                     /* no. of eigs less than sigma */
+  nneig = 0;                       /* no. of eigs less than sigma */
 
   blz->istor[0]  = PetscBLASIntCast(eps->nloc); /* number of rows of U, V, X*/
   blz->istor[1]  = PetscBLASIntCast(eps->nloc); /* leading dimension of U, V, X */
@@ -189,48 +191,46 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
   lflag = 0;           /* reverse communication interface flag */
 
   do {
-
-    BLZpack_( blz->istor, blz->rstor, &sigma, &nneig, blz->u, blz->v, 
-              &lflag, &nvopu, blz->eig, pV );
+    BLZpack_(blz->istor,blz->rstor,&sigma,&nneig,blz->u,blz->v,&lflag,&nvopu,blz->eig,pV);
 
     switch (lflag) {
     case 1:
       /* compute v = OP u */
       for (i=0;i<nvopu;i++) {
-        ierr = VecPlaceArray( x, blz->u+i*eps->nloc );CHKERRQ(ierr);
-        ierr = VecPlaceArray( y, blz->v+i*eps->nloc );CHKERRQ(ierr);
+        ierr = VecPlaceArray(x,blz->u+i*eps->nloc);CHKERRQ(ierr);
+        ierr = VecPlaceArray(y,blz->v+i*eps->nloc);CHKERRQ(ierr);
         if (blz->slice || eps->isgeneralized) { 
-          ierr = STAssociatedKSPSolve( eps->OP, x, y );CHKERRQ(ierr);
-	} else {
-          ierr = STApply( eps->OP, x, y ); CHKERRQ(ierr);
-	}
+          ierr = STAssociatedKSPSolve(eps->OP,x,y);CHKERRQ(ierr);
+        } else {
+          ierr = STApply(eps->OP,x,y);CHKERRQ(ierr);
+        }
         ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,eps->nds,PETSC_NULL,eps->DS,y,PETSC_NULL,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
         ierr = VecResetArray(x);CHKERRQ(ierr);
-        ierr = VecResetArray(y);CHKERRQ(ierr);	
+        ierr = VecResetArray(y);CHKERRQ(ierr);
       }
       /* monitor */
       eps->nconv  = BLZistorr_(blz->istor,"NTEIG",5);
-      EPSMonitor(eps,eps->its,eps->nconv,
+      ierr = EPSMonitor(eps,eps->its,eps->nconv,
         blz->rstor+BLZistorr_(blz->istor,"IRITZ",5),
         eps->eigi,
         blz->rstor+BLZistorr_(blz->istor,"IRITZ",5)+BLZistorr_(blz->istor,"JT",2),
-        BLZistorr_(blz->istor,"NRITZ",5));
+        BLZistorr_(blz->istor,"NRITZ",5));CHKERRQ(ierr);
       eps->its = eps->its + 1;
       if (eps->its >= eps->max_it || eps->nconv >= eps->nev) lflag = 5;
       break;
     case 2:  
       /* compute v = B u */
       for (i=0;i<nvopu;i++) {
-        ierr = VecPlaceArray( x, blz->u+i*eps->nloc );CHKERRQ(ierr);
-        ierr = VecPlaceArray( y, blz->v+i*eps->nloc );CHKERRQ(ierr);
-        ierr = IPApplyMatrix(eps->ip, x, y ); CHKERRQ(ierr);
+        ierr = VecPlaceArray(x,blz->u+i*eps->nloc);CHKERRQ(ierr);
+        ierr = VecPlaceArray(y,blz->v+i*eps->nloc);CHKERRQ(ierr);
+        ierr = IPApplyMatrix(eps->ip,x,y);CHKERRQ(ierr);
         ierr = VecResetArray(x);CHKERRQ(ierr);
-        ierr = VecResetArray(y);CHKERRQ(ierr);	
+        ierr = VecResetArray(y);CHKERRQ(ierr);
       }
       break;
     case 3:  
       /* update shift */
-      PetscInfo1(eps,"Factorization update (sigma=%g)\n",sigma);
+      ierr = PetscInfo1(eps,"Factorization update (sigma=%g)\n",sigma);CHKERRQ(ierr);
       ierr = STSetShift(eps->OP,sigma);CHKERRQ(ierr);
       ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
       ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
@@ -248,7 +248,7 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
     
   } while (lflag > 0);
 
-  ierr = VecRestoreArray( eps->V[0], &pV ); CHKERRQ(ierr);
+  ierr = VecRestoreArray(eps->V[0],&pV);CHKERRQ(ierr);
 
   eps->nconv  = BLZistorr_(blz->istor,"NTEIG",5);
   eps->reason = EPS_CONVERGED_TOL;
@@ -260,13 +260,12 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
   if (lflag!=0) { 
     char msg[2048] = "";
     for (i = 0; i < 33; i++) {
-      if (blz->istor[15] & (1 << i)) PetscStrcat(msg, blzpack_error[i]);
+      if (blz->istor[15] & (1 << i)) PetscStrcat(msg,blzpack_error[i]);
     }
-    SETERRQ2(PETSC_ERR_LIB,"Error in BLZPACK (code=%d): '%s'",blz->istor[15], msg); 
+    SETERRQ2(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in BLZPACK (code=%d): '%s'",blz->istor[15],msg); 
   }
-  ierr = VecDestroy(x);CHKERRQ(ierr);
-  ierr = VecDestroy(y);CHKERRQ(ierr);
-
+  ierr = VecDestroy(&x);CHKERRQ(ierr);
+  ierr = VecDestroy(&y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -285,23 +284,31 @@ PetscErrorCode EPSBackTransform_BLZPACK(EPS eps)
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSDestroy_BLZPACK"
-PetscErrorCode EPSDestroy_BLZPACK(EPS eps)
+#define __FUNCT__ "EPSReset_BLZPACK"
+PetscErrorCode EPSReset_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
   ierr = PetscFree(blz->istor);CHKERRQ(ierr);
   ierr = PetscFree(blz->rstor);CHKERRQ(ierr);
   ierr = PetscFree(blz->u);CHKERRQ(ierr);
   ierr = PetscFree(blz->v);CHKERRQ(ierr);
   ierr = PetscFree(blz->eig);CHKERRQ(ierr);
-  ierr = PetscFree(eps->data);CHKERRQ(ierr);
   ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSDestroy_BLZPACK"
+PetscErrorCode EPSDestroy_BLZPACK(EPS eps)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFree(eps->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetBlockSize_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetInterval_C","",PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetNSteps_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -311,16 +318,19 @@ PetscErrorCode EPSDestroy_BLZPACK(EPS eps)
 PetscErrorCode EPSView_BLZPACK(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  EPS_BLZPACK    *blz = (EPS_BLZPACK *) eps->data;
-  PetscTruth     isascii;
+  EPS_BLZPACK    *blz = (EPS_BLZPACK*)eps->data;
+  PetscBool      isascii;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) {
-    SETERRQ1(1,"Viewer type %s not supported for EPSBLZPACK",((PetscObject)viewer)->type_name);
+    SETERRQ1(((PetscObject)eps)->comm,1,"Viewer type %s not supported for EPSBLZPACK",((PetscObject)viewer)->type_name);
+  }
+  ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: block size=%d\n",blz->block_size);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: maximum number of steps per run=%d\n",blz->nsteps);CHKERRQ(ierr);
+  if (blz->slice) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  BLZPACK: computational interval [%f,%f]\n",eps->inta,eps->intb);CHKERRQ(ierr);
   }
-  ierr = PetscViewerASCIIPrintf(viewer,"block size of the block-Lanczos algorithm: %d\n",blz->block_size);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIPrintf(viewer,"computational interval: [%f,%f]\n",blz->initial,blz->final);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -331,13 +341,10 @@ PetscErrorCode EPSSetFromOptions_BLZPACK(EPS eps)
   PetscErrorCode ierr;
   EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
   PetscInt       bs,n;
-  PetscReal      interval[2];
-  PetscTruth     flg;
-  KSP            ksp;
-  PC             pc;
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsBegin(((PetscObject)eps)->comm,((PetscObject)eps)->prefix,"BLZPACK Options","EPS");CHKERRQ(ierr);
+  ierr = PetscOptionsHead("EPS BLZPACK Options");CHKERRQ(ierr);
 
   bs = blz->block_size;
   ierr = PetscOptionsInt("-eps_blzpack_block_size","Block size","EPSBlzpackSetBlockSize",bs,&bs,&flg);CHKERRQ(ierr);
@@ -347,24 +354,7 @@ PetscErrorCode EPSSetFromOptions_BLZPACK(EPS eps)
   ierr = PetscOptionsInt("-eps_blzpack_nsteps","Number of steps","EPSBlzpackSetNSteps",n,&n,&flg);CHKERRQ(ierr);
   if (flg) {ierr = EPSBlzpackSetNSteps(eps,n);CHKERRQ(ierr);}
 
-  interval[0] = blz->initial;
-  interval[1] = blz->final;
-  n = 2;
-  ierr = PetscOptionsRealArray("-eps_blzpack_interval","Computational interval","EPSBlzpackSetInterval",interval,&n,&flg);CHKERRQ(ierr);
-  if (flg) {
-    if (n==1) interval[1]=interval[0];
-    ierr = EPSBlzpackSetInterval(eps,interval[0],interval[1]);CHKERRQ(ierr);
-  }
-
-  if (blz->slice || eps->isgeneralized) {
-    ierr = STSetType(eps->OP,STSINVERT);CHKERRQ(ierr);
-    ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
-    ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
-    ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
-    ierr = PCSetType(pc,PCCHOLESKY);CHKERRQ(ierr);
-  }
-
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -373,12 +363,12 @@ EXTERN_C_BEGIN
 #define __FUNCT__ "EPSBlzpackSetBlockSize_BLZPACK"
 PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,PetscInt bs)
 {
-  EPS_BLZPACK *blz = (EPS_BLZPACK *) eps->data;;
+  EPS_BLZPACK *blz = (EPS_BLZPACK*)eps->data;;
 
   PetscFunctionBegin;
   if (bs == PETSC_DEFAULT) blz->block_size = 3;
   else if (bs <= 0) { 
-    SETERRQ(1, "Incorrect block size"); 
+    SETERRQ(((PetscObject)eps)->comm,1,"Incorrect block size"); 
   } else blz->block_size = PetscBLASIntCast(bs);
   PetscFunctionReturn(0);
 }
@@ -399,77 +389,15 @@ EXTERN_C_END
 .  -eps_blzpack_block_size - Sets the value of the block size
 
    Level: advanced
-
-.seealso: EPSBlzpackSetInterval()
 @*/
 PetscErrorCode EPSBlzpackSetBlockSize(EPS eps,PetscInt bs)
-{
-  PetscErrorCode ierr, (*f)(EPS,PetscInt);
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSBlzpackSetBlockSize_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,bs);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-EXTERN_C_BEGIN
-#undef __FUNCT__  
-#define __FUNCT__ "EPSBlzpackSetInterval_BLZPACK"
-PetscErrorCode EPSBlzpackSetInterval_BLZPACK(EPS eps,PetscReal initial,PetscReal final)
 {
   PetscErrorCode ierr;
-  EPS_BLZPACK *blz = (EPS_BLZPACK *) eps->data;;
 
   PetscFunctionBegin;
-  blz->initial    = initial;
-  blz->final      = final;
-  blz->slice      = 1;
-  ierr = STSetShift(eps->OP,initial);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-EXTERN_C_END
-
-#undef __FUNCT__  
-#define __FUNCT__ "EPSBlzpackSetInterval"
-/*@
-   EPSBlzpackSetInterval - Sets the computational interval for the BLZPACK
-   package.
-
-   Collective on EPS
-
-   Input Parameters:
-+  eps     - the eigenproblem solver context
-.  initial - lower bound of the interval
--  final   - upper bound of the interval
-
-   Options Database Key:
-.  -eps_blzpack_interval - Sets the bounds of the interval (two values
-   separated by commas)
-
-   Note:
-   The following possibilities are accepted (see Blzpack user's guide for
-   details).
-     initial>final: start seeking for eigenpairs in the upper bound
-     initial<final: start in the lower bound
-     initial=final: run around a single value (no interval)
-   
-   Level: advanced
-
-.seealso: EPSBlzpackSetBlockSize()
-@*/
-PetscErrorCode EPSBlzpackSetInterval(EPS eps,PetscReal initial,PetscReal final)
-{
-  PetscErrorCode ierr, (*f)(EPS,PetscReal,PetscReal);
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSBlzpackSetInterval_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,initial,final);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(eps,bs,2);
+  ierr = PetscTryMethod(eps,"EPSBlzpackSetBlockSize_C",(EPS,PetscInt),(eps,bs));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -478,7 +406,7 @@ EXTERN_C_BEGIN
 #define __FUNCT__ "EPSBlzpackSetNSteps_BLZPACK"
 PetscErrorCode EPSBlzpackSetNSteps_BLZPACK(EPS eps,PetscInt nsteps)
 {
-  EPS_BLZPACK *blz = (EPS_BLZPACK *) eps->data;
+  EPS_BLZPACK *blz = (EPS_BLZPACK*)eps->data;
 
   PetscFunctionBegin;
   if (nsteps == PETSC_DEFAULT) blz->nsteps = 0;
@@ -507,14 +435,12 @@ EXTERN_C_END
 @*/
 PetscErrorCode EPSBlzpackSetNSteps(EPS eps,PetscInt nsteps)
 {
-  PetscErrorCode ierr, (*f)(EPS,PetscInt);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSBlzpackSetNSteps_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,nsteps);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(eps,nsteps,2);
+  ierr = PetscTryMethod(eps,"EPSBlzpackSetNSteps_C",(EPS,PetscInt),(eps,nsteps));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -527,24 +453,21 @@ PetscErrorCode EPSCreate_BLZPACK(EPS eps)
   EPS_BLZPACK    *blzpack;
 
   PetscFunctionBegin;
-  ierr = PetscNew(EPS_BLZPACK,&blzpack);CHKERRQ(ierr);
-  PetscLogObjectMemory(eps,sizeof(EPS_BLZPACK));
-  eps->data                      = (void *) blzpack;
+  ierr = PetscNewLog(eps,EPS_BLZPACK,&blzpack);CHKERRQ(ierr);
+  eps->data                      = (void*)blzpack;
   eps->ops->setup                = EPSSetUp_BLZPACK;
   eps->ops->setfromoptions       = EPSSetFromOptions_BLZPACK;
   eps->ops->destroy              = EPSDestroy_BLZPACK;
+  eps->ops->reset                = EPSReset_BLZPACK;
   eps->ops->view                 = EPSView_BLZPACK;
   eps->ops->backtransform        = EPSBackTransform_BLZPACK;
   eps->ops->computevectors       = EPSComputeVectors_Default;
 
   blzpack->block_size = 3;
-  blzpack->initial = 0.0;
-  blzpack->final = 0.0;
   blzpack->slice = 0;
   blzpack->nsteps = 0;
 
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetBlockSize_C","EPSBlzpackSetBlockSize_BLZPACK",EPSBlzpackSetBlockSize_BLZPACK);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetInterval_C","EPSBlzpackSetInterval_BLZPACK",EPSBlzpackSetInterval_BLZPACK);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSBlzpackSetNSteps_C","EPSBlzpackSetNSteps_BLZPACK",EPSBlzpackSetNSteps_BLZPACK);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
