@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "src/eps/impls/external/trlan/trlanp.h"
+#include <private/epsimpl.h>          /*I "slepceps.h" I*/
+#include <../src/eps/impls/external/trlan/trlanp.h>
 
 PetscErrorCode EPSSolve_TRLAN(EPS);
 
@@ -36,37 +37,35 @@ PetscErrorCode EPSSetUp_TRLAN(EPS eps)
   EPS_TRLAN      *tr = (EPS_TRLAN *)eps->data;
 
   PetscFunctionBegin;
+  tr->maxlan = PetscBLASIntCast(PetscMax(7,eps->nev+PetscMin(eps->nev,6)));
   if (eps->ncv) {
-    if (eps->ncv<eps->nev) SETERRQ(1,"The value of ncv must be at least nev"); 
+    if (eps->ncv<eps->nev) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must be at least nev"); 
   }
-  else eps->ncv = eps->nev;
-  if (eps->mpd) PetscInfo(eps,"Warning: parameter mpd ignored\n");
+  else eps->ncv = tr->maxlan;
+  if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->max_it) eps->max_it = PetscMax(1000,eps->n);
   
   if (!eps->ishermitian)
-    SETERRQ(PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
 
   if (eps->isgeneralized)
-    SETERRQ(PETSC_ERR_SUP,"Requested method is not available for generalized problems");
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Requested method is not available for generalized problems");
 
   if (!eps->which) eps->which = EPS_LARGEST_REAL;
   if (eps->which!=EPS_LARGEST_REAL && eps->which!=EPS_SMALLEST_REAL && eps->which!=EPS_TARGET_REAL) 
-    SETERRQ(1,"Wrong value of eps->which");
+    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
 
   tr->restart = 0;
-  tr->maxlan = PetscBLASIntCast(eps->nev+PetscMin(eps->nev,6));
   if (tr->maxlan+1-eps->ncv<=0) { tr->lwork = PetscBLASIntCast(tr->maxlan*(tr->maxlan+10)); }
   else { tr->lwork = PetscBLASIntCast(eps->nloc*(tr->maxlan+1-eps->ncv) + tr->maxlan*(tr->maxlan+10)); }
   ierr = PetscMalloc(tr->lwork*sizeof(PetscReal),&tr->work);CHKERRQ(ierr);
 
-  if (eps->extraction) {
-     ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr);
-  }
+  if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
 
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  if (eps->leftvecs) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Left vectors not supported in this solver");
   eps->ops->solve = EPSSolve_TRLAN;
   PetscFunctionReturn(0);
 }
@@ -77,7 +76,7 @@ static PetscBLASInt MatMult_TRLAN(PetscBLASInt *n,PetscBLASInt *m,PetscReal *xin
 {
   PetscErrorCode ierr;
   Vec            x,y;
-  PetscBLASInt            i;
+  PetscBLASInt   i;
 
   PetscFunctionBegin;
   ierr = VecCreateMPIWithArray(((PetscObject)globaleps)->comm,*n,PETSC_DECIDE,PETSC_NULL,&x);CHKERRQ(ierr);
@@ -88,10 +87,10 @@ static PetscBLASInt MatMult_TRLAN(PetscBLASInt *n,PetscBLASInt *m,PetscReal *xin
     ierr = STApply(globaleps->OP,x,y);CHKERRQ(ierr);
     ierr = IPOrthogonalize(globaleps->ip,0,PETSC_NULL,globaleps->nds,PETSC_NULL,globaleps->DS,y,PETSC_NULL,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
     ierr = VecResetArray(x);CHKERRQ(ierr);
-    ierr = VecResetArray(y);CHKERRQ(ierr);	
+    ierr = VecResetArray(y);CHKERRQ(ierr);
   }
-  ierr = VecDestroy(x);CHKERRQ(ierr);
-  ierr = VecDestroy(y);CHKERRQ(ierr);
+  ierr = VecDestroy(&x);CHKERRQ(ierr);
+  ierr = VecDestroy(&y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -101,18 +100,17 @@ PetscErrorCode EPSSolve_TRLAN(EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       i;
-  PetscBLASInt   ipar[32], n, lohi, stat, ncv; 
-  EPS_TRLAN      *tr = (EPS_TRLAN *)eps->data;	   
-  PetscScalar    *pV;				   
+  PetscBLASInt   ipar[32],n,lohi,stat,ncv; 
+  EPS_TRLAN      *tr = (EPS_TRLAN *)eps->data;   
+  PetscScalar    *pV;
   
   PetscFunctionBegin;
-
   ncv = PetscBLASIntCast(eps->ncv);
   n = PetscBLASIntCast(eps->nloc);
   
   if (eps->which==EPS_LARGEST_REAL || eps->which==EPS_TARGET_REAL) lohi = 1;
   else if (eps->which==EPS_SMALLEST_REAL) lohi = -1;
-  else SETERRQ(1,"Wrong value of eps->which");
+  else SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
 
   globaleps = eps;
 
@@ -136,17 +134,29 @@ PetscErrorCode EPSSolve_TRLAN(EPS eps)
   ierr = EPSGetStartVector(eps,0,eps->V[0],PETSC_NULL);CHKERRQ(ierr);
   ierr = VecGetArray(eps->V[0],&pV);CHKERRQ(ierr);
 
-  TRLan_ ( MatMult_TRLAN, ipar, &n, &ncv, eps->eigr, pV, &n, tr->work, &tr->lwork );
+  TRLan_(MatMult_TRLAN,ipar,&n,&ncv,eps->eigr,pV,&n,tr->work,&tr->lwork);
 
-  ierr = VecRestoreArray( eps->V[0], &pV );CHKERRQ(ierr);
+  ierr = VecRestoreArray(eps->V[0],&pV);CHKERRQ(ierr);
 
   stat        = ipar[0];
   eps->nconv  = ipar[3];
   eps->its    = ipar[25];
   eps->reason = EPS_CONVERGED_TOL;
   
-  if (stat!=0) { SETERRQ1(PETSC_ERR_LIB,"Error in TRLAN (code=%d)",stat);}
+  if (stat!=0) { SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in TRLAN (code=%d)",stat);}
+  PetscFunctionReturn(0);
+}
 
+#undef __FUNCT__  
+#define __FUNCT__ "EPSReset_TRLAN"
+PetscErrorCode EPSReset_TRLAN(EPS eps)
+{
+  PetscErrorCode ierr;
+  EPS_TRLAN      *tr = (EPS_TRLAN *)eps->data;
+
+  PetscFunctionBegin;
+  ierr = PetscFree(tr->work);CHKERRQ(ierr);
+  ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -155,13 +165,9 @@ PetscErrorCode EPSSolve_TRLAN(EPS eps)
 PetscErrorCode EPSDestroy_TRLAN(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_TRLAN      *tr = (EPS_TRLAN *)eps->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscFree(tr->work);CHKERRQ(ierr);
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
-  ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -171,14 +177,12 @@ EXTERN_C_BEGIN
 PetscErrorCode EPSCreate_TRLAN(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_TRLAN      *trlan;
 
   PetscFunctionBegin;
-  ierr = PetscNew(EPS_TRLAN,&trlan);CHKERRQ(ierr);
-  PetscLogObjectMemory(eps,sizeof(EPS_TRLAN));
-  eps->data                      = (void *) trlan;
+  ierr = PetscNewLog(eps,EPS_TRLAN,&eps->data);CHKERRQ(ierr);
   eps->ops->setup                = EPSSetUp_TRLAN;
   eps->ops->destroy              = EPSDestroy_TRLAN;
+  eps->ops->reset                = EPSReset_TRLAN;
   eps->ops->backtransform        = EPSBackTransform_Default;
   eps->ops->computevectors       = EPSComputeVectors_Default;
   PetscFunctionReturn(0);
