@@ -17,7 +17,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -35,24 +35,24 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/epsimpl.h"                /*I "slepceps.h" I*/
-#include "slepcblaslapack.h"
+#include <private/epsimpl.h>                /*I "slepceps.h" I*/
+#include <slepcblaslapack.h>
 
-PetscErrorCode EPSSolve_ARNOLDI(EPS);
+PetscErrorCode EPSSolve_Arnoldi(EPS);
 
 typedef struct {
-  PetscTruth delayed;
+  PetscBool delayed;
 } EPS_ARNOLDI;
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSetUp_ARNOLDI"
-PetscErrorCode EPSSetUp_ARNOLDI(EPS eps)
+#define __FUNCT__ "EPSSetUp_Arnoldi"
+PetscErrorCode EPSSetUp_Arnoldi(EPS eps)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (eps->ncv) { /* ncv set */
-    if (eps->ncv<eps->nev) SETERRQ(1,"The value of ncv must be at least nev"); 
+    if (eps->ncv<eps->nev) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must be at least nev"); 
   }
   else if (eps->mpd) { /* mpd set */
     eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd);
@@ -62,11 +62,11 @@ PetscErrorCode EPSSetUp_ARNOLDI(EPS eps)
     else { eps->mpd = 500; eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd); }
   }
   if (!eps->mpd) eps->mpd = eps->ncv;
-  if (eps->ncv>eps->nev+eps->mpd) SETERRQ(1,"The value of ncv must not be larger than nev+mpd"); 
+  if (eps->ncv>eps->nev+eps->mpd) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must not be larger than nev+mpd"); 
   if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
-  if (!eps->which) eps->which = EPS_LARGEST_MAGNITUDE;
+  if (!eps->which) { ierr = EPSDefaultSetWhich(eps);CHKERRQ(ierr); }
   if (eps->ishermitian && (eps->which==EPS_LARGEST_IMAGINARY || eps->which==EPS_SMALLEST_IMAGINARY))
-    SETERRQ(1,"Wrong value of eps->which");
+    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
 
   if (!eps->extraction) {
     ierr = EPSSetExtraction(eps,EPS_RITZ);CHKERRQ(ierr);
@@ -78,15 +78,15 @@ PetscErrorCode EPSSetUp_ARNOLDI(EPS eps)
   if (eps->leftvecs) {
     ierr = PetscFree(eps->Tl);CHKERRQ(ierr);
     ierr = PetscMalloc(eps->ncv*eps->ncv*sizeof(PetscScalar),&eps->Tl);CHKERRQ(ierr);
-    PetscInfo(eps,"Warning: parameter mpd ignored\n");
+    ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr);
     ierr = EPSDefaultGetWork(eps,2);CHKERRQ(ierr);
   } else {
     ierr = EPSDefaultGetWork(eps,1);CHKERRQ(ierr);
   }
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(PETSC_ERR_SUP,"Left vectors not supported in this solver");
-  eps->ops->solve = EPSSolve_ARNOLDI;
+  if (eps->leftvecs) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  eps->ops->solve = EPSSolve_Arnoldi;
   PetscFunctionReturn(0);
 }
 
@@ -98,7 +98,7 @@ PetscErrorCode EPSSetUp_ARNOLDI(EPS eps)
    reorthogonalization is delayed to the next Arnoldi step. This version is
    more scalable but in some cases convergence may stagnate.
 */
-PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscReal *beta,PetscTruth *breakdown)
+PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscReal *beta,PetscBool *breakdown)
 {
   PetscErrorCode ierr;
   PetscInt       i,j,m=*M;
@@ -136,20 +136,20 @@ PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pets
       ierr = VecDotEnd(u,V[j-2],&dot2);CHKERRQ(ierr);
       if (PetscAbsScalar(dot2/norm2) > PETSC_MACHINE_EPSILON) {
         *breakdown = PETSC_TRUE;
-	*M = j-1;
-	*beta = norm2;
+        *M = j-1;
+        *beta = norm2;
 
-	if (m>100) { ierr = PetscFree(lhh);CHKERRQ(ierr); }
-	ierr = VecDestroy(u);CHKERRQ(ierr);
-	ierr = VecDestroy(t);CHKERRQ(ierr);
-	PetscFunctionReturn(0);
+        if (m>100) { ierr = PetscFree(lhh);CHKERRQ(ierr); }
+        ierr = VecDestroy(&u);CHKERRQ(ierr);
+        ierr = VecDestroy(&t);CHKERRQ(ierr);
+        PetscFunctionReturn(0);
       }
     }
     
     if (j>k) {      
-      norm1 = sqrt(PetscRealPart(dot));
+      norm1 = PetscSqrtReal(PetscRealPart(dot));
       for (i=0;i<j;i++)
-	H[ldh*j+i] = H[ldh*j+i]/norm1;
+        H[ldh*j+i] = H[ldh*j+i]/norm1;
       H[ldh*j+j] = H[ldh*j+j]/dot;
       
       ierr = VecCopy(V[j],t);CHKERRQ(ierr);
@@ -193,9 +193,8 @@ PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pets
   *breakdown = PETSC_FALSE;
   
   if (m>100) { ierr = PetscFree(lhh);CHKERRQ(ierr); }
-  ierr = VecDestroy(u);CHKERRQ(ierr);
-  ierr = VecDestroy(t);CHKERRQ(ierr);
-
+  ierr = VecDestroy(&u);CHKERRQ(ierr);
+  ierr = VecDestroy(&t);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -205,7 +204,7 @@ PetscErrorCode EPSDelayedArnoldi(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pets
    EPSDelayedArnoldi1 - This function is similar to EPSDelayedArnoldi1,
    but without reorthogonalization (only delayed normalization).
 */
-PetscErrorCode EPSDelayedArnoldi1(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscReal *beta,PetscTruth *breakdown)
+PetscErrorCode EPSDelayedArnoldi1(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscReal *beta,PetscBool *breakdown)
 {
   PetscErrorCode ierr;
   PetscInt       i,j,m=*M;
@@ -213,7 +212,6 @@ PetscErrorCode EPSDelayedArnoldi1(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pet
   PetscReal      norm=0.0;
 
   PetscFunctionBegin;
-
   for (j=k;j<m;j++) {
     ierr = STApply(eps->OP,V[j],f);CHKERRQ(ierr);
     ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,eps->nds,PETSC_NULL,eps->DS,f,PETSC_NULL,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
@@ -229,12 +227,12 @@ PetscErrorCode EPSDelayedArnoldi1(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pet
     }
     
     if (j>k) {      
-      norm = sqrt(PetscRealPart(dot));
+      norm = PetscSqrtReal(PetscRealPart(dot));
       ierr = VecScale(V[j],1.0/norm);CHKERRQ(ierr);
       H[ldh*(j-1)+j] = norm;
 
       for (i=0;i<j;i++)
-	H[ldh*j+i] = H[ldh*j+i]/norm;
+        H[ldh*j+i] = H[ldh*j+i]/norm;
       H[ldh*j+j] = H[ldh*j+j]/dot;      
       ierr = VecScale(f,1.0/norm);CHKERRQ(ierr);
     }
@@ -249,7 +247,6 @@ PetscErrorCode EPSDelayedArnoldi1(EPS eps,PetscScalar *H,PetscInt ldh,Vec *V,Pet
   ierr = IPNorm(eps->ip,f,beta);CHKERRQ(ierr);
   ierr = VecScale(f,1.0 / *beta);CHKERRQ(ierr);
   *breakdown = PETSC_FALSE;
-  
   PetscFunctionReturn(0);
 }
 
@@ -302,10 +299,10 @@ PetscErrorCode EPSProjectedArnoldi(EPS eps,PetscScalar *S,PetscInt lds,PetscScal
 PetscErrorCode EPSUpdateVectors(EPS eps,PetscInt n_,Vec *U,PetscInt s,PetscInt e,PetscScalar *Q,PetscInt ldq,PetscScalar *H,PetscInt ldh_)
 {
 #if defined(PETSC_MISSING_LAPACK_GESVD) 
-  SETERRQ(PETSC_ERR_SUP,"GESVD - Lapack routine is unavailable.");
+  SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"GESVD - Lapack routine is unavailable.");
 #else
   PetscErrorCode ierr;
-  PetscTruth     isrefined;
+  PetscBool      isrefined;
   PetscInt       i,j,k;
   PetscBLASInt   n1,lwork,idummy=1,info,n=n_,ldh=ldh_;
   PetscScalar    *B,sdummy,*work;
@@ -338,7 +335,7 @@ PetscErrorCode EPSUpdateVectors(EPS eps,PetscInt n_,Vec *U,PetscInt s,PetscInt e
   #else
       LAPACKgesvd_("N","O",&n1,&n,B,&n1,sigma,&sdummy,&idummy,&sdummy,&idummy,work,&lwork,sigma+n,&info);
   #endif
-      if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xGESVD %d",info);
+      if (info) SETERRQ1(((PetscObject)eps)->comm,PETSC_ERR_LIB,"Error in Lapack xGESVD %d",info);
       /* the smallest singular value is the new error estimate */
       eps->errest[k] = sigma[n-1];
       /* update vector with right singular vector associated to smallest singular value */
@@ -357,17 +354,17 @@ PetscErrorCode EPSUpdateVectors(EPS eps,PetscInt n_,Vec *U,PetscInt s,PetscInt e
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSolve_ARNOLDI"
-PetscErrorCode EPSSolve_ARNOLDI(EPS eps)
+#define __FUNCT__ "EPSSolve_Arnoldi"
+PetscErrorCode EPSSolve_Arnoldi(EPS eps)
 {
-  PetscErrorCode ierr;
-  PetscInt       i,k,lwork,nv;
-  Vec            f=eps->work[0];
-  PetscScalar    *H=eps->T,*U,*g,*work,*Hcopy;
-  PetscReal      beta,gnorm,corrf=1.0;
-  PetscTruth     breakdown;
-  IPOrthogonalizationRefinementType orthog_ref;
-  EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI *)eps->data;
+  PetscErrorCode     ierr;
+  PetscInt           i,k,lwork,nv;
+  Vec                f=eps->work[0];
+  PetscScalar        *H=eps->T,*U,*g,*work,*Hcopy;
+  PetscReal          beta,gnorm,corrf=1.0;
+  PetscBool          breakdown;
+  IPOrthogRefineType orthog_ref;
+  EPS_ARNOLDI        *arnoldi = (EPS_ARNOLDI *)eps->data;
 
   PetscFunctionBegin;
   ierr = PetscMemzero(eps->T,eps->ncv*eps->ncv*sizeof(PetscScalar));CHKERRQ(ierr);
@@ -394,7 +391,7 @@ PetscErrorCode EPSSolve_ARNOLDI(EPS eps)
     nv = PetscMin(eps->nconv+eps->mpd,eps->ncv);
     if (!arnoldi->delayed) {
       ierr = EPSBasicArnoldi(eps,PETSC_FALSE,H,eps->ncv,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
-    } else if (orthog_ref == IP_ORTH_REFINE_NEVER) {
+    } else if (orthog_ref == IP_ORTHOG_REFINE_NEVER) {
       ierr = EPSDelayedArnoldi1(eps,H,eps->ncv,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
     } else {
       ierr = EPSDelayedArnoldi(eps,H,eps->ncv,eps->V,eps->nconv,&nv,f,&beta,&breakdown);CHKERRQ(ierr);
@@ -412,25 +409,25 @@ PetscErrorCode EPSSolve_ARNOLDI(EPS eps)
       gnorm = 0.0;
       for (i=0;i<nv;i++)
         gnorm = gnorm + PetscRealPart(g[i]*PetscConj(g[i]));
-      corrf = sqrt(1.0+gnorm);
+      corrf = PetscSqrtReal(1.0+gnorm);
     }
 
     /* Solve projected problem */ 
     ierr = EPSProjectedArnoldi(eps,H,eps->ncv,U,nv);CHKERRQ(ierr);
 
     /* Check convergence */ 
-    ierr = EPSKrylovConvergence(eps,PETSC_FALSE,eps->nconv,nv-eps->nconv,H,eps->ncv,U,eps->V,nv,beta,corrf,&k,work);CHKERRQ(ierr);
+    ierr = EPSKrylovConvergence(eps,PETSC_FALSE,eps->trackall,eps->nconv,nv-eps->nconv,H,eps->ncv,U,eps->V,nv,beta,corrf,&k,work);CHKERRQ(ierr);
 
     ierr = EPSUpdateVectors(eps,nv,eps->V,eps->nconv,PetscMin(k+1,nv),U,nv,Hcopy,eps->ncv);CHKERRQ(ierr);
     eps->nconv = k;
 
-    EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,nv);
+    ierr = EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,nv);CHKERRQ(ierr);
     if (breakdown) {
-      PetscInfo2(eps,"Breakdown in Arnoldi method (it=%i norm=%g)\n",eps->its,beta);
+      ierr = PetscInfo2(eps,"Breakdown in Arnoldi method (it=%D norm=%G)\n",eps->its,beta);CHKERRQ(ierr);
       ierr = EPSGetStartVector(eps,k,eps->V[k],&breakdown);CHKERRQ(ierr);
       if (breakdown) {
         eps->reason = EPS_DIVERGED_BREAKDOWN;
-	PetscInfo(eps,"Unable to generate more start vectors\n");
+        ierr = PetscInfo(eps,"Unable to generate more start vectors\n");CHKERRQ(ierr);
       }
     }
     if (eps->its >= eps->max_it) eps->reason = EPS_DIVERGED_ITS;
@@ -449,29 +446,29 @@ PetscErrorCode EPSSolve_ARNOLDI(EPS eps)
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSetFromOptions_ARNOLDI"
-PetscErrorCode EPSSetFromOptions_ARNOLDI(EPS eps)
+#define __FUNCT__ "EPSSetFromOptions_Arnoldi"
+PetscErrorCode EPSSetFromOptions_Arnoldi(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscTruth     set,val;
+  PetscBool      set,val;
   EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI *)eps->data;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsBegin(((PetscObject)eps)->comm,((PetscObject)eps)->prefix,"ARNOLDI Options","EPS");CHKERRQ(ierr);
-  ierr = PetscOptionsTruth("-eps_arnoldi_delayed","Arnoldi with delayed reorthogonalization","EPSArnoldiSetDelayed",arnoldi->delayed,&val,&set);CHKERRQ(ierr);
+  ierr = PetscOptionsHead("EPS Arnoldi Options");CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-eps_arnoldi_delayed","Arnoldi with delayed reorthogonalization","EPSArnoldiSetDelayed",arnoldi->delayed,&val,&set);CHKERRQ(ierr);
   if (set) {
     ierr = EPSArnoldiSetDelayed(eps,val);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSArnoldiSetDelayed_ARNOLDI"
-PetscErrorCode EPSArnoldiSetDelayed_ARNOLDI(EPS eps,PetscTruth delayed)
+#define __FUNCT__ "EPSArnoldiSetDelayed_Arnoldi"
+PetscErrorCode EPSArnoldiSetDelayed_Arnoldi(EPS eps,PetscBool delayed)
 {
-  EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI *)eps->data;
+  EPS_ARNOLDI *arnoldi = (EPS_ARNOLDI *)eps->data;
 
   PetscFunctionBegin;
   arnoldi->delayed = delayed;
@@ -485,7 +482,7 @@ EXTERN_C_END
    EPSArnoldiSetDelayed - Activates or deactivates delayed reorthogonalization 
    in the Arnoldi iteration. 
 
-   Collective on EPS
+   Logically Collective on EPS
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -503,25 +500,23 @@ EXTERN_C_END
 
 .seealso: EPSArnoldiGetDelayed()
 @*/
-PetscErrorCode EPSArnoldiSetDelayed(EPS eps,PetscTruth delayed)
+PetscErrorCode EPSArnoldiSetDelayed(EPS eps,PetscBool delayed)
 {
-  PetscErrorCode ierr, (*f)(EPS,PetscTruth);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSArnoldiSetDelayed_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,delayed);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveBool(eps,delayed,2);
+  ierr = PetscTryMethod(eps,"EPSArnoldiSetDelayed_C",(EPS,PetscBool),(eps,delayed));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSArnoldiGetDelayed_ARNOLDI"
-PetscErrorCode EPSArnoldiGetDelayed_ARNOLDI(EPS eps,PetscTruth *delayed)
+#define __FUNCT__ "EPSArnoldiGetDelayed_Arnoldi"
+PetscErrorCode EPSArnoldiGetDelayed_Arnoldi(EPS eps,PetscBool *delayed)
 {
-  EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI *)eps->data;
+  EPS_ARNOLDI *arnoldi = (EPS_ARNOLDI *)eps->data;
 
   PetscFunctionBegin;
   *delayed = arnoldi->delayed;
@@ -535,7 +530,7 @@ EXTERN_C_END
    EPSArnoldiGetDelayed - Gets the type of reorthogonalization used during the Arnoldi
    iteration. 
 
-   Collective on EPS
+   Not Collective
 
    Input Parameter:
 .  eps - the eigenproblem solver context
@@ -547,75 +542,82 @@ EXTERN_C_END
 
 .seealso: EPSArnoldiSetDelayed()
 @*/
-PetscErrorCode EPSArnoldiGetDelayed(EPS eps,PetscTruth *delayed)
-{
-  PetscErrorCode ierr, (*f)(EPS,PetscTruth*);
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSArnoldiGetDelayed_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,delayed);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "EPSDestroy_ARNOLDI"
-PetscErrorCode EPSDestroy_ARNOLDI(EPS eps)
+PetscErrorCode EPSArnoldiGetDelayed(EPS eps,PetscBool *delayed)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = EPSDestroy_Default(eps);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidPointer(delayed,2);
+  ierr = PetscTryMethod(eps,"EPSArnoldiGetDelayed_C",(EPS,PetscBool*),(eps,delayed));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSReset_Arnoldi"
+PetscErrorCode EPSReset_Arnoldi(EPS eps)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFree(eps->T);CHKERRQ(ierr);
+  ierr = PetscFree(eps->Tl);CHKERRQ(ierr);
+  ierr = EPSReset_Default(eps);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSDestroy_Arnoldi"
+PetscErrorCode EPSDestroy_Arnoldi(EPS eps)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFree(eps->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiSetDelayed_C","",PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiGetDelayed_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSView_ARNOLDI"
-PetscErrorCode EPSView_ARNOLDI(EPS eps,PetscViewer viewer)
+#define __FUNCT__ "EPSView_Arnoldi"
+PetscErrorCode EPSView_Arnoldi(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  PetscTruth     isascii;
+  PetscBool      isascii;
   EPS_ARNOLDI    *arnoldi = (EPS_ARNOLDI *)eps->data;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) {
-    SETERRQ1(1,"Viewer type %s not supported for EPSARNOLDI",((PetscObject)viewer)->type_name);
+    SETERRQ1(((PetscObject)eps)->comm,1,"Viewer type %s not supported for EPS Arnoldi",((PetscObject)viewer)->type_name);
   }
   if (arnoldi->delayed) {
-    ierr = PetscViewerASCIIPrintf(viewer,"using delayed reorthogonalization\n");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  Arnoldi: using delayed reorthogonalization\n");CHKERRQ(ierr);
   }  
   PetscFunctionReturn(0);
 }
 
-EXTERN PetscErrorCode EPSSolve_TS_ARNOLDI(EPS);
+extern PetscErrorCode EPSSolve_TS_Arnoldi(EPS);
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSCreate_ARNOLDI"
-PetscErrorCode EPSCreate_ARNOLDI(EPS eps)
+#define __FUNCT__ "EPSCreate_Arnoldi"
+PetscErrorCode EPSCreate_Arnoldi(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_ARNOLDI    *arnoldi;
   
   PetscFunctionBegin;
-  ierr = PetscNew(EPS_ARNOLDI,&arnoldi);CHKERRQ(ierr);
-  PetscLogObjectMemory(eps,sizeof(EPS_ARNOLDI));
-  eps->data                      = (void *)arnoldi;
-  eps->ops->setup                = EPSSetUp_ARNOLDI;
-  eps->ops->setfromoptions       = EPSSetFromOptions_ARNOLDI;
-  eps->ops->destroy              = EPSDestroy_ARNOLDI;
-  eps->ops->view                 = EPSView_ARNOLDI;
+  ierr = PetscNewLog(eps,EPS_ARNOLDI,&eps->data);CHKERRQ(ierr);
+  eps->ops->setup                = EPSSetUp_Arnoldi;
+  eps->ops->setfromoptions       = EPSSetFromOptions_Arnoldi;
+  eps->ops->destroy              = EPSDestroy_Arnoldi;
+  eps->ops->reset                = EPSReset_Arnoldi;
+  eps->ops->view                 = EPSView_Arnoldi;
   eps->ops->backtransform        = EPSBackTransform_Default;
   eps->ops->computevectors       = EPSComputeVectors_Schur;
-  arnoldi->delayed               = PETSC_FALSE;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiSetDelayed_C","EPSArnoldiSetDelayed_ARNOLDI",EPSArnoldiSetDelayed_ARNOLDI);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiGetDelayed_C","EPSArnoldiGetDelayed_ARNOLDI",EPSArnoldiGetDelayed_ARNOLDI);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiSetDelayed_C","EPSArnoldiSetDelayed_Arnoldi",EPSArnoldiSetDelayed_Arnoldi);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSArnoldiGetDelayed_C","EPSArnoldiGetDelayed_Arnoldi",EPSArnoldiGetDelayed_Arnoldi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
