@@ -19,7 +19,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -37,9 +37,9 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/epsimpl.h"                /*I "slepceps.h" I*/
+#include <private/epsimpl.h>                /*I "slepceps.h" I*/
 
-PetscErrorCode EPSSolve_LANCZOS(EPS);
+PetscErrorCode EPSSolve_Lanczos(EPS);
 
 typedef struct {
   EPSLanczosReorthogType reorthog;
@@ -47,15 +47,15 @@ typedef struct {
 } EPS_LANCZOS;
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSetUp_LANCZOS"
-PetscErrorCode EPSSetUp_LANCZOS(EPS eps)
+#define __FUNCT__ "EPSSetUp_Lanczos"
+PetscErrorCode EPSSetUp_Lanczos(EPS eps)
 {
   EPS_LANCZOS    *lanczos = (EPS_LANCZOS *)eps->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (eps->ncv) { /* ncv set */
-    if (eps->ncv<eps->nev) SETERRQ(1,"The value of ncv must be at least nev"); 
+    if (eps->ncv<eps->nev) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must be at least nev"); 
   }
   else if (eps->mpd) { /* mpd set */
     eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd);
@@ -65,28 +65,28 @@ PetscErrorCode EPSSetUp_LANCZOS(EPS eps)
     else { eps->mpd = 500; eps->ncv = PetscMin(eps->n,eps->nev+eps->mpd); }
   }
   if (!eps->mpd) eps->mpd = eps->ncv;
-  if (eps->ncv>eps->nev+eps->mpd) SETERRQ(1,"The value of ncv must not be larger than nev+mpd"); 
+  if (eps->ncv>eps->nev+eps->mpd) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must not be larger than nev+mpd"); 
   if (!eps->max_it) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
 
-  if (!eps->which) eps->which = EPS_LARGEST_MAGNITUDE;
+  if (!eps->which) { ierr = EPSDefaultSetWhich(eps);CHKERRQ(ierr); }
   switch (eps->which) {
     case EPS_LARGEST_IMAGINARY:
     case EPS_SMALLEST_IMAGINARY:
     case EPS_TARGET_IMAGINARY:
-      SETERRQ(1,"Wrong value of eps->which");
+      SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
     default: ; /* default case to remove warning */
   }
   if (!eps->ishermitian)
-    SETERRQ(PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
   if (!eps->extraction) {
     ierr = EPSSetExtraction(eps,EPS_RITZ);CHKERRQ(ierr);
   } else if (eps->extraction!=EPS_RITZ) {
-    SETERRQ(PETSC_ERR_SUP,"Unsupported extraction type\n");
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Unsupported extraction type\n");
   }
 
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
   if (lanczos->reorthog == EPS_LANCZOS_REORTHOG_SELECTIVE) {
-    ierr = VecDuplicateVecs(eps->V[0],eps->ncv,&lanczos->AV);CHKERRQ(ierr);
+    ierr = VecDuplicateVecs(eps->t,eps->ncv,&lanczos->AV);CHKERRQ(ierr);
   }
   if (lanczos->reorthog == EPS_LANCZOS_REORTHOG_LOCAL) {
     ierr = EPSDefaultGetWork(eps,2);CHKERRQ(ierr);
@@ -95,8 +95,8 @@ PetscErrorCode EPSSetUp_LANCZOS(EPS eps)
   }
 
   /* dispatch solve method */
-  if (eps->leftvecs) SETERRQ(PETSC_ERR_SUP,"Left vectors not supported in this solver");
-  eps->ops->solve = EPSSolve_LANCZOS;
+  if (eps->leftvecs) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Left vectors not supported in this solver");
+  eps->ops->solve = EPSSolve_Lanczos;
   PetscFunctionReturn(0);
 }
 
@@ -111,17 +111,17 @@ PetscErrorCode EPSSetUp_LANCZOS(EPS eps)
    orthogonality that occurs in finite-precision arithmetic and, therefore, the 
    generated vectors are not guaranteed to be (semi-)orthogonal.
 */
-static PetscErrorCode EPSLocalLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscTruth *breakdown)
+static PetscErrorCode EPSLocalLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscBool *breakdown)
 {
   PetscErrorCode ierr;
   PetscInt       i,j,m = *M;
   PetscReal      norm;
-  PetscTruth     *which,lwhich[100];
+  PetscBool      *which,lwhich[100];
   PetscScalar    *hwork,lhwork[100];
   
   PetscFunctionBegin;  
   if (m > 100) {
-    ierr = PetscMalloc(sizeof(PetscTruth)*m,&which);CHKERRQ(ierr);
+    ierr = PetscMalloc(sizeof(PetscBool)*m,&which);CHKERRQ(ierr);
     ierr = PetscMalloc(m*sizeof(PetscScalar),&hwork);CHKERRQ(ierr);
   } else {
     which = lwhich;
@@ -165,14 +165,14 @@ static PetscErrorCode EPSLocalLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,V
 /*
    EPSSelectiveLanczos - Selective reorthogonalization.
 */
-static PetscErrorCode EPSSelectiveLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscTruth *breakdown,PetscReal anorm)
+static PetscErrorCode EPSSelectiveLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscBool *breakdown,PetscReal anorm)
 {
   PetscErrorCode ierr;
   EPS_LANCZOS    *lanczos = (EPS_LANCZOS *)eps->data;
   PetscInt       i,j,m = *M,n,nritz=0,nritzo;
   PetscReal      *d,*e,*ritz,norm;
   PetscScalar    *Y,*hwork,lhwork[100];
-  PetscTruth     *which,lwhich[100];
+  PetscBool      *which,lwhich[100];
 
   PetscFunctionBegin;
   ierr = PetscMalloc(m*sizeof(PetscReal),&d);CHKERRQ(ierr);
@@ -180,7 +180,7 @@ static PetscErrorCode EPSSelectiveLanczos(EPS eps,PetscReal *alpha,PetscReal *be
   ierr = PetscMalloc(m*sizeof(PetscReal),&ritz);CHKERRQ(ierr);
   ierr = PetscMalloc(m*m*sizeof(PetscScalar),&Y);CHKERRQ(ierr);
   if (m > 100) {
-    ierr = PetscMalloc(sizeof(PetscTruth)*m,&which);CHKERRQ(ierr);
+    ierr = PetscMalloc(sizeof(PetscBool)*m,&which);CHKERRQ(ierr);
     ierr = PetscMalloc(m*sizeof(PetscScalar),&hwork);CHKERRQ(ierr);
   } else {
     which = lwhich;
@@ -215,23 +215,23 @@ static PetscErrorCode EPSSelectiveLanczos(EPS eps,PetscReal *alpha,PetscReal *be
     nritzo = 0;
     for (i=0;i<n;i++)
       if (norm*PetscAbsScalar(Y[i*n+n-1]) < PETSC_SQRT_MACHINE_EPSILON*anorm)
-	nritzo++;
+        nritzo++;
 
     if (nritzo>nritz) {
       nritz = 0;
       for (i=0;i<n;i++) {
-	if (norm*PetscAbsScalar(Y[i*n+n-1]) < PETSC_SQRT_MACHINE_EPSILON*anorm) {
-	  ierr = SlepcVecMAXPBY(lanczos->AV[nritz],0.0,1.0,n,Y+i*n,V+k);CHKERRQ(ierr);
+        if (norm*PetscAbsScalar(Y[i*n+n-1]) < PETSC_SQRT_MACHINE_EPSILON*anorm) {
+          ierr = SlepcVecMAXPBY(lanczos->AV[nritz],0.0,1.0,n,Y+i*n,V+k);CHKERRQ(ierr);
           nritz++;
-	}
+        }
       }
     }
 
     if (nritz > 0) {
       ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,nritz,PETSC_NULL,lanczos->AV,f,hwork,&norm,breakdown);CHKERRQ(ierr);
       if (*breakdown) {
-	*M = j+1;
-	break;
+        *M = j+1;
+        break;
       }
     }
     
@@ -297,11 +297,11 @@ static void update_omega(PetscReal *omega,PetscReal *omega_old,PetscInt j,PetscR
 
 #undef __FUNCT__  
 #define __FUNCT__ "compute_int"
-static void compute_int(PetscTruth *which,PetscReal *mu,PetscInt j,PetscReal delta,PetscReal eta)
+static void compute_int(PetscBool *which,PetscReal *mu,PetscInt j,PetscReal delta,PetscReal eta)
 {
-  PetscInt   i,k,maxpos;
-  PetscReal  max;
-  PetscTruth found;
+  PetscInt  i,k,maxpos;
+  PetscReal max;
+  PetscBool found;
   
   PetscFunctionBegin;  
   /* initialize which */
@@ -325,12 +325,12 @@ static void compute_int(PetscTruth *which,PetscReal *mu,PetscInt j,PetscReal del
       /* find left interval */
       for (k=i;k>=0;k--) {
         if (PetscAbsReal(mu[k])<eta || which[k]) break;
-	else which[k] = PETSC_TRUE;
+        else which[k] = PETSC_TRUE;
       }
       /* find right interval */
       for (k=i+1;k<j;k++) {
         if (PetscAbsReal(mu[k])<eta || which[k]) break;
-	else which[k] = PETSC_TRUE;
+        else which[k] = PETSC_TRUE;
       }
     }
   PetscFunctionReturnVoid();
@@ -341,14 +341,15 @@ static void compute_int(PetscTruth *which,PetscReal *mu,PetscInt j,PetscReal del
 /*
    EPSPartialLanczos - Partial reorthogonalization.
 */
-static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscTruth *breakdown,PetscReal anorm)
+static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,Vec *V,PetscInt k,PetscInt *M,Vec f,PetscBool *breakdown,PetscReal anorm)
 {
   EPS_LANCZOS    *lanczos = (EPS_LANCZOS *)eps->data;
   PetscErrorCode ierr;
   PetscInt       i,j,m = *M;
   PetscReal      norm,*omega,lomega[100],*omega_old,lomega_old[100],eps1,delta,eta;
-  PetscTruth     *which,lwhich[100],*which2,lwhich2[100],
-                 reorth = PETSC_FALSE,force_reorth = PETSC_FALSE,fro = PETSC_FALSE,estimate_anorm = PETSC_FALSE;
+  PetscBool      *which,lwhich[100],*which2,lwhich2[100],
+                 reorth = PETSC_FALSE,force_reorth = PETSC_FALSE,
+                 fro = PETSC_FALSE,estimate_anorm = PETSC_FALSE;
   PetscScalar    *hwork,lhwork[100];
 
   PetscFunctionBegin;
@@ -360,8 +361,8 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
     omega_old = lomega_old;
   }
   if (m > 100) {
-    ierr = PetscMalloc(sizeof(PetscTruth)*m,&which);CHKERRQ(ierr);
-    ierr = PetscMalloc(sizeof(PetscTruth)*m,&which2);CHKERRQ(ierr);
+    ierr = PetscMalloc(sizeof(PetscBool)*m,&which);CHKERRQ(ierr);
+    ierr = PetscMalloc(sizeof(PetscBool)*m,&which2);CHKERRQ(ierr);
     ierr = PetscMalloc(m*sizeof(PetscScalar),&hwork);CHKERRQ(ierr);
   } else {
     which = lwhich;
@@ -369,9 +370,9 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
     hwork = lhwork;
   }
 
-  eps1 = sqrt((PetscReal)eps->n)*PETSC_MACHINE_EPSILON/2;
-  delta = PETSC_SQRT_MACHINE_EPSILON/sqrt((PetscReal)eps->ncv);
-  eta = pow(PETSC_MACHINE_EPSILON,3.0/4.0)/sqrt((PetscReal)eps->ncv);
+  eps1 = PetscSqrtReal((PetscReal)eps->n)*PETSC_MACHINE_EPSILON/2;
+  delta = PETSC_SQRT_MACHINE_EPSILON/PetscSqrtReal((PetscReal)eps->ncv);
+  eta = pow(PETSC_MACHINE_EPSILON,3.0/4.0)/PetscSqrtReal((PetscReal)eps->ncv);
   if (anorm < 0.0) {
     anorm = 1.0;
     estimate_anorm = PETSC_TRUE;
@@ -398,36 +399,36 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
       /* Estimate ||A|| if needed */ 
       if (estimate_anorm) {
         if (j>k) anorm = PetscMax(anorm,PetscAbsReal(alpha[j-k])+norm+beta[j-k-1]);
-	else anorm = PetscMax(anorm,PetscAbsReal(alpha[j-k])+norm);
+        else anorm = PetscMax(anorm,PetscAbsReal(alpha[j-k])+norm);
       }
 
       /* Check if reorthogonalization is needed */
       reorth = PETSC_FALSE;
       if (j>k) {      
-	update_omega(omega,omega_old,j-k,alpha,beta-1,eps1,anorm);
-	for (i=0;i<j-k;i++)
-	  if (PetscAbsScalar(omega[i]) > delta) reorth = PETSC_TRUE;
+        update_omega(omega,omega_old,j-k,alpha,beta-1,eps1,anorm);
+        for (i=0;i<j-k;i++)
+          if (PetscAbsScalar(omega[i]) > delta) reorth = PETSC_TRUE;
       }
 
       if (reorth || force_reorth) {
-	if (lanczos->reorthog == EPS_LANCZOS_REORTHOG_PERIODIC) {
-	  /* Periodic reorthogonalization */
-	  if (force_reorth) force_reorth = PETSC_FALSE;
-	  else force_reorth = PETSC_TRUE;
-	  ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,j-k,PETSC_NULL,V+k,f,hwork,&norm,breakdown);CHKERRQ(ierr);
-	  for (i=0;i<j-k;i++)
+        if (lanczos->reorthog == EPS_LANCZOS_REORTHOG_PERIODIC) {
+          /* Periodic reorthogonalization */
+          if (force_reorth) force_reorth = PETSC_FALSE;
+          else force_reorth = PETSC_TRUE;
+          ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,j-k,PETSC_NULL,V+k,f,hwork,&norm,breakdown);CHKERRQ(ierr);
+          for (i=0;i<j-k;i++)
             omega[i] = eps1;
-	} else {
-	  /* Partial reorthogonalization */
-	  if (force_reorth) force_reorth = PETSC_FALSE;
-	  else {
-	    force_reorth = PETSC_TRUE;
-	    compute_int(which2,omega,j-k,delta,eta);
-	    for (i=0;i<j-k;i++) 
-	      if (which2[i]) omega[i] = eps1;
-	  }
-	  ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,j-k,which2,V+k,f,hwork,&norm,breakdown);CHKERRQ(ierr);	
-	}	
+        } else {
+          /* Partial reorthogonalization */
+          if (force_reorth) force_reorth = PETSC_FALSE;
+          else {
+            force_reorth = PETSC_TRUE;
+            compute_int(which2,omega,j-k,delta,eta);
+            for (i=0;i<j-k;i++) 
+              if (which2[i]) omega[i] = eps1;
+          }
+          ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,j-k,which2,V+k,f,hwork,&norm,breakdown);CHKERRQ(ierr);
+        }
       }
     }
     
@@ -437,7 +438,7 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
     }
     if (!fro && norm*delta < anorm*eps1) {
       fro = PETSC_TRUE;
-      PetscInfo1(eps,"Switching to full reorthogonalization at iteration %i\n",eps->its);	
+      ierr = PetscInfo1(eps,"Switching to full reorthogonalization at iteration %D\n",eps->its);CHKERRQ(ierr);
     }
     
     beta[j-k] = norm;
@@ -450,8 +451,6 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
   if (m>100) {
     ierr = PetscFree(omega);CHKERRQ(ierr);
     ierr = PetscFree(omega_old);CHKERRQ(ierr);
-  }
-  if (m > 100) {
     ierr = PetscFree(which);CHKERRQ(ierr);
     ierr = PetscFree(which2);CHKERRQ(ierr);
     ierr = PetscFree(hwork);CHKERRQ(ierr);
@@ -478,14 +477,14 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
    This function simply calls another function which depends on the selected
    reorthogonalization strategy.
 */
-static PetscErrorCode EPSBasicLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,Vec *V,PetscInt k,PetscInt *m,Vec f,PetscTruth *breakdown,PetscReal anorm)
+static PetscErrorCode EPSBasicLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,Vec *V,PetscInt k,PetscInt *m,Vec f,PetscBool *breakdown,PetscReal anorm)
 {
-  EPS_LANCZOS *lanczos = (EPS_LANCZOS *)eps->data;
-  PetscErrorCode ierr;
-  IPOrthogonalizationRefinementType orthog_ref;
-  PetscScalar *T;
-  PetscInt i,n=*m;
-  PetscReal betam;
+  EPS_LANCZOS        *lanczos = (EPS_LANCZOS *)eps->data;
+  PetscScalar        *T;
+  PetscInt           i,n=*m;
+  PetscReal          betam;
+  PetscErrorCode     ierr;
+  IPOrthogRefineType orthog_ref;
 
   PetscFunctionBegin;
   switch (lanczos->reorthog) {
@@ -505,7 +504,7 @@ static PetscErrorCode EPSBasicLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,V
     case EPS_LANCZOS_REORTHOG_DELAYED:
       ierr = PetscMalloc(n*n*sizeof(PetscScalar),&T);CHKERRQ(ierr);
       ierr = IPGetOrthogonalization(eps->ip,PETSC_NULL,&orthog_ref,PETSC_NULL);CHKERRQ(ierr);
-      if (orthog_ref == IP_ORTH_REFINE_NEVER) {
+      if (orthog_ref == IP_ORTHOG_REFINE_NEVER) {
         ierr = EPSDelayedArnoldi1(eps,T,n,V,k,m,f,&betam,breakdown);CHKERRQ(ierr);       
       } else {
         ierr = EPSDelayedArnoldi(eps,T,n,V,k,m,f,&betam,breakdown);CHKERRQ(ierr);
@@ -516,22 +515,22 @@ static PetscErrorCode EPSBasicLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,V
       ierr = PetscFree(T);CHKERRQ(ierr);
       break;
     default:
-      SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid reorthogonalization type"); 
+      SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid reorthogonalization type"); 
   }
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSolve_LANCZOS"
-PetscErrorCode EPSSolve_LANCZOS(EPS eps)
+#define __FUNCT__ "EPSSolve_Lanczos"
+PetscErrorCode EPSSolve_Lanczos(EPS eps)
 {
-  EPS_LANCZOS *lanczos = (EPS_LANCZOS *)eps->data;
+  EPS_LANCZOS    *lanczos = (EPS_LANCZOS *)eps->data;
   PetscErrorCode ierr;
   PetscInt       nconv,i,j,k,l,x,n,m,*perm,restart,ncv=eps->ncv,r;
   Vec            w=eps->work[1],f=eps->work[0];
   PetscScalar    *Y,stmp;
   PetscReal      *d,*e,*ritz,*bnd,anorm,beta,norm,rtmp,resnorm;
-  PetscTruth     breakdown;
+  PetscBool      breakdown;
   char           *conv,ctmp;
 
   PetscFunctionBegin;
@@ -585,7 +584,7 @@ PetscErrorCode EPSSolve_LANCZOS(EPS eps)
 
     /* Compute restart vector */
     if (breakdown) {
-      PetscInfo2(eps,"Breakdown in Lanczos method (it=%i norm=%g)\n",eps->its,beta);
+      ierr = PetscInfo2(eps,"Breakdown in Lanczos method (it=%D norm=%G)\n",eps->its,beta);CHKERRQ(ierr);
     } else {
       restart = 0;
       while (restart<n && conv[restart] != 'N') restart++;
@@ -635,11 +634,11 @@ PetscErrorCode EPSSolve_LANCZOS(EPS eps)
     /* purge spurious ritz values */
     if (lanczos->reorthog == EPS_LANCZOS_REORTHOG_LOCAL) {
       for (i=0;i<k;i++) {
-	ierr = VecNorm(eps->V[nconv+i],NORM_2,&norm);CHKERRQ(ierr);
+        ierr = VecNorm(eps->V[nconv+i],NORM_2,&norm);CHKERRQ(ierr);
         ierr = VecScale(eps->V[nconv+i],1.0/norm);CHKERRQ(ierr);
         ierr = STApply(eps->OP,eps->V[nconv+i],w);CHKERRQ(ierr);
-	ierr = VecAXPY(w,-ritz[i],eps->V[nconv+i]);CHKERRQ(ierr);
-	ierr = VecNorm(w,NORM_2,&norm);CHKERRQ(ierr);
+        ierr = VecAXPY(w,-ritz[i],eps->V[nconv+i]);CHKERRQ(ierr);
+        ierr = VecNorm(w,NORM_2,&norm);CHKERRQ(ierr);
         ierr = (*eps->conv_func)(eps,ritz[i],eps->eigi[i],norm,&bnd[i],eps->conv_ctx);CHKERRQ(ierr);
         if (bnd[i]>=eps->tol) conv[i] = 'S';
       }
@@ -663,7 +662,7 @@ PetscErrorCode EPSSolve_LANCZOS(EPS eps)
       eps->eigr[nconv+i] = ritz[i];
       eps->errest[nconv+i] = bnd[i];
     }
-    EPSMonitor(eps,eps->its,nconv,eps->eigr,eps->eigi,eps->errest,nconv+n);
+    ierr = EPSMonitor(eps,eps->its,nconv,eps->eigr,eps->eigi,eps->errest,nconv+n);CHKERRQ(ierr);
     nconv = nconv+k;
     if (eps->its >= eps->max_it) eps->reason = EPS_DIVERGED_ITS;
     if (nconv >= eps->nev) eps->reason = EPS_CONVERGED_TOL;
@@ -671,17 +670,17 @@ PetscErrorCode EPSSolve_LANCZOS(EPS eps)
      if (eps->reason == EPS_CONVERGED_ITERATING) { /* copy restart vector */
       if (lanczos->reorthog == EPS_LANCZOS_REORTHOG_LOCAL && !breakdown) {
         /* Reorthonormalize restart vector */
-	ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,nconv,PETSC_NULL,eps->V,f,PETSC_NULL,&norm,&breakdown);CHKERRQ(ierr);
-	ierr = VecScale(f,1.0/norm);CHKERRQ(ierr);
+        ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,nconv,PETSC_NULL,eps->V,f,PETSC_NULL,&norm,&breakdown);CHKERRQ(ierr);
+        ierr = VecScale(f,1.0/norm);CHKERRQ(ierr);
       }
       if (breakdown) {
-	/* Use random vector for restarting */
-	PetscInfo(eps,"Using random vector for restart\n");
-	ierr = EPSGetStartVector(eps,nconv,f,&breakdown);CHKERRQ(ierr);
+        /* Use random vector for restarting */
+        ierr = PetscInfo(eps,"Using random vector for restart\n");CHKERRQ(ierr);
+        ierr = EPSGetStartVector(eps,nconv,f,&breakdown);CHKERRQ(ierr);
       }
       if (breakdown) { /* give up */
         eps->reason = EPS_DIVERGED_BREAKDOWN;
-        PetscInfo(eps,"Unable to generate more start vectors\n");
+        ierr = PetscInfo(eps,"Unable to generate more start vectors\n");CHKERRQ(ierr);
       } else {
         ierr = VecCopy(f,eps->V[nconv]);CHKERRQ(ierr);
       }
@@ -700,29 +699,27 @@ PetscErrorCode EPSSolve_LANCZOS(EPS eps)
   PetscFunctionReturn(0);
 }
 
-static const char *lanczoslist[6] = { "local", "full", "selective", "periodic", "partial" , "delayed" };
-
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSetFromOptions_LANCZOS"
-PetscErrorCode EPSSetFromOptions_LANCZOS(EPS eps)
+#define __FUNCT__ "EPSSetFromOptions_Lanczos"
+PetscErrorCode EPSSetFromOptions_Lanczos(EPS eps)
 {
-  PetscErrorCode ierr;
-  EPS_LANCZOS    *lanczos = (EPS_LANCZOS *)eps->data;
-  PetscTruth     flg;
-  PetscInt       i;
+  PetscErrorCode         ierr;
+  EPS_LANCZOS            *lanczos = (EPS_LANCZOS *)eps->data;
+  PetscBool              flg;
+  EPSLanczosReorthogType reorthog;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsBegin(((PetscObject)eps)->comm,((PetscObject)eps)->prefix,"LANCZOS Options","EPS");CHKERRQ(ierr);
-  ierr = PetscOptionsEList("-eps_lanczos_reorthog","Lanczos reorthogonalization","EPSLanczosSetReorthog",lanczoslist,6,lanczoslist[lanczos->reorthog],&i,&flg);CHKERRQ(ierr);
-  if (flg) lanczos->reorthog = (EPSLanczosReorthogType)i;
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  ierr = PetscOptionsHead("EPS Lanczos Options");CHKERRQ(ierr);
+  ierr = PetscOptionsEnum("-eps_lanczos_reorthog","Lanczos reorthogonalization","EPSLanczosSetReorthog",EPSLanczosReorthogTypes,(PetscEnum)lanczos->reorthog,(PetscEnum*)&reorthog,&flg);CHKERRQ(ierr);
+  if (flg) { ierr = EPSLanczosSetReorthog(eps,reorthog);CHKERRQ(ierr); }
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSLanczosSetReorthog_LANCZOS"
-PetscErrorCode EPSLanczosSetReorthog_LANCZOS(EPS eps,EPSLanczosReorthogType reorthog)
+#define __FUNCT__ "EPSLanczosSetReorthog_Lanczos"
+PetscErrorCode EPSLanczosSetReorthog_Lanczos(EPS eps,EPSLanczosReorthogType reorthog)
 {
   EPS_LANCZOS *lanczos = (EPS_LANCZOS *)eps->data;
 
@@ -737,7 +734,7 @@ PetscErrorCode EPSLanczosSetReorthog_LANCZOS(EPS eps,EPSLanczosReorthogType reor
       lanczos->reorthog = reorthog;
       break;
     default:
-      SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid reorthogonalization type");
+      SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid reorthogonalization type");
   }
   PetscFunctionReturn(0);
 }
@@ -749,7 +746,7 @@ EXTERN_C_END
    EPSLanczosSetReorthog - Sets the type of reorthogonalization used during the Lanczos
    iteration. 
 
-   Collective on EPS
+   Logically Collective on EPS
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -765,23 +762,22 @@ EXTERN_C_END
 @*/
 PetscErrorCode EPSLanczosSetReorthog(EPS eps,EPSLanczosReorthogType reorthog)
 {
-  PetscErrorCode ierr, (*f)(EPS,EPSLanczosReorthogType);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSLanczosSetReorthog_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,reorthog);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(eps,reorthog,2);
+  ierr = PetscTryMethod(eps,"EPSLanczosSetReorthog_C",(EPS,EPSLanczosReorthogType),(eps,reorthog));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSLanczosGetReorthog_LANCZOS"
-PetscErrorCode EPSLanczosGetReorthog_LANCZOS(EPS eps,EPSLanczosReorthogType *reorthog)
+#define __FUNCT__ "EPSLanczosGetReorthog_Lanczos"
+PetscErrorCode EPSLanczosGetReorthog_Lanczos(EPS eps,EPSLanczosReorthogType *reorthog)
 {
   EPS_LANCZOS *lanczos = (EPS_LANCZOS *)eps->data;
+
   PetscFunctionBegin;
   *reorthog = lanczos->reorthog;
   PetscFunctionReturn(0);
@@ -794,7 +790,7 @@ EXTERN_C_END
    EPSLanczosGetReorthog - Gets the type of reorthogonalization used during the Lanczos
    iteration. 
 
-   Collective on EPS
+   Not Collective
 
    Input Parameter:
 .  eps - the eigenproblem solver context
@@ -808,71 +804,76 @@ EXTERN_C_END
 @*/
 PetscErrorCode EPSLanczosGetReorthog(EPS eps,EPSLanczosReorthogType *reorthog)
 {
-  PetscErrorCode ierr, (*f)(EPS,EPSLanczosReorthogType*);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSLanczosGetReorthog_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,reorthog);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidPointer(reorthog,2);
+  ierr = PetscTryMethod(eps,"EPSLanczosGetReorthog_C",(EPS,EPSLanczosReorthogType*),(eps,reorthog));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSDestroy_LANCZOS"
-PetscErrorCode EPSDestroy_LANCZOS(EPS eps)
+#define __FUNCT__ "EPSReset_Lanczos"
+PetscErrorCode EPSReset_Lanczos(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_LANCZOS    *lanczos = (EPS_LANCZOS *)eps->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (lanczos->AV) { ierr = VecDestroyVecs(lanczos->AV,eps->ncv);CHKERRQ(ierr); }
-  ierr = EPSDestroy_Default(eps);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(eps->ncv,&lanczos->AV);CHKERRQ(ierr);
+  ierr = EPSReset_Default(eps);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSDestroy_Lanczos"
+PetscErrorCode EPSDestroy_Lanczos(EPS eps)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFree(eps->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSLanczosSetReorthog_C","",PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSLanczosGetReorthog_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSView_LANCZOS"
-PetscErrorCode EPSView_LANCZOS(EPS eps,PetscViewer viewer)
+#define __FUNCT__ "EPSView_Lanczos"
+PetscErrorCode EPSView_Lanczos(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   EPS_LANCZOS    *lanczos = (EPS_LANCZOS *)eps->data;
-  PetscTruth     isascii;
+  PetscBool      isascii;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) {
-    SETERRQ1(1,"Viewer type %s not supported for EPSLANCZOS",((PetscObject)viewer)->type_name);
+    SETERRQ1(((PetscObject)eps)->comm,1,"Viewer type %s not supported for EPS Lanczos",((PetscObject)viewer)->type_name);
   }  
-  ierr = PetscViewerASCIIPrintf(viewer,"reorthogonalization: %s\n",lanczoslist[lanczos->reorthog]);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  Lanczos: %s reorthogonalization\n",EPSLanczosReorthogTypes[lanczos->reorthog]);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSCreate_LANCZOS"
-PetscErrorCode EPSCreate_LANCZOS(EPS eps)
+#define __FUNCT__ "EPSCreate_Lanczos"
+PetscErrorCode EPSCreate_Lanczos(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_LANCZOS    *lanczos;
 
   PetscFunctionBegin;
-  ierr = PetscNew(EPS_LANCZOS,&lanczos);CHKERRQ(ierr);
-  PetscLogObjectMemory(eps,sizeof(EPS_LANCZOS));
-  eps->data                      = (void *) lanczos;
-  eps->ops->setup                = EPSSetUp_LANCZOS;
-  eps->ops->setfromoptions       = EPSSetFromOptions_LANCZOS;
-  eps->ops->destroy              = EPSDestroy_LANCZOS;
-  eps->ops->view                 = EPSView_LANCZOS;
+  ierr = PetscNewLog(eps,EPS_LANCZOS,&eps->data);CHKERRQ(ierr);
+  eps->ops->setup                = EPSSetUp_Lanczos;
+  eps->ops->setfromoptions       = EPSSetFromOptions_Lanczos;
+  eps->ops->destroy              = EPSDestroy_Lanczos;
+  eps->ops->reset                = EPSReset_Lanczos;
+  eps->ops->view                 = EPSView_Lanczos;
   eps->ops->backtransform        = EPSBackTransform_Default;
   eps->ops->computevectors       = EPSComputeVectors_Hermitian;
-  lanczos->reorthog              = EPS_LANCZOS_REORTHOG_LOCAL;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSLanczosSetReorthog_C","EPSLanczosSetReorthog_LANCZOS",EPSLanczosSetReorthog_LANCZOS);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSLanczosGetReorthog_C","EPSLanczosGetReorthog_LANCZOS",EPSLanczosGetReorthog_LANCZOS);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSLanczosSetReorthog_C","EPSLanczosSetReorthog_Lanczos",EPSLanczosSetReorthog_Lanczos);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSLanczosGetReorthog_C","EPSLanczosGetReorthog_Lanczos",EPSLanczosGetReorthog_Lanczos);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
