@@ -22,7 +22,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -40,48 +40,46 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/epsimpl.h"                /*I "slepceps.h" I*/
-#include "slepcblaslapack.h"
+#include <private/epsimpl.h>                /*I "slepceps.h" I*/
+#include <slepcblaslapack.h>
 
-PetscErrorCode EPSSolve_POWER(EPS);
-PetscErrorCode EPSSolve_TS_POWER(EPS);
+PetscErrorCode EPSSolve_Power(EPS);
+PetscErrorCode EPSSolve_TS_Power(EPS);
 
 typedef struct {
   EPSPowerShiftType shift_type;
 } EPS_POWER;
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSetUp_POWER"
-PetscErrorCode EPSSetUp_POWER(EPS eps)
+#define __FUNCT__ "EPSSetUp_Power"
+PetscErrorCode EPSSetUp_Power(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_POWER      *power = (EPS_POWER *)eps->data;
-  PetscTruth     flg;
+  PetscBool      flg;
   STMatMode      mode;
 
   PetscFunctionBegin;
   if (eps->ncv) {
-    if (eps->ncv<eps->nev) SETERRQ(1,"The value of ncv must be at least nev"); 
+    if (eps->ncv<eps->nev) SETERRQ(((PetscObject)eps)->comm,1,"The value of ncv must be at least nev"); 
   }
   else eps->ncv = eps->nev;
-  if (eps->mpd) PetscInfo(eps,"Warning: parameter mpd ignored\n");
+  if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
   if (!eps->max_it) eps->max_it = PetscMax(2000,100*eps->n);
-  if (!eps->which) eps->which = EPS_LARGEST_MAGNITUDE;
-  if (eps->which!=EPS_LARGEST_MAGNITUDE)
-    SETERRQ(1,"Wrong value of eps->which");
+  if (!eps->which) { ierr = EPSDefaultSetWhich(eps);CHKERRQ(ierr); }
+  if (eps->which!=EPS_LARGEST_MAGNITUDE && eps->which !=EPS_TARGET_MAGNITUDE)
+    SETERRQ(((PetscObject)eps)->comm,1,"Wrong value of eps->which");
   if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) {
-    ierr = PetscTypeCompare((PetscObject)eps->OP,STSINVERT,&flg);CHKERRQ(ierr);
+    ierr = PetscTypeCompareAny((PetscObject)eps->OP,&flg,STSINVERT,STCAYLEY,"");CHKERRQ(ierr);
     if (!flg) 
-      SETERRQ(PETSC_ERR_SUP,"Variable shifts only allowed in shift-and-invert ST");
+      SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Variable shifts only allowed in shift-and-invert or Cayley ST");
     ierr = STGetMatMode(eps->OP,&mode);CHKERRQ(ierr); 
     if (mode == ST_MATMODE_INPLACE)
-      SETERRQ(PETSC_ERR_SUP,"ST matrix mode inplace does not work with variable shifts");
-  }
-  if (eps->extraction) {
-     ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr);
+      SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"ST matrix mode inplace does not work with variable shifts");
   }
+  if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
   if (eps->balance!=EPS_BALANCE_NONE)
-    SETERRQ(PETSC_ERR_SUP,"Balancing not supported in this solver");
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Balancing not supported in this solver");
   ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
   if (eps->leftvecs) {
     ierr = EPSDefaultGetWork(eps,3);CHKERRQ(ierr);
@@ -90,23 +88,23 @@ PetscErrorCode EPSSetUp_POWER(EPS eps)
   }
 
   /* dispatch solve method */
-  if (eps->leftvecs) eps->ops->solve = EPSSolve_TS_POWER;
-  else eps->ops->solve = EPSSolve_POWER;
+  if (eps->leftvecs) eps->ops->solve = EPSSolve_TS_Power;
+  else eps->ops->solve = EPSSolve_Power;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSolve_POWER"
-PetscErrorCode EPSSolve_POWER(EPS eps)
+#define __FUNCT__ "EPSSolve_Power"
+PetscErrorCode EPSSolve_Power(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_POWER      *power = (EPS_POWER *)eps->data;
   PetscInt       i;
-  Vec            v, y, e;
+  Vec            v,y,e;
   Mat            A;
-  PetscReal      relerr, norm, rt1, rt2, cs1, anorm;
-  PetscScalar    theta, rho, delta, sigma, alpha2, beta1, sn1;
-  PetscTruth     breakdown,*select = PETSC_NULL,hasnorm;
+  PetscReal      relerr,norm,rt1,rt2,cs1,anorm;
+  PetscScalar    theta,rho,delta,sigma,alpha2,beta1,sn1;
+  PetscBool      breakdown,*select = PETSC_NULL,hasnorm;
 
   PetscFunctionBegin;
   v = eps->V[0];
@@ -119,7 +117,7 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
     ierr = MatHasOperation(A,MATOP_NORM,&hasnorm);CHKERRQ(ierr);
     if (hasnorm) {
       ierr = MatNorm(A,NORM_INFINITY,&anorm);CHKERRQ(ierr);
-      ierr = PetscMalloc(eps->nev*sizeof(PetscTruth),&select);CHKERRQ(ierr);
+      ierr = PetscMalloc(eps->nev*sizeof(PetscBool),&select);CHKERRQ(ierr);
     }
   }
 
@@ -154,7 +152,7 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
       delta = norm;
 
       /* compute relative error */
-      if (rho == 0.0) relerr = PETSC_MAX;
+      if (rho == 0.0) relerr = PETSC_MAX_REAL;
       else relerr = 1.0 / (norm*PetscAbsScalar(rho));
 
       /* approximate eigenvalue is the shift */
@@ -168,7 +166,7 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
         rho = rho + theta/(delta*delta);  /* Rayleigh quotient R(v) */
         if (power->shift_type == EPS_POWER_SHIFT_WILKINSON) {
 #if defined(SLEPC_MISSING_LAPACK_LAEV2)
-          SETERRQ(PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable.");
+          SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable.");
 #else 
           /* beta1 is the norm of the residual associated to R(v) */
           ierr = VecAXPY(v,-theta/(delta*delta),y);CHKERRQ(ierr);
@@ -188,21 +186,21 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
           else rho = rt2;
 #endif 
         }
-	/* update operator according to new shift */
-	PetscPushErrorHandler(PetscIgnoreErrorHandler,PETSC_NULL);
-	ierr = STSetShift(eps->OP,rho);
-	PetscPopErrorHandler();
-	if (ierr) {
+        /* update operator according to new shift */
+        PetscPushErrorHandler(PetscIgnoreErrorHandler,PETSC_NULL);
+        ierr = STSetShift(eps->OP,rho);
+        PetscPopErrorHandler();
+        if (ierr) {
           eps->eigr[eps->nconv] = rho;
-	  relerr = PETSC_MACHINE_EPSILON;
-	  rho = sigma;
+          relerr = PETSC_MACHINE_EPSILON;
+          rho = sigma;
           ierr = STSetShift(eps->OP,rho);CHKERRQ(ierr);
-	} 	
+        }
       }
     }
 
     eps->errest[eps->nconv] = relerr;
-    EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,eps->nconv+1); 
+    ierr = EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,eps->nconv+1);CHKERRQ(ierr);
 
     /* purge previously converged eigenvectors */
     if (select) {
@@ -224,33 +222,30 @@ PetscErrorCode EPSSolve_POWER(EPS eps)
       eps->nconv = eps->nconv + 1;
       if (eps->nconv==eps->nev) eps->reason = EPS_CONVERGED_TOL;
       else {
-	v = eps->V[eps->nconv];
-	ierr = EPSGetStartVector(eps,eps->nconv,v,&breakdown);CHKERRQ(ierr);
-	if (breakdown) {
+        v = eps->V[eps->nconv];
+        ierr = EPSGetStartVector(eps,eps->nconv,v,&breakdown);CHKERRQ(ierr);
+        if (breakdown) {
           eps->reason = EPS_DIVERGED_BREAKDOWN;
-	  PetscInfo(eps,"Unable to generate more start vectors\n");
-	}
+          ierr = PetscInfo(eps,"Unable to generate more start vectors\n");CHKERRQ(ierr);
+        }
       }
     }
-
     if (eps->its >= eps->max_it) eps->reason = EPS_DIVERGED_ITS;
   }
-
   ierr = PetscFree(select);CHKERRQ(ierr);
-
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSolve_TS_POWER"
-PetscErrorCode EPSSolve_TS_POWER(EPS eps)
+#define __FUNCT__ "EPSSolve_TS_Power"
+PetscErrorCode EPSSolve_TS_Power(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_POWER      *power = (EPS_POWER *)eps->data;
-  Vec            v, w, y, z, e;
+  Vec            v,w,y,z,e;
   Mat            A;
-  PetscReal      relerr, norm, rt1, rt2, cs1;
-  PetscScalar    theta, alpha, beta, rho, delta, sigma, alpha2, beta1, sn1;
+  PetscReal      relerr,norm,rt1,rt2,cs1;
+  PetscScalar    theta,alpha,beta,rho,delta,sigma,alpha2,beta1,sn1;
 
   PetscFunctionBegin;
   v = eps->V[0];
@@ -295,11 +290,11 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
 
       /* delta = sqrt(y,z)_B */
       ierr = IPInnerProduct(eps->ip,y,z,&alpha);CHKERRQ(ierr);
-      if (alpha==0.0) SETERRQ(1,"Breakdown in two-sided Power/RQI");
+      if (alpha==0.0) SETERRQ(((PetscObject)eps)->comm,1,"Breakdown in two-sided Power/RQI");
       delta = PetscSqrtScalar(alpha);
 
       /* compute relative error */
-      if (rho == 0.0) relerr = PETSC_MAX;
+      if (rho == 0.0) relerr = PETSC_MAX_REAL;
       else relerr = 1.0 / (PetscAbsScalar(delta*rho));
       eps->errest[eps->nconv] = relerr;
       eps->errest_left[eps->nconv] = relerr;
@@ -315,7 +310,7 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
         rho = rho + theta/(delta*delta);  /* Rayleigh quotient R(v,w) */
         if (power->shift_type == EPS_POWER_SHIFT_WILKINSON) {
 #if defined(SLEPC_MISSING_LAPACK_LAEV2)
-          SETERRQ(PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable.");
+          SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"LAEV2 - Lapack routine is unavailable.");
 #else 
           /* beta1 is the norm of the residual associated to R(v,w) */
           ierr = VecAXPY(v,-theta/(delta*delta),y);CHKERRQ(ierr);
@@ -335,22 +330,22 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
           else rho = rt2;
 #endif 
         }
-	/* update operator according to new shift */
-	PetscPushErrorHandler(PetscIgnoreErrorHandler,PETSC_NULL);
-	ierr = STSetShift(eps->OP,rho);
-	PetscPopErrorHandler();
-	if (ierr) {
+        /* update operator according to new shift */
+        PetscPushErrorHandler(PetscIgnoreErrorHandler,PETSC_NULL);
+        ierr = STSetShift(eps->OP,rho);
+        PetscPopErrorHandler();
+        if (ierr) {
           eps->eigr[eps->nconv] = rho;
-	  eps->errest[eps->nconv] = PETSC_MACHINE_EPSILON;
-	  eps->errest_left[eps->nconv] = PETSC_MACHINE_EPSILON;
-	  rho = sigma;
+          eps->errest[eps->nconv] = PETSC_MACHINE_EPSILON;
+          eps->errest_left[eps->nconv] = PETSC_MACHINE_EPSILON;
+          rho = sigma;
           ierr = STSetShift(eps->OP,rho);CHKERRQ(ierr);
-	}
+        }
       }
     }
 
-    EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,eps->nconv+1); 
-    EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest_left,eps->nconv+1); 
+    ierr = EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,eps->nconv+1);CHKERRQ(ierr);
+    ierr = EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest_left,eps->nconv+1);CHKERRQ(ierr);
 
     /* purge previously converged eigenvectors */
     ierr = IPBiOrthogonalize(eps->ip,eps->nconv,eps->V,eps->W,z,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
@@ -360,7 +355,7 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
     ierr = VecCopy(y,v);CHKERRQ(ierr);
     ierr = VecCopy(z,w);CHKERRQ(ierr);
     ierr = IPInnerProduct(eps->ip,y,z,&alpha);CHKERRQ(ierr);
-    if (alpha==0.0) SETERRQ(1,"Breakdown in two-sided Power/RQI");
+    if (alpha==0.0) SETERRQ(((PetscObject)eps)->comm,1,"Breakdown in two-sided Power/RQI");
     delta = PetscSqrtScalar(PetscAbsScalar(alpha));
     beta = 1.0/PetscConj(alpha/delta);
     delta = 1.0/delta;
@@ -377,19 +372,17 @@ PetscErrorCode EPSSolve_TS_POWER(EPS eps)
       ierr = EPSGetStartVectorLeft(eps,eps->nconv,w,PETSC_NULL);CHKERRQ(ierr);
     }
   }
-
-  if( eps->nconv == eps->nev ) eps->reason = EPS_CONVERGED_TOL;
+  if (eps->nconv == eps->nev) eps->reason = EPS_CONVERGED_TOL;
   else eps->reason = EPS_DIVERGED_ITS;
-
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSBackTransform_POWER"
-PetscErrorCode EPSBackTransform_POWER(EPS eps)
+#define __FUNCT__ "EPSBackTransform_Power"
+PetscErrorCode EPSBackTransform_Power(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_POWER *power = (EPS_POWER *)eps->data;
+  EPS_POWER      *power = (EPS_POWER *)eps->data;
 
   PetscFunctionBegin;
   if (power->shift_type == EPS_POWER_SHIFT_CONSTANT) {
@@ -399,30 +392,29 @@ PetscErrorCode EPSBackTransform_POWER(EPS eps)
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSSetFromOptions_POWER"
-PetscErrorCode EPSSetFromOptions_POWER(EPS eps)
+#define __FUNCT__ "EPSSetFromOptions_Power"
+PetscErrorCode EPSSetFromOptions_Power(EPS eps)
 {
-  PetscErrorCode ierr;
-  EPS_POWER      *power = (EPS_POWER *)eps->data;
-  PetscTruth     flg;
-  PetscInt       i;
-  const char     *shift_list[3] = { "constant", "rayleigh", "wilkinson" };
+  PetscErrorCode    ierr;
+  EPS_POWER         *power = (EPS_POWER *)eps->data;
+  PetscBool         flg;
+  EPSPowerShiftType shift;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsBegin(((PetscObject)eps)->comm,((PetscObject)eps)->prefix,"POWER Options","EPS");CHKERRQ(ierr);
-  ierr = PetscOptionsEList("-eps_power_shift_type","Shift type","EPSPowerSetShiftType",shift_list,3,shift_list[power->shift_type],&i,&flg);CHKERRQ(ierr);
-  if (flg ) power->shift_type = (EPSPowerShiftType)i;
+  ierr = PetscOptionsHead("EPS Power Options");CHKERRQ(ierr);
+  ierr = PetscOptionsEnum("-eps_power_shift_type","Shift type","EPSPowerSetShiftType",EPSPowerShiftTypes,(PetscEnum)power->shift_type,(PetscEnum*)&shift,&flg);CHKERRQ(ierr);
+  if (flg) { ierr = EPSPowerSetShiftType(eps,shift);CHKERRQ(ierr); }
   if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) {
     ierr = STSetType(eps->OP,STSINVERT);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSPowerSetShiftType_POWER"
-PetscErrorCode EPSPowerSetShiftType_POWER(EPS eps,EPSPowerShiftType shift)
+#define __FUNCT__ "EPSPowerSetShiftType_Power"
+PetscErrorCode EPSPowerSetShiftType_Power(EPS eps,EPSPowerShiftType shift)
 {
   EPS_POWER *power = (EPS_POWER *)eps->data;
 
@@ -434,7 +426,7 @@ PetscErrorCode EPSPowerSetShiftType_POWER(EPS eps,EPSPowerShiftType shift)
       power->shift_type = shift;
       break;
     default:
-      SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid shift type");
+      SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid shift type");
   }
   PetscFunctionReturn(0);
 }
@@ -447,7 +439,7 @@ EXTERN_C_END
    iteration. This can be used to emulate the Rayleigh Quotient Iteration
    (RQI) method.
 
-   Collective on EPS
+   Logically Collective on EPS
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -472,23 +464,22 @@ EXTERN_C_END
 @*/
 PetscErrorCode EPSPowerSetShiftType(EPS eps,EPSPowerShiftType shift)
 {
-  PetscErrorCode ierr, (*f)(EPS,EPSPowerShiftType);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSPowerSetShiftType_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,shift);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(eps,shift,2);
+  ierr = PetscTryMethod(eps,"EPSPowerSetShiftType_C",(EPS,EPSPowerShiftType),(eps,shift));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSPowerGetShiftType_POWER"
-PetscErrorCode EPSPowerGetShiftType_POWER(EPS eps,EPSPowerShiftType *shift)
+#define __FUNCT__ "EPSPowerGetShiftType_Power"
+PetscErrorCode EPSPowerGetShiftType_Power(EPS eps,EPSPowerShiftType *shift)
 {
   EPS_POWER  *power = (EPS_POWER *)eps->data;
+
   PetscFunctionBegin;
   *shift = power->shift_type;
   PetscFunctionReturn(0);
@@ -501,7 +492,7 @@ EXTERN_C_END
    EPSPowerGetShiftType - Gets the type of shifts used during the power
    iteration. 
 
-   Collective on EPS
+   Not Collective
 
    Input Parameter:
 .  eps - the eigenproblem solver context
@@ -515,70 +506,63 @@ EXTERN_C_END
 @*/
 PetscErrorCode EPSPowerGetShiftType(EPS eps,EPSPowerShiftType *shift)
 {
-  PetscErrorCode ierr, (*f)(EPS,EPSPowerShiftType*);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)eps,"EPSPowerGetShiftType_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(eps,shift);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidPointer(shift,2);
+  ierr = PetscTryMethod(eps,"EPSPowerGetShiftType_C",(EPS,EPSPowerShiftType*),(eps,shift));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSDestroy_POWER"
-PetscErrorCode EPSDestroy_POWER(EPS eps)
+#define __FUNCT__ "EPSDestroy_Power"
+PetscErrorCode EPSDestroy_Power(EPS eps)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = EPSDestroy_Default(eps);CHKERRQ(ierr);
+  ierr = PetscFree(eps->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerSetShiftType_C","",PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerGetShiftType_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "EPSView_POWER"
-PetscErrorCode EPSView_POWER(EPS eps,PetscViewer viewer)
+#define __FUNCT__ "EPSView_Power"
+PetscErrorCode EPSView_Power(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   EPS_POWER      *power = (EPS_POWER *)eps->data;
-  PetscTruth     isascii;
-  const char     *shift_list[3] = { "constant", "rayleigh", "wilkinson" };
+  PetscBool      isascii;
 
   PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) {
-    SETERRQ1(1,"Viewer type %s not supported for EPS_POWER",((PetscObject)viewer)->type_name);
+    SETERRQ1(((PetscObject)eps)->comm,1,"Viewer type %s not supported for EPS Power",((PetscObject)viewer)->type_name);
   }  
-  ierr = PetscViewerASCIIPrintf(viewer,"shift type: %s\n",shift_list[power->shift_type]);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  Power: %s shifts\n",EPSPowerShiftTypes[power->shift_type]);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "EPSCreate_POWER"
-PetscErrorCode EPSCreate_POWER(EPS eps)
+#define __FUNCT__ "EPSCreate_Power"
+PetscErrorCode EPSCreate_Power(EPS eps)
 {
   PetscErrorCode ierr;
-  EPS_POWER      *power;
 
   PetscFunctionBegin;
-  ierr = PetscNew(EPS_POWER,&power);CHKERRQ(ierr);
-  PetscLogObjectMemory(eps,sizeof(EPS_POWER));
-  eps->data                      = (void *) power;
-  eps->ops->setup                = EPSSetUp_POWER;
-  eps->ops->setfromoptions       = EPSSetFromOptions_POWER;
-  eps->ops->destroy              = EPSDestroy_POWER;
-  eps->ops->view                 = EPSView_POWER;
-  eps->ops->backtransform        = EPSBackTransform_POWER;
+  ierr = PetscNewLog(eps,EPS_POWER,&eps->data);CHKERRQ(ierr);
+  eps->ops->setup                = EPSSetUp_Power;
+  eps->ops->setfromoptions       = EPSSetFromOptions_Power;
+  eps->ops->destroy              = EPSDestroy_Power;
+  eps->ops->reset                = EPSReset_Default;
+  eps->ops->view                 = EPSView_Power;
+  eps->ops->backtransform        = EPSBackTransform_Power;
   eps->ops->computevectors       = EPSComputeVectors_Default;
-  power->shift_type              = EPS_POWER_SHIFT_CONSTANT;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerSetShiftType_C","EPSPowerSetShiftType_POWER",EPSPowerSetShiftType_POWER);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerGetShiftType_C","EPSPowerGetShiftType_POWER",EPSPowerGetShiftType_POWER);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerSetShiftType_C","EPSPowerSetShiftType_Power",EPSPowerSetShiftType_Power);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPowerGetShiftType_C","EPSPowerGetShiftType_Power",EPSPowerGetShiftType_Power);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
