@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -19,11 +19,11 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/fortranimpl.h"
-#include "slepceps.h"
-#include "private/epsimpl.h"
+#include <private/fortranimpl.h>
+#include <private/epsimpl.h>
 
-#ifdef PETSC_HAVE_FORTRAN_CAPS
+#if defined(PETSC_HAVE_FORTRAN_CAPS)
+#define epsdestroy_                 EPSDESTROY
 #define epsview_                    EPSVIEW
 #define epssetoptionsprefix_        EPSSETOPTIONSPREFIX
 #define epsappendoptionsprefix_     EPSAPPENDOPTIONSPREFIX
@@ -52,6 +52,7 @@
 #define epssetconvergencetestfunction_ EPSSETCONVERGENCETESTFUNCTION
 #define epsseteigenvaluecomparison_ EPSSETEIGENVALUECOMPARISON
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
+#define epsdestroy_                 epsdestroy
 #define epsview_                    epsview
 #define epssetoptionsprefix_        epssetoptionsprefix
 #define epsappendoptionsprefix_     epsappendoptionsprefix
@@ -124,10 +125,10 @@ static PetscErrorCode ourmonitor(EPS eps,PetscInt i,PetscInt nc,PetscScalar *er,
   return 0;
 }
 
-static PetscErrorCode ourdestroy(void* ctx)
+static PetscErrorCode ourdestroy(void** ctx)
 {
   PetscErrorCode ierr = 0;
-  EPS            eps = (EPS)ctx;
+  EPS            eps = *(EPS*)ctx;
   void           *mctx = (void*) ((PetscObject)eps)->fortran_func_pointers[1];
   (*(void (PETSC_STDCALL *)(void*,PetscErrorCode*))(((PetscObject)eps)->fortran_func_pointers[2]))(mctx,&ierr);CHKERRQ(ierr);
   return 0;
@@ -135,6 +136,11 @@ static PetscErrorCode ourdestroy(void* ctx)
 
 EXTERN_C_BEGIN
 
+void PETSC_STDCALL epsdestroy_(EPS *eps, PetscErrorCode *ierr)
+{
+  *ierr = EPSDestroy(eps);
+}
+
 void PETSC_STDCALL epsview_(EPS *eps,PetscViewer *viewer, PetscErrorCode *ierr)
 {
   PetscViewer v;
@@ -186,6 +192,7 @@ void PETSC_STDCALL epscreate_(MPI_Fint *comm,EPS *eps,PetscErrorCode *ierr)
 void PETSC_STDCALL epsmonitorset_(EPS *eps,void (PETSC_STDCALL *monitor)(EPS*,PetscInt*,PetscInt*,PetscScalar*,PetscScalar*,PetscReal*,PetscInt*,void*,PetscErrorCode*),
                                   void *mctx,void (PETSC_STDCALL *monitordestroy)(void *,PetscErrorCode *),PetscErrorCode *ierr)
 {
+  SlepcConvMonitor ctx;
   CHKFORTRANNULLFUNCTION(monitordestroy);
   PetscObjectAllocateFortranPointers(*eps,3);
   if ((PetscVoidFunction)monitor == (PetscVoidFunction)epsmonitorall_) {
@@ -195,13 +202,17 @@ void PETSC_STDCALL epsmonitorset_(EPS *eps,void (PETSC_STDCALL *monitor)(EPS*,Pe
   } else if ((PetscVoidFunction)monitor == (PetscVoidFunction)epsmonitorlgall_) {
     *ierr = EPSMonitorSet(*eps,EPSMonitorLGAll,0,0);
   } else if ((PetscVoidFunction)monitor == (PetscVoidFunction)epsmonitorconverged_) {
-    *ierr = EPSMonitorSet(*eps,EPSMonitorConverged,0,0);
+    if (!FORTRANNULLOBJECT(mctx)) { PetscError(((PetscObject)*eps)->comm,__LINE__,"epsmonitorset_",__FILE__,__SDIR__,PETSC_ERR_ARG_WRONG,PETSC_ERROR_INITIAL,"Must provide PETSC_NULL_OBJECT as a context in the Fortran interface to EPSMonitorSet"); *ierr = 1; return; }
+    *ierr = PetscNew(struct _n_SlepcConvMonitor,&ctx);
+    if (*ierr) return;
+    ctx->viewer = PETSC_NULL;
+    *ierr = EPSMonitorSet(*eps,EPSMonitorConverged,ctx,(PetscErrorCode (*)(void**))SlepcConvMonitorDestroy);
   } else if ((PetscVoidFunction)monitor == (PetscVoidFunction)epsmonitorfirst_) {
     *ierr = EPSMonitorSet(*eps,EPSMonitorFirst,0,0);
   } else {
     ((PetscObject)*eps)->fortran_func_pointers[0] = (PetscVoidFunction)monitor;
     ((PetscObject)*eps)->fortran_func_pointers[1] = (PetscVoidFunction)mctx;
-    if (FORTRANNULLFUNCTION(monitordestroy)) {
+    if (!monitordestroy) {
       *ierr = EPSMonitorSet(*eps,ourmonitor,*eps,0);
     } else {
       ((PetscObject)*eps)->fortran_func_pointers[2] = (PetscVoidFunction)monitordestroy;
