@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,24 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/epsimpl.h"   /*I "slepceps.h" I*/
+#include <private/epsimpl.h>   /*I "slepceps.h" I*/
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSMonitor"
+/*
+   Runs the user provided monitor routines, if any.
+*/
+PetscErrorCode EPSMonitor(EPS eps,PetscInt it,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,n = eps->numbermonitors;
+
+  PetscFunctionBegin;
+  for (i=0;i<n;i++) {
+    ierr = (*eps->monitor[i])(eps,it,nconv,eigr,eigi,errest,nest,eps->monitorcontext[i]);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "EPSMonitorSet"
@@ -29,7 +46,7 @@
    EPSMonitorSet - Sets an ADDITIONAL function to be called at every 
    iteration to monitor the error estimates for each requested eigenpair.
       
-   Collective on EPS
+   Logically Collective on EPS
 
    Input Parameters:
 +  eps     - eigensolver context obtained from EPSCreate()
@@ -71,15 +88,14 @@ $     monitor (EPS eps, int its, int nconv, PetscScalar *eigr, PetscScalar *eigi
 
    Level: intermediate
 
-.seealso: EPSMonitorFirst(), EPSMonitorAll(), EPSMonitorLG(), EPSMonitorLGAll(), EPSMonitorCancel()
+.seealso: EPSMonitorFirst(), EPSMonitorAll(), EPSMonitorCancel()
 @*/
-PetscErrorCode EPSMonitorSet(EPS eps,PetscErrorCode (*monitor)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*),
-                             void *mctx,PetscErrorCode (*monitordestroy)(void*))
+PetscErrorCode EPSMonitorSet(EPS eps,PetscErrorCode (*monitor)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*),void *mctx,PetscErrorCode (*monitordestroy)(void**))
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   if (eps->numbermonitors >= MAXEPSMONITORS) {
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Too many EPS monitors set");
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Too many EPS monitors set");
   }
   eps->monitor[eps->numbermonitors]           = monitor;
   eps->monitorcontext[eps->numbermonitors]    = (void*)mctx;
@@ -92,7 +108,7 @@ PetscErrorCode EPSMonitorSet(EPS eps,PetscErrorCode (*monitor)(EPS,PetscInt,Pets
 /*@
    EPSMonitorCancel - Clears all monitors for an EPS object.
 
-   Collective on EPS
+   Logically Collective on EPS
 
    Input Parameters:
 .  eps - eigensolver context obtained from EPSCreate()
@@ -112,10 +128,10 @@ PetscErrorCode EPSMonitorCancel(EPS eps)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   for (i=0; i<eps->numbermonitors; i++) {
     if (eps->monitordestroy[i]) {
-      ierr = (*eps->monitordestroy[i])(eps->monitorcontext[i]);CHKERRQ(ierr);
+      ierr = (*eps->monitordestroy[i])(&eps->monitorcontext[i]);CHKERRQ(ierr);
     }
   }
   eps->numbermonitors = 0;
@@ -126,7 +142,7 @@ PetscErrorCode EPSMonitorCancel(EPS eps)
 #define __FUNCT__ "EPSGetMonitorContext"
 /*@C
    EPSGetMonitorContext - Gets the monitor context, as set by 
-   EPSSetMonitor() for the FIRST monitor only.
+   EPSMonitorSet() for the FIRST monitor only.
 
    Not Collective
 
@@ -138,13 +154,13 @@ PetscErrorCode EPSMonitorCancel(EPS eps)
 
    Level: intermediate
 
-.seealso: EPSSetMonitor()
+.seealso: EPSMonitorSet()
 @*/
-PetscErrorCode EPSGetMonitorContext(EPS eps, void **ctx)
+PetscErrorCode EPSGetMonitorContext(EPS eps,void **ctx)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  *ctx =      (eps->monitorcontext[0]);
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  *ctx = eps->monitorcontext[0];
   PetscFunctionReturn(0);
 }
 
@@ -164,36 +180,36 @@ PetscErrorCode EPSGetMonitorContext(EPS eps, void **ctx)
 .  eigi   - imaginary part of the eigenvalues
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  dummy  - unused monitor context 
+-  monctx - monitor context (contains viewer, can be PETSC_NULL)
 
    Level: intermediate
 
-.seealso: EPSMonitorSet()
+.seealso: EPSMonitorSet(), EPSMonitorFirst(), EPSMonitorConverged()
 @*/
-PetscErrorCode EPSMonitorAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *dummy)
+PetscErrorCode EPSMonitorAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *monctx)
 {
-  PetscErrorCode          ierr;
-  PetscInt                i;
-  PetscScalar             er,ei;
-  PetscViewerASCIIMonitor viewer = (PetscViewerASCIIMonitor) dummy;
+  PetscErrorCode ierr;
+  PetscInt       i;
+  PetscScalar    er,ei;
+  PetscViewer    viewer = monctx? (PetscViewer)monctx: PETSC_VIEWER_STDOUT_(((PetscObject)eps)->comm);
 
   PetscFunctionBegin;
   if (its) {
-    if (!dummy) {ierr = PetscViewerASCIIMonitorCreate(((PetscObject)eps)->comm,"stdout",0,&viewer);CHKERRQ(ierr);}
-    ierr = PetscViewerASCIIMonitorPrintf(viewer,"%3d EPS nconv=%d Values (Errors)",its,nconv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%3D EPS nconv=%D Values (Errors)",its,nconv);CHKERRQ(ierr);
     for (i=0;i<nest;i++) {
       er = eigr[i]; ei = eigi[i];
-      ierr = STBackTransform(eps->OP, 1, &er, &ei); CHKERRQ(ierr);
+      ierr = STBackTransform(eps->OP,1,&er,&ei);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
-      ierr = PetscViewerASCIIMonitorPrintf(viewer," %g%+gi",PetscRealPart(er),PetscImaginaryPart(er));CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer," %G%+Gi",PetscRealPart(er),PetscImaginaryPart(er));CHKERRQ(ierr);
 #else
-      ierr = PetscViewerASCIIMonitorPrintf(viewer," %g",er);CHKERRQ(ierr);
-      if (ei!=0.0) { ierr = PetscViewerASCIIMonitorPrintf(viewer,"%+gi",ei);CHKERRQ(ierr); }
+      ierr = PetscViewerASCIIPrintf(viewer," %G",er);CHKERRQ(ierr);
+      if (ei!=0.0) { ierr = PetscViewerASCIIPrintf(viewer,"%+Gi",ei);CHKERRQ(ierr); }
 #endif
-      ierr = PetscViewerASCIIMonitorPrintf(viewer," (%10.8e)",errest[i]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer," (%10.8e)",(double)errest[i]);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIMonitorPrintf(viewer,"\n");CHKERRQ(ierr);
-    if (!dummy) {ierr = PetscViewerASCIIMonitorDestroy(viewer);CHKERRQ(ierr);}
+    ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+    ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -214,32 +230,32 @@ PetscErrorCode EPSMonitorAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *ei
 .  eigi   - imaginary part of the eigenvalues
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  dummy  - unused monitor context 
+-  monctx - monitor context (contains viewer, can be PETSC_NULL)
 
    Level: intermediate
 
-.seealso: EPSMonitorSet()
+.seealso: EPSMonitorSet(), EPSMonitorAll(), EPSMonitorConverged()
 @*/
-PetscErrorCode EPSMonitorFirst(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *dummy)
+PetscErrorCode EPSMonitorFirst(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *monctx)
 {
-  PetscErrorCode          ierr;
-  PetscScalar             er,ei;
-  PetscViewerASCIIMonitor viewer = (PetscViewerASCIIMonitor) dummy;
+  PetscErrorCode ierr;
+  PetscScalar    er,ei;
+  PetscViewer    viewer = monctx? (PetscViewer)monctx: PETSC_VIEWER_STDOUT_(((PetscObject)eps)->comm);
 
   PetscFunctionBegin;
   if (its && nconv<nest) {
-    if (!dummy) {ierr = PetscViewerASCIIMonitorCreate(((PetscObject)eps)->comm,"stdout",0,&viewer);CHKERRQ(ierr);}
-    ierr = PetscViewerASCIIMonitorPrintf(viewer,"%3d EPS nconv=%d first unconverged value (error)",its,nconv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%3D EPS nconv=%D first unconverged value (error)",its,nconv);CHKERRQ(ierr);
     er = eigr[nconv]; ei = eigi[nconv];
-    ierr = STBackTransform(eps->OP, 1, &er, &ei); CHKERRQ(ierr);
+    ierr = STBackTransform(eps->OP,1,&er,&ei);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
-    ierr = PetscViewerASCIIMonitorPrintf(viewer," %g%+gi",PetscRealPart(er),PetscImaginaryPart(er));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer," %G%+Gi",PetscRealPart(er),PetscImaginaryPart(er));CHKERRQ(ierr);
 #else
-    ierr = PetscViewerASCIIMonitorPrintf(viewer," %g",er);CHKERRQ(ierr);
-    if (ei!=0.0) { ierr = PetscViewerASCIIMonitorPrintf(viewer,"%+gi",ei);CHKERRQ(ierr); }
+    ierr = PetscViewerASCIIPrintf(viewer," %G",er);CHKERRQ(ierr);
+    if (ei!=0.0) { ierr = PetscViewerASCIIPrintf(viewer,"%+Gi",ei);CHKERRQ(ierr); }
 #endif
-    ierr = PetscViewerASCIIMonitorPrintf(viewer," (%10.8e)\n",errest[nconv]);CHKERRQ(ierr);
-    if (!dummy) {ierr = PetscViewerASCIIMonitorDestroy(viewer);CHKERRQ(ierr);}
+    ierr = PetscViewerASCIIPrintf(viewer," (%10.8e)\n",(double)errest[nconv]);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -260,49 +276,49 @@ PetscErrorCode EPSMonitorFirst(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *
 .  eigi   - imaginary part of the eigenvalues
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  dummy  - unused monitor context 
+-  monctx - monitor context
+
+   Note:
+   The monitor context must contain a struct with a PetscViewer and a
+   PetscInt. In Fortran, pass a PETSC_NULL_OBJECT.
 
    Level: intermediate
 
-.seealso: EPSMonitorSet()
+.seealso: EPSMonitorSet(), EPSMonitorFirst(), EPSMonitorAll()
 @*/
-PetscErrorCode EPSMonitorConverged(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *dummy)
+PetscErrorCode EPSMonitorConverged(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *monctx)
 {
-  PetscErrorCode  ierr;
-  PetscInt        i;
-  PetscScalar     er,ei;
-  EPSMONITOR_CONV *ctx = (EPSMONITOR_CONV*) dummy;
+  PetscErrorCode   ierr;
+  PetscInt         i;
+  PetscScalar      er,ei;
+  PetscViewer      viewer;
+  SlepcConvMonitor ctx = (SlepcConvMonitor)monctx;
 
   PetscFunctionBegin;
+  if (!monctx) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_WRONG,"Must provide a context for EPSMonitorConverged");
   if (!its) {
     ctx->oldnconv = 0;
   } else {
+    viewer = ctx->viewer? ctx->viewer: PETSC_VIEWER_STDOUT_(((PetscObject)eps)->comm);
     for (i=ctx->oldnconv;i<nconv;i++) {
-      ierr = PetscViewerASCIIMonitorPrintf(ctx->viewer,"%3d EPS converged value (error) #%d",its,i);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"%3D EPS converged value (error) #%D",its,i);CHKERRQ(ierr);
       er = eigr[i]; ei = eigi[i];
-      ierr = STBackTransform(eps->OP, 1, &er, &ei); CHKERRQ(ierr);
+      ierr = STBackTransform(eps->OP,1,&er,&ei);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
-      ierr = PetscViewerASCIIMonitorPrintf(ctx->viewer," %g%+gi",PetscRealPart(er),PetscImaginaryPart(er));CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer," %G%+Gi",PetscRealPart(er),PetscImaginaryPart(er));CHKERRQ(ierr);
 #else
-      ierr = PetscViewerASCIIMonitorPrintf(ctx->viewer," %g",er);CHKERRQ(ierr);
-      if (ei!=0.0) { ierr = PetscViewerASCIIMonitorPrintf(ctx->viewer,"%+gi",ei);CHKERRQ(ierr); }
+      ierr = PetscViewerASCIIPrintf(viewer," %G",er);CHKERRQ(ierr);
+      if (ei!=0.0) { ierr = PetscViewerASCIIPrintf(viewer,"%+Gi",ei);CHKERRQ(ierr); }
 #endif
-      ierr = PetscViewerASCIIMonitorPrintf(ctx->viewer," (%10.8e)\n",errest[i]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer," (%10.8e)\n",(double)errest[i]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
     }
     ctx->oldnconv = nconv;
   }
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode EPSMonitorDestroy_Converged(EPSMONITOR_CONV *ctx)
-{
-  PetscErrorCode  ierr;
-  PetscFunctionBegin;
-  ierr = PetscViewerASCIIMonitorDestroy(ctx->viewer);CHKERRQ(ierr);
-  ierr = PetscFree(ctx);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "EPSMonitorLG"
 PetscErrorCode EPSMonitorLG(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *monctx)
@@ -315,9 +331,7 @@ PetscErrorCode EPSMonitorLG(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eig
   PetscScalar    er,ei;
 
   PetscFunctionBegin;
-
   if (!viewer) { viewer = PETSC_VIEWER_DRAW_(((PetscObject)eps)->comm); }
-
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDrawLG(viewer,0,&lg);CHKERRQ(ierr);
   if (!its) {
@@ -346,7 +360,7 @@ PetscErrorCode EPSMonitorLG(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eig
   ierr = PetscDrawLGAddPoint(lg,&x,&y);CHKERRQ(ierr);
   if (eps->ishermitian) {
     er = eigr[nconv]; ei = eigi[nconv];
-    ierr = STBackTransform(eps->OP, 1, &er, &ei); CHKERRQ(ierr);
+    ierr = STBackTransform(eps->OP,1,&er,&ei);CHKERRQ(ierr);
     myeigr = PetscRealPart(er);
     ierr = PetscDrawLGAddPoint(lg1,&x,&myeigr);CHKERRQ(ierr);
     ierr = PetscDrawGetPause(draw1,&p);CHKERRQ(ierr);
@@ -358,7 +372,6 @@ PetscErrorCode EPSMonitorLG(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eig
   PetscFunctionReturn(0);
 } 
 
-
 #undef __FUNCT__  
 #define __FUNCT__ "EPSMonitorLGAll"
 PetscErrorCode EPSMonitorLGAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *monctx)
@@ -372,9 +385,7 @@ PetscErrorCode EPSMonitorLGAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *
   PetscScalar    er,ei;
 
   PetscFunctionBegin;
-
   if (!viewer) { viewer = PETSC_VIEWER_DRAW_(((PetscObject)eps)->comm); }
-
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDrawLG(viewer,0,&lg);CHKERRQ(ierr);
   if (!its) {
@@ -410,7 +421,7 @@ PetscErrorCode EPSMonitorLGAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *
     ierr = PetscMalloc(sizeof(PetscReal)*n,&myeigr);CHKERRQ(ierr);
     for(i=0;i<n;i++) {
       er = eigr[i]; ei = eigi[i];
-      ierr = STBackTransform(eps->OP, 1, &er, &ei); CHKERRQ(ierr);
+      ierr = STBackTransform(eps->OP,1,&er,&ei);CHKERRQ(ierr);
       if (i < nest) myeigr[i] = PetscRealPart(er);
       else myeigr[i] = 0.0;
     }
