@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/epsimpl.h"   /*I "slepceps.h" I*/
+#include <private/epsimpl.h>       /*I "slepceps.h" I*/
+#include <private/ipimpl.h>
 
 #undef __FUNCT__  
 #define __FUNCT__ "EPSSetUp"
@@ -47,70 +48,78 @@
 PetscErrorCode EPSSetUp(EPS eps)
 {
   PetscErrorCode ierr;
-  Vec            vds;  
   Mat            A,B; 
   PetscInt       i,k;
-  PetscTruth     flg,lindep;
-  PetscScalar    *pDS;
+  PetscBool      flg,lindep;
+  Vec            *newDS;
   PetscReal      norm;
 #if defined(PETSC_USE_COMPLEX)
   PetscScalar    sigma;
 #endif
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   if (eps->setupcalled) PetscFunctionReturn(0);
-
   ierr = PetscLogEventBegin(EPS_SetUp,eps,0,0,0);CHKERRQ(ierr);
 
-  /* Set default solver type */
+  /* Set default solver type (EPSSetFromOptions was not called) */
   if (!((PetscObject)eps)->type_name) {
     ierr = EPSSetType(eps,EPSKRYLOVSCHUR);CHKERRQ(ierr);
   }
-  if (!eps->balance) eps->balance = EPS_BALANCE_NONE;
+  if (!eps->OP) { ierr = EPSGetST(eps,&eps->OP);CHKERRQ(ierr); }
+  if (!((PetscObject)eps->OP)->type_name) {
+    ierr = PetscTypeCompareAny((PetscObject)eps,&flg,EPSGD,EPSJD,"");CHKERRQ(ierr);
+    ierr = STSetType(eps->OP,flg?STPRECOND:STSHIFT);CHKERRQ(ierr);
+  }
+  if (!eps->ip) { ierr = EPSGetIP(eps,&eps->ip);CHKERRQ(ierr); }
+  if (!((PetscObject)eps->ip)->type_name) {
+    ierr = IPSetDefaultType_Private(eps->ip);CHKERRQ(ierr);
+  }
+  if (!((PetscObject)eps->rand)->type_name) {
+    ierr = PetscRandomSetFromOptions(eps->rand);CHKERRQ(ierr);
+  }
   
   /* Set problem dimensions */
   ierr = STGetOperators(eps->OP,&A,&B);CHKERRQ(ierr);
+  if (!A) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_WRONGSTATE,"EPSSetOperators must be called first"); 
   ierr = MatGetSize(A,&eps->n,PETSC_NULL);CHKERRQ(ierr);
   ierr = MatGetLocalSize(A,&eps->nloc,PETSC_NULL);CHKERRQ(ierr);
+  ierr = VecDestroy(&eps->t);CHKERRQ(ierr);
+  ierr = SlepcMatGetVecsTemplate(A,&eps->t,PETSC_NULL);CHKERRQ(ierr);
 
   /* Set default problem type */
   if (!eps->problem_type) {
     if (B==PETSC_NULL) {
       ierr = EPSSetProblemType(eps,EPS_NHEP);CHKERRQ(ierr);
-    }
-    else {
+    } else {
       ierr = EPSSetProblemType(eps,EPS_GNHEP);CHKERRQ(ierr);
     }
-  } else if ((B && !eps->isgeneralized) || (!B && eps->isgeneralized)) {
-    SETERRQ(0,"Warning: Inconsistent EPS state"); 
+  } else if (!B && eps->isgeneralized) {
+    ierr = PetscInfo(eps,"Eigenproblem set as generalized but no matrix B was provided; reverting to a standard eigenproblem\n");CHKERRQ(ierr);
+    eps->isgeneralized = PETSC_FALSE;
+    eps->problem_type = eps->ishermitian? EPS_HEP: EPS_NHEP;
+  } else if (B && !eps->isgeneralized) {
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_INCOMP,"Inconsistent EPS state"); 
   }
 #if defined(PETSC_USE_COMPLEX)
   ierr = STGetShift(eps->OP,&sigma);CHKERRQ(ierr);
-  if (eps->ishermitian && PetscImaginaryPart(sigma) != 0.0) {
-    SETERRQ(1,"Hermitian problems are not compatible with complex shifts")
-  }
+  if (eps->ishermitian && PetscImaginaryPart(sigma) != 0.0)
+    SETERRQ(((PetscObject)eps)->comm,1,"Hermitian problems are not compatible with complex shifts");
 #endif
-  if (eps->ishermitian && eps->leftvecs) {
-    SETERRQ(1,"Requesting left eigenvectors not allowed in Hermitian problems")
-  }
+  if (eps->ishermitian && eps->leftvecs)
+    SETERRQ(((PetscObject)eps)->comm,1,"Requesting left eigenvectors not allowed in Hermitian problems");
   
   if (eps->ispositive) {
     ierr = STGetBilinearForm(eps->OP,&B);CHKERRQ(ierr);
-    ierr = IPSetBilinearForm(eps->ip,B,IP_INNER_HERMITIAN);CHKERRQ(ierr);
-    ierr = MatDestroy(B);CHKERRQ(ierr);
+    ierr = IPSetMatrix(eps->ip,B);CHKERRQ(ierr);
+    ierr = MatDestroy(&B);CHKERRQ(ierr);
   } else {
-    ierr = IPSetBilinearForm(eps->ip,PETSC_NULL,IP_INNER_HERMITIAN);CHKERRQ(ierr);
+    ierr = IPSetMatrix(eps->ip,PETSC_NULL);CHKERRQ(ierr);
   }
   
   if (eps->nev > eps->n) eps->nev = eps->n;
   if (eps->ncv > eps->n) eps->ncv = eps->n;
 
-  /* initialize the random number generator */
-  ierr = PetscRandomCreate(((PetscObject)eps)->comm,&eps->rand);CHKERRQ(ierr);
-  ierr = PetscRandomSetFromOptions(eps->rand);CHKERRQ(ierr);
-
   /* initialization of matrix norms */
   if (eps->nrma == PETSC_DETERMINE) {
     ierr = MatHasOperation(A,MATOP_NORM,&flg);CHKERRQ(ierr);
@@ -123,40 +132,58 @@ PetscErrorCode EPSSetUp(EPS eps)
     else eps->nrmb = 1.0;
   }
 
+  if (!eps->balance) eps->balance = EPS_BALANCE_NONE;
+
   /* call specific solver setup */
   ierr = (*eps->ops->setup)(eps);CHKERRQ(ierr);
-  ierr = STSetUp(eps->OP); CHKERRQ(ierr); 
-  
-  ierr = PetscTypeCompare((PetscObject)eps->OP,STCAYLEY,&flg);CHKERRQ(ierr);
-  if (flg && eps->problem_type == EPS_PGNHEP) {
-    SETERRQ(PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
+
+  /* set tolerance if not yet set */
+  if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL;
+
+  /* Build balancing matrix if required */
+  if (!eps->ishermitian && (eps->balance==EPS_BALANCE_ONESIDE || eps->balance==EPS_BALANCE_TWOSIDE)) {
+    if (!eps->D) {
+      ierr = VecDuplicate(eps->V[0],&eps->D);CHKERRQ(ierr);
+    } else {
+      ierr = VecSet(eps->D,1.0);CHKERRQ(ierr);
+    }
+    ierr = EPSBuildBalance_Krylov(eps);CHKERRQ(ierr);
+    ierr = STSetBalanceMatrix(eps->OP,eps->D);CHKERRQ(ierr);
   }
 
+  /* Setup ST */
+  ierr = STSetUp(eps->OP);CHKERRQ(ierr); 
+  
+  ierr = PetscTypeCompare((PetscObject)eps->OP,STCAYLEY,&flg);CHKERRQ(ierr);
+  if (flg && eps->problem_type == EPS_PGNHEP)
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
+
   ierr = PetscTypeCompare((PetscObject)eps->OP,STFOLD,&flg);CHKERRQ(ierr);
-  if (flg && !eps->ishermitian) {
-    SETERRQ(PETSC_ERR_SUP,"Fold spectral transformation requires a Hermitian problem");
-  }
+  if (flg && !eps->ishermitian)
+    SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_SUP,"Fold spectral transformation requires a Hermitian problem");
 
   if (eps->nds>0) {
     if (!eps->ds_ortho) {
       /* allocate memory and copy deflation basis vectors into DS */
-      ierr = PetscMalloc(eps->nds*eps->nloc*sizeof(PetscScalar),&pDS);CHKERRQ(ierr);
+      ierr = VecDuplicateVecs(eps->t,eps->nds,&newDS);CHKERRQ(ierr);
       for (i=0;i<eps->nds;i++) {
-        ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,eps->nloc,PETSC_DECIDE,pDS+i*eps->nloc,&vds);CHKERRQ(ierr);
-        ierr = VecCopy(eps->DS[i],vds);CHKERRQ(ierr);
-        ierr = VecDestroy(eps->DS[i]);CHKERRQ(ierr);
-        eps->DS[i] = vds;
+        ierr = VecCopy(eps->DS[i],newDS[i]);CHKERRQ(ierr);
+        ierr = VecDestroy(&eps->DS[i]);CHKERRQ(ierr);
       }
+      ierr = PetscFree(eps->DS);CHKERRQ(ierr);
+      eps->DS = newDS;
       /* orthonormalize vectors in DS */
       k = 0;
       for (i=0;i<eps->nds;i++) {
         ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,k,PETSC_NULL,eps->DS,eps->DS[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
-        if (norm==0.0 || lindep) PetscInfo(eps,"Linearly dependent deflation vector found, removing...\n");
-        else {
+        if (norm==0.0 || lindep) {
+          ierr = PetscInfo(eps,"Linearly dependent deflation vector found, removing...\n");CHKERRQ(ierr);
+        } else {
           ierr = VecScale(eps->DS[k],1.0/norm);CHKERRQ(ierr);
           k++;
         }
       }
+      for (i=k;i<eps->nds;i++) { ierr = VecDestroy(&eps->DS[i]);CHKERRQ(ierr); }
       eps->nds = k;
       eps->ds_ortho = PETSC_TRUE;
     }
@@ -166,14 +193,15 @@ PetscErrorCode EPSSetUp(EPS eps)
   /* process initial vectors */
   if (eps->nini<0) {
     eps->nini = -eps->nini;
-    if (eps->nini>eps->ncv) SETERRQ(1,"The number of initial vectors is larger than ncv")
+    if (eps->nini>eps->ncv) SETERRQ(((PetscObject)eps)->comm,1,"The number of initial vectors is larger than ncv");
     k = 0;
     for (i=0;i<eps->nini;i++) {
       ierr = VecCopy(eps->IS[i],eps->V[k]);CHKERRQ(ierr);
-      ierr = VecDestroy(eps->IS[i]);CHKERRQ(ierr);
+      ierr = VecDestroy(&eps->IS[i]);CHKERRQ(ierr);
       ierr = IPOrthogonalize(eps->ip,eps->nds,eps->DS,k,PETSC_NULL,eps->V,eps->V[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
-      if (norm==0.0 || lindep) PetscInfo(eps,"Linearly dependent initial vector found, removing...\n");
-      else {
+      if (norm==0.0 || lindep) {
+        ierr = PetscInfo(eps,"Linearly dependent initial vector found, removing...\n");CHKERRQ(ierr);
+      } else {
         ierr = VecScale(eps->V[k],1.0/norm);CHKERRQ(ierr);
         k++;
       }
@@ -182,17 +210,19 @@ PetscErrorCode EPSSetUp(EPS eps)
     ierr = PetscFree(eps->IS);CHKERRQ(ierr);
   }
   if (eps->ninil<0) {
-    if (!eps->leftvecs) PetscInfo(eps,"Ignoring initial left vectors\n");
-    else {
+    if (!eps->leftvecs) {
+      ierr = PetscInfo(eps,"Ignoring initial left vectors\n");CHKERRQ(ierr);
+    } else {
       eps->ninil = -eps->ninil;
-      if (eps->ninil>eps->ncv) SETERRQ(1,"The number of initial left vectors is larger than ncv")
+      if (eps->ninil>eps->ncv) SETERRQ(((PetscObject)eps)->comm,1,"The number of initial left vectors is larger than ncv");
       k = 0;
       for (i=0;i<eps->ninil;i++) {
         ierr = VecCopy(eps->ISL[i],eps->W[k]);CHKERRQ(ierr);
-        ierr = VecDestroy(eps->ISL[i]);CHKERRQ(ierr);
+        ierr = VecDestroy(&eps->ISL[i]);CHKERRQ(ierr);
         ierr = IPOrthogonalize(eps->ip,0,PETSC_NULL,k,PETSC_NULL,eps->W,eps->W[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
-        if (norm==0.0 || lindep) PetscInfo(eps,"Linearly dependent initial left vector found, removing...\n");
-        else {
+        if (norm==0.0 || lindep) {
+          ierr = PetscInfo(eps,"Linearly dependent initial left vector found, removing...\n");CHKERRQ(ierr);
+        } else {
           ierr = VecScale(eps->W[k],1.0/norm);CHKERRQ(ierr);
           k++;
         }
@@ -202,18 +232,6 @@ PetscErrorCode EPSSetUp(EPS eps)
     }
   }
 
-  /* Build balancing matrix if required */
-  if (!eps->ishermitian && (eps->balance==EPS_BALANCE_ONESIDE || eps->balance==EPS_BALANCE_TWOSIDE)) {
-    if (!eps->D) {
-      ierr = VecDuplicate(eps->V[0],&eps->D);CHKERRQ(ierr);
-    }
-    else {
-      ierr = VecSet(eps->D,1.0);CHKERRQ(ierr);
-    }
-    ierr = EPSBuildBalance_Krylov(eps);CHKERRQ(ierr);
-    ierr = STSetBalanceMatrix(eps->OP,eps->D);CHKERRQ(ierr);
-  }
-
   ierr = PetscLogEventEnd(EPS_SetUp,eps,0,0,0);CHKERRQ(ierr);
   eps->setupcalled = 1;
   PetscFunctionReturn(0);
@@ -234,9 +252,12 @@ PetscErrorCode EPSSetUp(EPS eps)
    Notes: 
    To specify a standard eigenproblem, use PETSC_NULL for parameter B.
 
+   It must be called after EPSSetUp(). If it is called again after EPSSetUp() then
+   the EPS object is reset.
+
    Level: beginner
 
-.seealso: EPSSolve(), EPSGetST(), STGetOperators()
+.seealso: EPSSolve(), EPSSetUp(), EPSReset(), EPSGetST(), STGetOperators()
 @*/
 PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 {
@@ -244,29 +265,24 @@ PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
   PetscInt       m,n,m0;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  PetscValidHeaderSpecific(A,MAT_COOKIE,2);
-  if (B) PetscValidHeaderSpecific(B,MAT_COOKIE,3);
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidHeaderSpecific(A,MAT_CLASSID,2);
+  if (B) PetscValidHeaderSpecific(B,MAT_CLASSID,3);
   PetscCheckSameComm(eps,1,A,2);
   if (B) PetscCheckSameComm(eps,1,B,3);
 
   /* Check for square matrices */
   ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
-  if (m!=n) { SETERRQ(1,"A is a non-square matrix"); }
+  if (m!=n) SETERRQ(((PetscObject)eps)->comm,1,"A is a non-square matrix");
   if (B) { 
     ierr = MatGetSize(B,&m0,&n);CHKERRQ(ierr);
-    if (m0!=n) { SETERRQ(1,"B is a non-square matrix"); }
-    if (m!=m0) { SETERRQ(1,"Dimensions of A and B do not match"); }
+    if (m0!=n) SETERRQ(((PetscObject)eps)->comm,1,"B is a non-square matrix");
+    if (m!=m0) SETERRQ(((PetscObject)eps)->comm,1,"Dimensions of A and B do not match");
   }
 
+  if (eps->setupcalled) { ierr = EPSReset(eps);CHKERRQ(ierr); }
+  if (!eps->OP) { ierr = EPSGetST(eps,&eps->OP);CHKERRQ(ierr); }
   ierr = STSetOperators(eps->OP,A,B);CHKERRQ(ierr);
-  eps->setupcalled = 0;  /* so that next solve call will call setup */
-
-  if (eps->D) {
-    ierr = VecDestroy(eps->D);CHKERRQ(ierr);
-    eps->D = PETSC_NULL;
-  }
-
   PetscFunctionReturn(0);
 }
 
@@ -288,13 +304,13 @@ PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 
 .seealso: EPSSolve(), EPSGetST(), STGetOperators(), STSetOperators()
 @*/
-PetscErrorCode EPSGetOperators(EPS eps, Mat *A, Mat *B)
+PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
 {
   PetscErrorCode ierr;
   ST             st;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   if (A) PetscValidPointer(A,2);
   if (B) PetscValidPointer(B,3);
   ierr = EPSGetST(eps,&st);CHKERRQ(ierr);
@@ -340,22 +356,25 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *ds)
   PetscInt       i;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (n<=0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Argument n out of range"); 
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(eps,n,2);
+  if (n<0) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n out of range"); 
 
   /* free previous vectors */
   ierr = EPSRemoveDeflationSpace(eps);CHKERRQ(ierr);
 
   /* get references of passed vectors */
-  ierr = PetscMalloc(n*sizeof(Vec),&eps->DS);CHKERRQ(ierr);
-  for (i=0;i<n;i++) {
-    ierr = PetscObjectReference((PetscObject)ds[i]);CHKERRQ(ierr);
-    eps->DS[i] = ds[i];
+  if (n>0) {
+    ierr = PetscMalloc(n*sizeof(Vec),&eps->DS);CHKERRQ(ierr);
+    for (i=0;i<n;i++) {
+      ierr = PetscObjectReference((PetscObject)ds[i]);CHKERRQ(ierr);
+      eps->DS[i] = ds[i];
+    }
+    eps->setupcalled = 0;
+    eps->ds_ortho = PETSC_FALSE;
   }
 
   eps->nds = n;
-  eps->setupcalled = 0;
-  eps->ds_ortho = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
@@ -376,22 +395,10 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec *ds)
 PetscErrorCode EPSRemoveDeflationSpace(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscInt       i;
-  PetscScalar    *pV;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (eps->nds > 0) {
-    ierr = VecGetArray(eps->DS[0],&pV);CHKERRQ(ierr);
-    ierr = VecRestoreArray(eps->DS[0],PETSC_NULL);CHKERRQ(ierr);
-    for (i=0;i<eps->nds;i++) {
-      ierr = VecDestroy(eps->DS[i]);CHKERRQ(ierr);
-    }
-    if (eps->setupcalled) {  /* before EPSSetUp, DS are just references */
-      ierr = PetscFree(pV);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(eps->DS);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  ierr = VecDestroyVecs(eps->nds,&eps->DS);CHKERRQ(ierr);
   eps->nds = 0;
   eps->setupcalled = 0;
   eps->ds_ortho = PETSC_FALSE;
@@ -434,26 +441,29 @@ PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec *is)
   PetscInt       i;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (n<0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(eps,n,2);
+  if (n<0) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
 
   /* free previous non-processed vectors */
   if (eps->nini<0) {
     for (i=0;i<-eps->nini;i++) {
-      ierr = VecDestroy(eps->IS[i]);CHKERRQ(ierr);
+      ierr = VecDestroy(&eps->IS[i]);CHKERRQ(ierr);
     }
     ierr = PetscFree(eps->IS);CHKERRQ(ierr);
   }
 
   /* get references of passed vectors */
-  ierr = PetscMalloc(n*sizeof(Vec),&eps->IS);CHKERRQ(ierr);
-  for (i=0;i<n;i++) {
-    ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
-    eps->IS[i] = is[i];
+  if (n>0) {
+    ierr = PetscMalloc(n*sizeof(Vec),&eps->IS);CHKERRQ(ierr);
+    for (i=0;i<n;i++) {
+      ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
+      eps->IS[i] = is[i];
+    }
+    eps->setupcalled = 0;
   }
 
   eps->nini = -n;
-  eps->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
@@ -494,26 +504,29 @@ PetscErrorCode EPSSetInitialSpaceLeft(EPS eps,PetscInt n,Vec *is)
   PetscInt       i;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (n<0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(eps,n,2);
+  if (n<0) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
 
   /* free previous non-processed vectors */
   if (eps->ninil<0) {
     for (i=0;i<-eps->ninil;i++) {
-      ierr = VecDestroy(eps->ISL[i]);CHKERRQ(ierr);
+      ierr = VecDestroy(&eps->ISL[i]);CHKERRQ(ierr);
     }
     ierr = PetscFree(eps->ISL);CHKERRQ(ierr);
   }
 
   /* get references of passed vectors */
-  ierr = PetscMalloc(n*sizeof(Vec),&eps->ISL);CHKERRQ(ierr);
-  for (i=0;i<n;i++) {
-    ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
-    eps->ISL[i] = is[i];
+  if (n>0) {
+    ierr = PetscMalloc(n*sizeof(Vec),&eps->ISL);CHKERRQ(ierr);
+    for (i=0;i<n;i++) {
+      ierr = PetscObjectReference((PetscObject)is[i]);CHKERRQ(ierr);
+      eps->ISL[i] = is[i];
+    }
+    eps->setupcalled = 0;
   }
 
   eps->ninil = -n;
-  eps->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
