@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -19,25 +19,27 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/fortranimpl.h"
-#include "slepcip.h"
+#include <private/fortranimpl.h>
+#include <slepcip.h>
 
-#ifdef PETSC_HAVE_FORTRAN_CAPS
+#if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define ipcreate_                 IPCREATE
+#define ipdestroy_                IPDESTROY
 #define ipsetoptionsprefix_       IPSETOPTIONSPREFIX
 #define ipappendoptionsprefix_    IPAPPENDOPTIONSPREFIX
 #define ipgetoptionsprefix_       IGSETOPTIONSPREFIX
 #define ipgetorthogonalization_   IPGETORTHOGONALIZATION
 #define ipview_                   IPVIEW
-#define ipgetbilinarform_         IPGETBILINARFORM
+#define ipgetmatrix_              IPGETMATRIX
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
 #define ipcreate_                 ipcreate
+#define ipdestroy_                ipdestroy
 #define ipsetoptionsprefix_       ipsetoptionsprefix
 #define ipappendoptionsprefix_    ipappendoptionsprefix
 #define ipgetoptionsprefix_       ipgetoptionsprefix
 #define ipgetorthogonalization_   ipgetorthogonalization
 #define ipview_                   ipview
-#define ipgetbilinarform_         ipgetbilinarform
+#define ipgetmatrix_              ipgetmatrix
 #endif
 
 EXTERN_C_BEGIN
@@ -47,6 +49,11 @@ void PETSC_STDCALL ipcreate_(MPI_Fint *comm,IP *newip,PetscErrorCode *ierr)
   *ierr = IPCreate(MPI_Comm_f2c(*(comm)),newip);
 }
 
+void PETSC_STDCALL ipdestroy_(IP *ip, PetscErrorCode *ierr)
+{
+  *ierr = IPDestroy(ip);
+}
+
 void PETSC_STDCALL ipsetoptionsprefix_(IP *ip,CHAR prefix PETSC_MIXED_LEN(len),
                                        PetscErrorCode *ierr PETSC_END_LEN(len))
 {
@@ -75,7 +82,7 @@ void PETSC_STDCALL ipgetoptionsprefix_(IP *ip,CHAR prefix PETSC_MIXED_LEN(len),P
   *ierr = PetscStrncpy(prefix,tname,len);
 }
 
-void PETSC_STDCALL ipgetorthogonalization_(IP *ip,IPOrthogonalizationType *type,IPOrthogonalizationRefinementType *refinement,PetscReal *eta,PetscErrorCode *ierr)
+void PETSC_STDCALL ipgetorthogonalization_(IP *ip,IPOrthogType *type,IPOrthogRefineType *refinement,PetscReal *eta,PetscErrorCode *ierr)
 {
   *ierr = IPGetOrthogonalization(*ip,type,refinement,eta);
 }
@@ -87,10 +94,9 @@ void PETSC_STDCALL ipview_(IP *ip,PetscViewer *viewer, PetscErrorCode *ierr)
   *ierr = IPView(*ip,v);
 }
 
-void PETSC_STDCALL ipgetbilinarform_(IP *ip,Mat *mat,IPBilinearForm* form,PetscErrorCode *ierr)
+void PETSC_STDCALL ipgetmatrix_(IP *ip,Mat *mat,PetscErrorCode *ierr)
 {
-  if (FORTRANNULLOBJECT(mat)) mat = PETSC_NULL;  
-  *ierr = IPGetBilinearForm(*ip,mat,form);
+  *ierr = IPGetMatrix(*ip,mat);
 }
 
 EXTERN_C_END
