@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,10 +21,32 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/ipimpl.h"      /*I "slepcip.h" I*/
+#include <private/ipimpl.h>      /*I "slepcip.h" I*/
 
-PetscCookie IP_COOKIE = 0;
-PetscLogEvent IP_InnerProduct = 0, IP_Orthogonalize = 0, IP_ApplyMatrix = 0;
+PetscFList       IPList = 0;
+PetscBool        IPRegisterAllCalled = PETSC_FALSE;
+PetscClassId     IP_CLASSID = 0;
+PetscLogEvent    IP_InnerProduct = 0,IP_Orthogonalize = 0,IP_ApplyMatrix = 0;
+static PetscBool IPPackageInitialized = PETSC_FALSE;
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPFinalizePackage"
+/*@C
+   IPFinalizePackage - This function destroys everything in the Slepc interface 
+   to the IP package. It is called from SlepcFinalize().
+
+   Level: developer
+
+.seealso: SlepcFinalize()
+@*/
+PetscErrorCode IPFinalizePackage(void) 
+{
+  PetscFunctionBegin;
+  IPPackageInitialized = PETSC_FALSE;
+  IPList               = 0;
+  IPRegisterAllCalled  = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "IPInitializePackage"
@@ -40,39 +62,41 @@ PetscLogEvent IP_InnerProduct = 0, IP_Orthogonalize = 0, IP_ApplyMatrix = 0;
 
 .seealso: SlepcInitialize()
 @*/
-PetscErrorCode IPInitializePackage(char *path) 
+PetscErrorCode IPInitializePackage(const char *path) 
 {
-  static PetscTruth initialized = PETSC_FALSE;
-  char              logList[256];
-  char              *className;
-  PetscTruth        opt;
-  PetscErrorCode    ierr;
+  char             logList[256];
+  char             *className;
+  PetscBool        opt;
+  PetscErrorCode   ierr;
 
   PetscFunctionBegin;
-  if (initialized) PetscFunctionReturn(0);
-  initialized = PETSC_TRUE;
+  if (IPPackageInitialized) PetscFunctionReturn(0);
+  IPPackageInitialized = PETSC_TRUE;
   /* Register Classes */
-  ierr = PetscCookieRegister("Inner product",&IP_COOKIE);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Inner product",&IP_CLASSID);CHKERRQ(ierr);
+  /* Register Constructors */
+  ierr = IPRegisterAll(path);CHKERRQ(ierr);
   /* Register Events */
-  ierr = PetscLogEventRegister("IPOrthogonalize",IP_COOKIE,&IP_Orthogonalize); CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("IPInnerProduct",IP_COOKIE,&IP_InnerProduct); CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("IPApplyMatrix",IP_COOKIE,&IP_ApplyMatrix); CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("IPOrthogonalize",IP_CLASSID,&IP_Orthogonalize);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("IPInnerProduct",IP_CLASSID,&IP_InnerProduct);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("IPApplyMatrix",IP_CLASSID,&IP_ApplyMatrix);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL, "-log_info_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "ip", &className);CHKERRQ(ierr);
+    ierr = PetscStrstr(logList,"ip",&className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscInfoDeactivateClass(IP_COOKIE);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(IP_CLASSID);CHKERRQ(ierr);
     }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL, "-log_summary_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "ip", &className);CHKERRQ(ierr);
+    ierr = PetscStrstr(logList,"ip",&className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscLogEventDeactivateClass(IP_COOKIE);CHKERRQ(ierr);
+      ierr = PetscLogEventDeactivateClass(IP_CLASSID);CHKERRQ(ierr);
     }
   }
+  ierr = PetscRegisterFinalize(IPFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -99,24 +123,21 @@ PetscErrorCode IPInitializePackage(char *path)
 @*/
 PetscErrorCode IPCreate(MPI_Comm comm,IP *newip)
 {
+  IP             ip;
   PetscErrorCode ierr;
-  IP ip;
 
   PetscFunctionBegin;
   PetscValidPointer(newip,2);
-  ierr = PetscHeaderCreate(ip,_p_IP,int,IP_COOKIE,-1,"IP",comm,IPDestroy,IPView);CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(ip,_p_IP,struct _IPOps,IP_CLASSID,-1,"IP","Inner Product","IP",comm,IPDestroy,IPView);CHKERRQ(ierr);
   *newip            = ip;
-  ip->orthog_type   = IP_ORTH_CGS;
-  ip->orthog_ref    = IP_ORTH_REFINE_IFNEEDED;
+  ip->orthog_type   = IP_ORTHOG_CGS;
+  ip->orthog_ref    = IP_ORTHOG_REFINE_IFNEEDED;
   ip->orthog_eta    = 0.7071;
-  ip->bilinear_form = IP_INNER_HERMITIAN;
   ip->innerproducts = 0;
   ip->matrix        = PETSC_NULL;
   ip->Bx            = PETSC_NULL;
   ip->xid           = 0;
   ip->xstate        = 0;
-
-  ierr = PetscPublishAll(ip);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -126,7 +147,7 @@ PetscErrorCode IPCreate(MPI_Comm comm,IP *newip)
    IPSetOptionsPrefix - Sets the prefix used for searching for all 
    IP options in the database.
 
-   Collective on IP
+   Logically Collective on IP
 
    Input Parameters:
 +  ip - the innerproduct context
@@ -144,8 +165,9 @@ PetscErrorCode IPCreate(MPI_Comm comm,IP *newip)
 PetscErrorCode IPSetOptionsPrefix(IP ip,const char *prefix)
 {
   PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
   ierr = PetscObjectSetOptionsPrefix((PetscObject)ip,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -156,7 +178,7 @@ PetscErrorCode IPSetOptionsPrefix(IP ip,const char *prefix)
    IPAppendOptionsPrefix - Appends to the prefix used for searching for all 
    IP options in the database.
 
-   Collective on IP
+   Logically Collective on IP
 
    Input Parameters:
 +  ip - the innerproduct context
@@ -173,8 +195,9 @@ PetscErrorCode IPSetOptionsPrefix(IP ip,const char *prefix)
 PetscErrorCode IPAppendOptionsPrefix(IP ip,const char *prefix)
 {
   PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)ip,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -202,14 +225,108 @@ PetscErrorCode IPAppendOptionsPrefix(IP ip,const char *prefix)
 @*/
 PetscErrorCode IPGetOptionsPrefix(IP ip,const char *prefix[])
 {
- PetscErrorCode ierr;
- PetscFunctionBegin;
- PetscValidHeaderSpecific(ip,IP_COOKIE,1);
- PetscValidPointer(prefix,2);
- ierr = PetscObjectGetOptionsPrefix((PetscObject)ip, prefix);CHKERRQ(ierr);
- PetscFunctionReturn(0);
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidPointer(prefix,2);
+  ierr = PetscObjectGetOptionsPrefix((PetscObject)ip,prefix);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "IPSetType"
+/*@C
+   IPSetType - Selects the type for the IP object.
+
+   Logically Collective on IP
+
+   Input Parameter:
++  ip   - the inner product context.
+-  type - a known type
+
+   Notes:
+   Two types are available: IPBILINEAR and IPSESQUILINEAR.
+
+   For complex scalars, the default is a sesquilinear form (x,y)=x^H*M*y and it is
+   also possible to choose a bilinear form (x,y)=x^T*M*y (without complex conjugation).
+   The latter could be useful e.g. in complex-symmetric eigensolvers.
+
+   In the case of real scalars, only the bilinear form (x,y)=x^T*M*y is available.
+
+   Level: advanced
+
+.seealso: IPGetType()
+
+@*/
+PetscErrorCode IPSetType(IP ip,const IPType type)
+{
+  PetscErrorCode ierr,(*r)(IP);
+  PetscBool      match;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidCharPointer(type,2);
+
+  ierr = PetscTypeCompare((PetscObject)ip,type,&match);CHKERRQ(ierr);
+  if (match) PetscFunctionReturn(0);
+
+  ierr =  PetscFListFind(IPList,((PetscObject)ip)->comm,type,PETSC_TRUE,(void (**)(void))&r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(((PetscObject)ip)->comm,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested IP type %s",type);
+
+  ierr = PetscMemzero(ip->ops,sizeof(struct _IPOps));CHKERRQ(ierr);
+
+  ierr = PetscObjectChangeTypeName((PetscObject)ip,type);CHKERRQ(ierr);
+  ierr = (*r)(ip);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPGetType"
+/*@C
+   IPGetType - Gets the IP type name (as a string) from the IP context.
+
+   Not Collective
+
+   Input Parameter:
+.  ip - the inner product context
+
+   Output Parameter:
+.  name - name of the inner product
+
+   Level: advanced
+
+.seealso: IPSetType()
+
+@*/
+PetscErrorCode IPGetType(IP ip,const IPType *type)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidPointer(type,2);
+  *type = ((PetscObject)ip)->type_name;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPSetDefaultType_Private"
+/*
+  Sets the default IP type, depending on whether complex arithmetic
+  is used or not.
+*/
+PetscErrorCode IPSetDefaultType_Private(IP ip)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+#if defined(PETSC_USE_COMPLEX)
+  ierr = IPSetType(ip,IPSESQUILINEAR);CHKERRQ(ierr);
+#else
+  ierr = IPSetType(ip,IPBILINEAR);CHKERRQ(ierr);
+#endif
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "IPSetFromOptions"
@@ -225,27 +342,31 @@ PetscErrorCode IPGetOptionsPrefix(IP ip,const char *prefix[])
    To see all options, run your program with the -help option.
 
    Level: beginner
-
-.seealso: 
 @*/
 PetscErrorCode IPSetFromOptions(IP ip)
 {
-  PetscErrorCode ierr;
-  const char     *orth_list[2] = { "mgs" , "cgs" };
-  const char     *ref_list[3] = { "never" , "ifneeded", "always" };
+  const char     *orth_list[2] = {"mgs","cgs"};
+  const char     *ref_list[3] = {"never","ifneeded","always"};
   PetscReal      r;
   PetscInt       i,j;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  if (!IPRegisterAllCalled) { ierr = IPRegisterAll(PETSC_NULL);CHKERRQ(ierr); }
+  /* Set default type (we do not allow changing it with -ip_type) */
+  if (!((PetscObject)ip)->type_name) {
+    ierr = IPSetDefaultType_Private(ip);CHKERRQ(ierr);
+  }
   ierr = PetscOptionsBegin(((PetscObject)ip)->comm,((PetscObject)ip)->prefix,"Inner Product (IP) Options","IP");CHKERRQ(ierr);
-  i = ip->orthog_type;
-  ierr = PetscOptionsEList("-orthog_type","Orthogonalization method","IPSetOrthogonalization",orth_list,2,orth_list[i],&i,PETSC_NULL);CHKERRQ(ierr);
-  j = ip->orthog_ref;
-  ierr = PetscOptionsEList("-orthog_refinement","Iterative refinement mode during orthogonalization","IPSetOrthogonalization",ref_list,3,ref_list[j],&j,PETSC_NULL);CHKERRQ(ierr);
-  r = ip->orthog_eta;
-  ierr = PetscOptionsReal("-orthog_eta","Parameter of iterative refinement during orthogonalization","IPSetOrthogonalization",r,&r,PETSC_NULL);CHKERRQ(ierr);
-  ierr = IPSetOrthogonalization(ip,(IPOrthogonalizationType)i,(IPOrthogonalizationRefinementType)j,r);CHKERRQ(ierr);
+    i = ip->orthog_type;
+    ierr = PetscOptionsEList("-ip_orthog_type","Orthogonalization method","IPSetOrthogonalization",orth_list,2,orth_list[i],&i,PETSC_NULL);CHKERRQ(ierr);
+    j = ip->orthog_ref;
+    ierr = PetscOptionsEList("-ip_orthog_refine","Iterative refinement mode during orthogonalization","IPSetOrthogonalization",ref_list,3,ref_list[j],&j,PETSC_NULL);CHKERRQ(ierr);
+    r = ip->orthog_eta;
+    ierr = PetscOptionsReal("-ip_orthog_eta","Parameter of iterative refinement during orthogonalization","IPSetOrthogonalization",r,&r,PETSC_NULL);CHKERRQ(ierr);
+    ierr = IPSetOrthogonalization(ip,(IPOrthogType)i,(IPOrthogRefineType)j,r);CHKERRQ(ierr);
+    ierr = PetscObjectProcessOptionsHandlers((PetscObject)ip);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -256,20 +377,18 @@ PetscErrorCode IPSetFromOptions(IP ip)
    IPSetOrthogonalization - Specifies the type of orthogonalization technique
    to be used (classical or modified Gram-Schmidt with or without refinement).
 
-   Collective on IP
+   Logically Collective on IP
 
    Input Parameters:
-+  ip         - the innerproduct context
-.  type       - the type of orthogonalization technique
-.  refinement - type of refinement
--  eta        - parameter for selective refinement
++  ip     - the innerproduct context
+.  type   - the type of orthogonalization technique
+.  refine - type of refinement
+-  eta    - parameter for selective refinement
 
    Options Database Keys:
-+  -orthog_type <type> -  Where <type> is cgs for Classical Gram-Schmidt
-                              orthogonalization (default)
-                              or mgs for Modified Gram-Schmidt orthogonalization
-.  -orthog_refinement <type> -  Where <type> is one of never, ifneeded
-                              (default) or always 
++  -orthog_type <type> - Where <type> is cgs for Classical Gram-Schmidt orthogonalization
+                         (default) or mgs for Modified Gram-Schmidt orthogonalization
+.  -orthog_refine <type> - Where <type> is one of never, ifneeded (default) or always 
 -  -orthog_eta <eta> -  For setting the value of eta
     
    Notes:  
@@ -282,34 +401,37 @@ PetscErrorCode IPSetFromOptions(IP ip)
 
    Level: advanced
 
-.seealso: IPOrthogonalize(), IPGetOrthogonalization(), IPOrthogonalizationType,
-          IPOrthogonalizationRefinementType
+.seealso: IPOrthogonalize(), IPGetOrthogonalization(), IPOrthogType,
+          IPOrthogRefineType
 @*/
-PetscErrorCode IPSetOrthogonalization(IP ip,IPOrthogonalizationType type, IPOrthogonalizationRefinementType refinement, PetscReal eta)
+PetscErrorCode IPSetOrthogonalization(IP ip,IPOrthogType type,IPOrthogRefineType refine,PetscReal eta)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(ip,type,2);
+  PetscValidLogicalCollectiveEnum(ip,refine,3);
+  PetscValidLogicalCollectiveReal(ip,eta,4);
   switch (type) {
-    case IP_ORTH_CGS:
-    case IP_ORTH_MGS:
+    case IP_ORTHOG_CGS:
+    case IP_ORTHOG_MGS:
       ip->orthog_type = type;
       break;
     default:
-      SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
+      SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
   }
-  switch (refinement) {
-    case IP_ORTH_REFINE_NEVER:
-    case IP_ORTH_REFINE_IFNEEDED:
-    case IP_ORTH_REFINE_ALWAYS:
-      ip->orthog_ref = refinement;
+  switch (refine) {
+    case IP_ORTHOG_REFINE_NEVER:
+    case IP_ORTHOG_REFINE_IFNEEDED:
+    case IP_ORTHOG_REFINE_ALWAYS:
+      ip->orthog_ref = refine;
       break;
     default:
-      SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown refinement type");
+      SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_WRONG,"Unknown refinement type");
   }
   if (eta == PETSC_DEFAULT) {
     ip->orthog_eta = 0.7071;
   } else {
-    if (eta <= 0.0 || eta > 1.0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid eta value");    
+    if (eta <= 0.0 || eta > 1.0) SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid eta value");    
     ip->orthog_eta = eta;
   }
   PetscFunctionReturn(0);
@@ -327,22 +449,22 @@ PetscErrorCode IPSetOrthogonalization(IP ip,IPOrthogonalizationType type, IPOrth
 .  ip - inner product context 
 
    Output Parameter:
-+  type       - type of orthogonalization technique
-.  refinement - type of refinement
--  eta        - parameter for selective refinement
++  type   - type of orthogonalization technique
+.  refine - type of refinement
+-  eta    - parameter for selective refinement
 
    Level: advanced
 
-.seealso: IPOrthogonalize(), IPSetOrthogonalization(), IPOrthogonalizationType,
-          IPOrthogonalizationRefinementType
+.seealso: IPOrthogonalize(), IPSetOrthogonalization(), IPOrthogType,
+          IPOrthogRefineType
 @*/
-PetscErrorCode IPGetOrthogonalization(IP ip,IPOrthogonalizationType *type,IPOrthogonalizationRefinementType *refinement, PetscReal *eta)
+PetscErrorCode IPGetOrthogonalization(IP ip,IPOrthogType *type,IPOrthogRefineType *refine,PetscReal *eta)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  if (type) *type = ip->orthog_type;
-  if (refinement) *refinement = ip->orthog_ref;
-  if (eta) *eta = ip->orthog_eta;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  if (type)   *type   = ip->orthog_type;
+  if (refine) *refine = ip->orthog_ref;
+  if (eta)    *eta    = ip->orthog_eta;
   PetscFunctionReturn(0);
 }
 
@@ -370,54 +492,88 @@ PetscErrorCode IPGetOrthogonalization(IP ip,IPOrthogonalizationType *type,IPOrth
 
    Level: beginner
 
-.seealso: IPView(), EPSView(), SVDView(), PetscViewerASCIIOpen()
+.seealso: EPSView(), SVDView(), PetscViewerASCIIOpen()
 @*/
 PetscErrorCode IPView(IP ip,PetscViewer viewer)
 {
+  PetscBool      isascii;
   PetscErrorCode ierr;
-  PetscTruth     isascii;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
   if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)ip)->comm);
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_COOKIE,2);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(ip,1,viewer,2);
-
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"IP Object:\n");CHKERRQ(ierr);
+    ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ip,viewer,"IP Object");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  orthogonalization method: ");CHKERRQ(ierr);
     switch (ip->orthog_type) {
-      case IP_ORTH_MGS:
+      case IP_ORTHOG_MGS:
         ierr = PetscViewerASCIIPrintf(viewer,"modified Gram-Schmidt\n");CHKERRQ(ierr);
         break;
-      case IP_ORTH_CGS:
+      case IP_ORTHOG_CGS:
         ierr = PetscViewerASCIIPrintf(viewer,"classical Gram-Schmidt\n");CHKERRQ(ierr);
         break;
-      default: SETERRQ(1,"Wrong value of ip->orth_type");
+      default: SETERRQ(((PetscObject)ip)->comm,1,"Wrong value of ip->orth_type");
     }
     ierr = PetscViewerASCIIPrintf(viewer,"  orthogonalization refinement: ");CHKERRQ(ierr);
     switch (ip->orthog_ref) {
-      case IP_ORTH_REFINE_NEVER:
+      case IP_ORTHOG_REFINE_NEVER:
         ierr = PetscViewerASCIIPrintf(viewer,"never\n");CHKERRQ(ierr);
         break;
-      case IP_ORTH_REFINE_IFNEEDED:
-        ierr = PetscViewerASCIIPrintf(viewer,"if needed (eta: %f)\n",ip->orthog_eta);CHKERRQ(ierr);
+      case IP_ORTHOG_REFINE_IFNEEDED:
+        ierr = PetscViewerASCIIPrintf(viewer,"if needed (eta: %G)\n",ip->orthog_eta);CHKERRQ(ierr);
         break;
-      case IP_ORTH_REFINE_ALWAYS:
+      case IP_ORTHOG_REFINE_ALWAYS:
         ierr = PetscViewerASCIIPrintf(viewer,"always\n");CHKERRQ(ierr);
         break;
-      default: SETERRQ(1,"Wrong value of ip->orth_ref");
+      default: SETERRQ(((PetscObject)ip)->comm,1,"Wrong value of ip->orth_ref");
+    }
+    if (ip->matrix) {
+      ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
+      ierr = MatView(ip->matrix,viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+      ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     }
   } else {
-    SETERRQ1(1,"Viewer type %s not supported for IP",((PetscObject)viewer)->type_name);
+    SETERRQ1(((PetscObject)ip)->comm,1,"Viewer type %s not supported for IP",((PetscObject)viewer)->type_name);
   }
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "IPReset"
+/*@C
+   IPReset - Resets the IP context to the initial state.
+
+   Collective on IP
+
+   Input Parameter:
+.  ip - the inner product context
+
+   Level: advanced
+
+.seealso: IPDestroy()
+@*/
+PetscErrorCode IPReset(IP ip)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  ierr = MatDestroy(&ip->matrix);CHKERRQ(ierr);
+  ierr = VecDestroy(&ip->Bx);CHKERRQ(ierr);
+  ip->xid    = 0;
+  ip->xstate = 0;
+  ierr = IPResetOperationCounters(ip);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "IPDestroy"
-/*@
+/*@C
    IPDestroy - Destroys IP context that was created with IPCreate().
 
    Collective on IP
@@ -429,16 +585,15 @@ PetscErrorCode IPView(IP ip,PetscViewer viewer)
 
 .seealso: IPCreate()
 @*/
-PetscErrorCode IPDestroy(IP ip)
+PetscErrorCode IPDestroy(IP *ip)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  if (--((PetscObject)ip)->refct > 0) PetscFunctionReturn(0);
-
-  if (ip->matrix) { ierr = MatDestroy(ip->matrix);CHKERRQ(ierr); }
-  if (ip->Bx) { ierr = VecDestroy(ip->Bx);CHKERRQ(ierr); }
+  if (!*ip) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(*ip,IP_CLASSID,1);
+  if (--((PetscObject)(*ip))->refct > 0) { *ip = 0; PetscFunctionReturn(0); }
+  ierr = IPReset(*ip);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(ip);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -464,7 +619,7 @@ PetscErrorCode IPDestroy(IP ip)
 PetscErrorCode IPGetOperationCounters(IP ip,PetscInt *dots)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
   PetscValidPointer(dots,2);
   *dots = ip->innerproducts;
   PetscFunctionReturn(0);
@@ -476,7 +631,7 @@ PetscErrorCode IPGetOperationCounters(IP ip,PetscInt *dots)
    IPResetOperationCounters - Resets the counters for inner product operations 
    made by of the IP object.
 
-   Collective on IP
+   Logically Collective on IP
 
    Input Parameter:
 .  ip - the inner product context
@@ -488,8 +643,80 @@ PetscErrorCode IPGetOperationCounters(IP ip,PetscInt *dots)
 PetscErrorCode IPResetOperationCounters(IP ip)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
   ip->innerproducts = 0;
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "IPRegister"
+/*@C
+   IPRegister - See IPRegisterDynamic()
+
+   Level: advanced
+@*/
+PetscErrorCode IPRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(IP))
+{
+  PetscErrorCode ierr;
+  char           fullname[PETSC_MAX_PATH_LEN];
+
+  PetscFunctionBegin;
+  ierr = PetscFListConcat(path,name,fullname);CHKERRQ(ierr);
+  ierr = PetscFListAdd(&IPList,sname,fullname,(void (*)(void))function);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPRegisterDestroy"
+/*@
+   IPRegisterDestroy - Frees the list of IP methods that were
+   registered by IPRegisterDynamic().
+
+   Not Collective
+
+   Level: advanced
+
+.seealso: IPRegisterDynamic(), IPRegisterAll()
+@*/
+PetscErrorCode IPRegisterDestroy(void)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFListDestroy(&IPList);CHKERRQ(ierr);
+  IPRegisterAllCalled = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
+EXTERN_C_BEGIN
+extern PetscErrorCode IPCreate_Bilinear(IP);
+#if defined(PETSC_USE_COMPLEX)
+extern PetscErrorCode IPCreate_Sesquilinear(IP);
+#endif
+EXTERN_C_END
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPRegisterAll"
+/*@C
+   IPRegisterAll - Registers all of the inner products in the IP package.
+
+   Not Collective
+
+   Input Parameter:
+.  path - the library where the routines are to be found (optional)
+
+   Level: advanced
+@*/
+PetscErrorCode IPRegisterAll(const char *path)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  IPRegisterAllCalled = PETSC_TRUE;
+  ierr = IPRegisterDynamic(IPBILINEAR,path,"IPCreate_Bilinear",IPCreate_Bilinear);CHKERRQ(ierr);
+#if defined(PETSC_USE_COMPLEX)
+  ierr = IPRegisterDynamic(IPSESQUILINEAR,path,"IPCreate_Sesquilinear",IPCreate_Sesquilinear);CHKERRQ(ierr);
+#endif
+  PetscFunctionReturn(0);
+}
+
