@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,21 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/ipimpl.h"      /*I "slepcip.h" I*/
+#include <private/ipimpl.h>      /*I "slepcip.h" I*/
+
+/* The following definitions are intended to avoid using the "T" versions
+   of dot products in the case of real scalars */
+#if defined(PETSC_USE_COMPLEX)
+#define VecXDotBegin  VecTDotBegin
+#define VecXDotEnd    VecTDotEnd  
+#define VecMXDotBegin VecMTDotBegin
+#define VecMXDotEnd   VecMTDotEnd  
+#else
+#define VecXDotBegin  VecDotBegin
+#define VecXDotEnd    VecDotEnd  
+#define VecMXDotBegin VecMDotBegin
+#define VecMXDotEnd   VecMDotEnd  
+#endif
 
 #undef __FUNCT__  
 #define __FUNCT__ "IPNorm"
@@ -41,7 +55,7 @@
    Notes:
    This function will usually compute the 2-norm of a vector, ||x||_2. But
    this behaviour may be different if using a non-standard inner product changed 
-   via IPSetBilinearForm(). For example, if using the B-inner product for 
+   via IPSetMatrix(). For example, if using the B-inner product for 
    positive definite B, (x,y)_B=y^H Bx, then the computed norm is ||x||_B = 
    sqrt( x^H Bx ).
 
@@ -50,31 +64,43 @@
 .seealso: IPInnerProduct()
 @*/
 PetscErrorCode IPNorm(IP ip,Vec x,PetscReal *norm)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,2);
+  PetscValidPointer(norm,3);
+  ierr = (*ip->ops->normbegin)(ip,x,norm);CHKERRQ(ierr);
+  ierr = (*ip->ops->normend)(ip,x,norm);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPNormBegin_Bilinear"
+PetscErrorCode IPNormBegin_Bilinear(IP ip,Vec x,PetscReal *norm)
 {
   PetscErrorCode ierr;
   PetscScalar    p;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,2);
-  PetscValidPointer(norm,3);
-  
-  if (!ip->matrix && ip->bilinear_form == IP_INNER_HERMITIAN) {
-    ierr = VecNorm(x,NORM_2,norm);CHKERRQ(ierr);
-  } else {
-    ierr = IPInnerProduct(ip,x,x,&p);CHKERRQ(ierr);
-    if (PetscAbsScalar(p)<PETSC_MACHINE_EPSILON)
-      PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");
-#if defined(PETSC_USE_COMPLEX)
-    if (PetscRealPart(p)<0.0 || PetscAbsReal(PetscImaginaryPart(p))>PETSC_MACHINE_EPSILON) 
-       SETERRQ(1,"IPNorm: The inner product is not well defined");
-    *norm = PetscSqrtScalar(PetscRealPart(p));
-#else
-    if (p<0.0) SETERRQ(1,"IPNorm: The inner product is not well defined");
-    *norm = PetscSqrtScalar(p);
-#endif
-  }
+  ierr = IPInnerProductBegin(ip,x,x,&p);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
 
+#undef __FUNCT__  
+#define __FUNCT__ "IPNormBegin_Sesquilinear"
+PetscErrorCode IPNormBegin_Sesquilinear(IP ip,Vec x,PetscReal *norm)
+{
+  PetscErrorCode ierr;
+  PetscScalar    p;
+
+  PetscFunctionBegin;
+  if (!ip->matrix) {
+    ierr = VecNormBegin(x,NORM_2,norm);CHKERRQ(ierr);
+  } else {
+    ierr = IPInnerProductBegin(ip,x,x,&p);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -83,6 +109,8 @@ PetscErrorCode IPNorm(IP ip,Vec x,PetscReal *norm)
 /*@
    IPNormBegin - Starts a split phase norm computation.
 
+   Collective on IP and Vec
+
    Input Parameters:
 +  ip   - the inner product context
 .  x    - input vector
@@ -95,24 +123,59 @@ PetscErrorCode IPNorm(IP ip,Vec x,PetscReal *norm)
 
 .seealso: IPNormEnd(), IPNorm(), IPInnerProduct(), IPMInnerProduct(), 
           IPInnerProductBegin(), IPInnerProductEnd()
-
 @*/
 PetscErrorCode IPNormBegin(IP ip,Vec x,PetscReal *norm)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,2);
+  PetscValidPointer(norm,3);
+  ierr = (*ip->ops->normbegin)(ip,x,norm);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPNormEnd_Bilinear"
+PetscErrorCode IPNormEnd_Bilinear(IP ip,Vec x,PetscReal *norm)
 {
   PetscErrorCode ierr;
   PetscScalar    p;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,2);
-  PetscValidPointer(norm,3);
-  
-  if (!ip->matrix && ip->bilinear_form == IP_INNER_HERMITIAN) {
-    ierr = VecNormBegin(x,NORM_2,norm);CHKERRQ(ierr);
-  } else {
-    ierr = IPInnerProductBegin(ip,x,x,&p);CHKERRQ(ierr);
-  }
+  ierr = IPInnerProductEnd(ip,x,x,&p);CHKERRQ(ierr);
+  if (PetscAbsScalar(p)<PETSC_MACHINE_EPSILON)
+    PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");
+#if defined(PETSC_USE_COMPLEX)
+  if (PetscRealPart(p)<0.0 || PetscAbsReal(PetscImaginaryPart(p))>PETSC_MACHINE_EPSILON) 
+     SETERRQ(((PetscObject)ip)->comm,1,"IPNorm: The inner product is not well defined");
+  *norm = PetscSqrtScalar(PetscRealPart(p));
+#else
+  if (p<0.0) SETERRQ(((PetscObject)ip)->comm,1,"IPNorm: The inner product is not well defined");
+  *norm = PetscSqrtScalar(p);
+#endif
+  PetscFunctionReturn(0);
+}
 
+#undef __FUNCT__  
+#define __FUNCT__ "IPNormEnd_Sesquilinear"
+PetscErrorCode IPNormEnd_Sesquilinear(IP ip,Vec x,PetscReal *norm)
+{
+  PetscErrorCode ierr;
+  PetscScalar    p;
+
+  PetscFunctionBegin;
+  if (!ip->matrix) {
+    ierr = VecNormEnd(x,NORM_2,norm);CHKERRQ(ierr);
+  } else {
+    ierr = IPInnerProductEnd(ip,x,x,&p);CHKERRQ(ierr);
+    if (PetscAbsScalar(p)<PETSC_MACHINE_EPSILON)
+      PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");
+    if (PetscRealPart(p)<0.0 || PetscAbsReal(PetscImaginaryPart(p))/PetscAbsScalar(p)>PETSC_MACHINE_EPSILON) 
+       SETERRQ(((PetscObject)ip)->comm,1,"IPNorm: The inner product is not well defined");
+    *norm = PetscSqrtScalar(PetscRealPart(p));
+  }
   PetscFunctionReturn(0);
 }
 
@@ -121,6 +184,8 @@ PetscErrorCode IPNormBegin(IP ip,Vec x,PetscReal *norm)
 /*@
    IPNormEnd - Ends a split phase norm computation.
 
+   Collective on IP and Vec
+
    Input Parameters:
 +  ip   - the inner product context
 -  x    - input vector
@@ -135,35 +200,16 @@ PetscErrorCode IPNormBegin(IP ip,Vec x,PetscReal *norm)
 
 .seealso: IPNormBegin(), IPNorm(), IPInnerProduct(), IPMInnerProduct(), 
           IPInnerProductBegin(), IPInnerProductEnd()
-
 @*/
 PetscErrorCode IPNormEnd(IP ip,Vec x,PetscReal *norm)
 {
   PetscErrorCode ierr;
-  PetscScalar    p;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,2);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,2);
   PetscValidPointer(norm,3);
-  
-  if (!ip->matrix && ip->bilinear_form == IP_INNER_HERMITIAN) {
-    ierr = VecNormEnd(x,NORM_2,norm);CHKERRQ(ierr);
-  } else {
-    ierr = IPInnerProductEnd(ip,x,x,&p);CHKERRQ(ierr);
-    if (PetscAbsScalar(p)<PETSC_MACHINE_EPSILON)
-      PetscInfo(ip,"Zero norm, either the vector is zero or a semi-inner product is being used\n");
-
-#if defined(PETSC_USE_COMPLEX)
-    if (PetscRealPart(p)<0.0 || PetscAbsReal(PetscImaginaryPart(p))>PETSC_MACHINE_EPSILON) 
-       SETERRQ(1,"IPNorm: The inner product is not well defined");
-    *norm = PetscSqrtScalar(PetscRealPart(p));
-#else
-    if (p<0.0) SETERRQ(1,"IPNorm: The inner product is not well defined");
-    *norm = PetscSqrtScalar(p);
-#endif
-  }
-
+  ierr = (*ip->ops->normend)(ip,x,norm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -185,41 +231,60 @@ PetscErrorCode IPNormEnd(IP ip,Vec x,PetscReal *norm)
    Notes:
    This function will usually compute the standard dot product of vectors
    x and y, (x,y)=y^H x. However this behaviour may be different if changed 
-   via IPSetBilinearForm(). This allows use of other inner products such as
+   via IPSetMatrix(). This allows use of other inner products such as
    the indefinite product y^T x for complex symmetric problems or the
    B-inner product for positive definite B, (x,y)_B=y^H Bx.
 
    Level: developer
 
-.seealso: IPSetBilinearForm(), IPApplyB(), VecDot(), IPMInnerProduct()
+.seealso: IPSetMatrix(), VecDot(), IPMInnerProduct()
 @*/
 PetscErrorCode IPInnerProduct(IP ip,Vec x,Vec y,PetscScalar *p)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,2);
-  PetscValidHeaderSpecific(y,VEC_COOKIE,3);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,2);
+  PetscValidHeaderSpecific(y,VEC_CLASSID,3);
   PetscValidScalarPointer(p,4);
-      
   ierr = PetscLogEventBegin(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   ip->innerproducts++;
+  ierr = (*ip->ops->innerproductbegin)(ip,x,y,p);CHKERRQ(ierr);
+  ierr = (*ip->ops->innerproductend)(ip,x,y,p);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPInnerProductBegin_Bilinear"
+PetscErrorCode IPInnerProductBegin_Bilinear(IP ip,Vec x,Vec y,PetscScalar *p)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
   if (ip->matrix) {
     ierr = IPApplyMatrix_Private(ip,x);CHKERRQ(ierr);
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecDot(ip->Bx,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecTDot(ip->Bx,y,p);CHKERRQ(ierr);
-    }
+    ierr = VecXDotBegin(ip->Bx,y,p);CHKERRQ(ierr);
   } else {
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecDot(x,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecTDot(x,y,p);CHKERRQ(ierr);
-    }
+    ierr = VecXDotBegin(x,y,p);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPInnerProductBegin_Sesquilinear"
+PetscErrorCode IPInnerProductBegin_Sesquilinear(IP ip,Vec x,Vec y,PetscScalar *p)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ip->matrix) {
+    ierr = IPApplyMatrix_Private(ip,x);CHKERRQ(ierr);
+    ierr = VecDotBegin(ip->Bx,y,p);CHKERRQ(ierr);
+  } else {
+    ierr = VecDotBegin(x,y,p);CHKERRQ(ierr);
   }
-  ierr = PetscLogEventEnd(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -228,6 +293,8 @@ PetscErrorCode IPInnerProduct(IP ip,Vec x,Vec y,PetscScalar *p)
 /*@
    IPInnerProductBegin - Starts a split phase inner product computation.
 
+   Collective on IP and Vec
+
    Input Parameters:
 +  ip - the inner product context
 .  x  - the first vector
@@ -241,43 +308,60 @@ PetscErrorCode IPInnerProduct(IP ip,Vec x,Vec y,PetscScalar *p)
 
 .seealso: IPInnerProductEnd(), IPInnerProduct(), IPNorm(), IPNormBegin(), 
           IPNormEnd(), IPMInnerProduct() 
-
 @*/
 PetscErrorCode IPInnerProductBegin(IP ip,Vec x,Vec y,PetscScalar *p)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,2);
-  PetscValidHeaderSpecific(y,VEC_COOKIE,3);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,2);
+  PetscValidHeaderSpecific(y,VEC_CLASSID,3);
   PetscValidScalarPointer(p,4);
-  
   ierr = PetscLogEventBegin(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   ip->innerproducts++;
-  if (ip->matrix) {
-    ierr = IPApplyMatrix_Private(ip,x);CHKERRQ(ierr);
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecDotBegin(ip->Bx,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecTDotBegin(ip->Bx,y,p);CHKERRQ(ierr);
-    }
-  } else {
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecDotBegin(x,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecTDotBegin(x,y,p);CHKERRQ(ierr);
-    }
-  }
+  ierr = (*ip->ops->innerproductbegin)(ip,x,y,p);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "IPInnerProductEnd_Bilinear"
+PetscErrorCode IPInnerProductEnd_Bilinear(IP ip,Vec x,Vec y,PetscScalar *p)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ip->matrix) {
+    ierr = VecXDotEnd(ip->Bx,y,p);CHKERRQ(ierr);
+  } else {
+    ierr = VecXDotEnd(x,y,p);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPInnerProductEnd_Sesquilinear"
+PetscErrorCode IPInnerProductEnd_Sesquilinear(IP ip,Vec x,Vec y,PetscScalar *p)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ip->matrix) {
+    ierr = VecDotEnd(ip->Bx,y,p);CHKERRQ(ierr);
+  } else {
+    ierr = VecDotEnd(x,y,p);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "IPInnerProductEnd"
 /*@
    IPInnerProductEnd - Ends a split phase inner product computation.
 
+   Collective on IP and Vec
+
    Input Parameters:
 +  ip - the inner product context
 .  x  - the first vector
@@ -293,32 +377,18 @@ PetscErrorCode IPInnerProductBegin(IP ip,Vec x,Vec y,PetscScalar *p)
 
 .seealso: IPInnerProductBegin(), IPInnerProduct(), IPNorm(), IPNormBegin(), 
           IPNormEnd(), IPMInnerProduct() 
-
 @*/
 PetscErrorCode IPInnerProductEnd(IP ip,Vec x,Vec y,PetscScalar *p)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,2);
-  PetscValidHeaderSpecific(y,VEC_COOKIE,3);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,2);
+  PetscValidHeaderSpecific(y,VEC_CLASSID,3);
   PetscValidScalarPointer(p,4);
-  
   ierr = PetscLogEventBegin(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
-  if (ip->matrix) {
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecDotEnd(ip->Bx,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecTDotEnd(ip->Bx,y,p);CHKERRQ(ierr);
-    }
-  } else {
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecDotEnd(x,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecTDotEnd(x,y,p);CHKERRQ(ierr);
-    }
-  }
+  ierr = (*ip->ops->innerproductend)(ip,x,y,p);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -343,42 +413,61 @@ PetscErrorCode IPInnerProductEnd(IP ip,Vec x,Vec y,PetscScalar *p)
    Notes:
    This function will usually compute the standard dot product of x and y_i, 
    (x,y_i)=y_i^H x, for each column of Y. However this behaviour may be different
-   if changed via IPSetBilinearForm(). This allows use of other inner products 
+   if changed via IPSetMatrix(). This allows use of other inner products 
    such as the indefinite product y_i^T x for complex symmetric problems or the
    B-inner product for positive definite B, (x,y_i)_B=y_i^H Bx.
 
    Level: developer
 
-.seealso: IPSetBilinearForm(), IPApplyB(), VecMDot(), IPInnerProduct()
+.seealso: IPSetMatrix(), VecMDot(), IPInnerProduct()
 @*/
 PetscErrorCode IPMInnerProduct(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,3);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,3);
   PetscValidPointer(y,4);
-  PetscValidHeaderSpecific(*y,VEC_COOKIE,4);
+  PetscValidHeaderSpecific(*y,VEC_CLASSID,4);
   PetscValidScalarPointer(p,5);
-  
   ierr = PetscLogEventBegin(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   ip->innerproducts += n;
+  ierr = (*ip->ops->minnerproductbegin)(ip,x,n,y,p);CHKERRQ(ierr);
+  ierr = (*ip->ops->minnerproductend)(ip,x,n,y,p);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPMInnerProductBegin_Bilinear"
+PetscErrorCode IPMInnerProductBegin_Bilinear(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
   if (ip->matrix) {
     ierr = IPApplyMatrix_Private(ip,x);CHKERRQ(ierr);
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecMDot(ip->Bx,n,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecMTDot(ip->Bx,n,y,p);CHKERRQ(ierr);
-    }
+    ierr = VecMXDotBegin(ip->Bx,n,y,p);CHKERRQ(ierr);
   } else {
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecMDot(x,n,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecMTDot(x,n,y,p);CHKERRQ(ierr);
-    }
+    ierr = VecMXDotBegin(x,n,y,p);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPMInnerProductBegin_Sesquilinear"
+PetscErrorCode IPMInnerProductBegin_Sesquilinear(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ip->matrix) {
+    ierr = IPApplyMatrix_Private(ip,x);CHKERRQ(ierr);
+    ierr = VecMDotBegin(ip->Bx,n,y,p);CHKERRQ(ierr);
+  } else {
+    ierr = VecMDotBegin(x,n,y,p);CHKERRQ(ierr);
   }
-  ierr = PetscLogEventEnd(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -387,6 +476,8 @@ PetscErrorCode IPMInnerProduct(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar
 /*@
    IPMInnerProductBegin - Starts a split phase multiple inner product computation.
 
+   Collective on IP and Vec
+
    Input Parameters:
 +  ip - the inner product context
 .  x  - the first input vector
@@ -401,45 +492,62 @@ PetscErrorCode IPMInnerProduct(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar
 
 .seealso: IPMInnerProductEnd(), IPMInnerProduct(), IPNorm(), IPNormBegin(), 
           IPNormEnd(), IPInnerProduct() 
-
 @*/
 PetscErrorCode IPMInnerProductBegin(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,3);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,3);
   if (n == 0) PetscFunctionReturn(0);
   PetscValidPointer(y,4);
-  PetscValidHeaderSpecific(*y,VEC_COOKIE,4);
+  PetscValidHeaderSpecific(*y,VEC_CLASSID,4);
   PetscValidScalarPointer(p,5);
-  
   ierr = PetscLogEventBegin(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   ip->innerproducts += n;
-  if (ip->matrix) {
-    ierr = IPApplyMatrix_Private(ip,x);CHKERRQ(ierr);
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecMDotBegin(ip->Bx,n,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecMTDotBegin(ip->Bx,n,y,p);CHKERRQ(ierr);
-    }
-  } else {
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecMDotBegin(x,n,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecMTDotBegin(x,n,y,p);CHKERRQ(ierr);
-    }
-  }
+  ierr = (*ip->ops->minnerproductbegin)(ip,x,n,y,p);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "IPMInnerProductEnd_Bilinear"
+PetscErrorCode IPMInnerProductEnd_Bilinear(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ip->matrix) {
+    ierr = VecMXDotEnd(ip->Bx,n,y,p);CHKERRQ(ierr);
+  } else {
+    ierr = VecMXDotEnd(x,n,y,p);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "IPMInnerProductEnd_Sesquilinear"
+PetscErrorCode IPMInnerProductEnd_Sesquilinear(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ip->matrix) {
+    ierr = VecMDotEnd(ip->Bx,n,y,p);CHKERRQ(ierr);
+  } else {
+    ierr = VecMDotEnd(x,n,y,p);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "IPMInnerProductEnd"
 /*@
    IPMInnerProductEnd - Ends a split phase multiple inner product computation.
 
+   Collective on IP and Vec
+
    Input Parameters:
 +  ip - the inner product context
 .  x  - the first input vector
@@ -456,34 +564,53 @@ PetscErrorCode IPMInnerProductBegin(IP ip,Vec x,PetscInt n,const Vec y[],PetscSc
 
 .seealso: IPMInnerProductBegin(), IPMInnerProduct(), IPNorm(), IPNormBegin(), 
           IPNormEnd(), IPInnerProduct() 
-
 @*/
 PetscErrorCode IPMInnerProductEnd(IP ip,Vec x,PetscInt n,const Vec y[],PetscScalar *p)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,3);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidHeaderSpecific(x,VEC_CLASSID,3);
   if (n == 0) PetscFunctionReturn(0);
   PetscValidPointer(y,4);
-  PetscValidHeaderSpecific(*y,VEC_COOKIE,4);
+  PetscValidHeaderSpecific(*y,VEC_CLASSID,4);
   PetscValidScalarPointer(p,5);
-  
   ierr = PetscLogEventBegin(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
-  if (ip->matrix) {
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecMDotEnd(ip->Bx,n,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecMTDotEnd(ip->Bx,n,y,p);CHKERRQ(ierr);
-    }
-  } else {
-    if (ip->bilinear_form == IP_INNER_HERMITIAN) {
-      ierr = VecMDotEnd(x,n,y,p);CHKERRQ(ierr);
-    } else {
-      ierr = VecMTDotEnd(x,n,y,p);CHKERRQ(ierr);
-    }
-  }
+  ierr = (*ip->ops->minnerproductend)(ip,x,n,y,p);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(IP_InnerProduct,ip,x,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+EXTERN_C_BEGIN
+#undef __FUNCT__  
+#define __FUNCT__ "IPCreate_Bilinear"
+PetscErrorCode IPCreate_Bilinear(IP ip)
+{
+  PetscFunctionBegin;
+  ip->ops->normbegin          = IPNormBegin_Bilinear;
+  ip->ops->normend            = IPNormEnd_Bilinear;
+  ip->ops->innerproductbegin  = IPInnerProductBegin_Bilinear;
+  ip->ops->innerproductend    = IPInnerProductEnd_Bilinear;
+  ip->ops->minnerproductbegin = IPMInnerProductBegin_Bilinear;
+  ip->ops->minnerproductend   = IPMInnerProductEnd_Bilinear;
+  PetscFunctionReturn(0);
+}
+
+#if defined(PETSC_USE_COMPLEX)
+#undef __FUNCT__  
+#define __FUNCT__ "IPCreate_Sesquilinear"
+PetscErrorCode IPCreate_Sesquilinear(IP ip)
+{
+  PetscFunctionBegin;
+  ip->ops->normbegin          = IPNormBegin_Sesquilinear;
+  ip->ops->normend            = IPNormEnd_Sesquilinear;
+  ip->ops->innerproductbegin  = IPInnerProductBegin_Sesquilinear;
+  ip->ops->innerproductend    = IPInnerProductEnd_Sesquilinear;
+  ip->ops->minnerproductbegin = IPMInnerProductBegin_Sesquilinear;
+  ip->ops->minnerproductend   = IPMInnerProductEnd_Sesquilinear;
+  PetscFunctionReturn(0);
+}
+#endif
+EXTERN_C_END
+
