@@ -1,9 +1,9 @@
 /*
-     Routines for setting the bilinear form
+     Routines for setting the matrix representation of the inner product.
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,75 +21,68 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/ipimpl.h"      /*I "slepcip.h" I*/
+#include <private/ipimpl.h>      /*I "slepcip.h" I*/
 
 #undef __FUNCT__  
-#define __FUNCT__ "IPSetBilinearForm"
+#define __FUNCT__ "IPSetMatrix"
 /*@
-   IPSetBilinearForm - Specifies the bilinear form to be used for
-   inner products.
+   IPSetMatrix - Specifies the matrix representation of the inner product.
 
    Collective on IP
 
    Input Parameters:
 +  ip    - the inner product context
-.  mat   - the matrix of the bilinear form (may be PETSC_NULL)
--  form  - the type of bilinear form
+-  mat   - the matrix (may be PETSC_NULL)
+
+   Notes:
+   A PETSC_NULL has the same effect as if the identity matrix was passed.
 
-   Note:
    This function is called by EPSSetProblemType() and usually need not be
    called by the user.
 
    Level: developer
 
-.seealso: IPGetBilinearForm(), IPInnerProduct(), IPNorm(), EPSSetProblemType(),
-          IPBilinearForm
+.seealso: IPGetMatrix(), IPInnerProduct(), IPNorm(), EPSSetProblemType()
 @*/
-PetscErrorCode IPSetBilinearForm(IP ip,Mat mat,IPBilinearForm form)
+PetscErrorCode IPSetMatrix(IP ip,Mat mat)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
   if (mat) {
-    PetscValidHeaderSpecific(mat,MAT_COOKIE,1);
+    PetscValidHeaderSpecific(mat,MAT_CLASSID,2);
     PetscObjectReference((PetscObject)mat);
   }
-  if (ip->matrix) {
-    ierr = MatDestroy(ip->matrix);CHKERRQ(ierr);
-    ierr = VecDestroy(ip->Bx);CHKERRQ(ierr);
-  }
+  ierr = IPReset(ip);CHKERRQ(ierr);
   ip->matrix = mat;
-  ip->bilinear_form = form;
-  ip->xid = ip->xstate = 0;
   if (mat) { ierr = MatGetVecs(mat,&ip->Bx,PETSC_NULL);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "IPGetBilinearForm"
+#define __FUNCT__ "IPGetMatrix"
 /*@C
-   IPGetBilinearForm - Retrieves the bilinear form to be used for
-   inner products.
+   IPGetMatrix - Retrieves the matrix representation of the inner product.
+
+   Not collective, though a parallel Mat may be returned
 
    Input Parameter:
 .  ip    - the inner product context
 
    Output Parameter:
-+  mat   - the matrix of the bilinear form (may be PETSC_NULL)
--  form  - the type of bilinear form
+.  mat   - the matrix of the inner product (may be PETSC_NULL)
 
    Level: developer
 
-.seealso: IPSetBilinearForm(), IPInnerProduct(), IPNorm(), EPSSetProblemType(),
-          IPBilinearForm
+.seealso: IPSetMatrix(), IPInnerProduct(), IPNorm(), EPSSetProblemType()
 @*/
-PetscErrorCode IPGetBilinearForm(IP ip,Mat* mat,IPBilinearForm* form)
+PetscErrorCode IPGetMatrix(IP ip,Mat* mat)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
-  if (mat) *mat = ip->matrix;
-  if (form) *form = ip->bilinear_form;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidPointer(mat,2);
+  *mat  = ip->matrix;
   PetscFunctionReturn(0);
 }
 
@@ -113,10 +106,9 @@ PetscErrorCode IPApplyMatrix_Private(IP ip,Vec x)
 #undef __FUNCT__  
 #define __FUNCT__ "IPApplyMatrix"
 /*@
-   IPApplyMatrix - Multiplies a vector with the matrix associated to the
-                   bilinear form.
+   IPApplyMatrix - Multiplies a vector by the matrix representing the IP.
 
-   Collective on IP
+   Neighbor-wise Collective on IP and Vec
 
    Input Parameters:
 +  ip    - the inner product context
@@ -126,18 +118,18 @@ PetscErrorCode IPApplyMatrix_Private(IP ip,Vec x)
 .  y     - the result  
 
    Note:
-   If the bilinear form has no associated matrix this function copies the vector.
+   If no matrix was specified this function copies the vector.
 
    Level: developer
 
-.seealso: IPSetBilinearForm(), IPInnerProduct(), IPNorm(), EPSSetProblemType() 
+.seealso: IPSetMatrix(), IPInnerProduct(), IPNorm(), EPSSetProblemType() 
 @*/
 PetscErrorCode IPApplyMatrix(IP ip,Vec x,Vec y)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(ip,IP_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
   if (ip->matrix) {
     ierr = IPApplyMatrix_Private(ip,x);CHKERRQ(ierr);
     ierr = VecCopy(ip->Bx,y);CHKERRQ(ierr);
