@@ -4,7 +4,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -22,15 +22,15 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/ipimpl.h"      /*I "slepcip.h" I*/
-#include "slepcblaslapack.h"
+#include <private/ipimpl.h>      /*I "slepcip.h" I*/
+#include <slepcblaslapack.h>
 
 /* 
    IPOrthogonalizeMGS1 - Compute one step of Modified Gram-Schmidt 
 */
 #undef __FUNCT__  
 #define __FUNCT__ "IPOrthogonalizeMGS1"
-static PetscErrorCode IPOrthogonalizeMGS1(IP ip,PetscInt n,PetscTruth *which,Vec *V,Vec v,PetscScalar *H)
+static PetscErrorCode IPOrthogonalizeMGS1(IP ip,PetscInt n,PetscBool *which,Vec *V,Vec v,PetscScalar *H)
 {
   PetscErrorCode ierr;
   PetscInt       j;
@@ -53,7 +53,7 @@ static PetscErrorCode IPOrthogonalizeMGS1(IP ip,PetscInt n,PetscTruth *which,Vec
 */
 #undef __FUNCT__  
 #define __FUNCT__ "IPOrthogonalizeCGS1"
-PetscErrorCode IPOrthogonalizeCGS1(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscTruth *which,Vec *V,Vec v,PetscScalar *H,PetscReal *onorm,PetscReal *norm)
+PetscErrorCode IPOrthogonalizeCGS1(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscBool *which,Vec *V,Vec v,PetscScalar *H,PetscReal *onorm,PetscReal *norm)
 {
   PetscErrorCode ierr;
   PetscInt       j;
@@ -100,7 +100,7 @@ PetscErrorCode IPOrthogonalizeCGS1(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscTr
   }
     
   /* compute |v| */
-  if (onorm) *onorm = sqrt(PetscRealPart(alpha));
+  if (onorm) *onorm = PetscSqrtReal(PetscRealPart(alpha));
 
   if (norm) {
     if (!ip->matrix) {
@@ -114,7 +114,7 @@ PetscErrorCode IPOrthogonalizeCGS1(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscTr
       *norm = PetscRealPart(alpha)-sum;
       if (*norm <= 0.0) {
         ierr = IPNorm(ip,v,norm);CHKERRQ(ierr);
-      } else *norm = sqrt(*norm);
+      } else *norm = PetscSqrtReal(*norm);
     } else {
       /* compute |v'| */
       ierr = IPNorm(ip,v,norm);CHKERRQ(ierr);
@@ -128,14 +128,13 @@ PetscErrorCode IPOrthogonalizeCGS1(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscTr
 */
 #undef __FUNCT__  
 #define __FUNCT__ "IPOrthogonalizeMGS"
-static PetscErrorCode IPOrthogonalizeMGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscTruth *which,Vec *V,Vec v,PetscScalar *H,PetscReal *norm,PetscTruth *lindep)
+static PetscErrorCode IPOrthogonalizeMGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscBool *which,Vec *V,Vec v,PetscScalar *H,PetscReal *norm,PetscBool *lindep)
 {
   PetscErrorCode ierr;
   PetscInt       i,k;
   PetscReal      onrm,nrm;
 
   PetscFunctionBegin;
-  
   if (H) { 
     for (i=0;i<n;i++) 
       H[i] = 0; 
@@ -143,7 +142,7 @@ static PetscErrorCode IPOrthogonalizeMGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
   
   switch (ip->orthog_ref) {
   
-  case IP_ORTH_REFINE_NEVER:
+  case IP_ORTHOG_REFINE_NEVER:
     ierr = IPOrthogonalizeMGS1(ip,nds,PETSC_NULL,DS,v,PETSC_NULL);CHKERRQ(ierr);
     ierr = IPOrthogonalizeMGS1(ip,n,which,V,v,H);CHKERRQ(ierr);
     /* compute |v| */
@@ -152,7 +151,7 @@ static PetscErrorCode IPOrthogonalizeMGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
     if (lindep) *lindep = PETSC_FALSE;
     break;
     
-  case IP_ORTH_REFINE_ALWAYS:
+  case IP_ORTHOG_REFINE_ALWAYS:
     /* first step */
     ierr = IPOrthogonalizeMGS1(ip,nds,PETSC_NULL,DS,v,PETSC_NULL);CHKERRQ(ierr);
     ierr = IPOrthogonalizeMGS1(ip,n,which,V,v,H);CHKERRQ(ierr);
@@ -168,7 +167,7 @@ static PetscErrorCode IPOrthogonalizeMGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
     if (norm) *norm = nrm;
     break;
   
-  case IP_ORTH_REFINE_IFNEEDED:
+  case IP_ORTHOG_REFINE_IFNEEDED:
     /* first step */
     ierr = IPNorm(ip,v,&onrm);CHKERRQ(ierr);
     ierr = IPOrthogonalizeMGS1(ip,nds,PETSC_NULL,DS,v,PETSC_NULL);CHKERRQ(ierr);
@@ -191,9 +190,8 @@ static PetscErrorCode IPOrthogonalizeMGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
     break;
     
   default:
-    SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown orthogonalization refinement");
+    SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_WRONG,"Unknown orthogonalization refinement");
   }
-
   PetscFunctionReturn(0);
 }
 
@@ -202,11 +200,11 @@ static PetscErrorCode IPOrthogonalizeMGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
 */
 #undef __FUNCT__  
 #define __FUNCT__ "IPOrthogonalizeCGS"
-static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscTruth *which,Vec *V,Vec v,PetscScalar *H,PetscReal *norm,PetscTruth *lindep)
+static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscBool *which,Vec *V,Vec v,PetscScalar *H,PetscReal *norm,PetscBool *lindep)
 {
   PetscErrorCode ierr;
   PetscScalar    lh[100],*h,lc[100],*c;
-  PetscTruth     allocatedh = PETSC_FALSE,allocatedc = PETSC_FALSE;
+  PetscBool      allocatedh = PETSC_FALSE,allocatedc = PETSC_FALSE;
   PetscReal      onrm,nrm;
   PetscInt       j,k;
 
@@ -219,7 +217,7 @@ static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
       allocatedh = PETSC_TRUE;
     }
   } else h = H;
-  if (ip->orthog_ref != IP_ORTH_REFINE_NEVER) {
+  if (ip->orthog_ref != IP_ORTHOG_REFINE_NEVER) {
     if (nds+n<=100) c = lc;
     else {
       ierr = PetscMalloc((nds+n)*sizeof(PetscScalar),&c);CHKERRQ(ierr);
@@ -230,7 +228,7 @@ static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
   /* orthogonalize and compute onorm */
   switch (ip->orthog_ref) {
   
-  case IP_ORTH_REFINE_NEVER:
+  case IP_ORTHOG_REFINE_NEVER:
     ierr = IPOrthogonalizeCGS1(ip,nds,DS,n,which,V,v,h,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
     /* compute |v| */
     if (norm) { ierr = IPNorm(ip,v,norm);CHKERRQ(ierr); }
@@ -238,7 +236,7 @@ static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
     if (lindep) *lindep = PETSC_FALSE;
     break;
     
-  case IP_ORTH_REFINE_ALWAYS:
+  case IP_ORTHOG_REFINE_ALWAYS:
     ierr = IPOrthogonalizeCGS1(ip,nds,DS,n,which,V,v,h,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr); 
     if (lindep) {
       ierr = IPOrthogonalizeCGS1(ip,nds,DS,n,which,V,v,c,&onrm,&nrm);CHKERRQ(ierr);
@@ -252,7 +250,7 @@ static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
       if (!which || which[j]) h[nds+j] += c[nds+j];
     break;
   
-  case IP_ORTH_REFINE_IFNEEDED:
+  case IP_ORTHOG_REFINE_IFNEEDED:
     ierr = IPOrthogonalizeCGS1(ip,nds,DS,n,which,V,v,h,&onrm,&nrm);CHKERRQ(ierr); 
     /* ||q|| < eta ||h|| */
     k = 1;
@@ -265,7 +263,7 @@ static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
         ierr = IPOrthogonalizeCGS1(ip,nds,DS,n,which,V,v,c,PETSC_NULL,&nrm);CHKERRQ(ierr); 
       }
       for (j=0;j<n;j++) 
-	if (!which || which[j]) h[nds+j] += c[nds+j];
+        if (!which || which[j]) h[nds+j] += c[nds+j];
     }
     if (norm) *norm = nrm;
     if (lindep) {
@@ -275,7 +273,7 @@ static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
     break;
 
   default:
-    SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown orthogonalization refinement");
+    SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_WRONG,"Unknown orthogonalization refinement");
   }
 
   /* recover H from workspace */
@@ -295,7 +293,7 @@ static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
 /*@
    IPOrthogonalize - Orthogonalize a vector with respect to two set of vectors.
 
-   Collective on IP
+   Collective on IP and Vec
 
    Input Parameters:
 +  ip     - the inner product (IP) context
@@ -327,29 +325,30 @@ static PetscErrorCode IPOrthogonalizeCGS(IP ip,PetscInt nds,Vec *DS,PetscInt n,P
 
 .seealso: IPSetOrthogonalization(), IPBiOrthogonalize()
 @*/
-PetscErrorCode IPOrthogonalize(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscTruth *which,Vec *V,Vec v,PetscScalar *H,PetscReal *norm,PetscTruth *lindep)
+PetscErrorCode IPOrthogonalize(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscBool *which,Vec *V,Vec v,PetscScalar *H,PetscReal *norm,PetscBool *lindep)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(ip,nds,2);
+  PetscValidLogicalCollectiveInt(ip,n,4);
   ierr = PetscLogEventBegin(IP_Orthogonalize,ip,0,0,0);CHKERRQ(ierr);
-  
   if (nds==0 && n==0) {
     if (norm) { ierr = IPNorm(ip,v,norm);CHKERRQ(ierr); }
     if (lindep) *lindep = PETSC_FALSE;
   } else {
     switch (ip->orthog_type) {
-    case IP_ORTH_CGS:
+    case IP_ORTHOG_CGS:
       ierr = IPOrthogonalizeCGS(ip,nds,DS,n,which,V,v,H,norm,lindep);CHKERRQ(ierr); 
       break;
-    case IP_ORTH_MGS:
+    case IP_ORTHOG_MGS:
       ierr = IPOrthogonalizeMGS(ip,nds,DS,n,which,V,v,H,norm,lindep);CHKERRQ(ierr); 
       break;
     default:
-      SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
+      SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
     }
   }
-  
   ierr = PetscLogEventEnd(IP_Orthogonalize,ip,0,0,0);CHKERRQ(ierr);  
   PetscFunctionReturn(0);
 }
@@ -359,7 +358,7 @@ PetscErrorCode IPOrthogonalize(IP ip,PetscInt nds,Vec *DS,PetscInt n,PetscTruth
 /*@
    IPQRDecomposition - Compute the QR factorization of a set of vectors.
 
-   Collective on IP
+   Collective on IP and Vec
 
    Input Parameters:
 +  ip - the eigenproblem solver context
@@ -391,11 +390,10 @@ PetscErrorCode IPQRDecomposition(IP ip,Vec *V,PetscInt m,PetscInt n,PetscScalar
   PetscErrorCode ierr;
   PetscInt       k;
   PetscReal      norm;
-  PetscTruth     lindep;
+  PetscBool      lindep;
   PetscRandom    rctx=PETSC_NULL;
   
   PetscFunctionBegin;
-
   for (k=m; k<n; k++) {
 
     /* orthogonalize v_k with respect to v_0, ..., v_{k-1} */
@@ -406,7 +404,8 @@ PetscErrorCode IPQRDecomposition(IP ip,Vec *V,PetscInt m,PetscInt n,PetscScalar
     if (norm==0.0 || lindep) { 
       PetscInfo(ip,"Linearly dependent vector found, generating a new random vector\n");
       if (!rctx) {
-        ierr = PetscRandomCreate(((PetscObject)ip)->comm,&rctx); CHKERRQ(ierr);
+        ierr = PetscRandomCreate(((PetscObject)ip)->comm,&rctx);CHKERRQ(ierr);
+        ierr = PetscRandomSetSeed(rctx,0x12345678);CHKERRQ(ierr);
         ierr = PetscRandomSetFromOptions(rctx);CHKERRQ(ierr);
       }
       ierr = SlepcVecSetRandom(V[k],rctx);CHKERRQ(ierr);
@@ -416,8 +415,7 @@ PetscErrorCode IPQRDecomposition(IP ip,Vec *V,PetscInt m,PetscInt n,PetscScalar
     if (R) R[k+ldr*k] = norm;
 
   }
-  if (rctx) { ierr = PetscRandomDestroy(rctx);CHKERRQ(ierr); }
-
+  ierr = PetscRandomDestroy(&rctx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -430,14 +428,13 @@ static PetscErrorCode IPCGSBiOrthogonalization(IP ip,PetscInt n_,Vec *V,Vec *W,V
 {
 #if defined(SLEPC_MISSING_LAPACK_GELQF) || defined(SLEPC_MISSING_LAPACK_ORMLQ)
   PetscFunctionBegin;
-  SETERRQ(PETSC_ERR_SUP,"xGELQF - Lapack routine is unavailable.");
+  SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_SUP,"xGELQF - Lapack routine is unavailable.");
 #else
   PetscErrorCode ierr;
   PetscBLASInt   j,ione=1,lwork,info,n=n_;
   PetscScalar    shh[100],*lhh,*vw,*tau,one=1.0,*work;
 
   PetscFunctionBegin;
-
   /* Don't allocate small arrays */
   if (n<=100) lhh = shh;
   else { ierr = PetscMalloc(n*sizeof(PetscScalar),&lhh);CHKERRQ(ierr); }
@@ -450,7 +447,7 @@ static PetscErrorCode IPCGSBiOrthogonalization(IP ip,PetscInt n_,Vec *V,Vec *W,V
   ierr = PetscMalloc(n*sizeof(PetscScalar),&tau);CHKERRQ(ierr);
   ierr = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
   LAPACKgelqf_(&n,&n,vw,&n,tau,work,&lwork,&info);
-  if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xGELQF %i",info);
+  if (info) SETERRQ1(((PetscObject)ip)->comm,PETSC_ERR_LIB,"Error in Lapack xGELQF %d",info);
   
   /*** First orthogonalization ***/
 
@@ -459,7 +456,7 @@ static PetscErrorCode IPCGSBiOrthogonalization(IP ip,PetscInt n_,Vec *V,Vec *W,V
   ierr = IPMInnerProduct(ip,v,n,W,H);CHKERRQ(ierr);
   BLAStrsm_("L","L","N","N",&n,&ione,&one,vw,&n,H,&n);
   LAPACKormlq_("L","N",&n,&ione,&n,vw,&n,tau,H,&n,work,&lwork,&info);
-  if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xORMLQ %i",info);
+  if (info) SETERRQ1(((PetscObject)ip)->comm,PETSC_ERR_LIB,"Error in Lapack xORMLQ %d",info);
   ierr = SlepcVecMAXPBY(v,1.0,-1.0,n,H,V);CHKERRQ(ierr);
 
   /* compute norm of v */
@@ -478,7 +475,7 @@ static PetscErrorCode IPCGSBiOrthogonalization(IP ip,PetscInt n_,Vec *V,Vec *W,V
 /*@
    IPBiOrthogonalize - Bi-orthogonalize a vector with respect to a set of vectors.
 
-   Collective on IP
+   Collective on IP and Vec
 
    Input Parameters:
 +  ip - the inner product context
@@ -510,14 +507,16 @@ PetscErrorCode IPBiOrthogonalize(IP ip,PetscInt n,Vec *V,Vec *W,Vec v,PetscScala
 {
   PetscErrorCode ierr;
   PetscScalar    lh[100],*h;
-  PetscTruth     allocated = PETSC_FALSE;
+  PetscBool      allocated = PETSC_FALSE;
   PetscReal      lhnrm,*hnrm,lnrm,*nrm;
+
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(ip,IP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(ip,n,2);
   if (n==0) {
     if (norm) { ierr = IPNorm(ip,v,norm);CHKERRQ(ierr); }
   } else {
     ierr = PetscLogEventBegin(IP_Orthogonalize,ip,0,0,0);CHKERRQ(ierr);
-    
     /* allocate H if needed */
     if (!H) {
       if (n<=100) h = lh;
@@ -528,7 +527,7 @@ PetscErrorCode IPBiOrthogonalize(IP ip,PetscInt n,Vec *V,Vec *W,Vec v,PetscScala
     } else h = H;
     
     /* retrieve hnrm and nrm for linear dependence check or conditional refinement */
-    if (ip->orthog_ref == IP_ORTH_REFINE_IFNEEDED) {
+    if (ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
       hnrm = &lhnrm;
       if (norm) nrm = norm;
       else nrm = &lnrm;
@@ -538,15 +537,14 @@ PetscErrorCode IPBiOrthogonalize(IP ip,PetscInt n,Vec *V,Vec *W,Vec v,PetscScala
     }
     
     switch (ip->orthog_type) {
-      case IP_ORTH_CGS:
+      case IP_ORTHOG_CGS:
         ierr = IPCGSBiOrthogonalization(ip,n,V,W,v,h,hnrm,nrm);CHKERRQ(ierr);
         break;
       default:
-        SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
+        SETERRQ(((PetscObject)ip)->comm,PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
     }
     
     if (allocated) { ierr = PetscFree(h);CHKERRQ(ierr); }
-    
     ierr = PetscLogEventEnd(IP_Orthogonalize,ip,0,0,0);CHKERRQ(ierr);
   } 
   PetscFunctionReturn(0);
