@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,20 +21,21 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/qepimpl.h"      /*I "slepcqep.h" I*/
+#include <private/qepimpl.h>      /*I "slepcqep.h" I*/
 
-PetscFList QEPList = 0;
-PetscCookie QEP_COOKIE = 0;
-PetscLogEvent QEP_SetUp = 0, QEP_Solve = 0, QEP_Dense = 0;
-static PetscTruth QEPPackageInitialized = PETSC_FALSE;
+PetscFList       QEPList = 0;
+PetscBool        QEPRegisterAllCalled = PETSC_FALSE;
+PetscClassId     QEP_CLASSID = 0;
+PetscLogEvent    QEP_SetUp = 0,QEP_Solve = 0,QEP_Dense = 0;
+static PetscBool QEPPackageInitialized = PETSC_FALSE;
 
 #undef __FUNCT__  
 #define __FUNCT__ "QEPFinalizePackage"
 /*@C
-  QEPFinalizePackage - This function destroys everything in the Slepc interface to the QEP package. It is
-  called from SlepcFinalize().
+   QEPFinalizePackage - This function destroys everything in the Slepc interface
+   to the QEP package. It is called from SlepcFinalize().
 
-  Level: developer
+   Level: developer
 
 .seealso: SlepcFinalize()
 @*/
@@ -43,46 +44,48 @@ PetscErrorCode QEPFinalizePackage(void)
   PetscFunctionBegin;
   QEPPackageInitialized = PETSC_FALSE;
   QEPList               = 0;
+  QEPRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "QEPInitializePackage"
 /*@C
-  QEPInitializePackage - This function initializes everything in the QEP package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to QEPCreate()
-  when using static libraries.
+   QEPInitializePackage - This function initializes everything in the QEP package. It is called
+   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to QEPCreate()
+   when using static libraries.
 
-  Input Parameter:
-  path - The dynamic library path, or PETSC_NULL
+   Input Parameter:
+.  path - The dynamic library path, or PETSC_NULL
 
-  Level: developer
+   Level: developer
 
 .seealso: SlepcInitialize()
 @*/
-PetscErrorCode QEPInitializePackage(char *path) {
+PetscErrorCode QEPInitializePackage(const char *path)
+{
   char           logList[256];
   char           *className;
-  PetscTruth     opt;
+  PetscBool      opt;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (QEPPackageInitialized) PetscFunctionReturn(0);
   QEPPackageInitialized = PETSC_TRUE;
   /* Register Classes */
-  ierr = PetscCookieRegister("Quadratic Eigenproblem Solver",&QEP_COOKIE);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Quadratic Eigenproblem Solver",&QEP_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = QEPRegisterAll(path);CHKERRQ(ierr);
   /* Register Events */
-  ierr = PetscLogEventRegister("QEPSetUp",QEP_COOKIE,&QEP_SetUp);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("QEPSolve",QEP_COOKIE,&QEP_Solve);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("QEPDense",QEP_COOKIE,&QEP_Dense);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("QEPSetUp",QEP_CLASSID,&QEP_SetUp);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("QEPSolve",QEP_CLASSID,&QEP_Solve);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("QEPDense",QEP_CLASSID,&QEP_Dense);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL,"-log_info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
     ierr = PetscStrstr(logList,"qep",&className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscInfoDeactivateClass(QEP_COOKIE);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(QEP_CLASSID);CHKERRQ(ierr);
     }
   }
   /* Process summary exclusions */
@@ -90,7 +93,7 @@ PetscErrorCode QEPInitializePackage(char *path) {
   if (opt) {
     ierr = PetscStrstr(logList,"qep",&className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscLogEventDeactivateClass(QEP_COOKIE);CHKERRQ(ierr);
+      ierr = PetscLogEventDeactivateClass(QEP_CLASSID);CHKERRQ(ierr);
     }
   }
   ierr = PetscRegisterFinalize(QEPFinalizePackage);CHKERRQ(ierr);
@@ -130,12 +133,12 @@ PetscErrorCode QEPView(QEP qep,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   const char     *type;
-  PetscTruth     isascii;
+  PetscBool      isascii;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)qep)->comm);
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_COOKIE,2);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(qep,1,viewer,2);
 
 #if defined(PETSC_USE_COMPLEX)
@@ -143,24 +146,23 @@ PetscErrorCode QEPView(QEP qep,PetscViewer viewer)
 #else
 #define HERM "symmetric"
 #endif
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"QEP Object:\n");CHKERRQ(ierr);
+    ierr = PetscObjectPrintClassNamePrefixType((PetscObject)qep,viewer,"QEP Object");CHKERRQ(ierr);
+    if (qep->ops->view) {
+      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
+      ierr = (*qep->ops->view)(qep,viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+    }
     if (qep->problem_type) {
       switch (qep->problem_type) {
         case QEP_GENERAL:    type = "general quadratic eigenvalue problem"; break;
         case QEP_HERMITIAN:  type = HERM " quadratic eigenvalue problem"; break;
         case QEP_GYROSCOPIC: type = "gyroscopic quadratic eigenvalue problem"; break;
-        default: SETERRQ(1,"Wrong value of qep->problem_type");
+        default: SETERRQ(((PetscObject)qep)->comm,1,"Wrong value of qep->problem_type");
       }
     } else type = "not yet set";
     ierr = PetscViewerASCIIPrintf(viewer,"  problem type: %s\n",type);CHKERRQ(ierr);
-    ierr = QEPGetType(qep,&type);CHKERRQ(ierr);
-    if (type) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  method: %s\n",type);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  method: not yet set\n");CHKERRQ(ierr);
-    }
     ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: ");CHKERRQ(ierr);
     if (!qep->which) {
       ierr = PetscViewerASCIIPrintf(viewer,"not yet set\n");CHKERRQ(ierr);
@@ -183,36 +185,137 @@ PetscErrorCode QEPView(QEP qep,PetscViewer viewer)
       case QEP_SMALLEST_IMAGINARY:
         ierr = PetscViewerASCIIPrintf(viewer,"smallest imaginary parts\n");CHKERRQ(ierr);
         break;
-      default: SETERRQ(1,"Wrong value of qep->which");
+      default: SETERRQ(((PetscObject)qep)->comm,1,"Wrong value of qep->which");
     }    
     if (qep->leftvecs) {
       ierr = PetscViewerASCIIPrintf(viewer,"  computing left eigenvectors also\n");CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %d\n",qep->nev);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %d\n",qep->ncv);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %d\n",qep->mpd);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %d\n", qep->max_it);
-    ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %g\n",qep->tol);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  scaling factor: %g\n",qep->sfactor);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %D\n",qep->nev);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %D\n",qep->ncv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %D\n",qep->mpd);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",qep->max_it);
+    ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %G\n",qep->tol);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  scaling factor: %G\n",qep->sfactor);CHKERRQ(ierr);
     if (qep->nini!=0) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %d\n",PetscAbs(qep->nini));CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %D\n",PetscAbs(qep->nini));CHKERRQ(ierr);
     }
     if (qep->ninil!=0) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial left space: %d\n",PetscAbs(qep->ninil));CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial left space: %D\n",PetscAbs(qep->ninil));CHKERRQ(ierr);
     }
-    if (qep->ops->view) {
-      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-      ierr = (*qep->ops->view)(qep,viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
-    }
-    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-    ierr = IPView(qep->ip,viewer); CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   } else {
     if (qep->ops->view) {
       ierr = (*qep->ops->view)(qep,viewer);CHKERRQ(ierr);
     }
   }
+  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
+  ierr = IPView(qep->ip,viewer);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "QEPPrintSolution"
+/*@
+   QEPPrintSolution - Prints the computed eigenvalues.
+
+   Collective on QEP
+
+   Input Parameters:
++  qep - the eigensolver context
+-  viewer - optional visualization context
+
+   Options Database:
+.  -qep_terse - print only minimal information
+
+   Note:
+   By default, this function prints a table with eigenvalues and associated
+   relative errors. With -qep_terse only the eigenvalues are printed.
+
+   Level: intermediate
+
+.seealso: PetscViewerASCIIOpen()
+@*/
+PetscErrorCode QEPPrintSolution(QEP qep,PetscViewer viewer)
+{
+  PetscBool      terse,errok,isascii;
+  PetscReal      error,re,im;
+  PetscScalar    kr,ki;
+  PetscInt       i,j;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)qep)->comm);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
+  PetscCheckSameComm(qep,1,viewer,2);
+  if (!qep->eigr || !qep->eigi || !qep->V) { 
+    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONGSTATE,"QEPSolve must be called first"); 
+  }
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (!isascii) PetscFunctionReturn(0);
+
+  ierr = PetscOptionsHasName(PETSC_NULL,"-qep_terse",&terse);CHKERRQ(ierr);
+  if (terse) {
+    if (qep->nconv<qep->nev) {
+      ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D eigenvalues converged\n\n",qep->nev);CHKERRQ(ierr);
+    } else {
+      errok = PETSC_TRUE;
+      for (i=0;i<qep->nev;i++) {
+        ierr = QEPComputeRelativeError(qep,i,&error);CHKERRQ(ierr);
+        errok = (errok && error<qep->tol)? PETSC_TRUE: PETSC_FALSE;
+      }
+      if (errok) {
+        ierr = PetscViewerASCIIPrintf(viewer," All requested eigenvalues computed up to the required tolerance:");CHKERRQ(ierr);
+        for (i=0;i<=(qep->nev-1)/8;i++) {
+          ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
+          for (j=0;j<PetscMin(8,qep->nev-8*i);j++) {
+            ierr = QEPGetEigenpair(qep,8*i+j,&kr,&ki,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+#if defined(PETSC_USE_COMPLEX)
+            re = PetscRealPart(kr);
+            im = PetscImaginaryPart(kr);
+#else
+            re = kr;
+            im = ki;
+#endif 
+            if (PetscAbs(re)/PetscAbs(im)<PETSC_SMALL) re = 0.0;
+            if (PetscAbs(im)/PetscAbs(re)<PETSC_SMALL) im = 0.0;
+            if (im!=0.0) {
+              ierr = PetscViewerASCIIPrintf(viewer,"%.5F%+.5Fi",re,im);CHKERRQ(ierr);
+            } else {
+              ierr = PetscViewerASCIIPrintf(viewer,"%.5F",re);CHKERRQ(ierr); 
+            }
+            if (8*i+j+1<qep->nev) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
+          }
+        }
+        ierr = PetscViewerASCIIPrintf(viewer,"\n\n");CHKERRQ(ierr);
+      } else {
+        ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",qep->nev);CHKERRQ(ierr);
+      }
+    }
+  } else {
+    ierr = PetscViewerASCIIPrintf(viewer," Number of converged approximate eigenpairs: %D\n\n",qep->nconv);CHKERRQ(ierr);
+    if (qep->nconv>0) {
+      ierr = PetscViewerASCIIPrintf(viewer,
+           "           k          ||(k^2M+Ck+K)x||/||kx||\n"
+           "   ----------------- -------------------------\n");CHKERRQ(ierr);
+      for (i=0;i<qep->nconv;i++) {
+        ierr = QEPGetEigenpair(qep,i,&kr,&ki,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+        ierr = QEPComputeRelativeError(qep,i,&error);CHKERRQ(ierr);
+#if defined(PETSC_USE_COMPLEX)
+        re = PetscRealPart(kr);
+        im = PetscImaginaryPart(kr);
+#else
+        re = kr;
+        im = ki;
+#endif 
+        if (im!=0.0) {
+          ierr = PetscViewerASCIIPrintf(viewer," % 9F%+9F i     %12G\n",re,im,error);CHKERRQ(ierr);
+        } else {
+          ierr = PetscViewerASCIIPrintf(viewer,"   % 12F           %12G\n",re,error);CHKERRQ(ierr); 
+        }
+      }
+      ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
@@ -244,19 +347,19 @@ PetscErrorCode QEPCreate(MPI_Comm comm,QEP *outqep)
   PetscFunctionBegin;
   PetscValidPointer(outqep,2);
   *outqep = 0;
+  ierr = PetscHeaderCreate(qep,_p_QEP,struct _QEPOps,QEP_CLASSID,-1,"QEP","Quadratic Eigenvalue Problem","QEP",comm,QEPDestroy,QEPView);CHKERRQ(ierr);
 
-  ierr = PetscHeaderCreate(qep,_p_QEP,struct _QEPOps,QEP_COOKIE,-1,"QEP",comm,QEPDestroy,QEPView);CHKERRQ(ierr);
-  *outqep = qep;
-
-  ierr = PetscMemzero(qep->ops,sizeof(struct _QEPOps));CHKERRQ(ierr);
-
+  qep->M               = 0;
+  qep->C               = 0;
+  qep->K               = 0;
   qep->max_it          = 0;
   qep->nev             = 1;
   qep->ncv             = 0;
   qep->mpd             = 0;
   qep->nini            = 0;
   qep->ninil           = 0;
-  qep->tol             = 1e-7;
+  qep->allocated_ncv   = 0;
+  qep->tol             = PETSC_DEFAULT;
   qep->sfactor         = 0.0;
   qep->conv_func       = QEPDefaultConverged;
   qep->conv_ctx        = PETSC_NULL;
@@ -266,6 +369,7 @@ PetscErrorCode QEPCreate(MPI_Comm comm,QEP *outqep)
   qep->leftvecs        = PETSC_FALSE;
   qep->problem_type    = (QEPProblemType)0;
   qep->V               = PETSC_NULL;
+  qep->W               = PETSC_NULL;
   qep->IS              = PETSC_NULL;
   qep->ISL             = PETSC_NULL;
   qep->T               = PETSC_NULL;
@@ -273,6 +377,7 @@ PetscErrorCode QEPCreate(MPI_Comm comm,QEP *outqep)
   qep->eigi            = PETSC_NULL;
   qep->errest          = PETSC_NULL;
   qep->data            = PETSC_NULL;
+  qep->t               = PETSC_NULL;
   qep->nconv           = 0;
   qep->its             = 0;
   qep->perm            = PETSC_NULL;
@@ -284,12 +389,12 @@ PetscErrorCode QEPCreate(MPI_Comm comm,QEP *outqep)
   qep->reason          = QEP_CONVERGED_ITERATING;
   qep->numbermonitors  = 0;
   qep->trackall        = PETSC_FALSE;
+  qep->rand            = 0;
 
-  ierr = IPCreate(comm,&qep->ip); CHKERRQ(ierr);
-  ierr = IPSetOptionsPrefix(qep->ip,((PetscObject)qep)->prefix);
-  ierr = IPAppendOptionsPrefix(qep->ip,"qep_");
-  PetscLogObjectParent(qep,qep->ip);
-  ierr = PetscPublishAll(qep);CHKERRQ(ierr);
+  ierr = PetscRandomCreate(comm,&qep->rand);CHKERRQ(ierr);
+  ierr = PetscRandomSetSeed(qep->rand,0x12345678);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent(qep,qep->rand);CHKERRQ(ierr);
+  *outqep = qep;
   PetscFunctionReturn(0);
 }
  
@@ -298,7 +403,7 @@ PetscErrorCode QEPCreate(MPI_Comm comm,QEP *outqep)
 /*@C
    QEPSetType - Selects the particular solver to be used in the QEP object. 
 
-   Collective on QEP
+   Logically Collective on QEP
 
    Input Parameters:
 +  qep      - the quadratic eigensolver context
@@ -327,30 +432,24 @@ PetscErrorCode QEPCreate(MPI_Comm comm,QEP *outqep)
 PetscErrorCode QEPSetType(QEP qep,const QEPType type)
 {
   PetscErrorCode ierr,(*r)(QEP);
-  PetscTruth match;
+  PetscBool      match;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidCharPointer(type,2);
 
   ierr = PetscTypeCompare((PetscObject)qep,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  if (qep->data) {
-    /* destroy the old private QEP context */
-    ierr = (*qep->ops->destroy)(qep); CHKERRQ(ierr);
-    qep->data = 0;
-  }
+  ierr = PetscFListFind(QEPList,((PetscObject)qep)->comm,type,PETSC_TRUE,(void (**)(void))&r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(((PetscObject)qep)->comm,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown QEP type given: %s",type);
 
-  ierr = PetscFListFind(QEPList,((PetscObject)qep)->comm,type,(void (**)(void))&r);CHKERRQ(ierr);
-
-  if (!r) SETERRQ1(1,"Unknown QEP type given: %s",type);
+  if (qep->ops->destroy) { ierr = (*qep->ops->destroy)(qep);CHKERRQ(ierr); }
+  ierr = PetscMemzero(qep->ops,sizeof(struct _QEPOps));CHKERRQ(ierr);
 
   qep->setupcalled = 0;
-  ierr = PetscMemzero(qep->ops,sizeof(struct _QEPOps));CHKERRQ(ierr);
-  ierr = (*r)(qep);CHKERRQ(ierr);
-
   ierr = PetscObjectChangeTypeName((PetscObject)qep,type);CHKERRQ(ierr);
+  ierr = (*r)(qep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -374,53 +473,12 @@ PetscErrorCode QEPSetType(QEP qep,const QEPType type)
 PetscErrorCode QEPGetType(QEP qep,const QEPType *type)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidPointer(type,2);
   *type = ((PetscObject)qep)->type_name;
   PetscFunctionReturn(0);
 }
 
-/*MC
-   QEPRegisterDynamic - Adds a method to the quadratic eigenproblem solver package.
-
-   Synopsis:
-   QEPRegisterDynamic(char *name_solver,char *path,char *name_create,PetscErrorCode (*routine_create)(QEP))
-
-   Not Collective
-
-   Input Parameters:
-+  name_solver - name of a new user-defined solver
-.  path - path (either absolute or relative) the library containing this solver
-.  name_create - name of routine to create the solver context
--  routine_create - routine to create the solver context
-
-   Notes:
-   QEPRegisterDynamic() may be called multiple times to add several user-defined solvers.
-
-   If dynamic libraries are used, then the fourth input argument (routine_create)
-   is ignored.
-
-   Sample usage:
-.vb
-   QEPRegisterDynamic("my_solver",/home/username/my_lib/lib/libO/solaris/mylib.a,
-               "MySolverCreate",MySolverCreate);
-.ve
-
-   Then, your solver can be chosen with the procedural interface via
-$     QEPSetType(qep,"my_solver")
-   or at runtime via the option
-$     -qep_type my_solver
-
-   Level: advanced
-
-   Environmental variables such as ${PETSC_ARCH}, ${SLEPC_DIR},
-   and others of the form ${any_environmental_variable} occuring in pathname will be 
-   replaced with appropriate values.
-
-.seealso: QEPRegisterDestroy(), QEPRegisterAll()
-
-M*/
-
 #undef __FUNCT__  
 #define __FUNCT__ "QEPRegister"
 /*@C
@@ -431,7 +489,7 @@ M*/
 PetscErrorCode QEPRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(QEP))
 {
   PetscErrorCode ierr;
-  char           fullname[256];
+  char           fullname[PETSC_MAX_PATH_LEN];
 
   PetscFunctionBegin;
   ierr = PetscFListConcat(path,name,fullname);CHKERRQ(ierr);
@@ -457,13 +515,47 @@ PetscErrorCode QEPRegisterDestroy(void)
 
   PetscFunctionBegin;
   ierr = PetscFListDestroy(&QEPList);CHKERRQ(ierr);
-  ierr = QEPRegisterAll(PETSC_NULL);CHKERRQ(ierr);
+  QEPRegisterAllCalled = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "QEPReset"
+/*@C
+   QEPReset - Resets the QEP context to the setupcalled=0 state and removes any
+   allocated objects.
+
+   Collective on QEP
+
+   Input Parameter:
+.  qep - eigensolver context obtained from QEPCreate()
+
+   Level: advanced
+
+.seealso: QEPDestroy()
+@*/
+PetscErrorCode QEPReset(QEP qep)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  if (qep->ops->reset) { ierr = (qep->ops->reset)(qep);CHKERRQ(ierr); }
+  if (qep->ip) { ierr = IPReset(qep->ip);CHKERRQ(ierr); }
+  ierr = MatDestroy(&qep->M);CHKERRQ(ierr);
+  ierr = MatDestroy(&qep->C);CHKERRQ(ierr);
+  ierr = MatDestroy(&qep->K);CHKERRQ(ierr);
+  ierr = VecDestroy(&qep->t);CHKERRQ(ierr);
+  ierr = QEPFreeSolution(qep);CHKERRQ(ierr);
+  qep->matvecs     = 0;
+  qep->linits      = 0;
+  qep->setupcalled = 0;  
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "QEPDestroy"
-/*@
+/*@C
    QEPDestroy - Destroys the QEP context.
 
    Collective on QEP
@@ -475,49 +567,20 @@ PetscErrorCode QEPRegisterDestroy(void)
 
 .seealso: QEPCreate(), QEPSetUp(), QEPSolve()
 @*/
-PetscErrorCode QEPDestroy(QEP qep)
+PetscErrorCode QEPDestroy(QEP *qep)
 {
   PetscErrorCode ierr;
-  PetscScalar    *pV;
-  PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
-  if (--((PetscObject)qep)->refct > 0) PetscFunctionReturn(0);
-
-  /* if memory was published with AMS then destroy it */
-  ierr = PetscObjectDepublish(qep);CHKERRQ(ierr);
-
-  if (qep->ops->destroy) {
-    ierr = (*qep->ops->destroy)(qep); CHKERRQ(ierr);
-  }
-  
-  ierr = PetscFree(qep->T);CHKERRQ(ierr);
-
-  if (qep->eigr) { 
-    ierr = PetscFree(qep->eigr);CHKERRQ(ierr);
-    ierr = PetscFree(qep->eigi);CHKERRQ(ierr);
-    ierr = PetscFree(qep->perm);CHKERRQ(ierr);
-    ierr = PetscFree(qep->errest);CHKERRQ(ierr);
-    ierr = VecGetArray(qep->V[0],&pV);CHKERRQ(ierr);
-    for (i=0;i<qep->ncv;i++) {
-      ierr = VecDestroy(qep->V[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(pV);CHKERRQ(ierr);
-    ierr = PetscFree(qep->V);CHKERRQ(ierr);
-  }
-
-  ierr = QEPMonitorCancel(qep);CHKERRQ(ierr);
-
-  ierr = IPDestroy(qep->ip);CHKERRQ(ierr);
-  if (qep->rand) {
-    ierr = PetscRandomDestroy(qep->rand);CHKERRQ(ierr);
-  }
-
-  if (qep->M) { ierr = MatDestroy(qep->M);CHKERRQ(ierr); }
-  if (qep->C) { ierr = MatDestroy(qep->C);CHKERRQ(ierr); }
-  if (qep->K) { ierr = MatDestroy(qep->K);CHKERRQ(ierr); }
-
+  if (!*qep) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(*qep,QEP_CLASSID,1);
+  if (--((PetscObject)(*qep))->refct > 0) { *qep = 0; PetscFunctionReturn(0); }
+  ierr = QEPReset(*qep);CHKERRQ(ierr);
+  ierr = PetscObjectDepublish(*qep);CHKERRQ(ierr);
+  if ((*qep)->ops->destroy) { ierr = (*(*qep)->ops->destroy)(*qep);CHKERRQ(ierr); }
+  ierr = IPDestroy(&(*qep)->ip);CHKERRQ(ierr);
+  ierr = PetscRandomDestroy(&(*qep)->rand);CHKERRQ(ierr);
+  ierr = QEPMonitorCancel(*qep);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(qep);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -527,7 +590,7 @@ PetscErrorCode QEPDestroy(QEP qep)
 /*@
    QEPSetIP - Associates an inner product object to the quadratic eigensolver. 
 
-   Collective on QEP and IP
+   Collective on QEP
 
    Input Parameters:
 +  qep - eigensolver context obtained from QEPCreate()
@@ -546,12 +609,13 @@ PetscErrorCode QEPSetIP(QEP qep,IP ip)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
-  PetscValidHeaderSpecific(ip,IP_COOKIE,2);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,2);
   PetscCheckSameComm(qep,1,ip,2);
   ierr = PetscObjectReference((PetscObject)ip);CHKERRQ(ierr);
-  ierr = IPDestroy(qep->ip); CHKERRQ(ierr);
+  ierr = IPDestroy(&qep->ip);CHKERRQ(ierr);
   qep->ip = ip;
+  ierr = PetscLogObjectParent(qep,qep->ip);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -575,9 +639,15 @@ PetscErrorCode QEPSetIP(QEP qep,IP ip)
 @*/
 PetscErrorCode QEPGetIP(QEP qep,IP *ip)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidPointer(ip,2);
+  if (!qep->ip) {
+    ierr = IPCreate(((PetscObject)qep)->comm,&qep->ip);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(qep,qep->ip);CHKERRQ(ierr);
+  }
   *ip = qep->ip;
   PetscFunctionReturn(0);
 }
