@@ -4,7 +4,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -22,7 +22,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/qepimpl.h"   /*I "slepcqep.h" I*/
+#include <private/qepimpl.h>       /*I "slepcqep.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "QEPSetFromOptions"
@@ -43,16 +43,18 @@
 @*/
 PetscErrorCode QEPSetFromOptions(QEP qep)
 {
-  PetscErrorCode ierr;
-  char           type[256],monfilename[PETSC_MAX_PATH_LEN];
-  PetscTruth     flg,val;
-  PetscReal      r;
-  PetscInt       i,j,k;
-  PetscViewerASCIIMonitor monviewer;
-  QEPMONITOR_CONV *ctx;
+  PetscErrorCode   ierr;
+  char             type[256],monfilename[PETSC_MAX_PATH_LEN];
+  PetscBool        flg,val;
+  PetscReal        r;
+  PetscInt         i,j,k;
+  PetscViewer      monviewer;
+  SlepcConvMonitor ctx;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  if (!QEPRegisterAllCalled) { ierr = QEPRegisterAll(PETSC_NULL);CHKERRQ(ierr); }
+  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
   ierr = PetscOptionsBegin(((PetscObject)qep)->comm,((PetscObject)qep)->prefix,"Quadratic Eigenvalue Problem (QEP) Solver Options","QEP");CHKERRQ(ierr);
     ierr = PetscOptionsList("-qep_type","Quadratic Eigenvalue Problem method","QEPSetType",QEPList,(char*)(((PetscObject)qep)->type_name?((PetscObject)qep)->type_name:QEPLINEAR),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
@@ -61,11 +63,11 @@ PetscErrorCode QEPSetFromOptions(QEP qep)
       ierr = QEPSetType(qep,QEPLINEAR);CHKERRQ(ierr);
     }
 
-    ierr = PetscOptionsTruthGroupBegin("-qep_general","general quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupBegin("-qep_general","general quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetProblemType(qep,QEP_GENERAL);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroup("-qep_hermitian","hermitian quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroup("-qep_hermitian","hermitian quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetProblemType(qep,QEP_HERMITIAN);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroupEnd("-qep_gyroscopic","gyroscopic quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupEnd("-qep_gyroscopic","gyroscopic quadratic eigenvalue problem","QEPSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetProblemType(qep,QEP_GYROSCOPIC);CHKERRQ(ierr);}
 
     r = PETSC_IGNORE;
@@ -74,11 +76,11 @@ PetscErrorCode QEPSetFromOptions(QEP qep)
 
     r = i = PETSC_IGNORE;
     ierr = PetscOptionsInt("-qep_max_it","Maximum number of iterations","QEPSetTolerances",qep->max_it,&i,PETSC_NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsReal("-qep_tol","Tolerance","QEPSetTolerances",qep->tol,&r,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-qep_tol","Tolerance","QEPSetTolerances",qep->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:qep->tol,&r,PETSC_NULL);CHKERRQ(ierr);
     ierr = QEPSetTolerances(qep,r,i);CHKERRQ(ierr);
-    ierr = PetscOptionsTruthGroupBegin("-qep_convergence_default","Default (relative error) convergence test","QEPSetConvergenceTest",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupBegin("-qep_convergence_default","Default (relative error) convergence test","QEPSetConvergenceTest",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetConvergenceTest(qep,QEPDefaultConverged,PETSC_NULL);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroupEnd("-qep_convergence_absolute","Absolute error convergence test","QEPSetConvergenceTest",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupEnd("-qep_convergence_absolute","Absolute error convergence test","QEPSetConvergenceTest",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetConvergenceTest(qep,QEPAbsoluteConverged,PETSC_NULL);CHKERRQ(ierr);}
 
     i = j = k = PETSC_IGNORE;
@@ -92,57 +94,57 @@ PetscErrorCode QEPSetFromOptions(QEP qep)
       Cancels all monitors hardwired into code before call to QEPSetFromOptions()
     */
     flg  = PETSC_FALSE;
-    ierr = PetscOptionsTruth("-qep_monitor_cancel","Remove any hardwired monitor routines","QEPMonitorCancel",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-qep_monitor_cancel","Remove any hardwired monitor routines","QEPMonitorCancel",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
     if (flg) {
-      ierr = QEPMonitorCancel(qep); CHKERRQ(ierr);
+      ierr = QEPMonitorCancel(qep);CHKERRQ(ierr);
     }
     /*
       Prints approximate eigenvalues and error estimates at each iteration
     */
     ierr = PetscOptionsString("-qep_monitor","Monitor first unconverged approximate eigenvalue and error estimate","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
     if (flg) {
-      ierr = PetscViewerASCIIMonitorCreate(((PetscObject)qep)->comm,monfilename,((PetscObject)qep)->tablevel,&monviewer);CHKERRQ(ierr);
-      ierr = QEPMonitorSet(qep,QEPMonitorFirst,monviewer,(PetscErrorCode (*)(void*))PetscViewerASCIIMonitorDestroy);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIOpen(((PetscObject)qep)->comm,monfilename,&monviewer);CHKERRQ(ierr);
+      ierr = QEPMonitorSet(qep,QEPMonitorFirst,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
     }
     ierr = PetscOptionsString("-qep_monitor_conv","Monitor approximate eigenvalues and error estimates as they converge","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
     if (flg) {
-      ierr = PetscNew(QEPMONITOR_CONV,&ctx);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIMonitorCreate(((PetscObject)qep)->comm,monfilename,((PetscObject)qep)->tablevel,&ctx->viewer);CHKERRQ(ierr);
-      ierr = QEPMonitorSet(qep,QEPMonitorConverged,ctx,(PetscErrorCode (*)(void*))QEPMonitorDestroy_Converged);CHKERRQ(ierr);
+      ierr = PetscNew(struct _n_SlepcConvMonitor,&ctx);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIOpen(((PetscObject)qep)->comm,monfilename,&ctx->viewer);CHKERRQ(ierr);
+      ierr = QEPMonitorSet(qep,QEPMonitorConverged,ctx,(PetscErrorCode (*)(void**))SlepcConvMonitorDestroy);CHKERRQ(ierr);
     }
     ierr = PetscOptionsString("-qep_monitor_all","Monitor approximate eigenvalues and error estimates","QEPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
     if (flg) {
-      ierr = PetscViewerASCIIMonitorCreate(((PetscObject)qep)->comm,monfilename,((PetscObject)qep)->tablevel,&monviewer);CHKERRQ(ierr);
-      ierr = QEPMonitorSet(qep,QEPMonitorAll,monviewer,(PetscErrorCode (*)(void*))PetscViewerASCIIMonitorDestroy);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIOpen(((PetscObject)qep)->comm,monfilename,&monviewer);CHKERRQ(ierr);
+      ierr = QEPMonitorSet(qep,QEPMonitorAll,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
       ierr = QEPSetTrackAll(qep,PETSC_TRUE);CHKERRQ(ierr);
     }
     flg = PETSC_FALSE;
-    ierr = PetscOptionsTruth("-qep_monitor_draw","Monitor first unconverged approximate error estimate graphically","QEPMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+    ierr = PetscOptionsBool("-qep_monitor_draw","Monitor first unconverged approximate error estimate graphically","QEPMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
     if (flg) {
       ierr = QEPMonitorSet(qep,QEPMonitorLG,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
     }
     flg = PETSC_FALSE;
-    ierr = PetscOptionsTruth("-qep_monitor_draw_all","Monitor error estimates graphically","QEPMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+    ierr = PetscOptionsBool("-qep_monitor_draw_all","Monitor error estimates graphically","QEPMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
     if (flg) {
       ierr = QEPMonitorSet(qep,QEPMonitorLGAll,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
       ierr = QEPSetTrackAll(qep,PETSC_TRUE);CHKERRQ(ierr);
     }
   /* -----------------------------------------------------------------------*/
 
-    ierr = PetscOptionsTruthGroupBegin("-qep_largest_magnitude","compute largest eigenvalues in magnitude","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupBegin("-qep_largest_magnitude","compute largest eigenvalues in magnitude","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_MAGNITUDE);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroup("-qep_smallest_magnitude","compute smallest eigenvalues in magnitude","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroup("-qep_smallest_magnitude","compute smallest eigenvalues in magnitude","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_MAGNITUDE);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroup("-qep_largest_real","compute largest real parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroup("-qep_largest_real","compute largest real parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_REAL);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroup("-qep_smallest_real","compute smallest real parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroup("-qep_smallest_real","compute smallest real parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_REAL);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroup("-qep_largest_imaginary","compute largest imaginary parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroup("-qep_largest_imaginary","compute largest imaginary parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_LARGEST_IMAGINARY);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroupEnd("-qep_smallest_imaginary","compute smallest imaginary parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBoolGroupEnd("-qep_smallest_imaginary","compute smallest imaginary parts","QEPSetWhichEigenpairs",&flg);CHKERRQ(ierr);
     if (flg) {ierr = QEPSetWhichEigenpairs(qep,QEP_SMALLEST_IMAGINARY);CHKERRQ(ierr);}
 
-    ierr = PetscOptionsTruth("-qep_left_vectors","Compute left eigenvectors also","QEPSetLeftVectorsWanted",qep->leftvecs,&val,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-qep_left_vectors","Compute left eigenvectors also","QEPSetLeftVectorsWanted",qep->leftvecs,&val,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = QEPSetLeftVectorsWanted(qep,val);CHKERRQ(ierr);
     }
@@ -154,9 +156,11 @@ PetscErrorCode QEPSetFromOptions(QEP qep)
     if (qep->ops->setfromoptions) {
       ierr = (*qep->ops->setfromoptions)(qep);CHKERRQ(ierr);
     }
+    ierr = PetscObjectProcessOptionsHandlers((PetscObject)qep);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
-  ierr = IPSetFromOptions(qep->ip); CHKERRQ(ierr);
+  ierr = IPSetFromOptions(qep->ip);CHKERRQ(ierr);
+  ierr = PetscRandomSetFromOptions(qep->rand);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -185,7 +189,7 @@ PetscErrorCode QEPSetFromOptions(QEP qep)
 PetscErrorCode QEPGetTolerances(QEP qep,PetscReal *tol,PetscInt *maxits)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   if (tol)    *tol    = qep->tol;
   if (maxits) *maxits = qep->max_it;
   PetscFunctionReturn(0);
@@ -197,7 +201,7 @@ PetscErrorCode QEPGetTolerances(QEP qep,PetscReal *tol,PetscInt *maxits)
    QEPSetTolerances - Sets the tolerance and maximum iteration count used
    by the QEP convergence tests. 
 
-   Collective on QEP
+   Logically Collective on QEP
 
    Input Parameters:
 +  qep - the quadratic eigensolver context
@@ -221,12 +225,14 @@ PetscErrorCode QEPGetTolerances(QEP qep,PetscReal *tol,PetscInt *maxits)
 PetscErrorCode QEPSetTolerances(QEP qep,PetscReal tol,PetscInt maxits)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveReal(qep,tol,2);
+  PetscValidLogicalCollectiveInt(qep,maxits,3);
   if (tol != PETSC_IGNORE) {
     if (tol == PETSC_DEFAULT) {
-      qep->tol = 1e-7;
+      qep->tol = PETSC_DEFAULT;
     } else {
-      if (tol < 0.0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
+      if (tol < 0.0) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
       qep->tol = tol;
     }
   }
@@ -235,7 +241,7 @@ PetscErrorCode QEPSetTolerances(QEP qep,PetscReal tol,PetscInt maxits)
       qep->max_it = 0;
       qep->setupcalled = 0;
     } else {
-      if (maxits < 0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
+      if (maxits < 0) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
       qep->max_it = maxits;
     }
   }
@@ -268,7 +274,7 @@ PetscErrorCode QEPSetTolerances(QEP qep,PetscReal tol,PetscInt maxits)
 PetscErrorCode QEPGetDimensions(QEP qep,PetscInt *nev,PetscInt *ncv,PetscInt *mpd)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   if (nev) *nev = qep->nev;
   if (ncv) *ncv = qep->ncv;
   if (mpd) *mpd = qep->mpd;
@@ -281,7 +287,7 @@ PetscErrorCode QEPGetDimensions(QEP qep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
    QEPSetDimensions - Sets the number of eigenvalues to compute
    and the dimension of the subspace.
 
-   Collective on QEP
+   Logically Collective on QEP
 
    Input Parameters:
 +  qep - the quadratic eigensolver context
@@ -301,9 +307,9 @@ PetscErrorCode QEPGetDimensions(QEP qep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
    dependent on the solution method.
 
    The parameters ncv and mpd are intimately related, so that the user is advised
-   to set one of them at most. Normal usage is the following
-+  - In cases where nev is small, the user sets ncv (a reasonable default is 2*nev).
--  - In cases where nev is large, the user sets mpd.
+   to set one of them at most. Normal usage is that
+   (a) in cases where nev is small, the user sets ncv (a reasonable default is 2*nev); and
+   (b) in cases where nev is large, the user sets mpd.
 
    The value of ncv should always be between nev and (nev+mpd), typically
    ncv=nev+mpd. If nev is not too large, mpd=nev is a reasonable choice, otherwise
@@ -316,27 +322,29 @@ PetscErrorCode QEPGetDimensions(QEP qep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
 PetscErrorCode QEPSetDimensions(QEP qep,PetscInt nev,PetscInt ncv,PetscInt mpd)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
-
-  if( nev != PETSC_IGNORE ) {
-    if (nev<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nev. Must be > 0");
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(qep,nev,2);
+  PetscValidLogicalCollectiveInt(qep,ncv,3);
+  PetscValidLogicalCollectiveInt(qep,mpd,4);
+  if (nev != PETSC_IGNORE) {
+    if (nev<1) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nev. Must be > 0");
     qep->nev = nev;
     qep->setupcalled = 0;
   }
-  if( ncv != PETSC_IGNORE ) {
+  if (ncv != PETSC_IGNORE) {
     if (ncv == PETSC_DECIDE || ncv == PETSC_DEFAULT) {
       qep->ncv = 0;
     } else {
-      if (ncv<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
+      if (ncv<1) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
       qep->ncv = ncv;
     }
     qep->setupcalled = 0;
   }
-  if( mpd != PETSC_IGNORE ) {
+  if (mpd != PETSC_IGNORE) {
     if (mpd == PETSC_DECIDE || mpd == PETSC_DEFAULT) {
       qep->mpd = 0;
     } else {
-      if (mpd<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
+      if (mpd<1) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
       qep->mpd = mpd;
     }
   }
@@ -349,7 +357,7 @@ PetscErrorCode QEPSetDimensions(QEP qep,PetscInt nev,PetscInt ncv,PetscInt mpd)
     QEPSetWhichEigenpairs - Specifies which portion of the spectrum is 
     to be sought.
 
-    Collective on QEP
+    Logically Collective on QEP
 
     Input Parameters:
 +   qep   - eigensolver context obtained from QEPCreate()
@@ -386,7 +394,8 @@ PetscErrorCode QEPSetDimensions(QEP qep,PetscInt nev,PetscInt ncv,PetscInt mpd)
 PetscErrorCode QEPSetWhichEigenpairs(QEP qep,QEPWhich which)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(qep,which,2);
   if (which!=PETSC_IGNORE) {
     if (which==PETSC_DECIDE || which==PETSC_DEFAULT) qep->which = (QEPWhich)0;
     else switch (which) {
@@ -402,7 +411,7 @@ PetscErrorCode QEPSetWhichEigenpairs(QEP qep,QEPWhich which)
         }
         break;
       default:
-        SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' value"); 
+        SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' value"); 
     }
   }
   PetscFunctionReturn(0);
@@ -432,7 +441,7 @@ PetscErrorCode QEPSetWhichEigenpairs(QEP qep,QEPWhich which)
 PetscErrorCode QEPGetWhichEigenpairs(QEP qep,QEPWhich *which) 
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidPointer(which,2);
   *which = qep->which;
   PetscFunctionReturn(0);
@@ -443,7 +452,7 @@ PetscErrorCode QEPGetWhichEigenpairs(QEP qep,QEPWhich *which)
 /*@
     QEPSetLeftVectorsWanted - Specifies which eigenvectors are required.
 
-    Collective on QEP
+    Logically Collective on QEP
 
     Input Parameters:
 +   qep      - the quadratic eigensolver context
@@ -459,12 +468,13 @@ PetscErrorCode QEPGetWhichEigenpairs(QEP qep,QEPWhich *which)
 
     Level: intermediate
 
-.seealso: QEPGetLeftVectorsWanted(), QEPGetEigenvectorLeft()
+.seealso: QEPGetLeftVectorsWanted()
 @*/
-PetscErrorCode QEPSetLeftVectorsWanted(QEP qep,PetscTruth leftvecs)
+PetscErrorCode QEPSetLeftVectorsWanted(QEP qep,PetscBool leftvecs)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveBool(qep,leftvecs,2);
   if (qep->leftvecs != leftvecs) {
     qep->leftvecs = leftvecs;
     qep->setupcalled = 0;
@@ -490,10 +500,10 @@ PetscErrorCode QEPSetLeftVectorsWanted(QEP qep,PetscTruth leftvecs)
 
 .seealso: QEPSetLeftVectorsWanted()
 @*/
-PetscErrorCode QEPGetLeftVectorsWanted(QEP qep,PetscTruth *leftvecs) 
+PetscErrorCode QEPGetLeftVectorsWanted(QEP qep,PetscBool *leftvecs) 
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidPointer(leftvecs,2);
   *leftvecs = qep->leftvecs;
   PetscFunctionReturn(0);
@@ -523,8 +533,9 @@ PetscErrorCode QEPGetLeftVectorsWanted(QEP qep,PetscTruth *leftvecs)
 PetscErrorCode QEPGetScaleFactor(QEP qep,PetscReal *alpha)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
-  if (alpha) *alpha = qep->sfactor;
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidPointer(alpha,2);
+  *alpha = qep->sfactor;
   PetscFunctionReturn(0);
 }
 
@@ -534,7 +545,7 @@ PetscErrorCode QEPGetScaleFactor(QEP qep,PetscReal *alpha)
    QEPSetScaleFactor - Sets the scaling factor to be used for scaling the
    quadratic problem before attempting to solve.
 
-   Collective on QEP
+   Logically Collective on QEP
 
    Input Parameters:
 +  qep   - the quadratic eigensolver context
@@ -556,12 +567,13 @@ PetscErrorCode QEPGetScaleFactor(QEP qep,PetscReal *alpha)
 PetscErrorCode QEPSetScaleFactor(QEP qep,PetscReal alpha)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveReal(qep,alpha,2);
   if (alpha != PETSC_IGNORE) {
     if (alpha == PETSC_DEFAULT || alpha == PETSC_DECIDE) {
       qep->sfactor = 0.0;
     } else {
-      if (alpha < 0.0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of alpha. Must be > 0");
+      if (alpha < 0.0) SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of alpha. Must be > 0");
       qep->sfactor = alpha;
     }
   }
@@ -573,7 +585,7 @@ PetscErrorCode QEPSetScaleFactor(QEP qep,PetscReal alpha)
 /*@
    QEPSetProblemType - Specifies the type of the quadratic eigenvalue problem.
 
-   Collective on QEP
+   Logically Collective on QEP
 
    Input Parameters:
 +  qep      - the quadratic eigensolver context
@@ -602,9 +614,10 @@ PetscErrorCode QEPSetScaleFactor(QEP qep,PetscReal alpha)
 PetscErrorCode QEPSetProblemType(QEP qep,QEPProblemType type)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(qep,type,2);
   if (type!=QEP_GENERAL && type!=QEP_HERMITIAN && type!=QEP_GYROSCOPIC)
-    SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown eigenvalue problem type");
+    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONG,"Unknown eigenvalue problem type");
   qep->problem_type = type;
   PetscFunctionReturn(0);
 }
@@ -629,7 +642,7 @@ PetscErrorCode QEPSetProblemType(QEP qep,QEPProblemType type)
 PetscErrorCode QEPGetProblemType(QEP qep,QEPProblemType *type)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidPointer(type,2);
   *type = qep->problem_type;
   PetscFunctionReturn(0);
@@ -641,7 +654,7 @@ PetscErrorCode QEPGetProblemType(QEP qep,QEPProblemType *type)
     QEPSetConvergenceTest - Sets a function to compute the error estimate used in 
     the convergence test.
 
-    Collective on QEP
+    Logically Collective on QEP
 
     Input Parameters:
 +   qep  - eigensolver context obtained from QEPCreate()
@@ -666,9 +679,10 @@ $   func(QEP qep,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal* erre
 
 .seealso: QEPSetTolerances()
 @*/
-EXTERN PetscErrorCode QEPSetConvergenceTest(QEP qep,PetscErrorCode (*func)(QEP,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*),void* ctx)
+extern PetscErrorCode QEPSetConvergenceTest(QEP qep,PetscErrorCode (*func)(QEP,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*),void* ctx)
 {
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   qep->conv_func = func;
   qep->conv_ctx = ctx;
   PetscFunctionReturn(0);
@@ -677,31 +691,32 @@ EXTERN PetscErrorCode QEPSetConvergenceTest(QEP qep,PetscErrorCode (*func)(QEP,P
 #undef __FUNCT__  
 #define __FUNCT__ "QEPSetTrackAll"
 /*@
-    QEPSetTrackAll - Specifies if the solver must compute the residual of all
-    approximate eigenpairs or not.
+   QEPSetTrackAll - Specifies if the solver must compute the residual of all
+   approximate eigenpairs or not.
 
-    Collective on QEP
+   Logically Collective on QEP
 
-    Input Parameters:
-+   qep      - the eigensolver context
--   trackall - whether compute all residuals or not
+   Input Parameters:
++  qep      - the eigensolver context
+-  trackall - whether compute all residuals or not
 
-    Notes:
-    If the user sets trackall=PETSC_TRUE then the solver explicitly computes
-    the residual for each eigenpair approximation. Computing the residual is
-    usually an expensive operation and solvers commonly compute the associated
-    residual to the first unconverged eigenpair.
-    The options '-qep_monitor_all' and '-qep_monitor_draw_all' automatically
-    activates this option.
+   Notes:
+   If the user sets trackall=PETSC_TRUE then the solver explicitly computes
+   the residual for each eigenpair approximation. Computing the residual is
+   usually an expensive operation and solvers commonly compute the associated
+   residual to the first unconverged eigenpair.
+   The options '-qep_monitor_all' and '-qep_monitor_draw_all' automatically
+   activates this option.
 
-    Level: intermediate
+   Level: intermediate
 
-.seealso: EPSGetTrackAll()
+.seealso: QEPGetTrackAll()
 @*/
-PetscErrorCode QEPSetTrackAll(QEP qep,PetscTruth trackall)
+PetscErrorCode QEPSetTrackAll(QEP qep,PetscBool trackall)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveBool(qep,trackall,2);
   qep->trackall = trackall;
   PetscFunctionReturn(0);
 }
@@ -709,25 +724,25 @@ PetscErrorCode QEPSetTrackAll(QEP qep,PetscTruth trackall)
 #undef __FUNCT__  
 #define __FUNCT__ "QEPGetTrackAll"
 /*@
-    QEPGetTrackAll - Returns the flag indicating whether all residual norms must
-    be computed or not.
+   QEPGetTrackAll - Returns the flag indicating whether all residual norms must
+   be computed or not.
 
-    Not Collective
+   Not Collective
 
-    Input Parameter:
-.   qep - the eigensolver context
+   Input Parameter:
+.  qep - the eigensolver context
 
-    Output Parameter:
-.   trackall - the returned flag
+   Output Parameter:
+.  trackall - the returned flag
 
-    Level: intermediate
+   Level: intermediate
 
-.seealso: EPSSetTrackAll()
+.seealso: QEPSetTrackAll()
 @*/
-PetscErrorCode QEPGetTrackAll(QEP qep,PetscTruth *trackall) 
+PetscErrorCode QEPGetTrackAll(QEP qep,PetscBool *trackall) 
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidPointer(trackall,2);
   *trackall = qep->trackall;
   PetscFunctionReturn(0);
@@ -739,7 +754,7 @@ PetscErrorCode QEPGetTrackAll(QEP qep,PetscTruth *trackall)
    QEPSetOptionsPrefix - Sets the prefix used for searching for all 
    QEP options in the database.
 
-   Collective on QEP
+   Logically Collective on QEP
 
    Input Parameters:
 +  qep - the quadratic eigensolver context
@@ -764,11 +779,12 @@ PetscErrorCode QEPGetTrackAll(QEP qep,PetscTruth *trackall)
 PetscErrorCode QEPSetOptionsPrefix(QEP qep,const char *prefix)
 {
   PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
-  ierr = PetscObjectSetOptionsPrefix((PetscObject)qep,prefix);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
   ierr = IPSetOptionsPrefix(qep->ip,prefix);CHKERRQ(ierr);
-  ierr = IPAppendOptionsPrefix(qep->ip,"qep_");CHKERRQ(ierr);
+  ierr = PetscObjectSetOptionsPrefix((PetscObject)qep,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);  
 }
  
@@ -778,7 +794,7 @@ PetscErrorCode QEPSetOptionsPrefix(QEP qep,const char *prefix)
    QEPAppendOptionsPrefix - Appends to the prefix used for searching for all 
    QEP options in the database.
 
-   Collective on QEP
+   Logically Collective on QEP
 
    Input Parameters:
 +  qep - the quadratic eigensolver context
@@ -795,11 +811,20 @@ PetscErrorCode QEPSetOptionsPrefix(QEP qep,const char *prefix)
 PetscErrorCode QEPAppendOptionsPrefix(QEP qep,const char *prefix)
 {
   PetscErrorCode ierr;
+  PetscBool      flg;
+  EPS            eps;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
-  ierr = PetscObjectAppendOptionsPrefix((PetscObject)qep, prefix);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
   ierr = IPSetOptionsPrefix(qep->ip,prefix);CHKERRQ(ierr);
-  ierr = IPAppendOptionsPrefix(qep->ip,"qep_");CHKERRQ(ierr);
+  ierr = PetscObjectAppendOptionsPrefix((PetscObject)qep,prefix);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)qep,QEPLINEAR,&flg);CHKERRQ(ierr);
+  if (flg) {
+    ierr = QEPLinearGetEPS(qep,&eps);CHKERRQ(ierr);
+    ierr = EPSSetOptionsPrefix(eps,((PetscObject)qep)->prefix);CHKERRQ(ierr);
+    ierr = EPSAppendOptionsPrefix(eps,"qep_");CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -827,8 +852,9 @@ PetscErrorCode QEPAppendOptionsPrefix(QEP qep,const char *prefix)
 PetscErrorCode QEPGetOptionsPrefix(QEP qep,const char *prefix[])
 {
   PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidPointer(prefix,2);
   ierr = PetscObjectGetOptionsPrefix((PetscObject)qep,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
