@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/qepimpl.h"   /*I "slepcqep.h" I*/
+#include <private/qepimpl.h>       /*I "slepcqep.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "QEPSolve"
@@ -47,17 +47,17 @@ PetscErrorCode QEPSolve(QEP qep)
   PetscErrorCode ierr;
   PetscInt       i;
   PetscReal      re,im;
-  PetscTruth     flg;
+  PetscBool      flg;
   PetscViewer    viewer;
   PetscDraw      draw;
   PetscDrawSP    drawsp;
   char           filename[PETSC_MAX_PATH_LEN];
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
 
   flg = PETSC_FALSE;
-  ierr = PetscOptionsGetTruth(((PetscObject)qep)->prefix,"-qep_view_binary",&flg,PETSC_NULL);CHKERRQ(ierr); 
+  ierr = PetscOptionsGetBool(((PetscObject)qep)->prefix,"-qep_view_binary",&flg,PETSC_NULL);CHKERRQ(ierr); 
   if (flg) {
     ierr = MatView(qep->M,PETSC_VIEWER_BINARY_(((PetscObject)qep)->comm));CHKERRQ(ierr);
     ierr = MatView(qep->C,PETSC_VIEWER_BINARY_(((PetscObject)qep)->comm));CHKERRQ(ierr);
@@ -67,24 +67,22 @@ PetscErrorCode QEPSolve(QEP qep)
   /* reset the convergence flag from the previous solves */
   qep->reason = QEP_CONVERGED_ITERATING;
 
+  if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
   if (!qep->setupcalled){ ierr = QEPSetUp(qep);CHKERRQ(ierr); }
-  ierr = IPResetOperationCounters(qep->ip);CHKERRQ(ierr);
   qep->nconv = 0;
   qep->its = 0;
-  qep->matvecs = 0;
-  qep->linits = 0;
   for (i=0;i<qep->ncv;i++) qep->eigr[i]=qep->eigi[i]=qep->errest[i]=0.0;
-  QEPMonitor(qep,qep->its,qep->nconv,qep->eigr,qep->eigi,qep->errest,qep->ncv);
+  ierr = QEPMonitor(qep,qep->its,qep->nconv,qep->eigr,qep->eigi,qep->errest,qep->ncv);CHKERRQ(ierr);
 
   ierr = PetscLogEventBegin(QEP_Solve,qep,0,0,0);CHKERRQ(ierr);
   ierr = (*qep->ops->solve)(qep);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(QEP_Solve,qep,0,0,0);CHKERRQ(ierr);
 
   if (!qep->reason) {
-    SETERRQ(1,"Internal error, solver returned without setting converged reason");
+    SETERRQ(((PetscObject)qep)->comm,1,"Internal error, solver returned without setting converged reason");
   }
 
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
   /* reorder conjugate eigenvalues (positive imaginary first) */
   for (i=0;i<qep->nconv-1;i++) {
     if (qep->eigi[i] != 0) {
@@ -109,11 +107,11 @@ PetscErrorCode QEPSolve(QEP qep)
   if (flg && !PetscPreLoadingOn) {
     ierr = PetscViewerASCIIOpen(((PetscObject)qep)->comm,filename,&viewer);CHKERRQ(ierr);
     ierr = QEPView(qep,viewer);CHKERRQ(ierr); 
-    ierr = PetscViewerDestroy(viewer);CHKERRQ(ierr);
+    ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
 
   flg = PETSC_FALSE;
-  ierr = PetscOptionsGetTruth(((PetscObject)qep)->prefix,"-qep_plot_eigs",&flg,PETSC_NULL);CHKERRQ(ierr); 
+  ierr = PetscOptionsGetBool(((PetscObject)qep)->prefix,"-qep_plot_eigs",&flg,PETSC_NULL);CHKERRQ(ierr); 
   if (flg) { 
     ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed Eigenvalues",
                              PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
@@ -130,13 +128,12 @@ PetscErrorCode QEPSolve(QEP qep)
       ierr = PetscDrawSPAddPoint(drawsp,&re,&im);CHKERRQ(ierr);
     }
     ierr = PetscDrawSPDraw(drawsp);CHKERRQ(ierr);
-    ierr = PetscDrawSPDestroy(drawsp);CHKERRQ(ierr);
-    ierr = PetscViewerDestroy(viewer);CHKERRQ(ierr);
+    ierr = PetscDrawSPDestroy(&drawsp);CHKERRQ(ierr);
+    ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
 
   /* Remove the initial subspace */
   qep->nini = 0;
-
   PetscFunctionReturn(0);
 }
 
@@ -169,7 +166,7 @@ PetscErrorCode QEPSolve(QEP qep)
 PetscErrorCode QEPGetIterationNumber(QEP qep,PetscInt *its)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidIntPointer(its,2);
   *its = qep->its;
   PetscFunctionReturn(0);
@@ -198,13 +195,12 @@ PetscErrorCode QEPGetIterationNumber(QEP qep,PetscInt *its)
 PetscErrorCode QEPGetConverged(QEP qep,PetscInt *nconv)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   PetscValidIntPointer(nconv,2);
   *nconv = qep->nconv;
   PetscFunctionReturn(0);
 }
 
-
 #undef __FUNCT__  
 #define __FUNCT__ "QEPGetConvergedReason"
 /*@C
@@ -234,8 +230,8 @@ PetscErrorCode QEPGetConverged(QEP qep,PetscInt *nconv)
 PetscErrorCode QEPGetConvergedReason(QEP qep,QEPConvergedReason *reason)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
-  PetscValidIntPointer(reason,2);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidPointer(reason,2);
   *reason = qep->reason;
   PetscFunctionReturn(0);
 }
@@ -272,25 +268,27 @@ PetscErrorCode QEPGetConvergedReason(QEP qep,QEPConvergedReason *reason)
 
 .seealso: QEPSolve(), QEPGetConverged(), QEPSetWhichEigenpairs()
 @*/
-PetscErrorCode QEPGetEigenpair(QEP qep, PetscInt i, PetscScalar *eigr, PetscScalar *eigi, Vec Vr, Vec Vi)
+PetscErrorCode QEPGetEigenpair(QEP qep,PetscInt i,PetscScalar *eigr,PetscScalar *eigi,Vec Vr,Vec Vi)
 {
   PetscInt       k;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  if (Vr) { PetscValidHeaderSpecific(Vr,VEC_CLASSID,6); PetscCheckSameComm(qep,1,Vr,6); }
+  if (Vi) { PetscValidHeaderSpecific(Vi,VEC_CLASSID,7); PetscCheckSameComm(qep,1,Vi,7); }
   if (!qep->eigr || !qep->eigi || !qep->V) { 
-    SETERRQ(PETSC_ERR_ARG_WRONGSTATE, "QEPSolve must be called first"); 
+    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONGSTATE,"QEPSolve must be called first"); 
   }
   if (i<0 || i>=qep->nconv) { 
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE, "Argument 2 out of range"); 
+    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range"); 
   }
 
   if (!qep->perm) k = i;
   else k = qep->perm[i];
 
   /* eigenvalue */
-#ifdef PETSC_USE_COMPLEX
+#if defined(PETSC_USE_COMPLEX)
   if (eigr) *eigr = qep->eigr[k];
   if (eigi) *eigi = 0;
 #else
@@ -299,7 +297,7 @@ PetscErrorCode QEPGetEigenpair(QEP qep, PetscInt i, PetscScalar *eigr, PetscScal
 #endif
   
   /* eigenvector */
-#ifdef PETSC_USE_COMPLEX
+#if defined(PETSC_USE_COMPLEX)
   if (Vr) { ierr = VecCopy(qep->V[k],Vr);CHKERRQ(ierr); }
   if (Vi) { ierr = VecSet(Vi,0.0);CHKERRQ(ierr); }
 #else
@@ -317,7 +315,6 @@ PetscErrorCode QEPGetEigenpair(QEP qep, PetscInt i, PetscScalar *eigr, PetscScal
     if (Vi) { ierr = VecSet(Vi,0.0);CHKERRQ(ierr); }
   }
 #endif
-  
   PetscFunctionReturn(0);
 }
 
@@ -345,15 +342,16 @@ PetscErrorCode QEPGetEigenpair(QEP qep, PetscInt i, PetscScalar *eigr, PetscScal
 
 .seealso: QEPComputeRelativeError()
 @*/
-PetscErrorCode QEPGetErrorEstimate(QEP qep, PetscInt i, PetscReal *errest)
+PetscErrorCode QEPGetErrorEstimate(QEP qep,PetscInt i,PetscReal *errest)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidPointer(errest,3);
   if (!qep->eigr || !qep->eigi) { 
-    SETERRQ(PETSC_ERR_ARG_WRONGSTATE, "QEPSolve must be called first"); 
+    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_WRONGSTATE,"QEPSolve must be called first"); 
   }
   if (i<0 || i>=qep->nconv) { 
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE, "Argument 2 out of range"); 
+    SETERRQ(((PetscObject)qep)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range"); 
   }
   if (qep->perm) i = qep->perm[i];  
   if (errest) *errest = qep->errest[i];
@@ -366,12 +364,12 @@ PetscErrorCode QEPGetErrorEstimate(QEP qep, PetscInt i, PetscReal *errest)
    QEPComputeResidualNorm_Private - Computes the norm of the residual vector 
    associated with an eigenpair.
 */
-PetscErrorCode QEPComputeResidualNorm_Private(QEP qep, PetscScalar kr, PetscScalar ki, Vec xr, Vec xi, PetscReal *norm)
+PetscErrorCode QEPComputeResidualNorm_Private(QEP qep,PetscScalar kr,PetscScalar ki,Vec xr,Vec xi,PetscReal *norm)
 {
   PetscErrorCode ierr;
   Vec            u,w;
   Mat            M=qep->M,C=qep->C,K=qep->K;
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
   Vec            v,y,z;
   PetscReal      ni,nr;
   PetscScalar    a1,a2;
@@ -381,7 +379,7 @@ PetscErrorCode QEPComputeResidualNorm_Private(QEP qep, PetscScalar kr, PetscScal
   ierr = VecDuplicate(qep->V[0],&u);CHKERRQ(ierr);
   ierr = VecDuplicate(u,&w);CHKERRQ(ierr);
   
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
   if (ki == 0 || 
     PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
 #endif
@@ -393,7 +391,7 @@ PetscErrorCode QEPComputeResidualNorm_Private(QEP qep, PetscScalar kr, PetscScal
       ierr = VecAXPY(u,kr*kr,w);CHKERRQ(ierr);            /* u=l^2*M*x+l*C*x+K*x */
     }
     ierr = VecNorm(u,NORM_2,norm);CHKERRQ(ierr);  
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
   } else {
     ierr = VecDuplicate(u,&v);CHKERRQ(ierr);
     ierr = VecDuplicate(u,&y);CHKERRQ(ierr);
@@ -421,14 +419,14 @@ PetscErrorCode QEPComputeResidualNorm_Private(QEP qep, PetscScalar kr, PetscScal
     }
     ierr = VecNorm(u,NORM_2,&ni);CHKERRQ(ierr);
     *norm = SlepcAbsEigenvalue(nr,ni);
-    ierr = VecDestroy(v);CHKERRQ(ierr);
-    ierr = VecDestroy(y);CHKERRQ(ierr);
-    ierr = VecDestroy(z);CHKERRQ(ierr);
+    ierr = VecDestroy(&v);CHKERRQ(ierr);
+    ierr = VecDestroy(&y);CHKERRQ(ierr);
+    ierr = VecDestroy(&z);CHKERRQ(ierr);
   }
 #endif
 
-  ierr = VecDestroy(w);CHKERRQ(ierr);
-  ierr = VecDestroy(u);CHKERRQ(ierr);
+  ierr = VecDestroy(&w);CHKERRQ(ierr);
+  ierr = VecDestroy(&u);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -458,21 +456,22 @@ PetscErrorCode QEPComputeResidualNorm_Private(QEP qep, PetscScalar kr, PetscScal
 
 .seealso: QEPSolve(), QEPGetConverged(), QEPSetWhichEigenpairs()
 @*/
-PetscErrorCode QEPComputeResidualNorm(QEP qep, PetscInt i, PetscReal *norm)
+PetscErrorCode QEPComputeResidualNorm(QEP qep,PetscInt i,PetscReal *norm)
 {
   PetscErrorCode ierr;
   Vec            xr,xi;
   PetscScalar    kr,ki;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(qep,i,2);
   PetscValidPointer(norm,3);
   ierr = VecDuplicate(qep->V[0],&xr);CHKERRQ(ierr);
   ierr = VecDuplicate(qep->V[0],&xi);CHKERRQ(ierr);
   ierr = QEPGetEigenpair(qep,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
   ierr = QEPComputeResidualNorm_Private(qep,kr,ki,xr,xi,norm);CHKERRQ(ierr);
-  ierr = VecDestroy(xr);CHKERRQ(ierr);
-  ierr = VecDestroy(xi);CHKERRQ(ierr);
+  ierr = VecDestroy(&xr);CHKERRQ(ierr);
+  ierr = VecDestroy(&xi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -482,18 +481,17 @@ PetscErrorCode QEPComputeResidualNorm(QEP qep, PetscInt i, PetscReal *norm)
    QEPComputeRelativeError_Private - Computes the relative error bound 
    associated with an eigenpair.
 */
-PetscErrorCode QEPComputeRelativeError_Private(QEP qep, PetscScalar kr, PetscScalar ki, Vec xr, Vec xi, PetscReal *error)
+PetscErrorCode QEPComputeRelativeError_Private(QEP qep,PetscScalar kr,PetscScalar ki,Vec xr,Vec xi,PetscReal *error)
 {
   PetscErrorCode ierr;
-  PetscReal      norm, er;
-#ifndef PETSC_USE_COMPLEX
+  PetscReal      norm,er;
+#if !defined(PETSC_USE_COMPLEX)
   PetscReal      ei;
 #endif
   
   PetscFunctionBegin;
   ierr = QEPComputeResidualNorm_Private(qep,kr,ki,xr,xi,&norm);CHKERRQ(ierr);
-
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
   if (ki == 0 || 
     PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
 #endif
@@ -503,7 +501,7 @@ PetscErrorCode QEPComputeRelativeError_Private(QEP qep, PetscScalar kr, PetscSca
     } else {
       *error = norm/er;
     }
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
   } else {
     ierr = VecNorm(xr,NORM_2,&er);CHKERRQ(ierr);  
     ierr = VecNorm(xi,NORM_2,&ei);CHKERRQ(ierr);  
@@ -514,7 +512,6 @@ PetscErrorCode QEPComputeRelativeError_Private(QEP qep, PetscScalar kr, PetscSca
     }
   }
 #endif    
-  
   PetscFunctionReturn(0);
 }
 
@@ -539,21 +536,22 @@ PetscErrorCode QEPComputeRelativeError_Private(QEP qep, PetscScalar kr, PetscSca
 
 .seealso: QEPSolve(), QEPComputeResidualNorm(), QEPGetErrorEstimate()
 @*/
-PetscErrorCode QEPComputeRelativeError(QEP qep, PetscInt i, PetscReal *error)
+PetscErrorCode QEPComputeRelativeError(QEP qep,PetscInt i,PetscReal *error)
 {
   PetscErrorCode ierr;
   Vec            xr,xi;  
   PetscScalar    kr,ki;  
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);  
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);  
+  PetscValidLogicalCollectiveInt(qep,i,2);
   PetscValidPointer(error,3);
   ierr = VecDuplicate(qep->V[0],&xr);CHKERRQ(ierr);
   ierr = VecDuplicate(qep->V[0],&xi);CHKERRQ(ierr);
   ierr = QEPGetEigenpair(qep,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
   ierr = QEPComputeRelativeError_Private(qep,kr,ki,xr,xi,error);CHKERRQ(ierr);  
-  ierr = VecDestroy(xr);CHKERRQ(ierr);
-  ierr = VecDestroy(xi);CHKERRQ(ierr);
+  ierr = VecDestroy(&xr);CHKERRQ(ierr);
+  ierr = VecDestroy(&xi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -590,13 +588,17 @@ PetscErrorCode QEPSortEigenvalues(QEP qep,PetscInt n,PetscScalar *eigr,PetscScal
   PetscInt       i,j,result,tmp;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);  
+  PetscValidScalarPointer(eigr,3);
+  PetscValidScalarPointer(eigi,4);
+  PetscValidIntPointer(perm,5);
   for (i=0; i<n; i++) { perm[i] = i; }
   /* insertion sort */
   for (i=n-1; i>=0; i--) {
     re = eigr[perm[i]];
     im = eigi[perm[i]];
     j = i + 1;
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
     if (im != 0) {
       /* complex eigenvalue */
       i--;
@@ -606,14 +608,14 @@ PetscErrorCode QEPSortEigenvalues(QEP qep,PetscInt n,PetscScalar *eigr,PetscScal
     while (j<n) {
       ierr = QEPCompareEigenvalues(qep,re,im,eigr[perm[j]],eigi[perm[j]],&result);CHKERRQ(ierr);
       if (result >= 0) break;
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
       /* keep together every complex conjugated eigenpair */
       if (im == 0) { 
         if (eigi[perm[j]] == 0) {
 #endif
           tmp = perm[j-1]; perm[j-1] = perm[j]; perm[j] = tmp;
           j++;
-#ifndef PETSC_USE_COMPLEX
+#if !defined(PETSC_USE_COMPLEX)
         } else {
           tmp = perm[j-1]; perm[j-1] = perm[j]; perm[j] = perm[j+1]; perm[j+1] = tmp;
           j+=2;
@@ -666,6 +668,9 @@ PetscErrorCode QEPSortEigenvaluesReal(QEP qep,PetscInt n,PetscReal *eig,PetscInt
   PetscInt       i,j,result,tmp;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);  
+  PetscValidScalarPointer(eig,3);
+  PetscValidIntPointer(perm,4);
   for (i=0; i<n; i++) { perm[i] = i; }
   /* insertion sort */
   for (i=1; i<n; i++) {
@@ -714,9 +719,11 @@ PetscErrorCode QEPSortEigenvaluesReal(QEP qep,PetscInt n,PetscReal *eig,PetscInt
 @*/
 PetscErrorCode QEPCompareEigenvalues(QEP qep,PetscScalar ar,PetscScalar ai,PetscScalar br,PetscScalar bi,PetscInt *result)
 {
-  PetscReal      a,b;
+  PetscReal a,b;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);  
+  PetscValidIntPointer(result,6);
   switch(qep->which) {
     case QEP_LARGEST_MAGNITUDE:
     case QEP_SMALLEST_MAGNITUDE:
@@ -738,7 +745,7 @@ PetscErrorCode QEPCompareEigenvalues(QEP qep,PetscScalar ar,PetscScalar ai,Petsc
       b = PetscAbsReal(bi);
 #endif
       break;
-    default: SETERRQ(1,"Wrong value of which");
+    default: SETERRQ(((PetscObject)qep)->comm,1,"Wrong value of which");
   }
   switch(qep->which) {
     case QEP_LARGEST_MAGNITUDE:
@@ -784,9 +791,10 @@ PetscErrorCode QEPGetOperationCounters(QEP qep,PetscInt* matvecs,PetscInt* dots,
   PetscErrorCode ierr;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(qep,QEP_COOKIE,1);
+  PetscValidHeaderSpecific(qep,QEP_CLASSID,1);
   if (matvecs) *matvecs = qep->matvecs; 
   if (dots) {
+    if (!qep->ip) { ierr = QEPGetIP(qep,&qep->ip);CHKERRQ(ierr); }
     ierr = IPGetOperationCounters(qep->ip,dots);CHKERRQ(ierr);
   }
   if (lits) *lits = qep->linits; 
