@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/stimpl.h"          /*I "slepcst.h" I*/
+#include <private/stimpl.h>          /*I "slepcst.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "STApply_Sinvert"
@@ -65,12 +65,13 @@ PetscErrorCode STApplyTranspose_Sinvert(ST st,Vec x,Vec y)
 #define __FUNCT__ "STBackTransform_Sinvert"
 PetscErrorCode STBackTransform_Sinvert(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
-  PetscInt j;
-#ifndef PETSC_USE_COMPLEX
+  PetscInt    j;
+#if !defined(PETSC_USE_COMPLEX)
   PetscScalar t;
+#endif
+
   PetscFunctionBegin;
-  PetscValidPointer(eigr,2);
-  PetscValidPointer(eigi,3);
+#if !defined(PETSC_USE_COMPLEX)
   for (j=0;j<n;j++) {
     if (eigi[j] == 0) eigr[j] = 1.0 / eigr[j] + st->sigma;
     else {
@@ -80,8 +81,6 @@ PetscErrorCode STBackTransform_Sinvert(ST st,PetscInt n,PetscScalar *eigr,PetscS
     }
   }
 #else
-  PetscFunctionBegin;
-  PetscValidPointer(eigr,2);
   for (j=0;j<n;j++) {
     eigr[j] = 1.0 / eigr[j] + st->sigma;
   }
@@ -97,10 +96,10 @@ PetscErrorCode STPostSolve_Sinvert(ST st)
 
   PetscFunctionBegin;
   if (st->shift_matrix == ST_MATMODE_INPLACE) {
-    if( st->B ) {
+    if (st->B) {
       ierr = MatAXPY(st->A,st->sigma,st->B,st->str);CHKERRQ(ierr);
     } else {
-      ierr = MatShift(st->A,st->sigma); CHKERRQ(ierr);
+      ierr = MatShift(st->A,st->sigma);CHKERRQ(ierr);
     }
     st->setupcalled = 0;
   }
@@ -114,11 +113,12 @@ PetscErrorCode STSetUp_Sinvert(ST st)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (st->mat) { ierr = MatDestroy(st->mat);CHKERRQ(ierr); }
+  ierr = MatDestroy(&st->mat);CHKERRQ(ierr);
 
   /* if the user did not set the shift, use the target value */
   if (!st->sigma_set) st->sigma = st->defsigma;
 
+  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
   switch (st->shift_matrix) {
   case ST_MATMODE_INPLACE:
     st->mat = PETSC_NULL;
@@ -138,18 +138,14 @@ PetscErrorCode STSetUp_Sinvert(ST st)
   default:
     if (st->sigma != 0.0) {
       ierr = MatDuplicate(st->A,MAT_COPY_VALUES,&st->mat);CHKERRQ(ierr);
-      if (st->B) { 
-        ierr = MatAXPY(st->mat,-st->sigma,st->B,st->str);CHKERRQ(ierr); 
-      } else { 
-        ierr = MatShift(st->mat,-st->sigma);CHKERRQ(ierr); 
-      }
+      if (st->B) { ierr = MatAXPY(st->mat,-st->sigma,st->B,st->str);CHKERRQ(ierr); }
+      else { ierr = MatShift(st->mat,-st->sigma);CHKERRQ(ierr); }
       ierr = KSPSetOperators(st->ksp,st->mat,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     } else {
       st->mat = PETSC_NULL;
       ierr = KSPSetOperators(st->ksp,st->A,st->A,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     }
   }
-
   ierr = KSPSetUp(st->ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -162,10 +158,9 @@ PetscErrorCode STSetShift_Sinvert(ST st,PetscScalar newshift)
   MatStructure   flg;
 
   PetscFunctionBegin;
-
   /* Nothing to be done if STSetUp has not been called yet */
   if (!st->setupcalled) PetscFunctionReturn(0);
-  
+
   /* Check if the new KSP matrix has the same zero structure */
   if (st->B && st->str == DIFFERENT_NONZERO_PATTERN && (st->sigma == 0.0 || newshift == 0.0)) {
     flg = DIFFERENT_NONZERO_PATTERN;
@@ -194,20 +189,17 @@ PetscErrorCode STSetShift_Sinvert(ST st,PetscScalar newshift)
     ierr = KSPSetOperators(st->ksp,st->A,st->A,flg);CHKERRQ(ierr);
     break;
   case ST_MATMODE_SHELL:
-    ierr = KSPSetOperators(st->ksp,st->mat,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);    
+    ierr = KSPSetOperators(st->ksp,st->mat,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     break;
   default:
     if (st->mat) {
-      ierr = MatCopy(st->A,st->mat,SUBSET_NONZERO_PATTERN); CHKERRQ(ierr);
+      ierr = MatCopy(st->A,st->mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     } else {
       ierr = MatDuplicate(st->A,MAT_COPY_VALUES,&st->mat);CHKERRQ(ierr);
     }
     if (newshift != 0.0) {   
-      if (st->B) {
-        ierr = MatAXPY(st->mat,-newshift,st->B,st->str);CHKERRQ(ierr);
-      } else {
-        ierr = MatShift(st->mat,-newshift);CHKERRQ(ierr);
-      }
+      if (st->B) { ierr = MatAXPY(st->mat,-newshift,st->B,st->str);CHKERRQ(ierr); }
+      else { ierr = MatShift(st->mat,-newshift);CHKERRQ(ierr); }
     }
     ierr = KSPSetOperators(st->ksp,st->mat,st->mat,flg);CHKERRQ(ierr);    
   }
@@ -226,7 +218,7 @@ PetscErrorCode STSetFromOptions_Sinvert(ST st)
   const KSPType  ksptype;
 
   PetscFunctionBegin;
-
+  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
   ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
   ierr = KSPGetType(st->ksp,&ksptype);CHKERRQ(ierr);
   ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
@@ -241,7 +233,6 @@ PetscErrorCode STSetFromOptions_Sinvert(ST st)
       ierr = PCSetType(pc,PCREDUNDANT);CHKERRQ(ierr);
     }
   }
-
   PetscFunctionReturn(0);
 }
 
@@ -251,8 +242,6 @@ EXTERN_C_BEGIN
 PetscErrorCode STCreate_Sinvert(ST st)
 {
   PetscFunctionBegin;
-  st->data                 = 0;
-
   st->ops->apply           = STApply_Sinvert;
   st->ops->getbilinearform = STGetBilinearForm_Default;
   st->ops->applytrans      = STApplyTranspose_Sinvert;
@@ -260,11 +249,8 @@ PetscErrorCode STCreate_Sinvert(ST st)
   st->ops->backtr          = STBackTransform_Sinvert;
   st->ops->setup           = STSetUp_Sinvert;
   st->ops->setshift        = STSetShift_Sinvert;
-  st->ops->view            = STView_Default;
-  st->ops->setfromoptions = STSetFromOptions_Sinvert;
-  
-  st->checknullspace      = STCheckNullSpace_Default;
-
+  st->ops->setfromoptions  = STSetFromOptions_Sinvert;
+  st->ops->checknullspace  = STCheckNullSpace_Default;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
