@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,19 +21,19 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/stimpl.h"            /*I "slepcst.h" I*/
+#include <private/stimpl.h>            /*I "slepcst.h" I*/
 
-PetscCookie ST_COOKIE = 0;
-PetscLogEvent ST_SetUp = 0, ST_Apply = 0, ST_ApplyTranspose = 0;
-static PetscTruth STPackageInitialized = PETSC_FALSE;
+PetscClassId     ST_CLASSID = 0;
+PetscLogEvent    ST_SetUp = 0,ST_Apply = 0,ST_ApplyTranspose = 0;
+static PetscBool STPackageInitialized = PETSC_FALSE;
 
 #undef __FUNCT__  
 #define __FUNCT__ "STFinalizePackage"
 /*@C
-  STFinalizePackage - This function destroys everything in the Slepc interface to the ST package. It is
-  called from SlepcFinalize().
+   STFinalizePackage - This function destroys everything in the Slepc interface 
+   to the ST package. It is called from SlepcFinalize().
 
-  Level: developer
+   Level: developer
 
 .seealso: SlepcFinalize()
 @*/
@@ -42,54 +42,56 @@ PetscErrorCode STFinalizePackage(void)
   PetscFunctionBegin;
   STPackageInitialized = PETSC_FALSE;
   STList               = 0;
+  STRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "STInitializePackage"
 /*@C
-  STInitializePackage - This function initializes everything in the ST package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to STCreate()
-  when using static libraries.
+   STInitializePackage - This function initializes everything in the ST package. It is called
+   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to STCreate()
+   when using static libraries.
 
-  Input Parameter:
-  path - The dynamic library path, or PETSC_NULL
+   Input Parameter:
+.  path - The dynamic library path, or PETSC_NULL
 
-  Level: developer
+   Level: developer
 
 .seealso: SlepcInitialize()
 @*/
-PetscErrorCode STInitializePackage(char *path) {
-  char              logList[256];
-  char             *className;
-  PetscTruth        opt;
+PetscErrorCode STInitializePackage(const char *path)
+{
+  char           logList[256];
+  char           *className;
+  PetscBool      opt;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (STPackageInitialized) PetscFunctionReturn(0);
   STPackageInitialized = PETSC_TRUE;
   /* Register Classes */
-  ierr = PetscCookieRegister("Spectral Transform",&ST_COOKIE);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Spectral Transform",&ST_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = STRegisterAll(path);CHKERRQ(ierr);
   /* Register Events */
-  ierr = PetscLogEventRegister("STSetUp",ST_COOKIE,&ST_SetUp);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("STApply",ST_COOKIE,&ST_Apply);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("STApplyTranspose",ST_COOKIE,&ST_ApplyTranspose); CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("STSetUp",ST_CLASSID,&ST_SetUp);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("STApply",ST_CLASSID,&ST_Apply);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("STApplyTranspose",ST_CLASSID,&ST_ApplyTranspose);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL, "-log_info_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "st", &className);CHKERRQ(ierr);
+    ierr = PetscStrstr(logList,"st",&className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscInfoDeactivateClass(ST_COOKIE);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(ST_CLASSID);CHKERRQ(ierr);
     }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL, "-log_summary_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "st", &className);CHKERRQ(ierr);
+    ierr = PetscStrstr(logList,"st",&className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscLogEventDeactivateClass(ST_COOKIE);CHKERRQ(ierr);
+      ierr = PetscLogEventDeactivateClass(ST_CLASSID);CHKERRQ(ierr);
     }
   }
   ierr = PetscRegisterFinalize(STFinalizePackage);CHKERRQ(ierr);
@@ -97,8 +99,43 @@ PetscErrorCode STInitializePackage(char *path) {
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "STDestroy"
+#define __FUNCT__ "STReset"
 /*@
+   STReset - Resets the ST context and removes any allocated objects.
+
+   Collective on ST
+
+   Input Parameter:
+.  st - the spectral transformation context
+
+   Level: advanced
+
+.seealso: STDestroy()
+@*/
+PetscErrorCode STReset(ST st)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  if (st->ops->reset) { ierr = (*st->ops->reset)(st);CHKERRQ(ierr); }
+  if (st->ksp) { ierr = KSPReset(st->ksp);CHKERRQ(ierr); }
+  ierr = MatDestroy(&st->A);CHKERRQ(ierr);
+  ierr = MatDestroy(&st->B);CHKERRQ(ierr);
+  ierr = VecDestroy(&st->w);CHKERRQ(ierr);
+  ierr = VecDestroy(&st->D);CHKERRQ(ierr);
+  ierr = VecDestroy(&st->wb);CHKERRQ(ierr);
+  if (st->shift_matrix != ST_MATMODE_INPLACE) { 
+    ierr = MatDestroy(&st->mat);CHKERRQ(ierr); 
+  }
+  ierr = STResetOperationCounters(st);CHKERRQ(ierr);
+  st->setupcalled = 0;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "STDestroy"
+/*@C
    STDestroy - Destroys ST context that was created with STCreate().
 
    Collective on ST
@@ -110,28 +147,18 @@ PetscErrorCode STInitializePackage(char *path) {
 
 .seealso: STCreate(), STSetUp()
 @*/
-PetscErrorCode STDestroy(ST st)
+PetscErrorCode STDestroy(ST *st)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  if (--((PetscObject)st)->refct > 0) PetscFunctionReturn(0);
-
-  /* if memory was published with AMS then destroy it */
-  ierr = PetscObjectDepublish(st);CHKERRQ(ierr);
-
-  if (st->ops->destroy) { ierr = (*st->ops->destroy)(st);CHKERRQ(ierr); }
-  if (st->A) { ierr = MatDestroy(st->A);CHKERRQ(ierr); }
-  if (st->B) { ierr = MatDestroy(st->B);CHKERRQ(ierr); }
-  if (st->ksp) { ierr = KSPDestroy(st->ksp);CHKERRQ(ierr); } 
-  if (st->w) { ierr = VecDestroy(st->w);CHKERRQ(ierr); } 
-  if (st->D) { ierr = VecDestroy(st->D);CHKERRQ(ierr); } 
-  if (st->wb){ ierr = VecDestroy(st->wb);CHKERRQ(ierr); } 
-  if (st->shift_matrix != ST_MATMODE_INPLACE && st->mat) { 
-    ierr = MatDestroy(st->mat);CHKERRQ(ierr); 
-  }
-
+  if (!*st) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(*st,ST_CLASSID,1);
+  if (--((PetscObject)(*st))->refct > 0) { *st = 0; PetscFunctionReturn(0); }
+  ierr = STReset(*st);CHKERRQ(ierr);
+  ierr = PetscObjectDepublish(*st);CHKERRQ(ierr);
+  if ((*st)->ops->destroy) { ierr = (*(*st)->ops->destroy)(*st);CHKERRQ(ierr); }
+  ierr = KSPDestroy(&(*st)->ksp);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(st);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -157,15 +184,11 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
 {
   PetscErrorCode ierr;
   ST             st;
-  const char     *prefix;
 
   PetscFunctionBegin;
   PetscValidPointer(newst,2);
   *newst = 0;
-
-  ierr = PetscHeaderCreate(st,_p_ST,struct _STOps,ST_COOKIE,-1,"ST",comm,STDestroy,STView);CHKERRQ(ierr);
-  ierr = PetscMemzero(st->ops,sizeof(struct _STOps));CHKERRQ(ierr);
-
+  ierr = PetscHeaderCreate(st,_p_ST,struct _STOps,ST_CLASSID,-1,"ST","Spectral Transformation","ST",comm,STDestroy,STView);CHKERRQ(ierr);
   st->A                   = 0;
   st->B                   = 0;
   st->sigma               = 0.0;
@@ -176,19 +199,11 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
   st->w                   = 0;
   st->D                   = 0;
   st->wb                  = 0;
+  st->mat                 = 0;
   st->shift_matrix        = ST_MATMODE_COPY;
   st->str                 = DIFFERENT_NONZERO_PATTERN;
-  
-  ierr = KSPCreate(((PetscObject)st)->comm,&st->ksp);CHKERRQ(ierr);
-  ierr = STGetOptionsPrefix(st,&prefix);CHKERRQ(ierr);
-  ierr = KSPSetOptionsPrefix(st->ksp,prefix);CHKERRQ(ierr);
-  ierr = KSPAppendOptionsPrefix(st->ksp,"st_");CHKERRQ(ierr);
-  ierr = PetscObjectIncrementTabLevel((PetscObject)st->ksp,(PetscObject)st,1);CHKERRQ(ierr);
-  
-  *newst                  = st;
-  ierr = PetscPublishAll(st);CHKERRQ(ierr);
+  *newst = st;
   PetscFunctionReturn(0);
-
 }
 
 #undef __FUNCT__  
@@ -206,26 +221,30 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
    Notes:
    To specify a standard eigenproblem, use PETSC_NULL for B.
 
+   It must be called after STSetUp(). If it is called again after STSetUp() then
+   the ST object is reset.
+
    Level: intermediate
 
-.seealso: STGetOperators()
+.seealso: STGetOperators(), STSetUp(), STReset()
  @*/
 PetscErrorCode STSetOperators(ST st,Mat A,Mat B)
 {
   PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  PetscValidHeaderSpecific(A,MAT_COOKIE,2);
-  if (B) PetscValidHeaderSpecific(B,MAT_COOKIE,3);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidHeaderSpecific(A,MAT_CLASSID,2);
+  if (B) PetscValidHeaderSpecific(B,MAT_CLASSID,3);
   PetscCheckSameComm(st,1,A,2);
   if (B) PetscCheckSameComm(st,1,B,3);
+  if (st->setupcalled) { ierr = STReset(st);CHKERRQ(ierr); }
+  ierr = MatDestroy(&st->A);CHKERRQ(ierr);
   ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
-  if (st->A) { ierr = MatDestroy(st->A);CHKERRQ(ierr); }
   st->A = A;
+  ierr = MatDestroy(&st->B);CHKERRQ(ierr);
   if (B) { ierr = PetscObjectReference((PetscObject)B);CHKERRQ(ierr); }
-  if (st->B) { ierr = MatDestroy(st->B);CHKERRQ(ierr); }
   st->B = B;
-  st->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
@@ -250,7 +269,7 @@ PetscErrorCode STSetOperators(ST st,Mat A,Mat B)
 PetscErrorCode STGetOperators(ST st,Mat *A,Mat *B)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
   if (A) *A = st->A;
   if (B) *B = st->B;
   PetscFunctionReturn(0);
@@ -261,7 +280,7 @@ PetscErrorCode STGetOperators(ST st,Mat *A,Mat *B)
 /*@
    STSetShift - Sets the shift associated with the spectral transformation.
 
-   Collective on ST
+   Logically Collective on ST
 
    Input Parameters:
 +  st - the spectral transformation context
@@ -279,10 +298,11 @@ PetscErrorCode STSetShift(ST st,PetscScalar shift)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidLogicalCollectiveScalar(st,shift,2);
   if (st->sigma != shift) {
     if (st->ops->setshift) {
-      ierr = (*st->ops->setshift)(st,shift); CHKERRQ(ierr);
+      ierr = (*st->ops->setshift)(st,shift);CHKERRQ(ierr);
     }
   }
   st->sigma = shift;
@@ -295,7 +315,7 @@ PetscErrorCode STSetShift(ST st,PetscScalar shift)
 /*@
    STGetShift - Gets the shift associated with the spectral transformation.
 
-   Not collective
+   Not Collective
 
    Input Parameter:
 .  st - the spectral transformation context
@@ -309,8 +329,9 @@ PetscErrorCode STSetShift(ST st,PetscScalar shift)
 PetscErrorCode STGetShift(ST st,PetscScalar* shift)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  if (shift)  *shift = st->sigma;
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidScalarPointer(shift,2);
+  *shift = st->sigma;
   PetscFunctionReturn(0);
 }
 
@@ -320,7 +341,7 @@ PetscErrorCode STGetShift(ST st,PetscScalar* shift)
    STSetDefaultShift - Sets the value of the shift that should be employed if
    the user did not specify one.
 
-   Collective on ST
+   Logically Collective on ST
 
    Input Parameters:
 +  st - the spectral transformation context
@@ -332,7 +353,8 @@ PetscErrorCode STGetShift(ST st,PetscScalar* shift)
 PetscErrorCode STSetDefaultShift(ST st,PetscScalar defaultshift)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidLogicalCollectiveScalar(st,defaultshift,2);
   st->defsigma = defaultshift;
   PetscFunctionReturn(0);
 }
@@ -363,17 +385,13 @@ PetscErrorCode STSetBalanceMatrix(ST st,Vec D)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  PetscValidHeaderSpecific(D,VEC_COOKIE,2);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidHeaderSpecific(D,VEC_CLASSID,2);
   PetscCheckSameComm(st,1,D,2);
   ierr = PetscObjectReference((PetscObject)D);CHKERRQ(ierr);
-  if (st->D) {
-    ierr = VecDestroy(st->D); CHKERRQ(ierr);
-  }
+  ierr = VecDestroy(&st->D);CHKERRQ(ierr);
   st->D = D;
-  if (!st->wb) {
-    ierr = VecDuplicate(st->D,&st->wb); CHKERRQ(ierr);
-  }
+  st->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
@@ -400,7 +418,7 @@ PetscErrorCode STSetBalanceMatrix(ST st,Vec D)
 PetscErrorCode STGetBalanceMatrix(ST st,Vec *D)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
   PetscValidPointer(D,2);
   *D = st->D;
   PetscFunctionReturn(0);
@@ -412,7 +430,7 @@ PetscErrorCode STGetBalanceMatrix(ST st,Vec *D)
    STSetOptionsPrefix - Sets the prefix used for searching for all 
    ST options in the database.
 
-   Collective on ST
+   Logically Collective on ST
 
    Input Parameters:
 +  st     - the spectral transformation context
@@ -432,10 +450,11 @@ PetscErrorCode STSetOptionsPrefix(ST st,const char *prefix)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  ierr = PetscObjectSetOptionsPrefix((PetscObject)st,prefix);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
   ierr = KSPSetOptionsPrefix(st->ksp,prefix);CHKERRQ(ierr);
   ierr = KSPAppendOptionsPrefix(st->ksp,"st_");CHKERRQ(ierr);
+  ierr = PetscObjectSetOptionsPrefix((PetscObject)st,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -445,7 +464,7 @@ PetscErrorCode STSetOptionsPrefix(ST st,const char *prefix)
    STAppendOptionsPrefix - Appends to the prefix used for searching for all 
    ST options in the database.
 
-   Collective on ST
+   Logically Collective on ST
 
    Input Parameters:
 +  st     - the spectral transformation context
@@ -465,8 +484,9 @@ PetscErrorCode STAppendOptionsPrefix(ST st,const char *prefix)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)st,prefix);CHKERRQ(ierr);
+  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
   ierr = KSPSetOptionsPrefix(st->ksp,((PetscObject)st)->prefix);CHKERRQ(ierr);
   ierr = KSPAppendOptionsPrefix(st->ksp,"st_");CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -498,8 +518,9 @@ PetscErrorCode STGetOptionsPrefix(ST st,const char *prefix[])
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  ierr = PetscObjectGetOptionsPrefix((PetscObject)st, prefix);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidPointer(prefix,2);
+  ierr = PetscObjectGetOptionsPrefix((PetscObject)st,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -511,7 +532,7 @@ PetscErrorCode STGetOptionsPrefix(ST st,const char *prefix[])
    Collective on ST
 
    Input Parameters:
-+  ST - the ST context
++  st - the ST context
 -  viewer - optional visualization context
 
    Note:
@@ -534,39 +555,37 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
   PetscErrorCode    ierr;
   const STType      cstr;
   const char*       str;
-  PetscTruth        isascii,isstring;
-  PetscViewerFormat format;
+  PetscBool         isascii,isstring,flg;
+  PC                pc;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
   if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)st)->comm);
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_COOKIE,2); 
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2); 
   PetscCheckSameComm(st,1,viewer,2);
 
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_STRING,&isstring);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"ST Object:\n");CHKERRQ(ierr);
-    ierr = STGetType(st,&cstr);CHKERRQ(ierr);
-    if (cstr) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  type: %s\n",cstr);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  type: not yet set\n");CHKERRQ(ierr);
+    ierr = PetscObjectPrintClassNamePrefixType((PetscObject)st,viewer,"ST Object");CHKERRQ(ierr);
+    if (st->ops->view) {
+      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
+      ierr = (*st->ops->view)(st,viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
 #if !defined(PETSC_USE_COMPLEX)
-    ierr = PetscViewerASCIIPrintf(viewer,"  shift: %g\n",st->sigma);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  shift: %G\n",st->sigma);CHKERRQ(ierr);
 #else
-    ierr = PetscViewerASCIIPrintf(viewer,"  shift: %g+%g i\n",PetscRealPart(st->sigma),PetscImaginaryPart(st->sigma));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  shift: %G+%G i\n",PetscRealPart(st->sigma),PetscImaginaryPart(st->sigma));CHKERRQ(ierr);
 #endif
     switch (st->shift_matrix) {
     case ST_MATMODE_COPY:
       break;
     case ST_MATMODE_INPLACE:
-      ierr = PetscViewerASCIIPrintf(viewer,"Shifting the matrix and unshifting at exit\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  shifting the matrix and unshifting at exit\n");CHKERRQ(ierr);
       break;
     case ST_MATMODE_SHELL:
-      ierr = PetscViewerASCIIPrintf(viewer,"Using a shell matrix\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  using a shell matrix\n");CHKERRQ(ierr);
       break;
     }
     if (st->B && st->shift_matrix != ST_MATMODE_SHELL) { 
@@ -574,97 +593,44 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
         case SAME_NONZERO_PATTERN:      str = "same nonzero pattern";break;
         case DIFFERENT_NONZERO_PATTERN: str = "different nonzero pattern";break;
         case SUBSET_NONZERO_PATTERN:    str = "subset nonzero pattern";break;
-        default:                        SETERRQ(1,"Wrong structure flag");
+        default:                        SETERRQ(((PetscObject)st)->comm,1,"Wrong structure flag");
       }
-      ierr = PetscViewerASCIIPrintf(viewer,"Matrices A and B have %s\n",str);CHKERRQ(ierr);
-    }
-    if (st->ops->view) {
-      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-      ierr = (*st->ops->view)(st,viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  matrices A and B have %s\n",str);CHKERRQ(ierr);
     }
   } else if (isstring) {
     ierr = STGetType(st,&cstr);CHKERRQ(ierr);
     ierr = PetscViewerStringSPrintf(viewer," %-7.7s",cstr);CHKERRQ(ierr);
     if (st->ops->view) {ierr = (*st->ops->view)(st,viewer);CHKERRQ(ierr);}
   } else {
-    SETERRQ1(1,"Viewer type %s not supported by ST",((PetscObject)viewer)->type_name);
+    SETERRQ1(((PetscObject)st)->comm,1,"Viewer type %s not supported by ST",((PetscObject)viewer)->type_name);
   }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "STView_Default"
-PetscErrorCode STView_Default(ST st,PetscViewer viewer) 
-{
-  PetscErrorCode ierr;
-  PetscTruth     isascii,isstring;
-
-  PetscFunctionBegin;
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_STRING,&isstring);CHKERRQ(ierr);
-  if (isascii) {
+  ierr = PetscTypeCompareAny((PetscObject)st,&flg,STSHIFT,STFOLD,"");CHKERRQ(ierr);
+  if (st->B || !flg) {
+    if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
+    /* Trick for PCView when an unused PC is showed */
+    ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
+    ierr = PetscTypeCompare((PetscObject)pc,PCNONE,&flg);
+    if (flg) {
+      ierr = PCSetOperators(pc,PETSC_NULL,PETSC_NULL,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
+    }
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"Associated KSP object\n");CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"------------------------------\n");CHKERRQ(ierr);
     ierr = KSPView(st->ksp,viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"------------------------------\n");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
-  } else if (isstring) {
-    ierr = KSPView(st->ksp,viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-/*MC
-   STRegisterDynamic - Adds a method to the spectral transformation package.
-
-   Synopsis:
-   STRegisterDynamic(char *name_solver,char *path,char *name_create,PetscErrorCode (*routine_create)(ST))
-
-   Not collective
-
-   Input Parameters:
-+  name_solver - name of a new user-defined solver
-.  path - path (either absolute or relative) the library containing this solver
-.  name_create - name of routine to create method context
--  routine_create - routine to create method context
-
-   Notes:
-   STRegisterDynamic() may be called multiple times to add several user-defined spectral transformations.
-
-   If dynamic libraries are used, then the fourth input argument (routine_create)
-   is ignored.
-
-   Sample usage:
-.vb
-   STRegisterDynamic("my_solver","/home/username/my_lib/lib/libO/solaris/mylib.a",
-              "MySolverCreate",MySolverCreate);
-.ve
-
-   Then, your solver can be chosen with the procedural interface via
-$     STSetType(st,"my_solver")
-   or at runtime via the option
-$     -st_type my_solver
-
-   Level: advanced
-
-   $PETSC_DIR, $PETSC_ARCH and $PETSC_LIB_DIR occuring in pathname will be replaced with appropriate values.
-
-.seealso: STRegisterDestroy(), STRegisterAll()
-M*/
-
 #undef __FUNCT__  
 #define __FUNCT__ "STRegister"
 /*@C
-  STRegister - See STRegisterDynamic()
+   STRegister - See STRegisterDynamic()
 
-  Level: advanced
+   Level: advanced
 @*/
 PetscErrorCode STRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(ST))
 {
   PetscErrorCode ierr;
-  char           fullname[256];
+  char           fullname[PETSC_MAX_PATH_LEN];
 
   PetscFunctionBegin;
   ierr = PetscFListConcat(path,name,fullname);CHKERRQ(ierr);
@@ -690,6 +656,6 @@ PetscErrorCode STRegisterDestroy(void)
 
   PetscFunctionBegin;
   ierr = PetscFListDestroy(&STList);CHKERRQ(ierr);
-  ierr = STRegisterAll(PETSC_NULL);CHKERRQ(ierr);
+  STRegisterAllCalled = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
