@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -19,15 +19,15 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/stimpl.h"          /*I   "slepcst.h"   I*/
+#include <private/stimpl.h>          /*I   "slepcst.h"   I*/
 
 EXTERN_C_BEGIN
-EXTERN PetscErrorCode STCreate_Shell(ST);
-EXTERN PetscErrorCode STCreate_Shift(ST);
-EXTERN PetscErrorCode STCreate_Sinvert(ST);
-EXTERN PetscErrorCode STCreate_Cayley(ST);
-EXTERN PetscErrorCode STCreate_Fold(ST);
-EXTERN PetscErrorCode STCreate_Precond(ST);
+extern PetscErrorCode STCreate_Shell(ST);
+extern PetscErrorCode STCreate_Shift(ST);
+extern PetscErrorCode STCreate_Sinvert(ST);
+extern PetscErrorCode STCreate_Cayley(ST);
+extern PetscErrorCode STCreate_Fold(ST);
+extern PetscErrorCode STCreate_Precond(ST);
 EXTERN_C_END
 
 #undef __FUNCT__  
@@ -44,11 +44,12 @@ EXTERN_C_END
 
 .seealso: STRegisterDynamic()
 @*/
-PetscErrorCode STRegisterAll(char *path)
+PetscErrorCode STRegisterAll(const char *path)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  STRegisterAllCalled = PETSC_TRUE;
   ierr = STRegisterDynamic(STSHELL  ,path,"STCreate_Shell",STCreate_Shell);CHKERRQ(ierr);
   ierr = STRegisterDynamic(STSHIFT  ,path,"STCreate_Shift",STCreate_Shift);CHKERRQ(ierr);
   ierr = STRegisterDynamic(STSINVERT,path,"STCreate_Sinvert",STCreate_Sinvert);CHKERRQ(ierr);
