@@ -4,7 +4,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -22,7 +22,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/stimpl.h"            /*I "slepcst.h" I*/
+#include <private/stimpl.h>            /*I "slepcst.h" I*/
+#include <slepcsys.h>
 
 #undef __FUNCT__  
 #define __FUNCT__ "STAssociatedKSPSolve"
@@ -39,21 +40,18 @@
 */
 PetscErrorCode STAssociatedKSPSolve(ST st,Vec b,Vec x)
 {
-  PetscErrorCode ierr;
-  PetscInt       its;
+  PetscErrorCode     ierr;
+  PetscInt           its;
   KSPConvergedReason reason;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  PetscValidHeaderSpecific(b,VEC_COOKIE,2);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,3);
-  if (!st->ksp) { SETERRQ(PETSC_ERR_SUP,"ST has no associated KSP"); }
+  if (!st->ksp) SETERRQ(((PetscObject)st)->comm,PETSC_ERR_SUP,"ST has no associated KSP");
   ierr = KSPSolve(st->ksp,b,x);CHKERRQ(ierr);
   ierr = KSPGetConvergedReason(st->ksp,&reason);CHKERRQ(ierr);
-  if (reason<0) { SETERRQ1(0,"Warning: KSP did not converge (%d)",reason); }
+  if (reason<0) SETERRQ1(((PetscObject)st)->comm,PETSC_ERR_NOT_CONVERGED,"KSP did not converge (reason=%s)",KSPConvergedReasons[reason]);
   ierr = KSPGetIterationNumber(st->ksp,&its);CHKERRQ(ierr);  
   st->lineariterations += its;
-  PetscInfo1(st,"Linear solve iterations=%d\n",its);
+  PetscInfo1(st,"Linear solve iterations=%D\n",its);
   PetscFunctionReturn(0);
 }
 
@@ -77,16 +75,13 @@ PetscErrorCode STAssociatedKSPSolveTranspose(ST st,Vec b,Vec x)
   KSPConvergedReason reason;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  PetscValidHeaderSpecific(b,VEC_COOKIE,2);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,3);
-  if (!st->ksp) { SETERRQ(PETSC_ERR_SUP,"ST has no associated KSP"); }
+  if (!st->ksp) SETERRQ(((PetscObject)st)->comm,PETSC_ERR_SUP,"ST has no associated KSP");
   ierr = KSPSolveTranspose(st->ksp,b,x);CHKERRQ(ierr);
   ierr = KSPGetConvergedReason(st->ksp,&reason);CHKERRQ(ierr);
-  if (reason<0) { SETERRQ1(0,"Warning: KSP did not converge (%d)",reason); }
+  if (reason<0) SETERRQ1(((PetscObject)st)->comm,PETSC_ERR_NOT_CONVERGED,"KSP did not converge (reason=%s)",KSPConvergedReasons[reason]);
   ierr = KSPGetIterationNumber(st->ksp,&its);CHKERRQ(ierr);  
   st->lineariterations += its;
-  PetscInfo1(st,"Linear solve iterations=%d\n",its);
+  PetscInfo1(st,"Linear solve iterations=%D\n",its);
   PetscFunctionReturn(0);
 }
 
@@ -96,7 +91,7 @@ PetscErrorCode STAssociatedKSPSolveTranspose(ST st,Vec b,Vec x)
    STSetKSP - Sets the KSP object associated with the spectral 
    transformation.
 
-   Not collective
+   Collective on ST
 
    Input Parameters:
 +  st   - the spectral transformation context
@@ -110,14 +105,13 @@ PetscErrorCode STSetKSP(ST st,KSP ksp)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  PetscValidHeaderSpecific(ksp,KSP_COOKIE,2);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidHeaderSpecific(ksp,KSP_CLASSID,2);
   PetscCheckSameComm(st,1,ksp,2);
   ierr = PetscObjectReference((PetscObject)ksp);CHKERRQ(ierr);
-  if (st->ksp) {
-    ierr = KSPDestroy(st->ksp);CHKERRQ(ierr);
-  }
+  ierr = KSPDestroy(&st->ksp);CHKERRQ(ierr);
   st->ksp = ksp;
+  ierr = PetscLogObjectParent(st,st->ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -127,7 +121,7 @@ PetscErrorCode STSetKSP(ST st,KSP ksp)
    STGetKSP - Gets the KSP object associated with the spectral
    transformation.
 
-   Not collective
+   Not Collective
 
    Input Parameter:
 .  st - the spectral transformation context
@@ -145,10 +139,20 @@ PetscErrorCode STSetKSP(ST st,KSP ksp)
 @*/
 PetscErrorCode STGetKSP(ST st,KSP* ksp)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
-  if (!((PetscObject)st)->type_name) { SETERRQ(PETSC_ERR_ARG_WRONGSTATE,"Must call STSetType first"); }
-  if (ksp)  *ksp = st->ksp;
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidPointer(ksp,2);
+  if (!st->ksp) {
+    ierr = KSPCreate(((PetscObject)st)->comm,&st->ksp);CHKERRQ(ierr);
+    ierr = KSPSetOptionsPrefix(st->ksp,((PetscObject)st)->prefix);CHKERRQ(ierr);
+    ierr = KSPAppendOptionsPrefix(st->ksp,"st_");CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)st->ksp,(PetscObject)st,1);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(st,st->ksp);CHKERRQ(ierr);
+    ierr = KSPSetTolerances(st->ksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
+  }
+  *ksp = st->ksp;
   PetscFunctionReturn(0);
 }
 
@@ -177,7 +181,7 @@ PetscErrorCode STGetKSP(ST st,KSP* ksp)
 PetscErrorCode STGetOperationCounters(ST st,PetscInt* ops,PetscInt* lits)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
   if (ops) *ops = st->applys;
   if (lits) *lits = st->lineariterations;
   PetscFunctionReturn(0);
@@ -190,7 +194,7 @@ PetscErrorCode STGetOperationCounters(ST st,PetscInt* ops,PetscInt* lits)
    inner product operations and total number of linear iterations used by 
    the ST object.
 
-   Collective on ST
+   Logically Collective on ST
 
    Input Parameter:
 .  st - the spectral transformation context
@@ -202,7 +206,7 @@ PetscErrorCode STGetOperationCounters(ST st,PetscInt* ops,PetscInt* lits)
 PetscErrorCode STResetOperationCounters(ST st)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(st,ST_COOKIE,1);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
   st->lineariterations = 0;
   st->applys = 0;
   PetscFunctionReturn(0);
@@ -222,6 +226,7 @@ PetscErrorCode STCheckNullSpace_Default(ST st,PetscInt n,const Vec V[])
   
   PetscFunctionBegin;
   ierr = PetscMalloc(n*sizeof(Vec),&T);CHKERRQ(ierr);
+  if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
   ierr = KSPGetPC(st->ksp,&pc);CHKERRQ(ierr);
   ierr = PCGetOperators(pc,&A,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
   ierr = MatGetVecs(A,PETSC_NULL,&w);CHKERRQ(ierr);
@@ -230,16 +235,16 @@ PetscErrorCode STCheckNullSpace_Default(ST st,PetscInt n,const Vec V[])
     ierr = MatMult(A,V[i],w);CHKERRQ(ierr);
     ierr = VecNorm(w,NORM_2,&norm);CHKERRQ(ierr);
     if (norm < 1e-8) {
-      PetscInfo2(st,"Vector %i norm=%g\n",i,norm);
+      PetscInfo2(st,"Vector %D norm=%g\n",i,(double)norm);
       T[c] = V[i];
       c++;
     }
   }
-  ierr = VecDestroy(w);CHKERRQ(ierr);
+  ierr = VecDestroy(&w);CHKERRQ(ierr);
   if (c>0) {
     ierr = MatNullSpaceCreate(((PetscObject)st)->comm,PETSC_FALSE,c,T,&nullsp);CHKERRQ(ierr);
     ierr = KSPSetNullSpace(st->ksp,nullsp);CHKERRQ(ierr);
-    ierr = MatNullSpaceDestroy(nullsp);CHKERRQ(ierr);
+    ierr = MatNullSpaceDestroy(&nullsp);CHKERRQ(ierr);
   }
   ierr = PetscFree(T);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -272,8 +277,12 @@ PetscErrorCode STCheckNullSpace(ST st,PetscInt n,const Vec V[])
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (n>0 && st->checknullspace) {
-    ierr = (*st->checknullspace)(st,n,V);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  PetscValidLogicalCollectiveInt(st,n,2);
+  if (n>0 && st->ops->checknullspace) {
+    PetscValidPointer(V,3);
+    PetscValidHeaderSpecific(V[0],VEC_CLASSID,3);
+    ierr = (*st->ops->checknullspace)(st,n,V);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
