@@ -8,7 +8,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -26,20 +26,19 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"                /*I "slepcsvd.h" I*/
-#include "private/epsimpl.h"                /*I "slepceps.h" I*/
-#include "slepceps.h"
+#include <private/svdimpl.h>                /*I "slepcsvd.h" I*/
+#include <private/epsimpl.h>                /*I "slepceps.h" I*/
 
 typedef struct {
-  EPS eps;
-  PetscTruth setfromoptionscalled;
-  Mat mat;
-  Vec w,diag;
+  EPS       eps;
+  PetscBool setfromoptionscalled;
+  Mat       mat;
+  Vec       w,diag;
 } SVD_CROSS;
 
 #undef __FUNCT__  
-#define __FUNCT__ "ShellMatMult_CROSS"
-PetscErrorCode ShellMatMult_CROSS(Mat B,Vec x, Vec y)
+#define __FUNCT__ "ShellMatMult_Cross"
+PetscErrorCode ShellMatMult_Cross(Mat B,Vec x,Vec y)
 {
   PetscErrorCode ierr;
   SVD            svd;
@@ -47,15 +46,15 @@ PetscErrorCode ShellMatMult_CROSS(Mat B,Vec x, Vec y)
   
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&svd);CHKERRQ(ierr);
-  cross = (SVD_CROSS *)svd->data;
+  cross = (SVD_CROSS*)svd->data;
   ierr = SVDMatMult(svd,PETSC_FALSE,x,cross->w);CHKERRQ(ierr);
   ierr = SVDMatMult(svd,PETSC_TRUE,cross->w,y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "ShellMatGetDiagonal_CROSS"
-PetscErrorCode ShellMatGetDiagonal_CROSS(Mat B,Vec d)
+#define __FUNCT__ "ShellMatGetDiagonal_Cross"
+PetscErrorCode ShellMatGetDiagonal_Cross(Mat B,Vec d)
 {
   PetscErrorCode    ierr;
   SVD               svd;
@@ -67,7 +66,7 @@ PetscErrorCode ShellMatGetDiagonal_CROSS(Mat B,Vec d)
   
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&svd);CHKERRQ(ierr);
-  cross = (SVD_CROSS *)svd->data;
+  cross = (SVD_CROSS*)svd->data;
   if (!cross->diag) {
     /* compute diagonal from rows and store in cross->diag */
     ierr = VecDuplicate(d,&cross->diag);CHKERRQ(ierr);
@@ -107,28 +106,22 @@ PetscErrorCode ShellMatGetDiagonal_CROSS(Mat B,Vec d)
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSetUp_CROSS"
-PetscErrorCode SVDSetUp_CROSS(SVD svd)
+#define __FUNCT__ "SVDSetUp_Cross"
+PetscErrorCode SVDSetUp_Cross(SVD svd)
 {
-  PetscErrorCode    ierr;
-  SVD_CROSS         *cross = (SVD_CROSS *)svd->data;
-  PetscInt          n,i;
-  PetscTruth        trackall;
+  PetscErrorCode ierr;
+  SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
+  PetscInt       n,i;
+  PetscBool      trackall;
 
   PetscFunctionBegin;
-  if (cross->mat) { 
-     ierr = MatDestroy(cross->mat);CHKERRQ(ierr);
-     ierr = VecDestroy(cross->w);CHKERRQ(ierr);
+  if (!cross->mat) {
+    ierr = SVDMatGetLocalSize(svd,PETSC_NULL,&n);CHKERRQ(ierr);
+    ierr = MatCreateShell(((PetscObject)svd)->comm,n,n,PETSC_DETERMINE,PETSC_DETERMINE,svd,&cross->mat);CHKERRQ(ierr);
+    ierr = MatShellSetOperation(cross->mat,MATOP_MULT,(void(*)(void))ShellMatMult_Cross);CHKERRQ(ierr);  
+    ierr = MatShellSetOperation(cross->mat,MATOP_GET_DIAGONAL,(void(*)(void))ShellMatGetDiagonal_Cross);CHKERRQ(ierr);  
+    ierr = SVDMatGetVecs(svd,PETSC_NULL,&cross->w);CHKERRQ(ierr);
   }
-  if (cross->diag) {
-     ierr = VecDestroy(cross->diag);CHKERRQ(ierr);
-  }
-  
-  ierr = SVDMatGetLocalSize(svd,PETSC_NULL,&n);CHKERRQ(ierr);
-  ierr = MatCreateShell(((PetscObject)svd)->comm,n,n,PETSC_DETERMINE,PETSC_DETERMINE,svd,&cross->mat);CHKERRQ(ierr);
-  ierr = MatShellSetOperation(cross->mat,MATOP_MULT,(void(*)(void))ShellMatMult_CROSS);CHKERRQ(ierr);  
-  ierr = MatShellSetOperation(cross->mat,MATOP_GET_DIAGONAL,(void(*)(void))ShellMatGetDiagonal_CROSS);CHKERRQ(ierr);  
-  ierr = SVDMatGetVecs(svd,PETSC_NULL,&cross->w);CHKERRQ(ierr);
 
   ierr = EPSSetOperators(cross->eps,cross->mat,PETSC_NULL);CHKERRQ(ierr);
   ierr = EPSSetProblemType(cross->eps,EPS_HEP);CHKERRQ(ierr);
@@ -149,7 +142,7 @@ PetscErrorCode SVDSetUp_CROSS(SVD svd)
   if (svd->nini < 0) {
     ierr = EPSSetInitialSpace(cross->eps,-svd->nini,svd->IS);CHKERRQ(ierr);
     for(i=0; i<-svd->nini; i++) {
-      ierr = VecDestroy(svd->IS[i]);CHKERRQ(ierr);
+      ierr = VecDestroy(&svd->IS[i]);CHKERRQ(ierr);
     }
     ierr = PetscFree(svd->IS);CHKERRQ(ierr);
     svd->nini = 0;
@@ -158,11 +151,11 @@ PetscErrorCode SVDSetUp_CROSS(SVD svd)
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSolve_CROSS"
-PetscErrorCode SVDSolve_CROSS(SVD svd)
+#define __FUNCT__ "SVDSolve_Cross"
+PetscErrorCode SVDSolve_Cross(SVD svd)
 {
   PetscErrorCode ierr;
-  SVD_CROSS      *cross = (SVD_CROSS *)svd->data;
+  SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
   PetscInt       i;
   PetscScalar    sigma;
   
@@ -173,14 +166,15 @@ PetscErrorCode SVDSolve_CROSS(SVD svd)
   ierr = EPSGetConvergedReason(cross->eps,(EPSConvergedReason*)&svd->reason);CHKERRQ(ierr);
   for (i=0;i<svd->nconv;i++) {
     ierr = EPSGetEigenpair(cross->eps,i,&sigma,PETSC_NULL,svd->V[i],PETSC_NULL);CHKERRQ(ierr);
-    svd->sigma[i] = sqrt(PetscRealPart(sigma));
+    if (PetscRealPart(sigma)<0.0) SETERRQ(((PetscObject)svd)->comm,1,"Negative eigenvalue computed by EPS");
+    svd->sigma[i] = PetscSqrtReal(PetscRealPart(sigma));
   }
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDMonitor_CROSS"
-PetscErrorCode SVDMonitor_CROSS(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
+#define __FUNCT__ "SVDMonitor_Cross"
+static PetscErrorCode SVDMonitor_Cross(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
   PetscInt       i;
   SVD            svd = (SVD)ctx;
@@ -188,21 +182,21 @@ PetscErrorCode SVDMonitor_CROSS(EPS eps,PetscInt its,PetscInt nconv,PetscScalar
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  for (i=0;i<nest;i++) {
+  for (i=0;i<PetscMin(nest,svd->ncv);i++) {
     er = eigr[i]; ei = eigi[i];
-    ierr = STBackTransform(eps->OP, 1, &er, &ei); CHKERRQ(ierr);
-    svd->sigma[i] = sqrt(PetscRealPart(er));
+    ierr = STBackTransform(eps->OP,1,&er,&ei);CHKERRQ(ierr);
+    svd->sigma[i] = PetscSqrtReal(PetscRealPart(er));
     svd->errest[i] = errest[i];
   }
-  SVDMonitor(svd,its,nconv,svd->sigma,svd->errest,nest);
+  ierr = SVDMonitor(svd,its,nconv,svd->sigma,svd->errest,nest);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSetFromOptions_CROSS"
-PetscErrorCode SVDSetFromOptions_CROSS(SVD svd)
+#define __FUNCT__ "SVDSetFromOptions_Cross"
+PetscErrorCode SVDSetFromOptions_Cross(SVD svd)
 {
-  SVD_CROSS      *cross = (SVD_CROSS *)svd->data;
+  SVD_CROSS *cross = (SVD_CROSS*)svd->data;
 
   PetscFunctionBegin;
   cross->setfromoptionscalled = PETSC_TRUE;
@@ -211,18 +205,17 @@ PetscErrorCode SVDSetFromOptions_CROSS(SVD svd)
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDCrossSetEPS_CROSS"
-PetscErrorCode SVDCrossSetEPS_CROSS(SVD svd,EPS eps)
+#define __FUNCT__ "SVDCrossSetEPS_Cross"
+PetscErrorCode SVDCrossSetEPS_Cross(SVD svd,EPS eps)
 {
   PetscErrorCode ierr;
-  SVD_CROSS      *cross = (SVD_CROSS *)svd->data;
+  SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,2);
-  PetscCheckSameComm(svd,1,eps,2);
   ierr = PetscObjectReference((PetscObject)eps);CHKERRQ(ierr);
-  ierr = EPSDestroy(cross->eps);CHKERRQ(ierr);  
+  ierr = EPSDestroy(&cross->eps);CHKERRQ(ierr);  
   cross->eps = eps;
+  ierr = PetscLogObjectParent(svd,cross->eps);CHKERRQ(ierr);
   svd->setupcalled = 0;
   PetscFunctionReturn(0);
 }
@@ -246,26 +239,24 @@ EXTERN_C_END
 @*/
 PetscErrorCode SVDCrossSetEPS(SVD svd,EPS eps)
 {
-  PetscErrorCode ierr, (*f)(SVD,EPS eps);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)svd,"SVDCrossSetEPS_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(svd,eps);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,2);
+  PetscCheckSameComm(svd,1,eps,2);
+  ierr = PetscTryMethod(svd,"SVDCrossSetEPS_C",(SVD,EPS),(svd,eps));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDCrossGetEPS_CROSS"
-PetscErrorCode SVDCrossGetEPS_CROSS(SVD svd,EPS *eps)
+#define __FUNCT__ "SVDCrossGetEPS_Cross"
+PetscErrorCode SVDCrossGetEPS_Cross(SVD svd,EPS *eps)
 {
-  SVD_CROSS *cross = (SVD_CROSS *)svd->data;
+  SVD_CROSS *cross = (SVD_CROSS*)svd->data;
 
   PetscFunctionBegin;
-  PetscValidPointer(eps,2);
   *eps = cross->eps;
   PetscFunctionReturn(0);
 }
@@ -291,45 +282,53 @@ EXTERN_C_END
 @*/
 PetscErrorCode SVDCrossGetEPS(SVD svd,EPS *eps)
 {
-  PetscErrorCode ierr, (*f)(SVD,EPS *eps);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)svd,"SVDCrossGetEPS_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(svd,eps);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidPointer(eps,2);
+  ierr = PetscTryMethod(svd,"SVDCrossGetEPS_C",(SVD,EPS*),(svd,eps));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDView_CROSS"
-PetscErrorCode SVDView_CROSS(SVD svd,PetscViewer viewer)
+#define __FUNCT__ "SVDView_Cross"
+PetscErrorCode SVDView_Cross(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  SVD_CROSS      *cross = (SVD_CROSS *)svd->data;
+  SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
 
   PetscFunctionBegin;
+  ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
   ierr = EPSView(cross->eps,viewer);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDDestroy_CROSS"
-PetscErrorCode SVDDestroy_CROSS(SVD svd)
+#define __FUNCT__ "SVDReset_Cross"
+PetscErrorCode SVDReset_Cross(SVD svd)
 {
   PetscErrorCode ierr;
-  SVD_CROSS      *cross = (SVD_CROSS *)svd->data;
+  SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
 
   PetscFunctionBegin;
-  ierr = EPSDestroy(cross->eps);CHKERRQ(ierr);
-  if (cross->mat) { 
-    ierr = MatDestroy(cross->mat);CHKERRQ(ierr);
-    ierr = VecDestroy(cross->w);CHKERRQ(ierr);
-  }
-  if (cross->diag) {
-    ierr = VecDestroy(cross->diag);CHKERRQ(ierr);
-  }
+  ierr = EPSReset(cross->eps);CHKERRQ(ierr);
+  ierr = MatDestroy(&cross->mat);CHKERRQ(ierr);
+  ierr = VecDestroy(&cross->w);CHKERRQ(ierr);
+  ierr = VecDestroy(&cross->diag);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDDestroy_Cross"
+PetscErrorCode SVDDestroy_Cross(SVD svd)
+{
+  PetscErrorCode ierr;
+  SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
+
+  PetscFunctionBegin;
+  ierr = EPSDestroy(&cross->eps);CHKERRQ(ierr);
   ierr = PetscFree(svd->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCrossSetEPS_C","",PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCrossGetEPS_C","",PETSC_NULL);CHKERRQ(ierr);
@@ -338,33 +337,34 @@ PetscErrorCode SVDDestroy_CROSS(SVD svd)
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDCreate_CROSS"
-PetscErrorCode SVDCreate_CROSS(SVD svd)
+#define __FUNCT__ "SVDCreate_Cross"
+PetscErrorCode SVDCreate_Cross(SVD svd)
 {
   PetscErrorCode ierr;
   SVD_CROSS      *cross;
   ST             st;
   
   PetscFunctionBegin;
-  ierr = PetscNew(SVD_CROSS,&cross);CHKERRQ(ierr);
-  PetscLogObjectMemory(svd,sizeof(SVD_CROSS));
-  svd->data                = (void *)cross;
-  svd->ops->solve          = SVDSolve_CROSS;
-  svd->ops->setup          = SVDSetUp_CROSS;
-  svd->ops->setfromoptions = SVDSetFromOptions_CROSS;
-  svd->ops->destroy        = SVDDestroy_CROSS;
-  svd->ops->view           = SVDView_CROSS;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCrossSetEPS_C","SVDCrossSetEPS_CROSS",SVDCrossSetEPS_CROSS);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCrossGetEPS_C","SVDCrossGetEPS_CROSS",SVDCrossGetEPS_CROSS);CHKERRQ(ierr);
+  ierr = PetscNewLog(svd,SVD_CROSS,&cross);CHKERRQ(ierr);
+  svd->data                = (void*)cross;
+  svd->ops->solve          = SVDSolve_Cross;
+  svd->ops->setup          = SVDSetUp_Cross;
+  svd->ops->setfromoptions = SVDSetFromOptions_Cross;
+  svd->ops->destroy        = SVDDestroy_Cross;
+  svd->ops->reset          = SVDReset_Cross;
+  svd->ops->view           = SVDView_Cross;
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCrossSetEPS_C","SVDCrossSetEPS_Cross",SVDCrossSetEPS_Cross);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDCrossGetEPS_C","SVDCrossGetEPS_Cross",SVDCrossGetEPS_Cross);CHKERRQ(ierr);
 
   ierr = EPSCreate(((PetscObject)svd)->comm,&cross->eps);CHKERRQ(ierr);
   ierr = EPSSetOptionsPrefix(cross->eps,((PetscObject)svd)->prefix);CHKERRQ(ierr);
   ierr = EPSAppendOptionsPrefix(cross->eps,"svd_");CHKERRQ(ierr);
   ierr = PetscObjectIncrementTabLevel((PetscObject)cross->eps,(PetscObject)svd,1);CHKERRQ(ierr);  
-  PetscLogObjectParent(svd,cross->eps);
+  ierr = PetscLogObjectParent(svd,cross->eps);CHKERRQ(ierr);
+  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
   ierr = EPSSetIP(cross->eps,svd->ip);CHKERRQ(ierr);
   ierr = EPSSetWhichEigenpairs(cross->eps,EPS_LARGEST_REAL);CHKERRQ(ierr);
-  ierr = EPSMonitorSet(cross->eps,SVDMonitor_CROSS,svd,PETSC_NULL);CHKERRQ(ierr);
+  ierr = EPSMonitorSet(cross->eps,SVDMonitor_Cross,svd,PETSC_NULL);CHKERRQ(ierr);
   ierr = EPSGetST(cross->eps,&st);CHKERRQ(ierr);
   ierr = STSetMatMode(st,ST_MATMODE_SHELL);CHKERRQ(ierr);
   cross->mat = PETSC_NULL;
