@@ -8,7 +8,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -26,27 +26,26 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"                /*I "slepcsvd.h" I*/
-#include "private/ipimpl.h"
-#include "slepcblaslapack.h"
+#include <private/svdimpl.h>                /*I "slepcsvd.h" I*/
+#include <private/ipimpl.h>
+#include <slepcblaslapack.h>
 
 typedef struct {
-  PetscTruth oneside;
+  PetscBool oneside;
 } SVD_LANCZOS;
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSetUp_LANCZOS"
-PetscErrorCode SVDSetUp_LANCZOS(SVD svd)
+#define __FUNCT__ "SVDSetUp_Lanczos"
+PetscErrorCode SVDSetUp_Lanczos(SVD svd)
 {
-  PetscErrorCode  ierr;
-  SVD_LANCZOS     *lanczos = (SVD_LANCZOS *)svd->data;
-  PetscInt        i,N,nloc;
-  PetscScalar     *pU;
+  PetscErrorCode ierr;
+  SVD_LANCZOS    *lanczos = (SVD_LANCZOS *)svd->data;
+  PetscInt       N;
 
   PetscFunctionBegin;
   ierr = SVDMatGetSize(svd,PETSC_NULL,&N);CHKERRQ(ierr);
   if (svd->ncv) { /* ncv set */
-    if (svd->ncv<svd->nsv) SETERRQ(1,"The value of ncv must be at least nsv"); 
+    if (svd->ncv<svd->nsv) SETERRQ(((PetscObject)svd)->comm,1,"The value of ncv must be at least nsv"); 
   }
   else if (svd->mpd) { /* mpd set */
     svd->ncv = PetscMin(N,svd->nsv+svd->mpd);
@@ -56,22 +55,11 @@ PetscErrorCode SVDSetUp_LANCZOS(SVD svd)
     else { svd->mpd = 500; svd->ncv = PetscMin(N,svd->nsv+svd->mpd); }
   }
   if (!svd->mpd) svd->mpd = svd->ncv;
-  if (svd->ncv>svd->nsv+svd->mpd) SETERRQ(1,"The value of ncv must not be larger than nev+mpd"); 
-  if (!svd->max_it)
-    svd->max_it = PetscMax(N/svd->ncv,100);
-  if (svd->U) {
-    ierr = VecGetArray(svd->U[0],&pU);CHKERRQ(ierr);
-    for (i=0;i<svd->n;i++) { ierr = VecDestroy(svd->U[i]); CHKERRQ(ierr); }
-    ierr = PetscFree(pU);CHKERRQ(ierr);
-    ierr = PetscFree(svd->U);CHKERRQ(ierr);
-  }
-  if (!lanczos->oneside) {
-    ierr = PetscMalloc(sizeof(Vec)*svd->ncv,&svd->U);CHKERRQ(ierr);
-    ierr = SVDMatGetLocalSize(svd,&nloc,PETSC_NULL);CHKERRQ(ierr);
-    ierr = PetscMalloc(svd->ncv*nloc*sizeof(PetscScalar),&pU);CHKERRQ(ierr);
-    for (i=0;i<svd->ncv;i++) {
-      ierr = VecCreateMPIWithArray(((PetscObject)svd)->comm,nloc,PETSC_DECIDE,pU+i*nloc,&svd->U[i]);CHKERRQ(ierr);
-    }
+  if (svd->ncv>svd->nsv+svd->mpd) SETERRQ(((PetscObject)svd)->comm,1,"The value of ncv must not be larger than nev+mpd"); 
+  if (!svd->max_it) svd->max_it = PetscMax(N/svd->ncv,100);
+  if (!lanczos->oneside && svd->ncv != svd->n) {
+    if (svd->n) { ierr = VecDestroyVecs(svd->n,&svd->U);CHKERRQ(ierr); }
+    ierr = VecDuplicateVecs(svd->tl,svd->ncv,&svd->U);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -151,12 +139,12 @@ static PetscErrorCode SVDOneSideLanczos(SVD svd,PetscReal *alpha,PetscReal *beta
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSolve_LANCZOS"
-PetscErrorCode SVDSolve_LANCZOS(SVD svd)
+#define __FUNCT__ "SVDSolve_Lanczos"
+PetscErrorCode SVDSolve_Lanczos(SVD svd)
 {
 #if defined(SLEPC_MISSING_LAPACK_BDSDC)
   PetscFunctionBegin;
-  SETERRQ(PETSC_ERR_SUP,"BDSDC - Lapack routine is unavailable.");
+  SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_SUP,"BDSDC - Lapack routine is unavailable.");
 #else
   PetscErrorCode ierr;
   SVD_LANCZOS    *lanczos = (SVD_LANCZOS *)svd->data;
@@ -164,8 +152,8 @@ PetscErrorCode SVDSolve_LANCZOS(SVD svd)
   PetscScalar    *swork;
   PetscBLASInt   n,info,*iwork;
   PetscInt       i,j,k,m,nv;
-  Vec            v,u,u_1;
-  PetscTruth     conv;
+  Vec            v,u=0,u_1=0;
+  PetscBool      conv;
   
   PetscFunctionBegin;
   /* allocate working space */
@@ -278,15 +266,13 @@ PetscErrorCode SVDSolve_LANCZOS(SVD svd)
     }
         
     svd->nconv += k;
-    SVDMonitor(svd,svd->its,svd->nconv,svd->sigma,svd->errest,nv);
+    ierr = SVDMonitor(svd,svd->its,svd->nconv,svd->sigma,svd->errest,nv);CHKERRQ(ierr);
   }
   
   /* free working space */
-  ierr = VecDestroy(v);CHKERRQ(ierr);
-  if (lanczos->oneside) {
-    ierr = VecDestroy(u);CHKERRQ(ierr);
-    ierr = VecDestroy(u_1);CHKERRQ(ierr);
-  }
+  ierr = VecDestroy(&v);CHKERRQ(ierr);
+  ierr = VecDestroy(&u);CHKERRQ(ierr);
+  ierr = VecDestroy(&u_1);CHKERRQ(ierr);
   ierr = PetscFree(alpha);CHKERRQ(ierr);
   ierr = PetscFree(beta);CHKERRQ(ierr);
   ierr = PetscFree(Q);CHKERRQ(ierr);
@@ -299,29 +285,29 @@ PetscErrorCode SVDSolve_LANCZOS(SVD svd)
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSetFromOptions_LANCZOS"
-PetscErrorCode SVDSetFromOptions_LANCZOS(SVD svd)
+#define __FUNCT__ "SVDSetFromOptions_Lanczos"
+PetscErrorCode SVDSetFromOptions_Lanczos(SVD svd)
 {
   PetscErrorCode ierr;
-  PetscTruth     set,val;
+  PetscBool      set,val;
   SVD_LANCZOS    *lanczos = (SVD_LANCZOS *)svd->data;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsBegin(((PetscObject)svd)->comm,((PetscObject)svd)->prefix,"LANCZOS Singular Value Solver Options","SVD");CHKERRQ(ierr);
-  ierr = PetscOptionsTruth("-svd_lanczos_oneside","Lanczos one-side reorthogonalization","SVDLanczosSetOneSide",lanczos->oneside,&val,&set);CHKERRQ(ierr);
+  ierr = PetscOptionsHead("SVD Lanczos Options");CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-svd_lanczos_oneside","Lanczos one-side reorthogonalization","SVDLanczosSetOneSide",lanczos->oneside,&val,&set);CHKERRQ(ierr);
   if (set) {
     ierr = SVDLanczosSetOneSide(svd,val);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDLanczosSetOneSide_LANCZOS"
-PetscErrorCode SVDLanczosSetOneSide_LANCZOS(SVD svd,PetscTruth oneside)
+#define __FUNCT__ "SVDLanczosSetOneSide_Lanczos"
+PetscErrorCode SVDLanczosSetOneSide_Lanczos(SVD svd,PetscBool oneside)
 {
-  SVD_LANCZOS    *lanczos = (SVD_LANCZOS *)svd->data;
+  SVD_LANCZOS *lanczos = (SVD_LANCZOS *)svd->data;
 
   PetscFunctionBegin;
   if (lanczos->oneside != oneside) {
@@ -338,7 +324,7 @@ EXTERN_C_END
    SVDLanczosSetOneSide - Indicate if the variant of the Lanczos method 
    to be used is one-sided or two-sided.
 
-   Collective on SVD
+   Logically Collective on SVD
 
    Input Parameters:
 +  svd     - singular value solver
@@ -357,16 +343,14 @@ EXTERN_C_END
 
 .seealso: SVDTRLanczosSetOneSide()
 @*/
-PetscErrorCode SVDLanczosSetOneSide(SVD svd,PetscTruth oneside)
+PetscErrorCode SVDLanczosSetOneSide(SVD svd,PetscBool oneside)
 {
-  PetscErrorCode ierr, (*f)(SVD,PetscTruth);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)svd,"SVDLanczosSetOneSide_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(svd,oneside);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveBool(svd,oneside,2);
+  ierr = PetscTryMethod(svd,"SVDLanczosSetOneSide_C",(SVD,PetscBool),(svd,oneside));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -376,7 +360,7 @@ PetscErrorCode SVDLanczosSetOneSide(SVD svd,PetscTruth oneside)
    SVDLanczosGetOneSide - Gets if the variant of the Lanczos method 
    to be used is one-sided or two-sided.
 
-   Collective on SVD
+   Not Collective
 
    Input Parameters:
 .  svd     - singular value solver
@@ -388,79 +372,84 @@ PetscErrorCode SVDLanczosSetOneSide(SVD svd,PetscTruth oneside)
 
 .seealso: SVDLanczosSetOneSide()
 @*/
-PetscErrorCode SVDLanczosGetOneSide(SVD svd,PetscTruth *oneside)
+PetscErrorCode SVDLanczosGetOneSide(SVD svd,PetscBool *oneside)
 {
-  PetscErrorCode ierr, (*f)(SVD,PetscTruth*);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)svd,"SVDLanczosGetOneSide_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(svd,oneside);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidPointer(oneside,2);
+  ierr = PetscTryMethod(svd,"SVDLanczosGetOneSide_C",(SVD,PetscBool*),(svd,oneside));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDLanczosGetOneSide_LANCZOS"
-PetscErrorCode SVDLanczosGetOneSide_LANCZOS(SVD svd,PetscTruth *oneside)
+#define __FUNCT__ "SVDLanczosGetOneSide_Lanczos"
+PetscErrorCode SVDLanczosGetOneSide_Lanczos(SVD svd,PetscBool *oneside)
 {
-  SVD_LANCZOS    *lanczos = (SVD_LANCZOS *)svd->data;
+  SVD_LANCZOS *lanczos = (SVD_LANCZOS *)svd->data;
 
   PetscFunctionBegin;
-  PetscValidPointer(oneside,2);
   *oneside = lanczos->oneside;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDDestroy_LANCZOS"
-PetscErrorCode SVDDestroy_LANCZOS(SVD svd)
+#define __FUNCT__ "SVDReset_Lanczos"
+PetscErrorCode SVDReset_Lanczos(SVD svd)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = SVDDestroy_Default(svd);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(svd->n,&svd->U);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDDestroy_Lanczos"
+PetscErrorCode SVDDestroy_Lanczos(SVD svd)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFree(svd->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDLanczosSetOneSide_C","",PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDLanczosGetOneSide_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDView_LANCZOS"
-PetscErrorCode SVDView_LANCZOS(SVD svd,PetscViewer viewer)
+#define __FUNCT__ "SVDView_Lanczos"
+PetscErrorCode SVDView_Lanczos(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   SVD_LANCZOS    *lanczos = (SVD_LANCZOS *)svd->data;
 
   PetscFunctionBegin;
-  ierr = PetscViewerASCIIPrintf(viewer,"Lanczos reorthogonalization: %s\n",lanczos->oneside ? "one-side" : "two-side");CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  Lanczos: %s-sided reorthogonalization\n",lanczos->oneside? "one": "two");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDCreate_LANCZOS"
-PetscErrorCode SVDCreate_LANCZOS(SVD svd)
+#define __FUNCT__ "SVDCreate_Lanczos"
+PetscErrorCode SVDCreate_Lanczos(SVD svd)
 {
   PetscErrorCode ierr;
-  SVD_LANCZOS    *lanczos;
 
   PetscFunctionBegin;
-  ierr = PetscNew(SVD_LANCZOS,&lanczos);CHKERRQ(ierr);
-  PetscLogObjectMemory(svd,sizeof(SVD_LANCZOS));
-  svd->data                = (void *)lanczos;
-  svd->ops->setup          = SVDSetUp_LANCZOS;
-  svd->ops->solve          = SVDSolve_LANCZOS;
-  svd->ops->destroy        = SVDDestroy_LANCZOS;
-  svd->ops->setfromoptions = SVDSetFromOptions_LANCZOS;
-  svd->ops->view           = SVDView_LANCZOS;
-  lanczos->oneside         = PETSC_FALSE;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDLanczosSetOneSide_C","SVDLanczosSetOneSide_LANCZOS",SVDLanczosSetOneSide_LANCZOS);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDLanczosGetOneSide_C","SVDLanczosGetOneSide_LANCZOS",SVDLanczosGetOneSide_LANCZOS);CHKERRQ(ierr);
+  ierr = PetscNewLog(svd,SVD_LANCZOS,&svd->data);CHKERRQ(ierr);
+  svd->ops->setup          = SVDSetUp_Lanczos;
+  svd->ops->solve          = SVDSolve_Lanczos;
+  svd->ops->destroy        = SVDDestroy_Lanczos;
+  svd->ops->reset          = SVDReset_Lanczos;
+  svd->ops->setfromoptions = SVDSetFromOptions_Lanczos;
+  svd->ops->view           = SVDView_Lanczos;
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDLanczosSetOneSide_C","SVDLanczosSetOneSide_Lanczos",SVDLanczosSetOneSide_Lanczos);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDLanczosGetOneSide_C","SVDLanczosGetOneSide_Lanczos",SVDLanczosGetOneSide_Lanczos);CHKERRQ(ierr);
+  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
