@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,16 +21,15 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"
-#include "slepcblaslapack.h"
+#include <private/svdimpl.h>      /*I "slepcsvd.h" I*/
+#include <slepcblaslapack.h>
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDSetUp_LAPACK"
 PetscErrorCode SVDSetUp_LAPACK(SVD svd)
 {
-  PetscErrorCode  ierr;
-  PetscInt        N,i,nloc;
-  PetscScalar     *pU;
+  PetscErrorCode ierr;
+  PetscInt       N;
 
   PetscFunctionBegin;
   ierr = SVDMatGetSize(svd,PETSC_NULL,&N);CHKERRQ(ierr);
@@ -38,18 +37,7 @@ PetscErrorCode SVDSetUp_LAPACK(SVD svd)
   if (svd->mpd) PetscInfo(svd,"Warning: parameter mpd ignored\n");
   svd->max_it = 1;
   if (svd->ncv!=svd->n) {  
-    if (svd->U) {
-      ierr = VecGetArray(svd->U[0],&pU);CHKERRQ(ierr);
-      for (i=0;i<svd->n;i++) { ierr = VecDestroy(svd->U[i]); CHKERRQ(ierr); }
-      ierr = PetscFree(pU);CHKERRQ(ierr);
-      ierr = PetscFree(svd->U);CHKERRQ(ierr);
-    }
-    ierr = PetscMalloc(sizeof(Vec)*svd->ncv,&svd->U);CHKERRQ(ierr);
-    ierr = SVDMatGetLocalSize(svd,&nloc,PETSC_NULL);CHKERRQ(ierr);
-    ierr = PetscMalloc(svd->ncv*nloc*sizeof(PetscScalar),&pU);CHKERRQ(ierr);
-    for (i=0;i<svd->ncv;i++) {
-      ierr = VecCreateMPIWithArray(((PetscObject)svd)->comm,nloc,PETSC_DECIDE,pU+i*nloc,&svd->U[i]);CHKERRQ(ierr);
-    }
+    ierr = VecDuplicateVecs(svd->tl,svd->ncv,&svd->U);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -58,11 +46,11 @@ PetscErrorCode SVDSetUp_LAPACK(SVD svd)
 #define __FUNCT__ "SVDSolve_LAPACK"
 PetscErrorCode SVDSolve_LAPACK(SVD svd)
 {
-  PetscErrorCode  ierr;
-  PetscInt        M,N,n,i,j,k;
-  Mat             mat;
-  PetscScalar     *pU,*pVT,*pmat,*pu,*pv;
-  PetscReal       *sigma;
+  PetscErrorCode ierr;
+  PetscInt       M,N,n,i,j,k;
+  Mat            mat;
+  PetscScalar    *pU,*pVT,*pmat,*pu,*pv;
+  PetscReal      *sigma;
   
   PetscFunctionBegin;
   ierr = MatConvert(svd->OP,MATSEQDENSE,MAT_INITIAL_MATRIX,&mat);CHKERRQ(ierr);
@@ -103,7 +91,7 @@ PetscErrorCode SVDSolve_LAPACK(SVD svd)
   svd->reason = SVD_CONVERGED_TOL;
 
   ierr = MatRestoreArray(mat,&pmat);CHKERRQ(ierr);
-  ierr = MatDestroy(mat);CHKERRQ(ierr);
+  ierr = MatDestroy(&mat);CHKERRQ(ierr);
   ierr = PetscFree(sigma);CHKERRQ(ierr);
   if (M>=N) {
      ierr = PetscFree(pVT);CHKERRQ(ierr);
@@ -113,6 +101,28 @@ PetscErrorCode SVDSolve_LAPACK(SVD svd)
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "SVDReset_LAPACK"
+PetscErrorCode SVDReset_LAPACK(SVD svd)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = VecDestroyVecs(svd->n,&svd->U);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDDestroy_LAPACK"
+PetscErrorCode SVDDestroy_LAPACK(SVD svd)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFree(svd->data);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "SVDCreate_LAPACK"
@@ -121,7 +131,8 @@ PetscErrorCode SVDCreate_LAPACK(SVD svd)
   PetscFunctionBegin;
   svd->ops->setup   = SVDSetUp_LAPACK;
   svd->ops->solve   = SVDSolve_LAPACK;
-  svd->ops->destroy = SVDDestroy_Default;
+  svd->ops->destroy = SVDDestroy_LAPACK;
+  svd->ops->reset   = SVDReset_LAPACK;
   if (svd->transmode == PETSC_DECIDE)
     svd->transmode = SVD_TRANSPOSE_IMPLICIT; /* don't build the transpose */
   PetscFunctionReturn(0);
