@@ -8,7 +8,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -26,26 +26,25 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"                /*I "slepcsvd.h" I*/
-#include "private/ipimpl.h"
-#include "slepcblaslapack.h"
+#include <private/svdimpl.h>                /*I "slepcsvd.h" I*/
+#include <private/ipimpl.h>
+#include <slepcblaslapack.h>
 
 typedef struct {
-  PetscTruth oneside;
+  PetscBool oneside;
 } SVD_TRLANCZOS;
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSetUp_TRLANCZOS"
-PetscErrorCode SVDSetUp_TRLANCZOS(SVD svd)
+#define __FUNCT__ "SVDSetUp_TRLanczos"
+PetscErrorCode SVDSetUp_TRLanczos(SVD svd)
 {
-  PetscErrorCode  ierr;
-  PetscInt        i,N,nloc;
-  PetscScalar     *pU;
+  PetscErrorCode ierr;
+  PetscInt       N;
 
   PetscFunctionBegin;
   ierr = SVDMatGetSize(svd,PETSC_NULL,&N);CHKERRQ(ierr);
   if (svd->ncv) { /* ncv set */
-    if (svd->ncv<svd->nsv) SETERRQ(1,"The value of ncv must be at least nsv"); 
+    if (svd->ncv<svd->nsv) SETERRQ(((PetscObject)svd)->comm,1,"The value of ncv must be at least nsv"); 
   }
   else if (svd->mpd) { /* mpd set */
     svd->ncv = PetscMin(N,svd->nsv+svd->mpd);
@@ -55,22 +54,10 @@ PetscErrorCode SVDSetUp_TRLANCZOS(SVD svd)
     else { svd->mpd = 500; svd->ncv = PetscMin(N,svd->nsv+svd->mpd); }
   }
   if (!svd->mpd) svd->mpd = svd->ncv;
-  if (svd->ncv>svd->nsv+svd->mpd) SETERRQ(1,"The value of ncv must not be larger than nev+mpd"); 
-  if (!svd->max_it)
-    svd->max_it = PetscMax(N/svd->ncv,100);
+  if (svd->ncv>svd->nsv+svd->mpd) SETERRQ(((PetscObject)svd)->comm,1,"The value of ncv must not be larger than nev+mpd"); 
+  if (!svd->max_it) svd->max_it = PetscMax(N/svd->ncv,100);
   if (svd->ncv!=svd->n) {  
-    if (svd->U) {
-      ierr = VecGetArray(svd->U[0],&pU);CHKERRQ(ierr);
-      for (i=0;i<svd->n;i++) { ierr = VecDestroy(svd->U[i]); CHKERRQ(ierr); }
-      ierr = PetscFree(pU);CHKERRQ(ierr);
-      ierr = PetscFree(svd->U);CHKERRQ(ierr);
-    }
-    ierr = PetscMalloc(sizeof(Vec)*svd->ncv,&svd->U);CHKERRQ(ierr);
-    ierr = SVDMatGetLocalSize(svd,&nloc,PETSC_NULL);CHKERRQ(ierr);
-    ierr = PetscMalloc(svd->ncv*nloc*sizeof(PetscScalar),&pU);CHKERRQ(ierr);
-    for (i=0;i<svd->ncv;i++) {
-      ierr = VecCreateMPIWithArray(((PetscObject)svd)->comm,nloc,PETSC_DECIDE,pU+i*nloc,&svd->U[i]);CHKERRQ(ierr);
-    }
+    ierr = VecDuplicateVecs(svd->tl,svd->ncv,&svd->U);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -126,12 +113,12 @@ static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal
   for (i=k+1;i<n;i++) {
     ierr = SVDMatMult(svd,PETSC_TRUE,U[i-1],V[i]);CHKERRQ(ierr);
     ierr = IPNormBegin(svd->ip,U[i-1],&a);CHKERRQ(ierr);
-    if (svd->ip->orthog_ref == IP_ORTH_REFINE_IFNEEDED) {
+    if (svd->ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
       ierr = IPInnerProductBegin(svd->ip,V[i],V[i],&dot);CHKERRQ(ierr);
     }
     ierr = IPMInnerProductBegin(svd->ip,V[i],i,V,work);CHKERRQ(ierr);
     ierr = IPNormEnd(svd->ip,U[i-1],&a);CHKERRQ(ierr);
-    if (svd->ip->orthog_ref == IP_ORTH_REFINE_IFNEEDED) {
+    if (svd->ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
       ierr = IPInnerProductEnd(svd->ip,V[i],V[i],&dot);CHKERRQ(ierr);
     }
     ierr = IPMInnerProductEnd(svd->ip,V[i],i,V,work);CHKERRQ(ierr);
@@ -141,20 +128,20 @@ static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal
     ierr = SlepcVecMAXPBY(V[i],1.0/a,-1.0,i,work,V);CHKERRQ(ierr);
 
     switch (svd->ip->orthog_ref) {
-    case IP_ORTH_REFINE_NEVER:
+    case IP_ORTHOG_REFINE_NEVER:
       ierr = IPNorm(svd->ip,V[i],&b);CHKERRQ(ierr);
       break;      
-    case IP_ORTH_REFINE_ALWAYS:
+    case IP_ORTHOG_REFINE_ALWAYS:
       ierr = IPOrthogonalizeCGS1(svd->ip,0,PETSC_NULL,i,PETSC_NULL,V,V[i],work,PETSC_NULL,&b);CHKERRQ(ierr);
       break;
-    case IP_ORTH_REFINE_IFNEEDED:
-      onorm = sqrt(PetscRealPart(dot)) / a;
+    case IP_ORTHOG_REFINE_IFNEEDED:
+      onorm = PetscSqrtReal(PetscRealPart(dot)) / a;
       sum = 0.0;
       for (j=0;j<i;j++) {
         sum += PetscRealPart(work[j] * PetscConj(work[j]));
       }
       b = PetscRealPart(dot)/(a*a) - sum;
-      if (b>0.0) b = sqrt(b);
+      if (b>0.0) b = PetscSqrtReal(b);
       else {
         ierr = IPNorm(svd->ip,V[i],&b);CHKERRQ(ierr);
       }
@@ -174,12 +161,12 @@ static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal
   }
   ierr = SVDMatMult(svd,PETSC_TRUE,U[n-1],v);CHKERRQ(ierr);
   ierr = IPNormBegin(svd->ip,U[n-1],&a);CHKERRQ(ierr);
-  if (svd->ip->orthog_ref == IP_ORTH_REFINE_IFNEEDED) {
+  if (svd->ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
     ierr = IPInnerProductBegin(svd->ip,v,v,&dot);CHKERRQ(ierr);
   }
   ierr = IPMInnerProductBegin(svd->ip,v,n,V,work);CHKERRQ(ierr);
   ierr = IPNormEnd(svd->ip,U[n-1],&a);CHKERRQ(ierr);
-  if (svd->ip->orthog_ref == IP_ORTH_REFINE_IFNEEDED) {
+  if (svd->ip->orthog_ref == IP_ORTHOG_REFINE_IFNEEDED) {
     ierr = IPInnerProductEnd(svd->ip,v,v,&dot);CHKERRQ(ierr);
   }
   ierr = IPMInnerProductEnd(svd->ip,v,n,V,work);CHKERRQ(ierr);
@@ -189,20 +176,20 @@ static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal
   ierr = SlepcVecMAXPBY(v,1.0/a,-1.0,n,work,V);CHKERRQ(ierr);
 
   switch (svd->ip->orthog_ref) {
-  case IP_ORTH_REFINE_NEVER:
+  case IP_ORTHOG_REFINE_NEVER:
     ierr = IPNorm(svd->ip,v,&b);CHKERRQ(ierr);
     break;      
-  case IP_ORTH_REFINE_ALWAYS:
+  case IP_ORTHOG_REFINE_ALWAYS:
     ierr = IPOrthogonalizeCGS1(svd->ip,0,PETSC_NULL,n,PETSC_NULL,V,v,work,PETSC_NULL,&b);CHKERRQ(ierr);
     break;
-  case IP_ORTH_REFINE_IFNEEDED:
-    onorm = sqrt(PetscRealPart(dot)) / a;
+  case IP_ORTHOG_REFINE_IFNEEDED:
+    onorm = PetscSqrtReal(PetscRealPart(dot)) / a;
     sum = 0.0;
     for (j=0;j<i;j++) {
       sum += PetscRealPart(work[j] * PetscConj(work[j]));
     }
     b = PetscRealPart(dot)/(a*a) - sum;
-    if (b>0.0) b = sqrt(b);
+    if (b>0.0) b = PetscSqrtReal(b);
     else {
       ierr = IPNorm(svd->ip,v,&b);CHKERRQ(ierr);
     }
@@ -218,8 +205,8 @@ static PetscErrorCode SVDOneSideTRLanczosCGS(SVD svd,PetscReal *alpha,PetscReal
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSolve_TRLANCZOS"
-PetscErrorCode SVDSolve_TRLANCZOS(SVD svd)
+#define __FUNCT__ "SVDSolve_TRLanczos"
+PetscErrorCode SVDSolve_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
   SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS *)svd->data;
@@ -227,8 +214,8 @@ PetscErrorCode SVDSolve_TRLANCZOS(SVD svd)
   PetscScalar    *b,*Q,*PT,*swork;
   PetscInt       i,j,k,l,m,n,nv;
   Vec            v;
-  PetscTruth     conv;
-  IPOrthogonalizationType orthog;
+  PetscBool      conv;
+  IPOrthogType   orthog;
   
   PetscFunctionBegin;
   /* allocate working space */
@@ -262,7 +249,7 @@ PetscErrorCode SVDSolve_TRLANCZOS(SVD svd)
     /* inner loop */
     nv = PetscMin(svd->nconv+svd->mpd,svd->n);
     if (lanczos->oneside) {
-      if (orthog == IP_ORTH_MGS) {
+      if (orthog == IP_ORTHOG_MGS) {
         ierr = SVDOneSideTRLanczosMGS(svd,alpha,beta,b+svd->nconv,svd->V,v,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
       } else {
         ierr = SVDOneSideTRLanczosCGS(svd,alpha,beta,b+svd->nconv,svd->V,v,svd->U,svd->nconv,l,nv,swork);CHKERRQ(ierr);
@@ -344,7 +331,7 @@ PetscErrorCode SVDSolve_TRLANCZOS(SVD svd)
     }
     
     svd->nconv += k;
-    SVDMonitor(svd,svd->its,svd->nconv,svd->sigma,svd->errest,nv);
+    ierr = SVDMonitor(svd,svd->its,svd->nconv,svd->sigma,svd->errest,nv);CHKERRQ(ierr);
   }
   
   /* orthonormalize U columns in one side method */
@@ -356,7 +343,7 @@ PetscErrorCode SVDSolve_TRLANCZOS(SVD svd)
   }
   
   /* free working space */
-  ierr = VecDestroy(v);CHKERRQ(ierr);
+  ierr = VecDestroy(&v);CHKERRQ(ierr);
 
   ierr = PetscFree(alpha);CHKERRQ(ierr);
   ierr = PetscFree(beta);CHKERRQ(ierr);
@@ -368,27 +355,27 @@ PetscErrorCode SVDSolve_TRLANCZOS(SVD svd)
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDSetFromOptions_TRLANCZOS"
-PetscErrorCode SVDSetFromOptions_TRLANCZOS(SVD svd)
+#define __FUNCT__ "SVDSetFromOptions_TRLanczos"
+PetscErrorCode SVDSetFromOptions_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
-  PetscTruth     set,val;
+  PetscBool      set,val;
   SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS *)svd->data;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsBegin(((PetscObject)svd)->comm,((PetscObject)svd)->prefix,"TRLANCZOS Singular Value Solver Options","SVD");CHKERRQ(ierr);
-  ierr = PetscOptionsTruth("-svd_trlanczos_oneside","Lanczos one-side reorthogonalization","SVDTRLanczosSetOneSide",lanczos->oneside,&val,&set);CHKERRQ(ierr);
+  ierr = PetscOptionsHead("SVD TRLanczos Options");CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-svd_trlanczos_oneside","Lanczos one-side reorthogonalization","SVDTRLanczosSetOneSide",lanczos->oneside,&val,&set);CHKERRQ(ierr);
   if (set) {
     ierr = SVDTRLanczosSetOneSide(svd,val);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDTRLanczosSetOneSide_TRLANCZOS"
-PetscErrorCode SVDTRLanczosSetOneSide_TRLANCZOS(SVD svd,PetscTruth oneside)
+#define __FUNCT__ "SVDTRLanczosSetOneSide_TRLanczos"
+PetscErrorCode SVDTRLanczosSetOneSide_TRLanczos(SVD svd,PetscBool oneside)
 {
   SVD_TRLANCZOS *lanczos = (SVD_TRLANCZOS *)svd->data;
 
@@ -404,7 +391,7 @@ EXTERN_C_END
    SVDTRLanczosSetOneSide - Indicate if the variant of the Lanczos method 
    to be used is one-sided or two-sided.
 
-   Collective on SVD
+   Logically Collective on SVD
 
    Input Parameters:
 +  svd     - singular value solver
@@ -422,16 +409,14 @@ EXTERN_C_END
 
 .seealso: SVDLanczosSetOneSide()
 @*/
-PetscErrorCode SVDTRLanczosSetOneSide(SVD svd,PetscTruth oneside)
+PetscErrorCode SVDTRLanczosSetOneSide(SVD svd,PetscBool oneside)
 {
-  PetscErrorCode ierr, (*f)(SVD,PetscTruth);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)svd,"SVDTRLanczosSetOneSide_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(svd,oneside);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveBool(svd,oneside,2);
+  ierr = PetscTryMethod(svd,"SVDTRLanczosSetOneSide_C",(SVD,PetscBool),(svd,oneside));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -441,7 +426,7 @@ PetscErrorCode SVDTRLanczosSetOneSide(SVD svd,PetscTruth oneside)
    SVDTRLanczosGetOneSide - Gets if the variant of the Lanczos method 
    to be used is one-sided or two-sided.
 
-   Collective on SVD
+   Not Collective
 
    Input Parameters:
 .  svd     - singular value solver
@@ -453,80 +438,84 @@ PetscErrorCode SVDTRLanczosSetOneSide(SVD svd,PetscTruth oneside)
 
 .seealso: SVDTRLanczosSetOneSide()
 @*/
-PetscErrorCode SVDTRLanczosGetOneSide(SVD svd,PetscTruth *oneside)
+PetscErrorCode SVDTRLanczosGetOneSide(SVD svd,PetscBool *oneside)
 {
-  PetscErrorCode ierr, (*f)(SVD,PetscTruth*);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = PetscObjectQueryFunction((PetscObject)svd,"SVDTRLanczosGetOneSide_C",(void (**)())&f);CHKERRQ(ierr);
-  if (f) {
-    ierr = (*f)(svd,oneside);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidPointer(oneside,2);
+  ierr = PetscTryMethod(svd,"SVDTRLanczosGetOneSide_C",(SVD,PetscBool*),(svd,oneside));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDTRLanczosGetOneSide_TRLANCZOS"
-PetscErrorCode SVDTRLanczosGetOneSide_TRLANCZOS(SVD svd,PetscTruth *oneside)
+#define __FUNCT__ "SVDTRLanczosGetOneSide_TRLanczos"
+PetscErrorCode SVDTRLanczosGetOneSide_TRLanczos(SVD svd,PetscBool *oneside)
 {
   SVD_TRLANCZOS    *lanczos = (SVD_TRLANCZOS *)svd->data;
 
   PetscFunctionBegin;
-  PetscValidPointer(oneside,2);
   *oneside = lanczos->oneside;
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
 
-
 #undef __FUNCT__  
-#define __FUNCT__ "SVDDestroy_TRLANCZOS"
-PetscErrorCode SVDDestroy_TRLANCZOS(SVD svd)
+#define __FUNCT__ "SVDReset_TRLanczos"
+PetscErrorCode SVDReset_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = SVDDestroy_Default(svd);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(svd->n,&svd->U);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDDestroy_TRLanczos"
+PetscErrorCode SVDDestroy_TRLanczos(SVD svd)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFree(svd->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosSetOneSide_C","",PETSC_NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosGetOneSide_C","",PETSC_NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
-#define __FUNCT__ "SVDView_TRLANCZOS"
-PetscErrorCode SVDView_TRLANCZOS(SVD svd,PetscViewer viewer)
+#define __FUNCT__ "SVDView_TRLanczos"
+PetscErrorCode SVDView_TRLanczos(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   SVD_TRLANCZOS  *lanczos = (SVD_TRLANCZOS *)svd->data;
 
   PetscFunctionBegin;
-  ierr = PetscViewerASCIIPrintf(viewer,"Lanczos reorthogonalization: %s\n",lanczos->oneside ? "one-side" : "two-side");CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  TRLanczos: %s-sided reorthogonalization\n",lanczos->oneside? "one": "two");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 EXTERN_C_BEGIN
 #undef __FUNCT__  
-#define __FUNCT__ "SVDCreate_TRLANCZOS"
-PetscErrorCode SVDCreate_TRLANCZOS(SVD svd)
+#define __FUNCT__ "SVDCreate_TRLanczos"
+PetscErrorCode SVDCreate_TRLanczos(SVD svd)
 {
   PetscErrorCode ierr;
-  SVD_TRLANCZOS  *lanczos;
 
   PetscFunctionBegin;
-  ierr = PetscNew(SVD_TRLANCZOS,&lanczos);CHKERRQ(ierr);
-  PetscLogObjectMemory(svd,sizeof(SVD_TRLANCZOS));
-  svd->data                = (void *)lanczos;
-  svd->ops->setup          = SVDSetUp_TRLANCZOS;
-  svd->ops->solve          = SVDSolve_TRLANCZOS;
-  svd->ops->destroy        = SVDDestroy_TRLANCZOS;
-  svd->ops->setfromoptions = SVDSetFromOptions_TRLANCZOS;
-  svd->ops->view           = SVDView_TRLANCZOS;
-  lanczos->oneside         = PETSC_FALSE;
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosSetOneSide_C","SVDTRLanczosSetOneSide_TRLANCZOS",SVDTRLanczosSetOneSide_TRLANCZOS);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosGetOneSide_C","SVDTRLanczosGetOneSide_TRLANCZOS",SVDTRLanczosGetOneSide_TRLANCZOS);CHKERRQ(ierr);
+  ierr = PetscNewLog(svd,SVD_TRLANCZOS,&svd->data);CHKERRQ(ierr);
+  svd->ops->setup          = SVDSetUp_TRLanczos;
+  svd->ops->solve          = SVDSolve_TRLanczos;
+  svd->ops->destroy        = SVDDestroy_TRLanczos;
+  svd->ops->reset          = SVDReset_TRLanczos;
+  svd->ops->setfromoptions = SVDSetFromOptions_TRLanczos;
+  svd->ops->view           = SVDView_TRLanczos;
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosSetOneSide_C","SVDTRLanczosSetOneSide_TRLanczos",SVDTRLanczosSetOneSide_TRLanczos);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunctionDynamic((PetscObject)svd,"SVDTRLanczosGetOneSide_C","SVDTRLanczosGetOneSide_TRLanczos",SVDTRLanczosGetOneSide_TRLanczos);CHKERRQ(ierr);
+  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
