@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,20 +21,21 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"      /*I "slepcsvd.h" I*/
+#include <private/svdimpl.h>      /*I "slepcsvd.h" I*/
 
-PetscFList SVDList = 0;
-PetscCookie SVD_COOKIE = 0;
-PetscLogEvent SVD_SetUp = 0, SVD_Solve = 0, SVD_Dense = 0;
-static PetscTruth SVDPackageInitialized = PETSC_FALSE;
+PetscFList       SVDList = 0;
+PetscBool        SVDRegisterAllCalled = PETSC_FALSE;
+PetscClassId     SVD_CLASSID = 0;
+PetscLogEvent    SVD_SetUp = 0,SVD_Solve = 0,SVD_Dense = 0;
+static PetscBool SVDPackageInitialized = PETSC_FALSE;
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDFinalizePackage"
 /*@C
-  SVDFinalizePackage - This function destroys everything in the Slepc interface to the SVD package. It is
-  called from SlepcFinalize().
+   SVDFinalizePackage - This function destroys everything in the Slepc interface
+   to the SVD package. It is called from SlepcFinalize().
 
-  Level: developer
+   Level: developer
 
 .seealso: SlepcFinalize()
 @*/
@@ -43,55 +44,56 @@ PetscErrorCode SVDFinalizePackage(void)
   PetscFunctionBegin;
   SVDPackageInitialized = PETSC_FALSE;
   SVDList               = 0;
+  SVDRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDInitializePackage"
 /*@C
-  SVDInitializePackage - This function initializes everything in the SVD package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to SVDCreate()
-  when using static libraries.
+   SVDInitializePackage - This function initializes everything in the SVD package. It is called
+   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to SVDCreate()
+   when using static libraries.
 
-  Input Parameter:
-  path - The dynamic library path, or PETSC_NULL
+   Input Parameter:
+.  path - The dynamic library path, or PETSC_NULL
 
-  Level: developer
+   Level: developer
 
 .seealso: SlepcInitialize()
 @*/
-PetscErrorCode SVDInitializePackage(char *path)
+PetscErrorCode SVDInitializePackage(const char *path)
 {
-  char              logList[256];
-  char              *className;
-  PetscTruth        opt;
-  PetscErrorCode    ierr;
+  char           logList[256];
+  char           *className;
+  PetscBool      opt;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (SVDPackageInitialized) PetscFunctionReturn(0);
   SVDPackageInitialized = PETSC_TRUE;
   /* Register Classes */
-  ierr = PetscCookieRegister("Singular Value Solver",&SVD_COOKIE);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Singular Value Solver",&SVD_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = SVDRegisterAll(path);CHKERRQ(ierr);
   /* Register Events */
-  ierr = PetscLogEventRegister("SVDSetUp",SVD_COOKIE,&SVD_SetUp);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("SVDSolve",SVD_COOKIE,&SVD_Solve);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("SVDDense",SVD_COOKIE,&SVD_Dense);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("SVDSetUp",SVD_CLASSID,&SVD_SetUp);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("SVDSolve",SVD_CLASSID,&SVD_Solve);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("SVDDense",SVD_CLASSID,&SVD_Dense);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL, "-log_info_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "svd", &className);CHKERRQ(ierr);
+    ierr = PetscStrstr(logList,"svd",&className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscInfoDeactivateClass(SVD_COOKIE);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(SVD_CLASSID);CHKERRQ(ierr);
     }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(PETSC_NULL, "-log_summary_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(PETSC_NULL,"-log_summary_exclude",logList,256,&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "svd", &className);CHKERRQ(ierr);
+    ierr = PetscStrstr(logList,"svd",&className);CHKERRQ(ierr);
     if (className) {
-      ierr = PetscLogEventDeactivateClass(SVD_COOKIE);CHKERRQ(ierr);
+      ierr = PetscLogEventDeactivateClass(SVD_CLASSID);CHKERRQ(ierr);
     }
   }
   ierr = PetscRegisterFinalize(SVDFinalizePackage);CHKERRQ(ierr);
@@ -130,31 +132,29 @@ PetscErrorCode SVDInitializePackage(char *path)
 PetscErrorCode SVDView(SVD svd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  const SVDType  type;
-  PetscTruth     isascii;
+  PetscBool      isascii;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)svd)->comm);
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_COOKIE,2);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(svd,1,viewer,2);
 
-  ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"SVD Object:\n");CHKERRQ(ierr);
-    ierr = SVDGetType(svd,&type);CHKERRQ(ierr);
-    if (type) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  method: %s\n",type);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  method: not yet set\n");CHKERRQ(ierr);
+    ierr = PetscObjectPrintClassNamePrefixType((PetscObject)svd,viewer,"SVD Object");CHKERRQ(ierr);
+    if (svd->ops->view) {
+      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
+      ierr = (*svd->ops->view)(svd,viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
     switch (svd->transmode) {
       case SVD_TRANSPOSE_EXPLICIT:
         ierr = PetscViewerASCIIPrintf(viewer,"  transpose mode: explicit\n");CHKERRQ(ierr);
-	break;
+        break;
       case SVD_TRANSPOSE_IMPLICIT:
         ierr = PetscViewerASCIIPrintf(viewer,"  transpose mode: implicit\n");CHKERRQ(ierr);
-	break;
+        break;
       default:
         ierr = PetscViewerASCIIPrintf(viewer,"  transpose mode: not yet set\n");CHKERRQ(ierr);
     }
@@ -163,27 +163,103 @@ PetscErrorCode SVDView(SVD svd,PetscViewer viewer)
     } else {
       ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: smallest\n");CHKERRQ(ierr);
     }  
-    ierr = PetscViewerASCIIPrintf(viewer,"  number of singular values (nsv): %d\n",svd->nsv);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %d\n",svd->ncv);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %d\n",svd->mpd);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %d\n",svd->max_it);
-    ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %g\n",svd->tol);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  number of singular values (nsv): %D\n",svd->nsv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %D\n",svd->ncv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %D\n",svd->mpd);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",svd->max_it);
+    ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %G\n",svd->tol);CHKERRQ(ierr);
     if (svd->nini!=0) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %d\n",PetscAbs(svd->nini));CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided initial space: %D\n",PetscAbs(svd->nini));CHKERRQ(ierr);
     }
-    if (svd->ops->view) {
-      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-      ierr = (*svd->ops->view)(svd,viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
-    }
-    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-    ierr = IPView(svd->ip,viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   } else {
     if (svd->ops->view) {
       ierr = (*svd->ops->view)(svd,viewer);CHKERRQ(ierr);
     }
   }
+  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
+  ierr = IPView(svd->ip,viewer);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__
+#define __FUNCT__ "SVDPrintSolution"
+/*@
+   SVDPrintSolution - Prints the computed singular values.
+
+   Collective on SVD
+
+   Input Parameters:
++  svd - the singular value solver context
+-  viewer - optional visualization context
+
+   Options Database:
+.  -svd_terse - print only minimal information
+
+   Note:
+   By default, this function prints a table with singular values and associated
+   relative errors. With -svd_terse only the singular values are printed.
+
+   Level: intermediate
+
+.seealso: PetscViewerASCIIOpen()
+@*/
+PetscErrorCode SVDPrintSolution(SVD svd,PetscViewer viewer)
+{
+  PetscBool      terse,errok,isascii;
+  PetscReal      error,sigma;
+  PetscInt       i,j;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)svd)->comm);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
+  PetscCheckSameComm(svd,1,viewer,2);
+  if (!svd->sigma) { 
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first"); 
+  }
+  ierr = PetscTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (!isascii) PetscFunctionReturn(0);
+
+  ierr = PetscOptionsHasName(PETSC_NULL,"-svd_terse",&terse);CHKERRQ(ierr);
+  if (terse) {
+    if (svd->nconv<svd->nsv) {
+      ierr = PetscViewerASCIIPrintf(viewer," Problem: less than %D singular values converged\n\n",svd->nsv);CHKERRQ(ierr);
+    } else {
+      errok = PETSC_TRUE;
+      for (i=0;i<svd->nsv;i++) {
+        ierr = SVDComputeRelativeError(svd,i,&error);CHKERRQ(ierr);
+        errok = (errok && error<svd->tol)? PETSC_TRUE: PETSC_FALSE;
+      }
+      if (errok) {
+        ierr = PetscViewerASCIIPrintf(viewer," All requested singular values computed up to the required tolerance:");CHKERRQ(ierr);
+        for (i=0;i<=(svd->nsv-1)/8;i++) {
+          ierr = PetscViewerASCIIPrintf(viewer,"\n     ");CHKERRQ(ierr);
+          for (j=0;j<PetscMin(8,svd->nsv-8*i);j++) {
+            ierr = SVDGetSingularTriplet(svd,8*i+j,&sigma,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+            ierr = PetscViewerASCIIPrintf(viewer,"%.5F",sigma);CHKERRQ(ierr); 
+            if (8*i+j+1<svd->nsv) { ierr = PetscViewerASCIIPrintf(viewer,", ");CHKERRQ(ierr); }
+          }
+        }
+        ierr = PetscViewerASCIIPrintf(viewer,"\n\n");CHKERRQ(ierr);
+      } else {
+        ierr = PetscViewerASCIIPrintf(viewer," Problem: some of the first %D relative errors are higher than the tolerance\n\n",svd->nsv);CHKERRQ(ierr);
+      }
+    }
+  } else {
+    ierr = PetscViewerASCIIPrintf(viewer," Number of converged approximate singular triplets: %D\n\n",svd->nconv);CHKERRQ(ierr);
+    if (svd->nconv>0) {
+      ierr = PetscViewerASCIIPrintf(viewer,
+           "          sigma            relative error\n"
+           "   --------------------- ------------------\n");CHKERRQ(ierr);
+      for (i=0;i<svd->nconv;i++) {
+        ierr = SVDGetSingularTriplet(svd,i,&sigma,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+        ierr = SVDComputeRelativeError(svd,i,&error);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"       % 6F          %12G\n",sigma,error);CHKERRQ(ierr); 
+      }
+      ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
@@ -214,51 +290,90 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
 
   PetscFunctionBegin;
   PetscValidPointer(outsvd,2);
+  *outsvd = 0;
+  ierr = PetscHeaderCreate(svd,_p_SVD,struct _SVDOps,SVD_CLASSID,-1,"SVD","Singular Value Decomposition","SVD",comm,SVDDestroy,SVDView);CHKERRQ(ierr);
 
-  ierr = PetscHeaderCreate(svd,_p_SVD,struct _SVDOps,SVD_COOKIE,-1,"SVD",comm,SVDDestroy,SVDView);CHKERRQ(ierr);
-  *outsvd = svd;
-
-  ierr = PetscMemzero(svd->ops,sizeof(struct _SVDOps));CHKERRQ(ierr);
-
-  svd->OP          = PETSC_NULL;
-  svd->A           = PETSC_NULL;
-  svd->AT          = PETSC_NULL;
-  svd->transmode   = (SVDTransposeMode)PETSC_DECIDE;
-  svd->sigma       = PETSC_NULL;
-  svd->perm        = PETSC_NULL;
-  svd->U           = PETSC_NULL;
-  svd->V           = PETSC_NULL;
-  svd->IS          = PETSC_NULL;
-  svd->which       = SVD_LARGEST;
-  svd->n           = 0;
-  svd->nconv       = 0;
-  svd->nsv         = 1;    
-  svd->ncv         = 0;    
-  svd->mpd         = 0;    
-  svd->nini        = 0;
-  svd->its         = 0;
-  svd->max_it      = 0;  
-  svd->tol         = 1e-7;    
-  svd->errest      = PETSC_NULL;
-  svd->data        = PETSC_NULL;
-  svd->setupcalled = 0;
-  svd->reason      = SVD_CONVERGED_ITERATING;
+  svd->OP             = PETSC_NULL;
+  svd->A              = PETSC_NULL;
+  svd->AT             = PETSC_NULL;
+  svd->transmode      = (SVDTransposeMode)PETSC_DECIDE;
+  svd->sigma          = PETSC_NULL;
+  svd->perm           = PETSC_NULL;
+  svd->U              = PETSC_NULL;
+  svd->V              = PETSC_NULL;
+  svd->IS             = PETSC_NULL;
+  svd->tl             = PETSC_NULL;
+  svd->tr             = PETSC_NULL;
+  svd->rand           = PETSC_NULL;
+  svd->which          = SVD_LARGEST;
+  svd->n              = 0;
+  svd->nconv          = 0;
+  svd->nsv            = 1;    
+  svd->ncv            = 0;    
+  svd->mpd            = 0;    
+  svd->nini           = 0;
+  svd->its            = 0;
+  svd->max_it         = 0;  
+  svd->tol            = PETSC_DEFAULT;    
+  svd->errest         = PETSC_NULL;
+  svd->data           = PETSC_NULL;
+  svd->setupcalled    = 0;
+  svd->reason         = SVD_CONVERGED_ITERATING;
   svd->numbermonitors = 0;
-  svd->matvecs = 0;
-  svd->trackall    = PETSC_FALSE;
+  svd->matvecs        = 0;
+  svd->trackall       = PETSC_FALSE;
 
-  ierr = IPCreate(comm,&svd->ip);CHKERRQ(ierr);
-  ierr = IPSetOptionsPrefix(svd->ip,((PetscObject)svd)->prefix);
-  ierr = IPAppendOptionsPrefix(svd->ip,"svd_");
-  PetscLogObjectParent(svd,svd->ip);
-
-  ierr = PetscPublishAll(svd);CHKERRQ(ierr);
+  ierr = PetscRandomCreate(comm,&svd->rand);CHKERRQ(ierr);
+  ierr = PetscRandomSetSeed(svd->rand,0x12345678);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent(svd,svd->rand);CHKERRQ(ierr);
+  *outsvd = svd;
   PetscFunctionReturn(0);
 }
  
 #undef __FUNCT__  
-#define __FUNCT__ "SVDDestroy"
+#define __FUNCT__ "SVDReset"
 /*@
+   SVDReset - Resets the SVD context to the setupcalled=0 state and removes any
+   allocated objects.
+
+   Collective on SVD
+
+   Input Parameter:
+.  svd - singular value solver context obtained from SVDCreate()
+
+   Level: advanced
+
+.seealso: SVDDestroy()
+@*/
+PetscErrorCode SVDReset(SVD svd)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  if (svd->ops->reset) { ierr = (svd->ops->reset)(svd);CHKERRQ(ierr); }
+  if (svd->ip) { ierr = IPReset(svd->ip);CHKERRQ(ierr); }
+  ierr = MatDestroy(&svd->OP);CHKERRQ(ierr);
+  ierr = MatDestroy(&svd->A);CHKERRQ(ierr);
+  ierr = MatDestroy(&svd->AT);CHKERRQ(ierr);
+  ierr = VecDestroy(&svd->tl);CHKERRQ(ierr);
+  ierr = VecDestroy(&svd->tr);CHKERRQ(ierr);
+  if (svd->n) { 
+    ierr = PetscFree(svd->sigma);CHKERRQ(ierr);
+    ierr = PetscFree(svd->perm);CHKERRQ(ierr);
+    ierr = PetscFree(svd->errest);CHKERRQ(ierr);
+    ierr = VecDestroyVecs(svd->n,&svd->U);CHKERRQ(ierr);
+    ierr = VecDestroyVecs(svd->n,&svd->V);CHKERRQ(ierr);
+  }
+  svd->transmode   = (SVDTransposeMode)PETSC_DECIDE;
+  svd->matvecs     = 0;
+  svd->setupcalled = 0;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDDestroy"
+/*@C
    SVDDestroy - Destroys the SVD context.
 
    Collective on SVD
@@ -270,74 +385,30 @@ PetscErrorCode SVDCreate(MPI_Comm comm,SVD *outsvd)
 
 .seealso: SVDCreate(), SVDSetUp(), SVDSolve()
 @*/
-PetscErrorCode SVDDestroy(SVD svd)
+PetscErrorCode SVDDestroy(SVD *svd)
 {
   PetscErrorCode ierr;
-  PetscInt       i;
-  PetscScalar    *p;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  if (--((PetscObject)svd)->refct > 0) PetscFunctionReturn(0);
-
-  /* if memory was published with AMS then destroy it */
-  ierr = PetscObjectDepublish(svd);CHKERRQ(ierr);
-
-  if (svd->ops->destroy) {
-    ierr = (*svd->ops->destroy)(svd); CHKERRQ(ierr);
-  }
-
-  if (svd->OP) { ierr = MatDestroy(svd->OP);CHKERRQ(ierr); }
-  if (svd->A) { ierr = MatDestroy(svd->A);CHKERRQ(ierr); }
-  if (svd->AT) { ierr = MatDestroy(svd->AT);CHKERRQ(ierr); }
-  if (svd->n) { 
-    ierr = PetscFree(svd->sigma);CHKERRQ(ierr);
-    ierr = PetscFree(svd->perm);CHKERRQ(ierr);
-    ierr = PetscFree(svd->errest);CHKERRQ(ierr);
-    if (svd->U) {
-      ierr = VecGetArray(svd->U[0],&p);CHKERRQ(ierr);
-      for (i=0;i<svd->n;i++) {
-        ierr = VecDestroy(svd->U[i]); CHKERRQ(ierr);
-      }
-      ierr = PetscFree(p);CHKERRQ(ierr);
-      ierr = PetscFree(svd->U);CHKERRQ(ierr);
-    }
-    ierr = VecGetArray(svd->V[0],&p);CHKERRQ(ierr);
-    for (i=0;i<svd->n;i++) {
-      ierr = VecDestroy(svd->V[i]);CHKERRQ(ierr); 
-    }
-    ierr = PetscFree(p);CHKERRQ(ierr);
-    ierr = PetscFree(svd->V);CHKERRQ(ierr);
-  }
-  ierr = SVDMonitorCancel(svd);CHKERRQ(ierr);
-  
-  ierr = IPDestroy(svd->ip);CHKERRQ(ierr);
-  if (svd->rand) {
-    ierr = PetscRandomDestroy(svd->rand);CHKERRQ(ierr);
-  }
-  
+  if (!*svd) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(*svd,SVD_CLASSID,1);
+  if (--((PetscObject)(*svd))->refct > 0) { *svd = 0; PetscFunctionReturn(0); }
+  ierr = SVDReset(*svd);CHKERRQ(ierr);
+  ierr = PetscObjectDepublish(*svd);CHKERRQ(ierr);
+  if ((*svd)->ops->destroy) { ierr = (*(*svd)->ops->destroy)(*svd);CHKERRQ(ierr); }
+  ierr = IPDestroy(&(*svd)->ip);CHKERRQ(ierr);
+  ierr = PetscRandomDestroy(&(*svd)->rand);CHKERRQ(ierr);
+  ierr = SVDMonitorCancel(*svd);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(svd);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "SVDDestroy_Default"
-PetscErrorCode SVDDestroy_Default(SVD svd)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  ierr = PetscFree(svd->data);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "SVDSetType"
 /*@C
    SVDSetType - Selects the particular solver to be used in the SVD object. 
 
-   Collective on SVD
+   Logically Collective on SVD
 
    Input Parameters:
 +  svd      - the singular value solver context
@@ -366,30 +437,24 @@ PetscErrorCode SVDDestroy_Default(SVD svd)
 PetscErrorCode SVDSetType(SVD svd,const SVDType type)
 {
   PetscErrorCode ierr,(*r)(SVD);
-  PetscTruth match;
+  PetscBool      match;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidCharPointer(type,2);
 
   ierr = PetscTypeCompare((PetscObject)svd,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  if (svd->data) {
-    /* destroy the old private SVD context */
-    ierr = (*svd->ops->destroy)(svd); CHKERRQ(ierr);
-    svd->data = 0;
-  }
+  ierr = PetscFListFind(SVDList,((PetscObject)svd)->comm,type,PETSC_TRUE,(void (**)(void)) &r);CHKERRQ(ierr);
+  if (!r) SETERRQ1(((PetscObject)svd)->comm,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown SVD type given: %s",type);
 
-  ierr = PetscFListFind(SVDList,((PetscObject)svd)->comm,type,(void (**)(void)) &r);CHKERRQ(ierr);
-
-  if (!r) SETERRQ1(1,"Unknown SVD type given: %s",type);
+  if (svd->ops->destroy) { ierr = (*svd->ops->destroy)(svd);CHKERRQ(ierr); }
+  ierr = PetscMemzero(svd->ops,sizeof(struct _SVDOps));CHKERRQ(ierr);
 
   svd->setupcalled = 0;
-  ierr = PetscMemzero(svd->ops,sizeof(struct _SVDOps));CHKERRQ(ierr);
-  ierr = (*r)(svd); CHKERRQ(ierr);
-
   ierr = PetscObjectChangeTypeName((PetscObject)svd,type);CHKERRQ(ierr);
+  ierr = (*r)(svd);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -413,64 +478,23 @@ PetscErrorCode SVDSetType(SVD svd,const SVDType type)
 PetscErrorCode SVDGetType(SVD svd,const SVDType *type)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(type,2);
   *type = ((PetscObject)svd)->type_name;
   PetscFunctionReturn(0);
 }
 
-/*MC
-   SVDRegisterDynamic - Adds a method to the singular value solver package.
-
-   Synopsis:
-   SVDRegisterDynamic(char *name_solver,char *path,char *name_create,PetscErrorCode (*routine_create)(SVD))
-
-   Not Collective
-
-   Input Parameters:
-+  name_solver - name of a new user-defined solver
-.  path - path (either absolute or relative) the library containing this solver
-.  name_create - name of routine to create the solver context
--  routine_create - routine to create the solver context
-
-   Notes:
-   SVDRegisterDynamic() may be called multiple times to add several user-defined solvers.
-
-   If dynamic libraries are used, then the fourth input argument (routine_create)
-   is ignored.
-
-   Sample usage:
-.vb
-   SVDRegisterDynamic("my_solver",/home/username/my_lib/lib/libO/solaris/mylib.a,
-               "MySolverCreate",MySolverCreate);
-.ve
-
-   Then, your solver can be chosen with the procedural interface via
-$     SVDSetType(svd,"my_solver")
-   or at runtime via the option
-$     -svd_type my_solver
-
-   Level: advanced
-
-   Environmental variables such as ${PETSC_ARCH}, ${SLEPC_DIR},
-   and others of the form ${any_environmental_variable} occuring in pathname will be 
-   replaced with appropriate values.
-
-.seealso: SVDRegisterDestroy(), SVDRegisterAll()
-
-M*/
-
 #undef __FUNCT__  
 #define __FUNCT__ "SVDRegister"
 /*@C
-  SVDRegister - See SVDRegisterDynamic()
+   SVDRegister - See SVDRegisterDynamic()
 
-  Level: advanced
+   Level: advanced
 @*/
 PetscErrorCode SVDRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(SVD))
 {
   PetscErrorCode ierr;
-  char           fullname[256];
+  char           fullname[PETSC_MAX_PATH_LEN];
 
   PetscFunctionBegin;
   ierr = PetscFListConcat(path,name,fullname);CHKERRQ(ierr);
@@ -496,7 +520,7 @@ PetscErrorCode SVDRegisterDestroy(void)
 
   PetscFunctionBegin;
   ierr = PetscFListDestroy(&SVDList);CHKERRQ(ierr);
-  ierr = SVDRegisterAll(PETSC_NULL);CHKERRQ(ierr);
+  SVDRegisterAllCalled = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
@@ -525,12 +549,13 @@ PetscErrorCode SVDSetIP(SVD svd,IP ip)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  PetscValidHeaderSpecific(ip,IP_COOKIE,2);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidHeaderSpecific(ip,IP_CLASSID,2);
   PetscCheckSameComm(svd,1,ip,2);
   ierr = PetscObjectReference((PetscObject)ip);CHKERRQ(ierr);
-  ierr = IPDestroy(svd->ip); CHKERRQ(ierr);
+  ierr = IPDestroy(&svd->ip);CHKERRQ(ierr);
   svd->ip = ip;
+  ierr = PetscLogObjectParent(svd,svd->ip);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -554,9 +579,15 @@ PetscErrorCode SVDSetIP(SVD svd,IP ip)
 @*/
 PetscErrorCode SVDGetIP(SVD svd,IP *ip)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(ip,2);
+  if (!svd->ip) {
+    ierr = IPCreate(((PetscObject)svd)->comm,&svd->ip);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent(svd,svd->ip);CHKERRQ(ierr);
+  }
   *ip = svd->ip;
   PetscFunctionReturn(0);
 }
