@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,18 +21,15 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"      /*I "slepcsvd.h" I*/
+#include <private/svdimpl.h>      /*I "slepcsvd.h" I*/
 
 #undef __FUNCT__
 #define __FUNCT__ "SVDMatMult"
-PetscErrorCode SVDMatMult(SVD svd,PetscTruth trans,Vec x,Vec y)
+PetscErrorCode SVDMatMult(SVD svd,PetscBool trans,Vec x,Vec y)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  PetscValidHeaderSpecific(x,VEC_COOKIE,3);
-  PetscValidHeaderSpecific(y,VEC_COOKIE,4);
   svd->matvecs++;
   if (trans) {
     if (svd->AT) {
@@ -57,7 +54,6 @@ PetscErrorCode SVDMatGetVecs(SVD svd,Vec *x,Vec *y)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
   if (svd->A) {
     ierr = MatGetVecs(svd->A,x,y);CHKERRQ(ierr);
   } else {
@@ -73,7 +69,6 @@ PetscErrorCode SVDMatGetSize(SVD svd,PetscInt *m,PetscInt *n)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
   if (svd->A) {
     ierr = MatGetSize(svd->A,m,n);CHKERRQ(ierr);
   } else {
@@ -89,7 +84,6 @@ PetscErrorCode SVDMatGetLocalSize(SVD svd,PetscInt *m,PetscInt *n)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
   if (svd->A) {
     ierr = MatGetLocalSize(svd->A,m,n);CHKERRQ(ierr);
   } else {
