@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,24 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"   /*I "slepcsvd.h" I*/
+#include <private/svdimpl.h>   /*I "slepcsvd.h" I*/
+
+#undef __FUNCT__  
+#define __FUNCT__ "SVDMonitor"
+/*
+   Runs the user provided monitor routines, if any.
+*/
+PetscErrorCode SVDMonitor(SVD svd,PetscInt it,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,n = svd->numbermonitors;
+
+  PetscFunctionBegin;
+  for (i=0;i<n;i++) {
+    ierr = (*svd->monitor[i])(svd,it,nconv,sigma,errest,nest,svd->monitorcontext[i]);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDMonitorSet"
@@ -68,15 +85,14 @@ $     monitor (SVD svd, PetscInt its, PetscInt nconv, PetscReal *sigma, PetscRea
 
    Level: intermediate
 
-.seealso: SVDMonitorFirst(), SVDMonitorAll(), SVDMonitorLG(), SVDMonitorLGAll(), SVDMonitorCancel()
+.seealso: SVDMonitorFirst(), SVDMonitorAll(), SVDMonitorCancel()
 @*/
-PetscErrorCode SVDMonitorSet(SVD svd,PetscErrorCode (*monitor)(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,void*),
-                             void *mctx,PetscErrorCode (*monitordestroy)(void*))
+PetscErrorCode SVDMonitorSet(SVD svd,PetscErrorCode (*monitor)(SVD,PetscInt,PetscInt,PetscReal*,PetscReal*,PetscInt,void*),void *mctx,PetscErrorCode (*monitordestroy)(void**))
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (svd->numbermonitors >= MAXSVDMONITORS) {
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Too many SVD monitors set");
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Too many SVD monitors set");
   }
   svd->monitor[svd->numbermonitors]           = monitor;
   svd->monitorcontext[svd->numbermonitors]    = (void*)mctx;
@@ -109,10 +125,10 @@ PetscErrorCode SVDMonitorCancel(SVD svd)
   PetscInt       i;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   for (i=0; i<svd->numbermonitors; i++) {
     if (svd->monitordestroy[i]) {
-      ierr = (*svd->monitordestroy[i])(svd->monitorcontext[i]);CHKERRQ(ierr);
+      ierr = (*svd->monitordestroy[i])(&svd->monitorcontext[i]);CHKERRQ(ierr);
     }
   }
   svd->numbermonitors = 0;
@@ -137,11 +153,11 @@ PetscErrorCode SVDMonitorCancel(SVD svd)
 
 .seealso: SVDMonitorSet()
 @*/
-PetscErrorCode SVDGetMonitorContext(SVD svd, void **ctx)
+PetscErrorCode SVDGetMonitorContext(SVD svd,void **ctx)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  *ctx =      (svd->monitorcontext[0]);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  *ctx = (svd->monitorcontext[0]);
   PetscFunctionReturn(0);
 }
 
@@ -160,27 +176,27 @@ PetscErrorCode SVDGetMonitorContext(SVD svd, void **ctx)
 .  sigma  - singular values
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  dummy  - unused monitor context 
+-  monctx - monitor context (contains viewer, can be PETSC_NULL)
 
    Level: intermediate
 
-.seealso: SVDMonitorSet()
+.seealso: SVDMonitorSet(), SVDMonitorFirst(), SVDMonitorConverged()
 @*/
-PetscErrorCode SVDMonitorAll(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest,void *dummy)
+PetscErrorCode SVDMonitorAll(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest,void *monctx)
 {
-  PetscErrorCode          ierr;
-  PetscInt                i;
-  PetscViewerASCIIMonitor viewer = (PetscViewerASCIIMonitor) dummy;
+  PetscErrorCode ierr;
+  PetscInt       i;
+  PetscViewer    viewer = monctx? (PetscViewer)monctx: PETSC_VIEWER_STDOUT_(((PetscObject)svd)->comm);
 
   PetscFunctionBegin;
   if (its) {
-    if (!dummy) {ierr = PetscViewerASCIIMonitorCreate(((PetscObject)svd)->comm,"stdout",0,&viewer);CHKERRQ(ierr);}
-    ierr = PetscViewerASCIIMonitorPrintf(viewer,"%3d SVD nconv=%d Values (Errors)",its,nconv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)svd)->tablevel);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%3D SVD nconv=%D Values (Errors)",its,nconv);CHKERRQ(ierr);
     for (i=0;i<nest;i++) {
-      ierr = PetscViewerASCIIMonitorPrintf(viewer," %g (%10.8e)",sigma[i],errest[i]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer," %G (%10.8e)",sigma[i],(double)errest[i]);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIMonitorPrintf(viewer,"\n");CHKERRQ(ierr);
-    if (!dummy) {ierr = PetscViewerASCIIMonitorDestroy(viewer);CHKERRQ(ierr);}
+    ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+    ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)svd)->tablevel);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -200,23 +216,23 @@ PetscErrorCode SVDMonitorAll(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigm
 .  sigma  - singular values
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  dummy  - unused monitor context 
+-  monctx - monitor context (contains viewer, can be PETSC_NULL)
 
    Level: intermediate
 
-.seealso: SVDMonitorSet()
+.seealso: SVDMonitorSet(), SVDMonitorAll(), SVDMonitorConverged()
 @*/
-PetscErrorCode SVDMonitorFirst(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest,void *dummy)
+PetscErrorCode SVDMonitorFirst(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest,void *monctx)
 {
-  PetscErrorCode          ierr;
-  PetscViewerASCIIMonitor viewer = (PetscViewerASCIIMonitor) dummy;
+  PetscErrorCode ierr;
+  PetscViewer    viewer = monctx? (PetscViewer)monctx: PETSC_VIEWER_STDOUT_(((PetscObject)svd)->comm);
 
   PetscFunctionBegin;
   if (its && nconv<nest) {
-    if (!dummy) {ierr = PetscViewerASCIIMonitorCreate(((PetscObject)svd)->comm,"stdout",0,&viewer);CHKERRQ(ierr);}
-    ierr = PetscViewerASCIIMonitorPrintf(viewer,"%3d SVD nconv=%d first unconverged value (error)",its,nconv);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIMonitorPrintf(viewer," %g (%10.8e)\n",sigma[nconv],errest[nconv]);CHKERRQ(ierr);
-    if (!dummy) {ierr = PetscViewerASCIIMonitorDestroy(viewer);CHKERRQ(ierr);}
+    ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)svd)->tablevel);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%3D SVD nconv=%D first unconverged value (error)",its,nconv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer," %G (%10.8e)\n",sigma[nconv],(double)errest[nconv]);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)svd)->tablevel);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -235,56 +251,52 @@ PetscErrorCode SVDMonitorFirst(SVD svd,PetscInt its,PetscInt nconv,PetscReal *si
 .  sigma  - singular values
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  dummy  - unused monitor context 
+-  monctx - monitor context 
+
+   Note:
+   The monitor context must contain a struct with a PetscViewer and a
+   PetscInt. In Fortran, pass a PETSC_NULL_OBJECT.
 
    Level: intermediate
 
-.seealso: SVDMonitorSet()
+.seealso: SVDMonitorSet(), SVDMonitorFirst(), SVDMonitorAll()
 @*/
-PetscErrorCode SVDMonitorConverged(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest,void *dummy)
+PetscErrorCode SVDMonitorConverged(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest,void *monctx)
 {
-  PetscErrorCode  ierr;
-  PetscInt        i;
-  SVDMONITOR_CONV *ctx = (SVDMONITOR_CONV*) dummy;
+  PetscErrorCode   ierr;
+  PetscInt         i;
+  PetscViewer      viewer;
+  SlepcConvMonitor ctx = (SlepcConvMonitor) monctx;
 
   PetscFunctionBegin;
+  if (!monctx) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONG,"Must provide a context for SVDMonitorConverged");
   if (!its) {
     ctx->oldnconv = 0;
   } else {
+    viewer = ctx->viewer? ctx->viewer: PETSC_VIEWER_STDOUT_(((PetscObject)svd)->comm);
     for (i=ctx->oldnconv;i<nconv;i++) {
-      ierr = PetscViewerASCIIMonitorPrintf(ctx->viewer,"%3d SVD converged value (error) #%d",its,i);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIMonitorPrintf(ctx->viewer," %g (%10.8e)\n",sigma[i],errest[i]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)svd)->tablevel);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"%3D SVD converged value (error) #%D",its,i);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer," %G (%10.8e)\n",sigma[i],(double)errest[i]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)svd)->tablevel);CHKERRQ(ierr);
     }
     ctx->oldnconv = nconv;
   }
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode SVDMonitorDestroy_Converged(SVDMONITOR_CONV *ctx)
-{
-  PetscErrorCode  ierr;
-  PetscFunctionBegin;
-  ierr = PetscViewerASCIIMonitorDestroy(ctx->viewer);CHKERRQ(ierr);
-  ierr = PetscFree(ctx);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "SVDMonitorLG"
 PetscErrorCode SVDMonitorLG(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest,void *monctx)
 {
   PetscViewer    viewer = (PetscViewer) monctx;
-  PetscDraw      draw;
-  PetscDrawLG    lg;
+  PetscDraw      draw,draw1;
+  PetscDrawLG    lg,lg1;
   PetscErrorCode ierr;
   PetscReal      x,y,p;
-  PetscDraw      draw1;
-  PetscDrawLG    lg1;
 
   PetscFunctionBegin;
-
   if (!viewer) { viewer = PETSC_VIEWER_DRAW_(((PetscObject)svd)->comm); }
-
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDrawLG(viewer,0,&lg);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDraw(viewer,1,&draw1);CHKERRQ(ierr);
@@ -323,19 +335,14 @@ PetscErrorCode SVDMonitorLG(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma
 PetscErrorCode SVDMonitorLGAll(SVD svd,PetscInt its,PetscInt nconv,PetscReal *sigma,PetscReal *errest,PetscInt nest,void *monctx)
 {
   PetscViewer    viewer = (PetscViewer) monctx;
-  PetscDraw      draw;
-  PetscDrawLG    lg;
+  PetscDraw      draw,draw1;
+  PetscDrawLG    lg,lg1;
   PetscErrorCode ierr;
   PetscReal      *x,*y,p;
-  int            n = PetscMin(svd->nsv,255);
-  PetscInt       i;
-  PetscDraw      draw1;
-  PetscDrawLG    lg1;
+  PetscInt       i,n = PetscMin(svd->nsv,255);
 
   PetscFunctionBegin;
-
   if (!viewer) { viewer = PETSC_VIEWER_DRAW_(((PetscObject)svd)->comm); }
-
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDrawLG(viewer,0,&lg);CHKERRQ(ierr);
   ierr = PetscViewerDrawGetDraw(viewer,1,&draw1);CHKERRQ(ierr);
