@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"      /*I "slepcsvd.h" I*/
+#include <private/svdimpl.h>      /*I "slepcsvd.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDSetTransposeMode"
@@ -29,7 +29,7 @@
    SVDSetTransposeMode - Sets how to handle the transpose of the matrix 
    associated with the singular value problem.
 
-   Collective on SVD
+   Logically Collective on SVD
 
    Input Parameters:
 +  svd  - the singular value solver context
@@ -60,16 +60,19 @@
 PetscErrorCode SVDSetTransposeMode(SVD svd,SVDTransposeMode mode)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(svd,mode,2);
   if (mode == PETSC_DEFAULT || mode == PETSC_DECIDE) mode = (SVDTransposeMode)PETSC_DECIDE;
   else switch (mode) {
     case SVD_TRANSPOSE_EXPLICIT:
     case SVD_TRANSPOSE_IMPLICIT:
-      svd->transmode = mode;
-      svd->setupcalled = 0;
+      if (svd->transmode!=mode) {
+        svd->transmode = mode;
+        svd->setupcalled = 0;
+      }
       break;
     default:
-      SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid transpose mode"); 
+      SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid transpose mode"); 
   }
   PetscFunctionReturn(0);
 }
@@ -80,7 +83,7 @@ PetscErrorCode SVDSetTransposeMode(SVD svd,SVDTransposeMode mode)
    SVDGetTransposeMode - Gets the mode used to compute the transpose 
    of the matrix associated with the singular value problem.
 
-   Not collective
+   Not Collective
 
    Input Parameter:
 .  svd  - the singular value solver context
@@ -97,7 +100,7 @@ PetscErrorCode SVDSetTransposeMode(SVD svd,SVDTransposeMode mode)
 PetscErrorCode SVDGetTransposeMode(SVD svd,SVDTransposeMode *mode)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(mode,2);
   *mode = svd->transmode;
   PetscFunctionReturn(0);
@@ -109,7 +112,7 @@ PetscErrorCode SVDGetTransposeMode(SVD svd,SVDTransposeMode *mode)
    SVDSetTolerances - Sets the tolerance and maximum
    iteration count used by the default SVD convergence testers. 
 
-   Collective on SVD
+   Logically Collective on SVD
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -131,12 +134,14 @@ PetscErrorCode SVDGetTransposeMode(SVD svd,SVDTransposeMode *mode)
 PetscErrorCode SVDSetTolerances(SVD svd,PetscReal tol,PetscInt maxits)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveReal(svd,tol,2);
+  PetscValidLogicalCollectiveInt(svd,maxits,3);
   if (tol != PETSC_IGNORE) {
     if (tol == PETSC_DEFAULT) {
-      tol = 1e-7;
+      tol = PETSC_DEFAULT;
     } else {
-      if (tol < 0.0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
+      if (tol < 0.0) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
       svd->tol = tol;
     }
   }
@@ -145,7 +150,7 @@ PetscErrorCode SVDSetTolerances(SVD svd,PetscReal tol,PetscInt maxits)
       svd->max_it = 0;
       svd->setupcalled = 0;
     } else {
-      if (maxits < 0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
+      if (maxits < 0) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
       svd->max_it = maxits;
     }
   }
@@ -177,7 +182,7 @@ PetscErrorCode SVDSetTolerances(SVD svd,PetscReal tol,PetscInt maxits)
 PetscErrorCode SVDGetTolerances(SVD svd,PetscReal *tol,PetscInt *maxits)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (tol)    *tol    = svd->tol;
   if (maxits) *maxits = svd->max_it;
   PetscFunctionReturn(0);
@@ -189,7 +194,7 @@ PetscErrorCode SVDGetTolerances(SVD svd,PetscReal *tol,PetscInt *maxits)
    SVDSetDimensions - Sets the number of singular values to compute
    and the dimension of the subspace.
 
-   Collective on SVD
+   Logically Collective on SVD
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -209,9 +214,9 @@ PetscErrorCode SVDGetTolerances(SVD svd,PetscReal *tol,PetscInt *maxits)
    dependent on the solution method and the number of singular values required.
 
    The parameters ncv and mpd are intimately related, so that the user is advised
-   to set one of them at most. Normal usage is the following:
-+  - In cases where nsv is small, the user sets ncv (a reasonable default is 2*nsv).
--  - In cases where nsv is large, the user sets mpd.
+   to set one of them at most. Normal usage is that
+   (a) in cases where nsv is small, the user sets ncv (a reasonable default is 2*nsv); and
+   (b) in cases where nsv is large, the user sets mpd.
 
    The value of ncv should always be between nsv and (nsv+mpd), typically
    ncv=nsv+mpd. If nev is not too large, mpd=nsv is a reasonable choice, otherwise
@@ -224,26 +229,28 @@ PetscErrorCode SVDGetTolerances(SVD svd,PetscReal *tol,PetscInt *maxits)
 PetscErrorCode SVDSetDimensions(SVD svd,PetscInt nsv,PetscInt ncv,PetscInt mpd)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveInt(svd,nsv,2);
+  PetscValidLogicalCollectiveInt(svd,ncv,3);
+  PetscValidLogicalCollectiveInt(svd,mpd,4);
   if (nsv != PETSC_IGNORE) {
-    if (nsv<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nsv. Must be > 0");
+    if (nsv<1) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nsv. Must be > 0");
     svd->nsv = nsv;
   }
   if (ncv != PETSC_IGNORE) {
     if (ncv == PETSC_DEFAULT || ncv == PETSC_DECIDE) {
       svd->ncv = 0;
     } else {
-      if (ncv<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
+      if (ncv<1) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
       svd->ncv = ncv;
     }
     svd->setupcalled = 0;
   }
-  if( mpd != PETSC_IGNORE ) {
+  if (mpd != PETSC_IGNORE) {
     if (mpd == PETSC_DECIDE || mpd == PETSC_DEFAULT) {
       svd->mpd = 0;
     } else {
-      if (mpd<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
+      if (mpd<1) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
       svd->mpd = mpd;
     }
   }
@@ -276,7 +283,7 @@ PetscErrorCode SVDSetDimensions(SVD svd,PetscInt nsv,PetscInt ncv,PetscInt mpd)
 PetscErrorCode SVDGetDimensions(SVD svd,PetscInt *nsv,PetscInt *ncv,PetscInt *mpd)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (nsv) *nsv = svd->nsv;
   if (ncv) *ncv = svd->ncv;
   if (mpd) *mpd = svd->mpd;
@@ -289,7 +296,7 @@ PetscErrorCode SVDGetDimensions(SVD svd,PetscInt *nsv,PetscInt *ncv,PetscInt *mp
     SVDSetWhichSingularTriplets - Specifies which singular triplets are 
     to be sought.
 
-    Collective on SVD
+    Logically Collective on SVD
 
     Input Parameter:
 .   svd - singular value solver context obtained from SVDCreate()
@@ -314,7 +321,8 @@ PetscErrorCode SVDGetDimensions(SVD svd,PetscInt *nsv,PetscInt *ncv,PetscInt *mp
 PetscErrorCode SVDSetWhichSingularTriplets(SVD svd,SVDWhich which)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveEnum(svd,which,2);
   switch (which) {
     case SVD_LARGEST:
     case SVD_SMALLEST:
@@ -324,7 +332,7 @@ PetscErrorCode SVDSetWhichSingularTriplets(SVD svd,SVDWhich which)
       }
       break;
   default:
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' parameter");    
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' parameter");    
   }
   PetscFunctionReturn(0);
 }
@@ -353,7 +361,7 @@ PetscErrorCode SVDSetWhichSingularTriplets(SVD svd,SVDWhich which)
 PetscErrorCode SVDGetWhichSingularTriplets(SVD svd,SVDWhich *which) 
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(which,2);
   *which = svd->which;
   PetscFunctionReturn(0);
@@ -380,18 +388,19 @@ PetscErrorCode SVDGetWhichSingularTriplets(SVD svd,SVDWhich *which)
 @*/
 PetscErrorCode SVDSetFromOptions(SVD svd)
 {
-  PetscErrorCode ierr;
-  char           type[256],monfilename[PETSC_MAX_PATH_LEN];
-  PetscTruth     flg;
-  const char     *mode_list[2] = { "explicit", "implicit" };
-  PetscInt       i,j,k;
-  PetscReal      r;
-  PetscViewerASCIIMonitor monviewer;
-  SVDMONITOR_CONV *ctx;
+  PetscErrorCode   ierr;
+  char             type[256],monfilename[PETSC_MAX_PATH_LEN];
+  PetscBool        flg;
+  const char       *mode_list[2] = {"explicit","implicit"};
+  PetscInt         i,j,k;
+  PetscReal        r;
+  PetscViewer      monviewer;
+  SlepcConvMonitor ctx;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   svd->setupcalled = 0;
+  if (!SVDRegisterAllCalled) { ierr = SVDRegisterAll(PETSC_NULL);CHKERRQ(ierr); }
   ierr = PetscOptionsBegin(((PetscObject)svd)->comm,((PetscObject)svd)->prefix,"Singular Value Solver (SVD) Options","SVD");CHKERRQ(ierr);
 
   ierr = PetscOptionsList("-svd_type","Singular Value Solver method","SVDSetType",SVDList,(char*)(((PetscObject)svd)->type_name?((PetscObject)svd)->type_name:SVDCROSS),type,256,&flg);CHKERRQ(ierr);
@@ -401,7 +410,7 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
     ierr = SVDSetType(svd,SVDCROSS);CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsName("-svd_view","Print detailed information on solver used","SVDView",0);CHKERRQ(ierr);
+  ierr = PetscOptionsName("-svd_view","Print detailed information on solver used","SVDView",&flg);CHKERRQ(ierr);
 
   ierr = PetscOptionsEList("-svd_transpose_mode","Transpose SVD mode","SVDSetTransposeMode",mode_list,2,svd->transmode == PETSC_DECIDE ? "decide" : mode_list[svd->transmode],&i,&flg);CHKERRQ(ierr);
   if (flg) {
@@ -410,7 +419,7 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
 
   r = i = PETSC_IGNORE;
   ierr = PetscOptionsInt("-svd_max_it","Maximum number of iterations","SVDSetTolerances",svd->max_it,&i,PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-svd_tol","Tolerance","SVDSetTolerances",svd->tol,&r,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-svd_tol","Tolerance","SVDSetTolerances",svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:svd->tol,&r,PETSC_NULL);CHKERRQ(ierr);
   ierr = SVDSetTolerances(svd,r,i);CHKERRQ(ierr);
 
   i = j = k = PETSC_IGNORE;
@@ -419,83 +428,88 @@ PetscErrorCode SVDSetFromOptions(SVD svd)
   ierr = PetscOptionsInt("-svd_mpd","Maximum dimension of projected problem","SVDSetDimensions",svd->mpd,&k,PETSC_NULL);CHKERRQ(ierr);
   ierr = SVDSetDimensions(svd,i,j,k);CHKERRQ(ierr);
 
-  ierr = PetscOptionsTruthGroupBegin("-svd_largest","compute largest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsBoolGroupBegin("-svd_largest","compute largest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
   if (flg) { ierr = SVDSetWhichSingularTriplets(svd,SVD_LARGEST);CHKERRQ(ierr); }
-  ierr = PetscOptionsTruthGroupEnd("-svd_smallest","compute smallest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsBoolGroupEnd("-svd_smallest","compute smallest singular values","SVDSetWhichSingularTriplets",&flg);CHKERRQ(ierr);
   if (flg) { ierr = SVDSetWhichSingularTriplets(svd,SVD_SMALLEST);CHKERRQ(ierr); }
 
   flg = PETSC_FALSE;
-  ierr = PetscOptionsTruth("-svd_monitor_cancel","Remove any hardwired monitor routines","SVDMonitorCancel",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-svd_monitor_cancel","Remove any hardwired monitor routines","SVDMonitorCancel",flg,&flg,PETSC_NULL);CHKERRQ(ierr);
   if (flg) {
     ierr = SVDMonitorCancel(svd);CHKERRQ(ierr); 
   }
 
   ierr = PetscOptionsString("-svd_monitor_all","Monitor approximate singular values and error estimates","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
   if (flg) {
-    ierr = PetscViewerASCIIMonitorCreate(((PetscObject)svd)->comm,monfilename,((PetscObject)svd)->tablevel,&monviewer);CHKERRQ(ierr);
-    ierr = SVDMonitorSet(svd,SVDMonitorAll,monviewer,(PetscErrorCode (*)(void*))PetscViewerASCIIMonitorDestroy);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,monfilename,&monviewer);CHKERRQ(ierr);
+    ierr = SVDMonitorSet(svd,SVDMonitorAll,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
     ierr = SVDSetTrackAll(svd,PETSC_TRUE);CHKERRQ(ierr);
   }
   ierr = PetscOptionsString("-svd_monitor_conv","Monitor approximate singular values and error estimates as they converge","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
   if (flg) {
-      ierr = PetscNew(SVDMONITOR_CONV,&ctx);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIMonitorCreate(((PetscObject)svd)->comm,monfilename,((PetscObject)svd)->tablevel,&ctx->viewer);CHKERRQ(ierr);
-      ierr = SVDMonitorSet(svd,SVDMonitorConverged,ctx,(PetscErrorCode (*)(void*))SVDMonitorDestroy_Converged);CHKERRQ(ierr);
+      ierr = PetscNew(struct _n_SlepcConvMonitor,&ctx);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,monfilename,&ctx->viewer);CHKERRQ(ierr);
+      ierr = SVDMonitorSet(svd,SVDMonitorConverged,ctx,(PetscErrorCode (*)(void**))SlepcConvMonitorDestroy);CHKERRQ(ierr);
   }
   ierr = PetscOptionsString("-svd_monitor","Monitor first unconverged approximate singular value and error estimate","SVDMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
   if (flg) {
-    ierr = PetscViewerASCIIMonitorCreate(((PetscObject)svd)->comm,monfilename,((PetscObject)svd)->tablevel,&monviewer);CHKERRQ(ierr);
-    ierr = SVDMonitorSet(svd,SVDMonitorFirst,monviewer,(PetscErrorCode (*)(void*))PetscViewerASCIIMonitorDestroy);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,monfilename,&monviewer);CHKERRQ(ierr);
+    ierr = SVDMonitorSet(svd,SVDMonitorFirst,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
   }
   flg = PETSC_FALSE;
-  ierr = PetscOptionsTruth("-svd_monitor_draw","Monitor first unconverged approximate singular value and error estimate graphically","SVDMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+  ierr = PetscOptionsBool("-svd_monitor_draw","Monitor first unconverged approximate singular value and error estimate graphically","SVDMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
   if (flg) {
     ierr = SVDMonitorSet(svd,SVDMonitorLG,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
   }
   flg = PETSC_FALSE;
-  ierr = PetscOptionsTruth("-svd_monitor_draw_all","Monitor error estimates graphically","SVDMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
+  ierr = PetscOptionsBool("-svd_monitor_draw_all","Monitor error estimates graphically","SVDMonitorSet",flg,&flg,PETSC_NULL);CHKERRQ(ierr); 
   if (flg) {
     ierr = SVDMonitorSet(svd,SVDMonitorLGAll,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
     ierr = SVDSetTrackAll(svd,PETSC_TRUE);CHKERRQ(ierr);
   }
-
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
-  ierr = IPSetFromOptions(svd->ip);CHKERRQ(ierr);
   if (svd->ops->setfromoptions) {
     ierr = (*svd->ops->setfromoptions)(svd);CHKERRQ(ierr);
   }
+
+  ierr = PetscObjectProcessOptionsHandlers((PetscObject)svd);CHKERRQ(ierr);
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+
+  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
+  ierr = IPSetFromOptions(svd->ip);CHKERRQ(ierr);
+  ierr = PetscRandomSetFromOptions(svd->rand);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDSetTrackAll"
 /*@
-    SVDSetTrackAll - Specifies if the solver must compute the residual norm of all
-    approximate singular value or not.
+   SVDSetTrackAll - Specifies if the solver must compute the residual norm of all
+   approximate singular value or not.
 
-    Collective on SVD
+   Logically Collective on SVD
 
-    Input Parameters:
-+   svd      - the singular value solver context
--   trackall - whether to compute all residuals or not
+   Input Parameters:
++  svd      - the singular value solver context
+-  trackall - whether to compute all residuals or not
 
-    Notes:
-    If the user sets trackall=PETSC_TRUE then the solver computes (or estimates)
-    the residual norm for each singular value approximation. Computing the residual is
-    usually an expensive operation and solvers commonly compute only the residual 
-    associated to the first unconverged singular value.
+   Notes:
+   If the user sets trackall=PETSC_TRUE then the solver computes (or estimates)
+   the residual norm for each singular value approximation. Computing the residual is
+   usually an expensive operation and solvers commonly compute only the residual 
+   associated to the first unconverged singular value.
 
-    The options '-svd_monitor_all' and '-svd_monitor_draw_all' automatically
-    activate this option.
+   The options '-svd_monitor_all' and '-svd_monitor_draw_all' automatically
+   activate this option.
 
-    Level: intermediate
+   Level: intermediate
 
 .seealso: SVDGetTrackAll()
 @*/
-PetscErrorCode SVDSetTrackAll(SVD svd,PetscTruth trackall)
+PetscErrorCode SVDSetTrackAll(SVD svd,PetscBool trackall)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveBool(svd,trackall,2);
   svd->trackall = trackall;
   PetscFunctionReturn(0);
 }
@@ -503,25 +517,25 @@ PetscErrorCode SVDSetTrackAll(SVD svd,PetscTruth trackall)
 #undef __FUNCT__  
 #define __FUNCT__ "SVDGetTrackAll"
 /*@
-    SVDGetTrackAll - Returns the flag indicating whether all residual norms must
-    be computed or not.
+   SVDGetTrackAll - Returns the flag indicating whether all residual norms must
+   be computed or not.
 
-    Not Collective
+   Not Collective
 
-    Input Parameter:
-.   svd - the singular value solver context
+   Input Parameter:
+.  svd - the singular value solver context
 
-    Output Parameter:
-.   trackall - the returned flag
+   Output Parameter:
+.  trackall - the returned flag
 
-    Level: intermediate
+   Level: intermediate
 
 .seealso: SVDSetTrackAll()
 @*/
-PetscErrorCode SVDGetTrackAll(SVD svd,PetscTruth *trackall) 
+PetscErrorCode SVDGetTrackAll(SVD svd,PetscBool *trackall) 
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(trackall,2);
   *trackall = svd->trackall;
   PetscFunctionReturn(0);
@@ -534,7 +548,7 @@ PetscErrorCode SVDGetTrackAll(SVD svd,PetscTruth *trackall)
    SVDSetOptionsPrefix - Sets the prefix used for searching for all 
    SVD options in the database.
 
-   Collective on SVD
+   Logically Collective on SVD
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -559,11 +573,13 @@ PetscErrorCode SVDGetTrackAll(SVD svd,PetscTruth *trackall)
 PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg1,flg2;
+  PetscBool      flg1,flg2;
   EPS            eps;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
+  ierr = IPSetOptionsPrefix(svd->ip,prefix);CHKERRQ(ierr);
   ierr = PetscObjectSetOptionsPrefix((PetscObject)svd,prefix);CHKERRQ(ierr);
   ierr = PetscTypeCompare((PetscObject)svd,SVDCROSS,&flg1);CHKERRQ(ierr);
   ierr = PetscTypeCompare((PetscObject)svd,SVDCYCLIC,&flg2);CHKERRQ(ierr);
@@ -576,8 +592,6 @@ PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
     ierr = EPSSetOptionsPrefix(eps,prefix);CHKERRQ(ierr);
     ierr = EPSAppendOptionsPrefix(eps,"svd_");CHKERRQ(ierr);
   }
-  ierr = IPSetOptionsPrefix(svd->ip,prefix);CHKERRQ(ierr);
-  ierr = IPAppendOptionsPrefix(svd->ip,"svd_");CHKERRQ(ierr);
   PetscFunctionReturn(0);  
 }
 
@@ -587,7 +601,7 @@ PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
    SVDAppendOptionsPrefix - Appends to the prefix used for searching for all 
    SVD options in the database.
 
-   Collective on SVD
+   Logically Collective on SVD
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -604,11 +618,13 @@ PetscErrorCode SVDSetOptionsPrefix(SVD svd,const char *prefix)
 PetscErrorCode SVDAppendOptionsPrefix(SVD svd,const char *prefix)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg1,flg2;
+  PetscBool      flg1,flg2;
   EPS            eps;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
+  ierr = IPSetOptionsPrefix(svd->ip,prefix);CHKERRQ(ierr);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)svd,prefix);CHKERRQ(ierr);
   ierr = PetscTypeCompare((PetscObject)svd,SVDCROSS,&flg1);CHKERRQ(ierr);
   ierr = PetscTypeCompare((PetscObject)svd,SVDCYCLIC,&flg2);CHKERRQ(ierr);
@@ -621,8 +637,6 @@ PetscErrorCode SVDAppendOptionsPrefix(SVD svd,const char *prefix)
     ierr = EPSSetOptionsPrefix(eps,((PetscObject)svd)->prefix);CHKERRQ(ierr);
     ierr = EPSAppendOptionsPrefix(eps,"svd_");CHKERRQ(ierr);
   }
-  ierr = IPSetOptionsPrefix(svd->ip,((PetscObject)svd)->prefix);CHKERRQ(ierr);
-  ierr = IPAppendOptionsPrefix(svd->ip,"svd_");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -650,8 +664,9 @@ PetscErrorCode SVDAppendOptionsPrefix(SVD svd,const char *prefix)
 PetscErrorCode SVDGetOptionsPrefix(SVD svd,const char *prefix[])
 {
   PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(prefix,2);
   ierr = PetscObjectGetOptionsPrefix((PetscObject)svd,prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
