@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -19,43 +19,37 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"  /*I "slepcsvd.h" I*/
+#include <private/svdimpl.h>       /*I "slepcsvd.h" I*/
 
 EXTERN_C_BEGIN
-EXTERN PetscErrorCode SVDCreate_CROSS(SVD);
-EXTERN PetscErrorCode SVDCreate_CYCLIC(SVD);
-EXTERN PetscErrorCode SVDCreate_LAPACK(SVD);
-EXTERN PetscErrorCode SVDCreate_LANCZOS(SVD);
-EXTERN PetscErrorCode SVDCreate_TRLANCZOS(SVD);
+extern PetscErrorCode SVDCreate_Cross(SVD);
+extern PetscErrorCode SVDCreate_Cyclic(SVD);
+extern PetscErrorCode SVDCreate_LAPACK(SVD);
+extern PetscErrorCode SVDCreate_Lanczos(SVD);
+extern PetscErrorCode SVDCreate_TRLanczos(SVD);
 EXTERN_C_END
   
 #undef __FUNCT__  
 #define __FUNCT__ "SVDRegisterAll"
 /*@C
-  SVDRegisterAll - Registers all the singular value solvers in the SVD package.
+   SVDRegisterAll - Registers all the singular value solvers in the SVD package.
 
-  Not Collective
+   Not Collective
 
-  Level: advanced
+   Level: advanced
 
 .seealso:  SVDRegisterDynamic()
 @*/
-PetscErrorCode SVDRegisterAll(char *path)
+PetscErrorCode SVDRegisterAll(const char *path)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  
-  ierr = SVDRegisterDynamic(SVDCROSS, path,"SVDCreate_CROSS", 
-		  SVDCreate_CROSS);CHKERRQ(ierr);
-  ierr = SVDRegisterDynamic(SVDCYCLIC, path,"SVDCreate_CYCLIC", 
-		  SVDCreate_CYCLIC);CHKERRQ(ierr);
-  ierr = SVDRegisterDynamic(SVDLAPACK, path,"SVDCreate_LAPACK", 
-		  SVDCreate_LAPACK);CHKERRQ(ierr);
-  ierr = SVDRegisterDynamic(SVDLANCZOS, path,"SVDCreate_LANCZOS", 
-		  SVDCreate_LANCZOS);CHKERRQ(ierr);
-  ierr = SVDRegisterDynamic(SVDTRLANCZOS, path,"SVDCreate_TRLANCZOS", 
-		  SVDCreate_TRLANCZOS);CHKERRQ(ierr);
-  
+  SVDRegisterAllCalled = PETSC_TRUE;
+  ierr = SVDRegisterDynamic(SVDCROSS,path,"SVDCreate_Cross",SVDCreate_Cross);CHKERRQ(ierr);
+  ierr = SVDRegisterDynamic(SVDCYCLIC,path,"SVDCreate_Cyclic",SVDCreate_Cyclic);CHKERRQ(ierr);
+  ierr = SVDRegisterDynamic(SVDLAPACK,path,"SVDCreate_LAPACK",SVDCreate_LAPACK);CHKERRQ(ierr);
+  ierr = SVDRegisterDynamic(SVDLANCZOS,path,"SVDCreate_Lanczos",SVDCreate_Lanczos);CHKERRQ(ierr);
+  ierr = SVDRegisterDynamic(SVDTRLANCZOS,path,"SVDCreate_TRLanczoS",SVDCreate_TRLanczos);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
