@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,8 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"      /*I "slepcsvd.h" I*/
+#include <private/svdimpl.h>      /*I "slepcsvd.h" I*/
+#include <private/ipimpl.h>
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDSetOperator"
@@ -43,15 +44,13 @@ PetscErrorCode SVDSetOperator(SVD svd,Mat mat)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  PetscValidHeaderSpecific(mat,MAT_COOKIE,2);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,2);
   PetscCheckSameComm(svd,1,mat,2);
+  if (svd->setupcalled) { ierr = SVDReset(svd);CHKERRQ(ierr); }
   ierr = PetscObjectReference((PetscObject)mat);CHKERRQ(ierr);
-  if (svd->OP) {
-    ierr = MatDestroy(svd->OP);CHKERRQ(ierr);
-  }
+  ierr = MatDestroy(&svd->OP);CHKERRQ(ierr);
   svd->OP = mat;
-  svd->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
@@ -75,7 +74,7 @@ PetscErrorCode SVDSetOperator(SVD svd,Mat mat)
 PetscErrorCode SVDGetOperator(SVD svd,Mat *A)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidPointer(A,2);
   *A = svd->OP;
   PetscFunctionReturn(0);
@@ -104,24 +103,29 @@ PetscErrorCode SVDGetOperator(SVD svd,Mat *A)
 PetscErrorCode SVDSetUp(SVD svd)
 {
   PetscErrorCode ierr;
-  PetscTruth     flg,lindep;
-  PetscInt       i,k,M,N,nloc;
-  PetscScalar    *pV;
+  PetscBool      flg,lindep;
+  PetscInt       i,k,M,N;
   PetscReal      norm;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (svd->setupcalled) PetscFunctionReturn(0);
   ierr = PetscLogEventBegin(SVD_SetUp,svd,0,0,0);CHKERRQ(ierr);
 
-  /* Set default solver type */
+  /* Set default solver type (SVDSetFromOptions was not called) */
   if (!((PetscObject)svd)->type_name) {
     ierr = SVDSetType(svd,SVDCROSS);CHKERRQ(ierr);
   }
+  if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
+  if (!((PetscObject)svd->ip)->type_name) {
+    ierr = IPSetDefaultType_Private(svd->ip);CHKERRQ(ierr);
+  }
+  if (!((PetscObject)svd->rand)->type_name) {
+    ierr = PetscRandomSetFromOptions(svd->rand);CHKERRQ(ierr);
+  }
 
   /* check matrix */
-  if (!svd->OP)
-    SETERRQ(PETSC_ERR_ARG_WRONGSTATE, "SVDSetOperator must be called first"); 
+  if (!svd->OP) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSetOperator must be called first"); 
   
   /* determine how to build the transpose */
   if (svd->transmode == PETSC_DECIDE) {
@@ -131,25 +135,25 @@ PetscErrorCode SVDSetUp(SVD svd)
   }
   
   /* build transpose matrix */
-  if (svd->A) { ierr = MatDestroy(svd->A);CHKERRQ(ierr); }
-  if (svd->AT) { ierr = MatDestroy(svd->AT);CHKERRQ(ierr); }
+  ierr = MatDestroy(&svd->A);CHKERRQ(ierr);
+  ierr = MatDestroy(&svd->AT);CHKERRQ(ierr);
   ierr = MatGetSize(svd->OP,&M,&N);CHKERRQ(ierr);
   ierr = PetscObjectReference((PetscObject)svd->OP);CHKERRQ(ierr);
   switch (svd->transmode) {
     case SVD_TRANSPOSE_EXPLICIT:
       ierr = MatHasOperation(svd->OP,MATOP_TRANSPOSE,&flg);CHKERRQ(ierr);
-      if (!flg) SETERRQ(1,"Matrix has not defined the MatTranpose operation");
+      if (!flg) SETERRQ(((PetscObject)svd)->comm,1,"Matrix has not defined the MatTranpose operation");
       if (M>=N) {
         svd->A = svd->OP;
-        ierr = MatTranspose(svd->OP, MAT_INITIAL_MATRIX,&svd->AT);CHKERRQ(ierr);
+        ierr = MatTranspose(svd->OP,MAT_INITIAL_MATRIX,&svd->AT);CHKERRQ(ierr);
       } else {
-        ierr = MatTranspose(svd->OP, MAT_INITIAL_MATRIX,&svd->A);CHKERRQ(ierr);
+        ierr = MatTranspose(svd->OP,MAT_INITIAL_MATRIX,&svd->A);CHKERRQ(ierr);
         svd->AT = svd->OP;
       }
       break;
     case SVD_TRANSPOSE_IMPLICIT:
       ierr = MatHasOperation(svd->OP,MATOP_MULT_TRANSPOSE,&flg);CHKERRQ(ierr);
-      if (!flg) SETERRQ(1,"Matrix has not defined the MatMultTranpose operation");
+      if (!flg) SETERRQ(((PetscObject)svd)->comm,1,"Matrix has not defined the MatMultTranpose operation");
       if (M>=N) {
         svd->A = svd->OP;
         svd->AT = PETSC_NULL;    
@@ -159,20 +163,27 @@ PetscErrorCode SVDSetUp(SVD svd)
       }
       break;
     default:
-      SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid transpose mode"); 
+      SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Invalid transpose mode"); 
   }
 
-  /* initialize the random number generator */
-  ierr = PetscRandomCreate(((PetscObject)svd)->comm,&svd->rand);CHKERRQ(ierr);
-  ierr = PetscRandomSetFromOptions(svd->rand);CHKERRQ(ierr);
+  ierr = VecDestroy(&svd->tr);CHKERRQ(ierr);
+  ierr = VecDestroy(&svd->tl);CHKERRQ(ierr);
+  if (svd->A) {
+    ierr = SlepcMatGetVecsTemplate(svd->A,&svd->tr,&svd->tl);CHKERRQ(ierr);
+  } else {
+    ierr = SlepcMatGetVecsTemplate(svd->AT,&svd->tl,&svd->tr);CHKERRQ(ierr);
+  }
 
   /* call specific solver setup */
   ierr = (*svd->ops->setup)(svd);CHKERRQ(ierr);
 
+  /* set tolerance if not yet set */
+  if (svd->tol==PETSC_DEFAULT) svd->tol = SLEPC_DEFAULT_TOL;
+
   if (svd->ncv > M || svd->ncv > N)
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"ncv bigger than matrix dimensions");
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"ncv bigger than matrix dimensions");
   if (svd->nsv > svd->ncv)
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"nsv bigger than ncv");
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"nsv bigger than ncv");
 
   if (svd->ncv != svd->n) {
     /* free memory for previous solution  */
@@ -180,38 +191,24 @@ PetscErrorCode SVDSetUp(SVD svd)
       ierr = PetscFree(svd->sigma);CHKERRQ(ierr);
       ierr = PetscFree(svd->perm);CHKERRQ(ierr);
       ierr = PetscFree(svd->errest);CHKERRQ(ierr);
-      ierr = VecGetArray(svd->V[0],&pV);CHKERRQ(ierr);
-      for (i=0;i<svd->n;i++) {
-        ierr = VecDestroy(svd->V[i]);CHKERRQ(ierr);
-      }
-      ierr = PetscFree(pV);CHKERRQ(ierr);
-      ierr = PetscFree(svd->V);CHKERRQ(ierr);
+      ierr = VecDestroyVecs(svd->n,&svd->V);CHKERRQ(ierr);
     }
     /* allocate memory for next solution */
     ierr = PetscMalloc(svd->ncv*sizeof(PetscReal),&svd->sigma);CHKERRQ(ierr);
     ierr = PetscMalloc(svd->ncv*sizeof(PetscInt),&svd->perm);CHKERRQ(ierr);
     ierr = PetscMalloc(svd->ncv*sizeof(PetscReal),&svd->errest);CHKERRQ(ierr);
-    ierr = PetscMalloc(svd->ncv*sizeof(Vec),&svd->V);CHKERRQ(ierr);
-    if (svd->A) {
-      ierr = MatGetLocalSize(svd->A,PETSC_NULL,&nloc);CHKERRQ(ierr);
-    } else {
-      ierr = MatGetLocalSize(svd->AT,&nloc,PETSC_NULL);CHKERRQ(ierr);
-    }
-    ierr = PetscMalloc(svd->ncv*nloc*sizeof(PetscScalar),&pV);CHKERRQ(ierr);
-    for (i=0;i<svd->ncv;i++) {
-      ierr = VecCreateMPIWithArray(((PetscObject)svd)->comm,nloc,PETSC_DECIDE,pV+i*nloc,&svd->V[i]);CHKERRQ(ierr);
-    }
+    ierr = VecDuplicateVecs(svd->tr,svd->ncv,&svd->V);CHKERRQ(ierr);
     svd->n = svd->ncv;
   }
 
   /* process initial vectors */
   if (svd->nini<0) {
     svd->nini = -svd->nini;
-    if (svd->nini>svd->ncv) SETERRQ(1,"The number of initial vectors is larger than ncv")
+    if (svd->nini>svd->ncv) SETERRQ(((PetscObject)svd)->comm,1,"The number of initial vectors is larger than ncv");
     k = 0;
     for (i=0;i<svd->nini;i++) {
       ierr = VecCopy(svd->IS[i],svd->V[k]);CHKERRQ(ierr);
-      ierr = VecDestroy(svd->IS[i]);CHKERRQ(ierr);
+      ierr = VecDestroy(&svd->IS[i]);CHKERRQ(ierr);
       ierr = IPOrthogonalize(svd->ip,0,PETSC_NULL,k,PETSC_NULL,svd->V,svd->V[k],PETSC_NULL,&norm,&lindep);CHKERRQ(ierr); 
       if (norm==0.0 || lindep) PetscInfo(svd,"Linearly dependent initial vector found, removing...\n");
       else {
@@ -262,13 +259,14 @@ PetscErrorCode SVDSetInitialSpace(SVD svd,PetscInt n,Vec *is)
   PetscInt       i;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  if (n<0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveInt(svd,n,2);
+  if (n<0) SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative"); 
 
   /* free previous non-processed vectors */
   if (svd->nini<0) {
     for (i=0;i<-svd->nini;i++) {
-      ierr = VecDestroy(svd->IS[i]);CHKERRQ(ierr);
+      ierr = VecDestroy(&svd->IS[i]);CHKERRQ(ierr);
     }
     ierr = PetscFree(svd->IS);CHKERRQ(ierr);
   }
