@@ -3,7 +3,7 @@
 
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2010, Universidad Politecnica de Valencia, Spain
+   Copyright (c) 2002-2011, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
       
@@ -21,7 +21,7 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "private/svdimpl.h"   /*I "slepcsvd.h" I*/
+#include <private/svdimpl.h>   /*I "slepcsvd.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "SVDSolve"
@@ -43,22 +43,19 @@
 PetscErrorCode SVDSolve(SVD svd) 
 {
   PetscErrorCode ierr;
-  PetscTruth     flg;
+  PetscBool      flg;
   PetscInt       i,*workperm;
   char           filename[PETSC_MAX_PATH_LEN];
   PetscViewer    viewer;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (!svd->setupcalled) { ierr = SVDSetUp(svd);CHKERRQ(ierr); }
   svd->its = 0;
-  svd->matvecs = 0;
   svd->nconv = 0;
   svd->reason = SVD_CONVERGED_ITERATING;
-  ierr = IPResetOperationCounters(svd->ip);CHKERRQ(ierr);
   for (i=0;i<svd->ncv;i++) svd->sigma[i]=svd->errest[i]=0.0;
-  SVDMonitor(svd,svd->its,svd->nconv,svd->sigma,svd->errest,svd->ncv);
+  ierr = SVDMonitor(svd,svd->its,svd->nconv,svd->sigma,svd->errest,svd->ncv);CHKERRQ(ierr);
 
   ierr = PetscLogEventBegin(SVD_Solve,svd,0,0,0);CHKERRQ(ierr);
   ierr = (*svd->ops->solve)(svd);CHKERRQ(ierr);
@@ -80,12 +77,11 @@ PetscErrorCode SVDSolve(SVD svd)
   if (flg && !PetscPreLoadingOn) {
     ierr = PetscViewerASCIIOpen(((PetscObject)svd)->comm,filename,&viewer);CHKERRQ(ierr);
     ierr = SVDView(svd,viewer);CHKERRQ(ierr); 
-    ierr = PetscViewerDestroy(viewer);CHKERRQ(ierr);
+    ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   }
 
   /* Remove the initial subspace */
   svd->nini = 0;
-
   PetscFunctionReturn(0);
 }
 
@@ -117,7 +113,7 @@ PetscErrorCode SVDSolve(SVD svd)
 PetscErrorCode SVDGetIterationNumber(SVD svd,PetscInt *its)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidIntPointer(its,2);
   *its = svd->its;
   PetscFunctionReturn(0);
@@ -152,7 +148,7 @@ PetscErrorCode SVDGetIterationNumber(SVD svd,PetscInt *its)
 PetscErrorCode SVDGetConvergedReason(SVD svd,SVDConvergedReason *reason)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidIntPointer(reason,2);
   *reason = svd->reason;
   PetscFunctionReturn(0);
@@ -180,10 +176,10 @@ PetscErrorCode SVDGetConvergedReason(SVD svd,SVDConvergedReason *reason)
 PetscErrorCode SVDGetConverged(SVD svd,PetscInt *nconv)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidIntPointer(nconv,2);
   if (svd->reason == SVD_CONVERGED_ITERATING) { 
-    SETERRQ(PETSC_ERR_ARG_WRONGSTATE, "SVDSolve must be called first"); 
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first"); 
   }
   *nconv = svd->nconv;
   PetscFunctionReturn(0);
@@ -196,7 +192,7 @@ PetscErrorCode SVDGetConverged(SVD svd,PetscInt *nconv)
    as computed by SVDSolve(). The solution consists in the singular value and its left 
    and right singular vectors.
 
-   Not Collective
+   Not Collective, but vectors are shared by all processors that share the SVD
 
    Input Parameters:
 +  svd - singular value solver context 
@@ -215,35 +211,29 @@ PetscErrorCode SVDGetConverged(SVD svd,PetscInt *nconv)
 
 .seealso: SVDSolve(),  SVDGetConverged()
 @*/
-PetscErrorCode SVDGetSingularTriplet(SVD svd, PetscInt i, PetscReal *sigma, Vec u, Vec v)
+PetscErrorCode SVDGetSingularTriplet(SVD svd,PetscInt i,PetscReal *sigma,Vec u,Vec v)
 {
   PetscErrorCode ierr;
   PetscReal      norm;
-  PetscInt       j,nloc,M,N;
-  PetscScalar    *pU;
+  PetscInt       j,M,N;
   Vec            w;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  PetscValidPointer(sigma,3);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  if (u) { PetscValidHeaderSpecific(u,VEC_CLASSID,4); PetscCheckSameComm(svd,1,u,4); }
+  if (v) { PetscValidHeaderSpecific(v,VEC_CLASSID,5); PetscCheckSameComm(svd,1,v,5); }
   if (svd->reason == SVD_CONVERGED_ITERATING) { 
-    SETERRQ(PETSC_ERR_ARG_WRONGSTATE, "SVDSolve must be called first"); 
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first"); 
   }
   if (i<0 || i>=svd->nconv) { 
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE, "Argument 2 out of range"); 
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range"); 
   }
   *sigma = svd->sigma[svd->perm[i]];
   ierr = MatGetSize(svd->OP,&M,&N);CHKERRQ(ierr);
   if (M<N) { w = u; u = v; v = w; }
   if (u) {
-    PetscValidHeaderSpecific(u,VEC_COOKIE,4);
     if (!svd->U) {
-      ierr = PetscMalloc(sizeof(Vec)*svd->ncv,&svd->U);CHKERRQ(ierr);
-      ierr = SVDMatGetLocalSize(svd,&nloc,PETSC_NULL);CHKERRQ(ierr);
-      ierr = PetscMalloc(svd->ncv*nloc*sizeof(PetscScalar),&pU);CHKERRQ(ierr);
-      for (j=0;j<svd->ncv;j++) {
-        ierr = VecCreateMPIWithArray(((PetscObject)svd)->comm,nloc,PETSC_DECIDE,pU+j*nloc,&svd->U[j]);CHKERRQ(ierr);
-      }
+      ierr = VecDuplicateVecs(svd->tl,svd->ncv,&svd->U);CHKERRQ(ierr);
       for (j=0;j<svd->nconv;j++) {
         ierr = SVDMatMult(svd,PETSC_FALSE,svd->V[j],svd->U[j]);CHKERRQ(ierr);
         ierr = IPOrthogonalize(svd->ip,0,PETSC_NULL,j,PETSC_NULL,svd->U,svd->U[j],PETSC_NULL,&norm,PETSC_NULL);CHKERRQ(ierr);
@@ -253,7 +243,6 @@ PetscErrorCode SVDGetSingularTriplet(SVD svd, PetscInt i, PetscReal *sigma, Vec
     ierr = VecCopy(svd->U[svd->perm[i]],u);CHKERRQ(ierr);
   }
   if (v) {
-    PetscValidHeaderSpecific(v,VEC_COOKIE,5);   
     ierr = VecCopy(svd->V[svd->perm[i]],v);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -284,7 +273,7 @@ PetscErrorCode SVDGetSingularTriplet(SVD svd, PetscInt i, PetscReal *sigma, Vec
 
 .seealso: SVDSolve(), SVDGetConverged(), SVDComputeRelativeError()
 @*/
-PetscErrorCode SVDComputeResidualNorms(SVD svd, PetscInt i, PetscReal *norm1, PetscReal *norm2)
+PetscErrorCode SVDComputeResidualNorms(SVD svd,PetscInt i,PetscReal *norm1,PetscReal *norm2)
 {
   PetscErrorCode ierr;
   Vec            u,v,x = PETSC_NULL,y = PETSC_NULL;
@@ -292,12 +281,13 @@ PetscErrorCode SVDComputeResidualNorms(SVD svd, PetscInt i, PetscReal *norm1, Pe
   PetscInt       M,N;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveInt(svd,i,2);
   if (svd->reason == SVD_CONVERGED_ITERATING) { 
-    SETERRQ(PETSC_ERR_ARG_WRONGSTATE, "SVDSolve must be called first"); 
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_WRONGSTATE,"SVDSolve must be called first"); 
   }
   if (i<0 || i>=svd->nconv) { 
-    SETERRQ(PETSC_ERR_ARG_OUTOFRANGE, "Argument 2 out of range"); 
+    SETERRQ(((PetscObject)svd)->comm,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range"); 
   }
   
   ierr = MatGetVecs(svd->OP,&v,&u);CHKERRQ(ierr);
@@ -324,10 +314,10 @@ PetscErrorCode SVDComputeResidualNorms(SVD svd, PetscInt i, PetscReal *norm1, Pe
     ierr = VecNorm(y,NORM_2,norm2);CHKERRQ(ierr);
   }
 
-  ierr = VecDestroy(v);CHKERRQ(ierr);
-  ierr = VecDestroy(u);CHKERRQ(ierr);
-  if (x) { ierr = VecDestroy(x);CHKERRQ(ierr); }
-  if (y) { ierr = VecDestroy(y);CHKERRQ(ierr); }
+  ierr = VecDestroy(&v);CHKERRQ(ierr);
+  ierr = VecDestroy(&u);CHKERRQ(ierr);
+  ierr = VecDestroy(&x);CHKERRQ(ierr);
+  ierr = VecDestroy(&y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -359,11 +349,12 @@ PetscErrorCode SVDComputeRelativeError(SVD svd,PetscInt i,PetscReal *error)
   PetscReal      sigma,norm1,norm2;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
-  PetscValidPointer(error,2);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
+  PetscValidLogicalCollectiveInt(svd,i,2);
+  PetscValidPointer(error,3);
   ierr = SVDGetSingularTriplet(svd,i,&sigma,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
   ierr = SVDComputeResidualNorms(svd,i,&norm1,&norm2);CHKERRQ(ierr);
-  *error = sqrt(norm1*norm1+norm2*norm2);
+  *error = PetscSqrtReal(norm1*norm1+norm2*norm2);
   if (sigma>*error) *error /= sigma;
   PetscFunctionReturn(0);
 }
@@ -394,9 +385,10 @@ PetscErrorCode SVDGetOperationCounters(SVD svd,PetscInt* matvecs,PetscInt* dots)
   PetscErrorCode ierr;
   
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(svd,SVD_COOKIE,1);
+  PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   if (matvecs) *matvecs = svd->matvecs; 
   if (dots) {
+    if (!svd->ip) { ierr = SVDGetIP(svd,&svd->ip);CHKERRQ(ierr); }
     ierr = IPGetOperationCounters(svd->ip,dots);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
